/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2003, 06, 07 by Ralf Baechle (ralf@linux-mips.org)
 */
#ifndef __ASM_CMPXCHG_H
#define __ASM_CMPXCHG_H

#include <linux/irqflags.h>
#include <asm-generic/cmpxchg-local.h>

#define __HAVE_ARCH_CMPXCHG 1

#define __cmpxchg_asm(m, old, new)					\
({									\
	__typeof(*(m)) __ret;						\
									\
	unsigned long __flags;						\
									\
	raw_local_irq_save(__flags);					\
	__ret = *m;							\
	if (__ret == old)						\
		*m = new;						\
	raw_local_irq_restore(__flags);					\
									\
	__ret;								\
})

/*
 * This function doesn't exist, so you'll get a linker error
 * if something tries to do an invalid cmpxchg().
 */
extern void __cmpxchg_called_with_bad_pointer(void);

#define __cmpxchg(ptr, old, new, barrier)				\
({									\
	__typeof__(ptr) __ptr = (ptr);					\
	__typeof__(*(ptr)) __old = (old);				\
	__typeof__(*(ptr)) __new = (new);				\
	__typeof__(*(ptr)) __res = 0;					\
									\
	barrier;							\
									\
	switch (sizeof(*(__ptr))) {					\
        case 4:
		__res = __cmpxchg_asm(__ptr, __old, __new);		\
		break;							\
	default:							\
		__cmpxchg_called_with_bad_pointer();			\
		break;							\
	}								\
									\
	barrier;							\
									\
	__res;								\
})

#define cmpxchg(ptr, old, new)		__cmpxchg(ptr, old, new, )
#define cmpxchg_local(ptr, old, new)	__cmpxchg(ptr, old, new, )

#endif /* __ASM_CMPXCHG_H */
