#ifndef __AEONR2_DMA_MAPPING_H__
#define __AEONR2_DMA_MAPPING_H__

#include <asm/scatterlist.h>
#include <asm/cacheflush.h>
#include <asm/io.h>

static inline int dma_supported(struct device *dev, u64 mask)
{
	return 0;
}

static inline int dma_set_mask(struct device *dev, u64 dma_mask)
{
	BUG();
	return 0;
}

#define dma_alloc_noncoherent(d, s, h, f) dma_alloc_coherent(d, s, h, f)
#define dma_free_noncoherent(d, s, v, h) dma_free_coherent(d, s, v, h)
#define dma_is_consistent(d, h) (1)

void *dma_alloc_coherent(struct device *dev, size_t size,
                         dma_addr_t *dma_handle, gfp_t flag);

void dma_free_coherent(struct device *dev, size_t size,
                       void *cpu_addr, dma_addr_t dma_handle);

static inline dma_addr_t dma_map_single(struct device *dev, void *cpu_addr,
                                        size_t size,
	                                enum dma_data_direction direction)
{
	/* Assumes that the proc. does memory access in-order */
	return virt_to_phys(cpu_addr);
}

static inline void dma_unmap_single(struct device *dev, dma_addr_t dma_addr,
                                    size_t size,
		                    enum dma_data_direction direction)
{
	if (direction == DMA_FROM_DEVICE) {
		/* The device DMA-d some memory someplace .. flush the cache */
		inv_dcache_phys_range(dma_addr, size);
	} else if (direction == DMA_TO_DEVICE) {
		flush_dcache_phys_range(dma_addr, size);
	} else if (direction == DMA_BIDIRECTIONAL) {
		flush_inv_dcache_phys_range(dma_addr, size);
	}
}

static inline dma_addr_t dma_map_page(struct device *dev, struct page *page,
                                      unsigned long offset, size_t size,
                                      enum dma_data_direction direction)
{
	BUG();
	return 0;
}

static inline void dma_unmap_page(struct device *dev, dma_addr_t dma_address,
                                  size_t size,
	                          enum dma_data_direction direction)
{
	BUG();
}

static inline int dma_map_sg(struct device *dev, struct scatterlist *sg,
                             int nents, enum dma_data_direction direction)
{
	BUG();
	return 0;
}

static inline void dma_unmap_sg(struct device *dev, struct scatterlist *sg,
                                int nhwentries,
	                        enum dma_data_direction direction)
{
	BUG();
}

static inline void dma_sync_single_for_cpu(struct device *dev,
                                           dma_addr_t dma_addr, size_t size,
			                   enum dma_data_direction direction)
{
	if (direction == DMA_FROM_DEVICE) {
		/* The device DMA-d some memory someplace .. flush the cache */
		inv_dcache_phys_range(dma_addr, size);
	} else if (direction == DMA_TO_DEVICE) {
		flush_dcache_phys_range(dma_addr, size);
	} else if (direction == DMA_BIDIRECTIONAL) {
		flush_inv_dcache_phys_range(dma_addr, size);
	}
}

static inline void dma_sync_single_for_device(struct device *dev,
                                              dma_addr_t dma_addr,
                                              size_t size,
			                      enum dma_data_direction direction)
{
	/* Assumes that the proc. does memory access in-order */
}

static inline void dma_sync_sg_for_cpu(struct device *dev,
                                       struct scatterlist *sg, int nelems,
		                       enum dma_data_direction direction)
{
	BUG();
}

static inline void dma_sync_sg_for_device(struct device *dev,
                                          struct scatterlist *sg, int nelems,
		                          enum dma_data_direction direction)
{
	BUG();
}

static inline int dma_error(dma_addr_t dma_addr)
{
	return 0;
}

#endif /* __AEONR2_DMA_MAPPING_H__ */
