#ifndef __AEONR2_FLAT_H__
#define __AEONR2_FLAT_H__

#include <asm/unaligned.h>

#define	flat_stack_align(sp)		/* nothing needed */
#define	flat_argvp_envp_on_stack()	0
#define	flat_old_ram_flag(flags)	(flags)
#define	flat_reloc_valid(reloc, size)	((reloc) <= (size))

#define BA_FLAG_MASK	0xf0000000

#define BA_CONST	0x10000000
#define BA_CONSTH	0x20000000
#define BA_CONSTH_S	0x30000000
#define BA_CONST_SPLIT	0x40000000
#define BA_32		0x50000000

struct {
	unsigned long value;
	unsigned short *addr;
	char handled;
	char s;
} hi_part;

/* Convert a relocation entry into an address.  */
static inline unsigned long flat_get_relocate_addr (unsigned long relval) {
	return relval & ~BA_FLAG_MASK;
}

long flat_get_addr_from_rp(unsigned long *ptr, unsigned long relval, unsigned long flags) {
	unsigned short *usptr = (unsigned short *)ptr;
	unsigned long type = relval & BA_FLAG_MASK;
	unsigned long val = 0;

	switch (type) {
		case BA_32:
			val = get_unaligned(ptr) << 4;
			val = (long)val >> 4;
			pr_debug("*ptr = %lx", val);
			break;
		case BA_CONSTH_S:
			hi_part.s = 1;
			hi_part.value = get_unaligned(usptr) << 16;
			hi_part.addr = usptr;
			hi_part.handled = 0;
			pr_debug("*usptr = %lx", val);
			break;
		case BA_CONSTH:
			hi_part.s = 0;
			hi_part.value = get_unaligned(usptr) << 16;
			hi_part.addr = usptr;
			hi_part.handled = 0;
			pr_debug("*usptr = %lx", val);
			break;
		case BA_CONST:
			val = hi_part.value + get_unaligned(usptr);
			pr_debug("*usptr = %lx", val);
			break;
		case BA_CONST_SPLIT:
			val = hi_part.value + ((get_unaligned(ptr) & 0x03e00000) >> 10) + (get_unaligned(ptr) & 0x7ff);
			pr_debug("*usptr = %lx", val);
			break;
		default:
			pr_debug("BINFMT_FLAT: Unknown relocation type %lx\n", type);
			return 0;
	}

	return val;
}

void flat_put_addr_at_rp(unsigned long *ptr, unsigned long addr, unsigned long relval) {
	unsigned short *usptr = (unsigned short *)ptr;
	unsigned long type = relval & BA_FLAG_MASK;
	unsigned long val;

	switch (type) {
		case BA_32:
			put_unaligned(addr, ptr);
			pr_debug("new value %lx", get_unaligned(ptr));
			break;
		case BA_CONST:
			if(!hi_part.handled) {
				if(hi_part.s && (addr & 0x8000))
					addr += 0x10000;
				put_unaligned((addr >> 16) & 0xffff, hi_part.addr);
				pr_debug("new value %x", get_unaligned(hi_part.addr));
				hi_part.handled = 1;
			}
			put_unaligned(addr & 0xffff, usptr);
			pr_debug("new value %x", get_unaligned(usptr));
			break;
		case BA_CONST_SPLIT:
			if(!hi_part.handled) {
				if(hi_part.s && (addr & 0x8000))
					addr += 0x10000;
				put_unaligned((addr >> 16) & 0xffff, hi_part.addr);
				pr_debug("new value %x", get_unaligned(hi_part.addr));
				hi_part.handled = 1;
			}
			val = get_unaligned(ptr) & 0xfc1ff800;
			put_unaligned(val + ((addr << 10) & 0x03e00000) + (addr & 0x7ff), ptr);
			pr_debug("new value %lx", get_unaligned(ptr));
			break;
		default:
			pr_debug("BINFMT_FLAT: Unknown relocation type %lx\n", type);
			return;
	}
}

#endif /* __AEONR2_FLAT_H__ */
