#ifndef __AEONR2_STRING_H__
#define __AEONR2_STRING_H__

#define __HAVE_ARCH_MEMCPY
extern void *memcpy(void *, const void *, size_t);

#define __HAVE_ARCH_MEMSET
extern void *__memset_generic(void *s, int c, size_t count);
extern void *__memset_aligned32(void *s, unsigned long cccc, size_t count);
extern void *__memset_unaligned(void *s, unsigned long cccc, size_t count);

/*
 * Compiler should be able to optimize this away into a very neat code
 */
static __always_inline void * __constant_c_and_count_memset(void * s, unsigned long pattern, size_t count)
{
	if (__builtin_constant_p((unsigned int)s & 0x3)) {

		if (((unsigned int)s & 0x3) == 0) {
			switch (count) {
			case 0:
				return s;
			case 1:
				*(unsigned char *)s = (unsigned char) pattern;
				return s;
			case 2:
				*(unsigned short *)s = (unsigned short) pattern;
				return s;
			case 3:
				*(unsigned short *)s = (unsigned short) pattern;
				*(2+(unsigned char *)s) = (unsigned char) pattern;
				return s;
			case 4:
				*(unsigned long *)s = (unsigned long) pattern;
				return s;
			default:
				return(__memset_aligned32(s, pattern, count));
			}
		}
	} else {
		switch (count) {
		case 8:
			*((unsigned char *)s+7) = pattern;
		case 7:
			*((unsigned char *)s+6) = pattern;
		case 6:
			*((unsigned char *)s+5) = pattern;
		case 5:
			*((unsigned char *)s+4) = pattern;
		case 4:
			*((unsigned char *)s+3) = pattern;
		case 3:
			*((unsigned char *)s+2) = pattern;
		case 2:
			*((unsigned char *)s+1) = pattern;
		case 1:
			*((unsigned char *)s+0) = pattern;
		case 0:
			return s;
		}
	}
	return(__memset_unaligned(s, pattern, count));
}

static __always_inline void *__constant_c_generic_count_memset(void *s, unsigned long pattern, size_t count)
{
	if (__builtin_constant_p((unsigned int)s & 0x3) && (((unsigned int)s & 0x3) == 0))
		return(__memset_aligned32(s, pattern, count));
	else
		return(__memset_unaligned(s, pattern, count));
}

/* we might get some code bloat here because of inlining 'c' -> 'cccc' expansion here */
static __always_inline void *__generic_c_memset(void *s, int c, size_t count)
{
	if (__builtin_constant_p((unsigned int)s & 0x3))
		return(__memset_aligned32(s, 0x01010101UL*(unsigned char)(c), count));
	else
		return(__memset_generic(s, c, count));
}

#define __constant_c_memset(s, c, count)				\
	(__builtin_constant_p(count) ?					\
	 __constant_c_and_count_memset((s),(c),(count)) :		\
	 __constant_c_generic_count_memset((s),(c),(count)))

#define memset(s, c, count)						\
	(__builtin_constant_p(c) ?					\
	 __constant_c_memset((s),(0x01010101UL*(unsigned char)(c)),(count)) : \
	 __generic_c_memset((s),(c),(count)))

#endif /* __AEONR2_STRING_H__ */
