/*
 *  linux/arch/ba/kernel/alternatives_asm.S
 *
 *  BA version
 *    author(s): Gyorgy Jeney (nog@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@bsemi.com or visit website http://www.bsemi.com.
 *
 *  changes:
 *  04. 07. 2007: Gyorgy Jeney (nog@beyondsemi.com)
 *    Initial bright spark idea
 */

#include <asm/page.h>

#include <asm/spr_defs.h>
#include <asm/aeonR2_asm.h>
#include <asm/board.h>
#include "ba_funcs.S"

/* DESC: this function can be used to switch position independant code
 *
 * PRMS: 
 *
 */
ENTRY(___switch_pic_alternate)
	/* Reason I disable the DMMU is because the kernel's text may have
	 * been write protected.  The IMMU is disabled because this is used
	 * to switch the tlb handlers.  Just for further paranoia, the
	 * caches are also disabled. */

	b.addi	r1,r1,-12
	b.sw	0(r1),r9
	b.sw	8(r1),r6

	b.mfspr	r6,r0,SPR_SR
	b.addi	r7,r0,-1
	b.xori	r7,r7,SPR_SR_IEE|SPR_SR_TEE
	b.and	r7,r6,r7
	b.mtspr	r0,r7,SPR_SR

	b.sw	4(r1),r6

	b.movhi	r6,hi(1f)
	b.ori	r6,r6,lo(1f)
	tophys	(r8,r6)

	b.ori	r7,r0,SPR_SR_FO|SPR_SR_SM
#ifdef CONFIG_AEONR2_ICACHE_ENABLED
	b.ori	r7,r7,SPR_SR_ICE
#endif
#ifdef CONFIG_AEONR2_DCACHE_ENABLED
	b.ori	r7,r7,SPR_SR_DCE
#endif

	/* ensure that page boundy never comes in between
	 * setting SPR_E* registers and b.rfe
	 */
	b.j	2f
	.align 0x10
2:
	b.mtspr	r0,r7,SPR_ESR_BASE
	b.mtspr	r0,r8,SPR_EPCR_BASE
	b.rfe
1:
	tophys	(r6,r1)
	b.ori	r1,r6,0

	tophys	(r6,r3)
	b.ori	r3,r6,0

	tophys	(r6,r4)
	b.ori	r4,r6,0

	b.jal	_memcpy

#ifdef CONFIG_AEONR2_DCACHE_ENABLED
	// flush the whole d-cache line
	b.addi	r3,r0,0
	// index
	b.addi	r4,r0,(DC_SIZE/DC_WAY)
99:
	b.flush_line 0(r3),0
	b.flush_line 0(r3),1
	b.flush_line 0(r3),2
	b.flush_line 0(r3),3
	b.addi	r3,r3,DC_LINE
	b.sfleu	r3,r4
	b.bf	99b

	// flush the write buffer
	b.syncwritebuffer

#endif // CONFIG_AEONR2_DCACHE_ENABLED
#ifdef CONFIG_AEONR2_ICACHE_ENABLED
#if 0//AEONR2
	// disable IC
	b.mfspr	r3,r0,SPR_SR
	LOAD_SYMBOL_2_GPR(r4,~SPR_SR_ICE)
	b.and	r3,r3,r4
	b.mtspr	r0,r3,SPR_SR
	b.nop	0x0
	b.nop	0x0
	b.nop	0x0
	b.nop	0x0
	b.nop	0x0
#endif

	// invaliate the while i-cache line
	b.addi	r3,r0,0
	b.addi	r4,r0,IC_SIZE
	b.ori	r5,r0,SPR_ICBIR
99:
	b.mtspr	r5,r3,0
	b.addi	r3,r3,IC_LINE
	b.sfleu	r3,r4
	b.bf	99b

#if 0//AEONR2
	// enable IC
	b.mfspr	r3,r0,SPR_SR
	b.ori	r3,r3,SPR_SR_ICE
	b.mtspr	r0,r3,SPR_SR
	b.nop	0x0
	b.nop	0x0
	b.nop	0x0
	b.nop	0x0
	b.nop	0x0
#endif
#endif // CONFIG_AEONR2_ICACHE_ENABLED

	b.lwz	r4,8(r1)

	b.sfeqi	r4,0
	b.bf	1f

	b.jal	_flush_tlb_all_nommu

1:
	b.lwz	r9,0(r1)
	b.lwz	r3,4(r1)

	tovirt(r4,r1)
	b.ori	r1,r4,0
	b.addi	r1,r1,12

	/* ensure that page boundy never comes in between
	 * setting SPR_E* registers and b.rfe
	 */
	b.j	3f
	.align	0x10
3:
	b.mtspr	r0,r3,SPR_ESR_BASE
	b.mtspr	r0,r9,SPR_EPCR_BASE

	b.rfe

