/*
 *  linux/arch/ba/kernel/ba_defs.c
 *
 *  BA version
 *  
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 * This program is used to generate definitions needed by
 * assembly language modules.
 *
 * We use the technique used in the OSF Mach kernel code:
 * generate asm statements containing #defines,
 * compile this file to assembler, and then extract the
 * #defines from the assembly-language output.
 */

#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/types.h>
#include <linux/ptrace.h>
#include <linux/mman.h>
#include <linux/mm.h>
#include <asm/io.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/processor.h>
#include <asm/thread_info.h>

#define DEFINE(sym, val) \
        asm volatile("\n->" #sym " %0 " #val : : "i" (val))
        
#define BLANK() asm volatile("\n->" : : )
        
int
main(void)
{
  /* offsets into thread_info */
  DEFINE(TI_TASK, offsetof(struct thread_info, task));
  DEFINE(TI_EXEC_DOMAIN, offsetof(struct thread_info, exec_domain));
  DEFINE(TI_FLAGS, offsetof(struct thread_info, flags));
  DEFINE(TI_CPU, offsetof(struct thread_info, cpu));
  DEFINE(TI_PREEMPT, offsetof(struct thread_info, preempt_count));
  DEFINE(TI_ADDR_LIMIT, offsetof(struct thread_info, addr_limit));
  DEFINE(TI_RESTART_BLOCK, offsetof(struct thread_info, restart_block));
  DEFINE(TI_KSP, offsetof(struct thread_info, ksp));
#ifdef CONFIG_AEONR2_TLB_STAT
  DEFINE(TI_K_ITLB_MISS, offsetof(struct thread_info, k_itlb_miss));
  DEFINE(TI_K_DTLB_MISS, offsetof(struct thread_info, k_dtlb_miss));
  DEFINE(TI_U_ITLB_MISS, offsetof(struct thread_info, u_itlb_miss));
  DEFINE(TI_U_DTLB_MISS, offsetof(struct thread_info, u_dtlb_miss));
#endif
#ifdef CONFIG_AEONR2_EXCEPTION_TRACE
  DEFINE(TI_EXCEPT_TRACE_POS, offsetof(struct thread_info, except_trace_pos));
  DEFINE(TI_EXCEPT_TRACE, offsetof(struct thread_info, except_trace));
#endif
  
  /* Interrupt register frame */
  DEFINE(STACK_FRAME_OVERHEAD, STACK_FRAME_OVERHEAD);
  DEFINE(INT_FRAME_SIZE, STACK_FRAME_OVERHEAD + sizeof(struct pt_regs));
  
  DEFINE(NUM_USER_SEGMENTS, TASK_SIZE>>28);
  return 0;
}
