/*
 *  linux/arch/ba/kernel/ba_funcs.S
 *
 *  BA version
 *    author(s): Matjaz Breskvar (phoenix@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  some helper functions for BA assembler programming
 *
 */

#ifndef ENTRY
#  define ENTRY(symbol)			\
	.global symbol			;\
	symbol:	
#endif
	
#define CLEAR_GPR(gpr)			\
	b.or	gpr,r0,r0

#define LOAD_SYMBOL_2_GPR(gpr,symbol)	\
	b.movhi	gpr,hi(symbol)		;\
	b.ori	gpr,gpr,lo(symbol)

/*
 * DSCR: lower bits of SPR_SR defined by mask will be set to 1
 *
 * PRMS: t1 is temporary (and destroyed)
 */
#define SR_ENABLE_LO_BITS(mask,t1)	\
	b.mfspr	t1,r0,SPR_SR		;\
	b.ori	t1,t1,lo(mask)		;\
	b.mtspr	r0,t1,SPR_SR

/*
 * DSCR: bits set in mask will be set to 1
 *
 * PRMS: t1 is temporary register
 *       t2 is temporary register
 */
#define SR_ENABLE_BITS(mask,t1,t2)	\
	b.mfspr	t2,r0,SPR_SR		;\
	LOAD_SYMBOL_2_GPR(t1,mask)	;\
	b.or	t2,t2,t1		;\
	b.mtspr	r0,t2,SPR_SR

/*
 * DSCR: bits set in mask will be set to 0
 *
 * PRMS: t1 is temporary register
 *       t2 is temporary register
 */ 
#define SR_DISABLE_BITS(mask,t1,t2)	\
	b.mfspr	t2,r0,SPR_SR		;\
	LOAD_SYMBOL_2_GPR(t1,(~mask))	;\
	b.and	t2,t2,t1		;\
	b.mtspr	r0,t2,SPR_SR


/*
 * DSCR: lower bits of SPR defined by mask will be set to 1
 *
 * PRMS: t1 is temporary (and destroyed)
 */
#define SPR_ENABLE_LO_BITS(SPR,mask,t1)	\
	b.mfspr t1,r0,SPR		;\
	b.ori  t1,t1,lo(mask)		;\
	b.mtspr r0,t1,SPR
	
/*
 * DSCR: lower bits of SPR defined by mask will be set to 0
 *
 * PRMS: t1 is temporary (and destroyed)
 */
#define SPR_DISABLE_LO_BITS(SPR,mask,t1)	\
	b.mfspr t1,r0,SPR			;\
	b.andi  t1,t1,lo(~mask)			;\
	b.mtspr r0,t1,SPR
	

#define DISABLE_INTERRUPTS(t1,t2)	\
	SR_DISABLE_BITS((SPR_SR_IEE|SPR_SR_TEE),t1,t2)

#define ENABLE_INTERRUPTS(t1)		\
	SR_ENABLE_LO_BITS((SPR_SR_IEE|SPR_SR_TEE),t1)

