/*
 *  linux/arch/ba/kernel/entry.S
 *
 *  BA version
 *    author(s): Matjaz Breskvar (phoenix@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    initial port to BA architecture
 *  06. 12. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    major changes in handling exceptions
 *    physical addresses used only by code in head.S
 *    cleand exception handlers and moved them to entry.S
 *  31. 12. 2005: Gyorgy Jeney (nog@beyondsemi.com)
 *    Added syscall traceing code
 *
 * syscall table entries are based on ppc port, so they will probably
 * need some more cleaning.
 *
 * Stack layout in 'ret_from_system_call':
 *      ptrace needs to have all regs on the stack.
 *      if the order here is changed, it needs to be
 *      updated in fork.c:copy_process, signal.c:do_signal,
 *      ptrace.c and ptrace.h
 *
 * syscall table entries are based on ppc port, so it will probably need
 * some more cleaning.
 */

#include <linux/linkage.h>
#include <linux/sys.h>
#include <asm/processor.h>
#include <asm/thread_info.h>
#include <asm/unistd.h>
#include <asm/errno.h>
#include <asm/aeonR2_asm.h>
#include <asm/spr_defs.h>

#include <asm/page.h>
#include <asm/unistd.h>
#include <asm/pgtable.h>
#include <asm/asm-offsets.h>
#include <asm/board.h>

#include <asm/abi.h>

/* some defines to ease BA assembly programming */
#include "ba_funcs.S"

// ((ksp >> PAGE_SHIFT) << PAGE_SHIFT)
// == (ksp &= ~(PAGE_SIZE - 1)
// aligned to page boundary
#define GET_CURRENT_THREAD_INFO(reg,ksp)			\
	b.srli	reg,ksp,PAGE_SHIFT				;\
	b.slli	reg,reg,PAGE_SHIFT

/* =========================================================[ macros ]=== */

#define PUSH_PT_REGS_NO_TRANS					\
	b.addi	r1,r1,-(INT_FRAME_SIZE)				;\
	b.sw	PC(r1),r9					;\
	b.sw	GPR9(r1),r9					;\
	b.mfspr	r9,r0,SPR_SR					;\
	b.sw	SR(r1),r9					;\
	b.addi	r9,r1,INT_FRAME_SIZE				;\
	b.sw	SP(r1),r9					;\
	b.sw	GPR2(r1),r2					;\
	b.sw	GPR3(r1),r3					;\
	b.sw	ORIG_GPR3(r1),r3				;\
	b.sw	GPR4(r1),r4					;\
	b.sw	GPR5(r1),r5					;\
	b.sw	GPR6(r1),r6					;\
	b.sw	GPR7(r1),r7					;\
	b.sw	GPR8(r1),r8					;\
	b.sw	GPR10(r1),r10					;\
	b.sw	GPR11(r1),r11					;\
	b.sw	GPR12(r1),r12					;\
	b.sw	GPR13(r1),r13					;\
	b.sw	GPR14(r1),r14					;\
	b.sw	GPR15(r1),r15					;\
	b.sw	GPR16(r1),r16					;\
	b.sw	GPR17(r1),r17					;\
	b.sw	GPR18(r1),r18					;\
	b.sw	GPR19(r1),r19					;\
	b.sw	GPR20(r1),r20					;\
	b.sw	GPR21(r1),r21					;\
	b.sw	GPR22(r1),r22					;\
	b.sw	GPR23(r1),r23					;\
	b.sw	GPR24(r1),r24					;\
	b.sw	GPR25(r1),r25					;\
	b.sw	GPR26(r1),r26					;\
	b.sw	GPR27(r1),r27					;\
	b.sw	GPR28(r1),r28					;\
	b.sw	GPR29(r1),r29					;\
	b.sw	GPR30(r1),r30					;\
	b.sw	GPR31(r1),r31					;\
	b.sw	RESULT(r1),r0


/*
 * We need to disable interrupts at beginning of RESTORE_ALL
 * since interrupt might come in after we've loaded EPC return address
 * and overwrite EPC with address somewhere in RESTORE_ALL
 * which is of course wrong!
 */

#define RESTORE_ALL						\
	DISABLE_INTERRUPTS(r3,r4)				;\
	b.lwz	r3,PC(r1)					;\
	b.mtspr	r0,r3,SPR_EPCR_BASE				;\
	b.lwz	r3,SR(r1)					;\
	b.mtspr	r0,r3,SPR_ESR_BASE				;\
	b.lwz	r2,GPR2(r1)					;\
	b.lwz	r3,GPR3(r1)					;\
	b.lwz	r4,GPR4(r1)					;\
	b.lwz	r5,GPR5(r1)					;\
	b.lwz	r6,GPR6(r1)					;\
	b.lwz	r7,GPR7(r1)					;\
	b.lwz	r8,GPR8(r1)					;\
	b.lwz	r9,GPR9(r1)					;\
	b.lwz	r10,GPR10(r1)					;\
	b.lwz	r11,GPR11(r1)					;\
	b.lwz	r12,GPR12(r1)					;\
	b.lwz	r13,GPR13(r1)					;\
	b.lwz	r14,GPR14(r1)					;\
	b.lwz	r15,GPR15(r1)					;\
	b.lwz	r16,GPR16(r1)					;\
	b.lwz	r17,GPR17(r1)					;\
	b.lwz	r18,GPR18(r1)					;\
	b.lwz	r19,GPR19(r1)					;\
	b.lwz	r20,GPR20(r1)					;\
	b.lwz	r21,GPR21(r1)					;\
	b.lwz	r22,GPR22(r1)					;\
	b.lwz	r23,GPR23(r1)					;\
	b.lwz	r24,GPR24(r1)					;\
	b.lwz	r25,GPR25(r1)					;\
	b.lwz	r26,GPR26(r1)					;\
	b.lwz	r27,GPR27(r1)					;\
	b.lwz	r28,GPR28(r1)					;\
	b.lwz	r29,GPR29(r1)					;\
	b.lwz	r30,GPR30(r1)					;\
	b.lwz	r31,GPR31(r1)					;\
	b.lwz	r1,SP(r1)					;\
	b.rfe

#define RESTORE_ALL_NO_R3					\
	DISABLE_INTERRUPTS(r5,r4)				;\
	b.lwz	r4,PC(r1)					;\
	b.mtspr	r0,r4,SPR_EPCR_BASE				;\
	b.lwz	r4,SR(r1)					;\
	b.mtspr	r0,r4,SPR_ESR_BASE				;\
	b.lwz	r2,GPR2(r1)					;\
	b.lwz	r4,GPR4(r1)					;\
	b.lwz	r5,GPR5(r1)					;\
	b.lwz	r6,GPR6(r1)					;\
	b.lwz	r7,GPR7(r1)					;\
	b.lwz	r8,GPR8(r1)					;\
	b.lwz	r9,GPR9(r1)					;\
	b.lwz	r10,GPR10(r1)					;\
	b.lwz	r11,GPR11(r1)					;\
	b.lwz	r12,GPR12(r1)					;\
	b.lwz	r13,GPR13(r1)					;\
	b.lwz	r14,GPR14(r1)					;\
	b.lwz	r15,GPR15(r1)					;\
	b.lwz	r16,GPR16(r1)					;\
	b.lwz	r17,GPR17(r1)					;\
	b.lwz	r18,GPR18(r1)					;\
	b.lwz	r19,GPR19(r1)					;\
	b.lwz	r20,GPR20(r1)					;\
	b.lwz	r21,GPR21(r1)					;\
	b.lwz	r22,GPR22(r1)					;\
	b.lwz	r23,GPR23(r1)					;\
	b.lwz	r24,GPR24(r1)					;\
	b.lwz	r25,GPR25(r1)					;\
	b.lwz	r26,GPR26(r1)					;\
	b.lwz	r27,GPR27(r1)					;\
	b.lwz	r28,GPR28(r1)					;\
	b.lwz	r29,GPR29(r1)					;\
	b.lwz	r30,GPR30(r1)					;\
	b.lwz	r31,GPR31(r1)					;\
	b.lwz	r1,SP(r1)					;\
	b.rfe

#define RESTORE_ALL_NO_R11					\
	DISABLE_INTERRUPTS(r3,r4)				;\
	b.lwz	r3,PC(r1)					;\
	b.mtspr	r0,r3,SPR_EPCR_BASE				;\
	b.lwz	r3,SR(r1)					;\
	b.mtspr	r0,r3,SPR_ESR_BASE				;\
	b.lwz	r2,GPR2(r1)					;\
	b.lwz	r3,GPR3(r1)					;\
	b.lwz	r4,GPR4(r1)					;\
	b.lwz	r5,GPR5(r1)					;\
	b.lwz	r6,GPR6(r1)					;\
	b.lwz	r7,GPR7(r1)					;\
	b.lwz	r8,GPR8(r1)					;\
	b.lwz	r9,GPR9(r1)					;\
	b.lwz	r10,GPR10(r1)					;\
	b.lwz	r12,GPR12(r1)					;\
	b.lwz	r13,GPR13(r1)					;\
	b.lwz	r14,GPR14(r1)					;\
	b.lwz	r15,GPR15(r1)					;\
	b.lwz	r16,GPR16(r1)					;\
	b.lwz	r17,GPR17(r1)					;\
	b.lwz	r18,GPR18(r1)					;\
	b.lwz	r19,GPR19(r1)					;\
	b.lwz	r20,GPR20(r1)					;\
	b.lwz	r21,GPR21(r1)					;\
	b.lwz	r22,GPR22(r1)					;\
	b.lwz	r23,GPR23(r1)					;\
	b.lwz	r24,GPR24(r1)					;\
	b.lwz	r25,GPR25(r1)					;\
	b.lwz	r26,GPR26(r1)					;\
	b.lwz	r27,GPR27(r1)					;\
	b.lwz	r28,GPR28(r1)					;\
	b.lwz	r29,GPR29(r1)					;\
	b.lwz	r30,GPR30(r1)					;\
	b.lwz	r31,GPR31(r1)					;\
	b.lwz	r1,SP(r1)					;\
	b.rfe

#if REG_RETVAL == 3
# define RESTORE_ALL_NO_RETVAL	RESTORE_ALL_NO_R3
#elif REG_RETVAL == 11
# define RESTORE_ALL_NO_RETVAL	RESTORE_ALL_NO_R11
#else
# error "Unknown retval register"
#endif

#define EXCEPTION_ENTRY(handler)				\
	.global	handler						;\
handler:							;\
	/* r1, EPCR, ESR already saved */			;\
	b.sw	GPR2(r1),r2					;\
	b.sw	GPR3(r1),r3					;\
	b.sw	ORIG_GPR3(r1),r3				;\
	/* r4 already saved */					;\
	b.sw	GPR5(r1),r5					;\
	b.sw	GPR6(r1),r6					;\
	b.sw	GPR7(r1),r7					;\
	b.sw	GPR8(r1),r8					;\
	b.sw	GPR9(r1),r9					;\
	b.sw	GPR10(r1),r10					;\
	b.sw	GPR11(r1),r11					;\
	/* r12,r13 already saved */				;\
	b.sw	GPR14(r1),r14					;\
	b.sw	GPR15(r1),r15					;\
	b.sw	GPR16(r1),r16					;\
	b.sw	GPR17(r1),r17					;\
	b.sw	GPR18(r1),r18					;\
	b.sw	GPR19(r1),r19					;\
	b.sw	GPR20(r1),r20					;\
	b.sw	GPR21(r1),r21					;\
	b.sw	GPR22(r1),r22					;\
	b.sw	GPR23(r1),r23					;\
	b.sw	GPR24(r1),r24					;\
	b.sw	GPR25(r1),r25					;\
	b.sw	GPR26(r1),r26					;\
	b.sw	GPR27(r1),r27					;\
	b.sw	GPR28(r1),r28					;\
	/* r29 already saved */					;\
	b.sw	GPR30(r1),r30					;\
	/* r31 already save */					;\
	b.sw	RESULT(r1),r0

/*
 * NOTE: one should never assume that SPR_EPC, SPR_ESR, SPR_EEAR
 *       contain the same values as when exception we're handling
 *	 occured. in fact they never do. if you need them use
 *	 values saved on stack (for SPR_EPC, SPR_ESR) or content
 *       of r4 (for SPR_EEAR). for details look at EXCEPTION_HANDLE()
 *       in 'arch/ba/kernel/head.S'
 */

/* =====================================================[ exceptions] === */

/* ---[ 0x100: RESET exception ]----------------------------------------- */

EXCEPTION_ENTRY(_tng_kernel_start)
	b.jal	_start

/* ---[ 0x200: BUS exception ]------------------------------------------- */



/* ---[ 0x300: Data Page Fault exception ]------------------------------- */

EXCEPTION_ENTRY(_data_page_fault_handler)
	/* set up parameters for do_page_fault */
	b.addi	r3,r1,0				// pt_regs
	/* r4 set be EXCEPTION_HANDLE */	// effective address of fault

	b.addi	r1,r1,-16
	b.sw	0x0(r1),r7
	b.sw	0x4(r1),r8

	b.lwz	r6,PC(r3)		// address of an offending insn
	b.lbz	r6,0(r6)		// instruction OP code that caused pf
	b.srli	r6,r6,2			// check opcode(6bits)
	b.andi	r6,r6,0x3f

	// check bn.sb
	b.sfeqi	r6,0x6
	b.bf	1f

	// check bg.sb
	b.sfeqi	r6,0x3e
	b.bf	1f

	// check bn.sh,bn.sw
	b.sfeqi	r6,0x3
	b.bnf	99f // not bn.sh,bn.sw
	// check bn.sh
	b.lwz	r7,PC(r3)
	b.lbz	r7,0x2(r7)
	b.andi	r8,r7,0x1
	b.sfeqi	r8,0x1 // is bn.sh
	b.bf	1f // is bn.sh
	// check bn.sw
	b.andi	r8,r7,0x3
	b.sfeqi	r8,0x0 // is bn.sw
	b.bf	1f // is bn.sw
	b.j	2f // not bn.sh,bn.sw

99:
	// check bg.sh,bg.sw
	b.sfeqi	r6,0x3b
	b.bnf	2f
	// check bg.sh
	b.lwz	r7,PC(r3)
	b.lbz	r7,0x3(r7)
	b.andi	r8,r7,0x1
	b.sfeqi	r8,0x1 // is bg.sh
	b.bf	1f // is bg.sh
	b.andi	r8,r7,0x3
	b.sfeqi	r8,0x0 // is bg.sw
	b.bf	1f // is bg.sw
	b.j	2f // not bg.sh,bg.sw

1:
	b.ori	r6,r0,0x1		// write access
	b.j	3f
2:	b.ori	r6,r0,0x0		// !write access
3:	b.ori	r5,r0,0x300		// exception vector

	b.lwz	r8,0x4(r1)
	b.lwz	r7,0x0(r1)
	b.addi	r1,r1,16

	/* call fault.c handler in ba/mm/fault.c */
	b.jal	_do_page_fault
	b.j	_ret_from_exception

/* ---[ 0x400: Insn Page Fault exception ]------------------------------- */

EXCEPTION_ENTRY(_insn_page_fault_handler)
	/* set up parameters for do_page_fault */
	b.addi	r3,r1,0				// pt_regs
	/* r4 set be EXCEPTION_HANDLE */	// effective address of fault
	b.ori	r5,r0,0x400			// exception vector
	b.ori	r6,r0,0x0			// !write access

	/* call fault.c handler in ba/mm/fault.c */
	b.jal	_do_page_fault
	b.j	_ret_from_exception

/* ---[ 0x500: Timer exception ]----------------------------------------- */

EXCEPTION_ENTRY(_timer_handler)
	b.addi r3,r1,0
	LOAD_SYMBOL_2_GPR(r8,_timer_interrupt)
	b.jalr r8
	b.j	_ret_from_intr

/* ---[ 0x600: Aligment exception ]-------------------------------------- */


/* ---[ 0x700: Illegal insn exception ]---------------------------------- */


/* ---[ 0x800: External interrupt exception ]---------------------------- */

EXCEPTION_ENTRY(_external_irq_handler)
	b.addi r3,r1,0
	LOAD_SYMBOL_2_GPR(r8,_do_IRQ)
	b.jalr r8
	b.j	_ret_from_intr

/* ---[ 0x900: DTLB miss exception ]------------------------------------- */


/* ---[ 0xa00: ITLB miss exception ]------------------------------------- */


/* ---[ 0xb00: Range exception ]----------------------------------------- */


/* ---[ 0xc00: Syscall exception ]--------------------------------------- */

EXCEPTION_ENTRY(_sys_call_handler)
	/* r3 is not tainted by EXCEPTION_HANDLE() */

	/* r4 holds the EEAR address of the fault, load the original r4 */
	b.lwz	r4,GPR4(r1)

	b.lwz	r30,TI_FLAGS(r29)
	b.andi	r30,r30,_TIF_SYSCALL_TRACE
	b.sfne	r30,r0
	b.bf	_syscall_trace_entry

	/* check that userspace didn't try to fool us with
	 * syscall number greater then __NR_syscall.
	 */
	b.ori	r10,r0,NR_syscalls
	b.sfgeu	r11,r10
	b.bf	_syscall_badsys

	ENABLE_INTERRUPTS(r9)		// enable interrupts, r9 is temp

_syscall_call:
	b.movhi	r30,hi(_sys_call_table)
	b.ori	r30,r30,lo(_sys_call_table)
	b.slli	r11,r11,2
	b.add	r30,r30,r11
	b.lwz	r30,0(r30)

	b.addi	r8,r1,0			// regs pointer
	b.jalr	r30

	b.sw	GPR3(r1),REG_RETVAL_ASM   // save return value
	b.sw	RESULT(r1),REG_RETVAL_ASM // save result

_syscall_exit:
	DISABLE_INTERRUPTS(r3,r4)
	b.lwz	r30,TI_FLAGS(r29)
	b.andi	r3,r30,_TIF_ALLWORK_MASK
	b.sfne	r3,r0
	b.bf	_syscall_exit_work
	b.j	_restore_all

_syscall_trace_entry:
	b.sw	SYSCALLNO(r1),r11
	b.addi	r3,r1,0
	b.addi	r4,r0,0
	b.jal	_do_syscall_trace

	/* Restore arguments */
	b.lwz	r11,SYSCALLNO(r1)
	b.lwz	r3,GPR3(r1)
	b.lwz	r4,GPR4(r1)
	b.lwz	r5,GPR5(r1)
	b.lwz	r6,GPR6(r1)
	b.lwz	r7,GPR7(r1)
	b.ori	r10,r0,NR_syscalls
	b.sfltu	r11,r10
	b.bf	_syscall_call

	b.addi	r30,r0,-ENOSYS
	b.sw	GPR11(r1),r30
	b.sw	RESULT(r1),r30
	b.j	_syscall_exit

_syscall_badsys:
	/* Just return -ENOSYS */
	b.addi	r30,r0,-ENOSYS
	b.sw	RESULT(r1),r30
	b.sw	GPR11(r1),r30
	b.j	_resume_userspace

_syscall_exit_work:
	/* r30 holds TI_flags */
	b.andi	r3,r30,_TIF_SYSCALL_TRACE|_TIF_SINGLESTEP
	b.sfeq	r3,r0
	b.bf	_work_pending

	ENABLE_INTERRUPTS(r3)

	b.addi	r3,r1,0
	b.addi	r4,r0,1
	b.jal	_do_syscall_trace
	b.j	_resume_userspace

/* ---[ 0xd00: Trap exception ]------------------------------------------ */


/* ---[ 0xe00: Trap exception ]------------------------------------------ */


/* ---[ 0xf00: Reserved exception ]-------------------------------------- */


/* ---[ 0x1000: Reserved exception ]------------------------------------- */


/* ---[ 0x1100: Reserved exception ]------------------------------------- */


/* ---[ 0x1200: Reserved exception ]------------------------------------- */


/* ---[ 0x1300: Reserved exception ]------------------------------------- */


/* ---[ 0x1400: Reserved exception ]------------------------------------- */


/* ---[ 0x1500: Reserved exception ]------------------------------------- */


/* ---[ 0x1600: Reserved exception ]------------------------------------- */


/* ---[ 0x1700: Reserved exception ]------------------------------------- */


/* ---[ 0x1800: Reserved exception ]------------------------------------- */


/* ---[ 0x1900: Reserved exception ]------------------------------------- */


/* ---[ 0x1a00: Reserved exception ]------------------------------------- */


/* ---[ 0x1b00: Reserved exception ]------------------------------------- */


/* ---[ 0x1c00: Reserved exception ]------------------------------------- */


/* ---[ 0x1d00: Reserved exception ]------------------------------------- */


/* ---[ 0x1e00: Reserved exception ]------------------------------------- */


/* ---[ 0x1f00: Reserved exception ]------------------------------------- */


/* ========================================================[ return ] === */

_work_pending:
	/*
	 * if (current_thread_info->flags & _TIF_NEED_RESCHED)
	 *     schedule();
	 */
	b.lwz	r3,TI_FLAGS(r29)
	b.andi	r3,r3,_TIF_NEED_RESCHED
	b.sfeqi	r3,0
	b.bf	work_notifysig

work_resched:
	b.jal	_schedule

	DISABLE_INTERRUPTS(r3,r4)

	b.lwz	r3,TI_FLAGS(r29)
	b.andi	r30,r3,_TIF_WORK_MASK
	b.sfeqi	r30,0
	b.bf	_restore_all
	b.andi	r30,r3,_TIF_NEED_RESCHED
	b.sfnei	r30,0
	b.bf	work_resched
	/*
	 * if (current_thread_info->flags & _TIF_SIGPENDING)
	 *     do_signal(r3,r4);
	 */
work_notifysig:
	b.lwz	r5,TI_FLAGS(r29)
	b.andi	r5,r5,_TIF_SIGPENDING
	b.addi	r3,r0,1
	b.sfeqi r5,0
	b.bf	_resume_userspace
	b.addi	r4,r0,0
	b.addi	r5,r1,0
	b.jal	_do_signal
	b.j	_resume_userspace


ENTRY(_ret_from_fork)
	b.jal	_schedule_tail
	b.j	_syscall_exit

ENTRY(_restore_all)
	// we need to save KSP here too
	// or else fork's 'eat' stack space
	b.addi	r4,r1,INT_FRAME_SIZE
	b.sw	(TI_KSP)(r29),r4
	RESTORE_ALL

ENTRY(_ret_from_intr)
ENTRY(_ret_from_exception)
	b.lwz	r4,SR(r1)
	b.andi	r3,r4,SPR_SR_SM
	b.sfnei	r3,0
	b.bf	_restore_all
	/*
	 * return to usermode
	 */
_resume_userspace:
	DISABLE_INTERRUPTS(r3,r4)
	b.lwz	r3,TI_FLAGS(r29)
	b.andi	r3,r3,_TIF_WORK_MASK
	b.sfnei	r3,0
	b.bf	_work_pending
	b.j	_restore_all

/* ========================================================[ switch ] === */

/*
 * This routine switches between two different tasks.  The process
 * state of one is saved on its kernel stack.  Then the state
 * of the other is restored from its kernel stack.  The memory
 * management hardware is updated to the second process's state.
 * Finally, we can return to the second process, via the 'return'.
 *
 * Note: there are two ways to get to the "going out" portion
 * of this code; either by coming in via the entry (_switch)
 * or via "fork" which must set up an environment equivalent
 * to the "_switch" path.  If you change this (or in particular, the
 * SAVE_REGS macro), you'll have to change the fork code also.
 */


/* __switch MUST never lay on page boundry, cause it runs from
 * effective addresses and beeing interrupted by iTLB miss would kill it.
 * dTLB miss seams to never accour in the bad place since data accesses
 * are from task structures which are always page aligned.
 *
 * The problem happens in RESTORE_ALL_NO_RETVAL where we first set the EPCR
 * register, then load the previous register values and only at the end call
 * the b.rfe instruction. If get TLB miss in beetwen the EPCR register gets
 * garbled and we end up calling b.rfe with the wrong EPCR. (same probably
 * holds for ESR)
 *
 * To avoid this problems it is sufficient to align __switch to
 * some nice round number smaller than it's size...
 */

	.align 0x400
ENTRY(__switch)
	PUSH_PT_REGS_NO_TRANS

	b.sw	TI_KSP(r3),r1			/* Save old stack pointer */
	b.lwz	REG_RETVAL_ASM,TI_TASK(r29)	/* Save old 'current_task' to 'last' return value*/

	b.lwz	r1,TI_KSP(r4)			/* Load new stack pointer */
	RESTORE_ALL_NO_RETVAL

ENTRY(_kernel_execve)
	b.addi	r1,r1,-4
	b.sw	0(r1),r11
	b.ori	r11,r0,__NR_execve
	b.sys
	b.lwz	r11,0(r1)
	b.addi	r1,r1,4
	b.jr	r9

/* =================================================[ syscall table ] === */

	.data
	.align 4
ENTRY(_sys_call_table)
	.long _sys_ni_syscall	/* 0 */		/* old "setup()" system call */
	.long _sys_exit
	.long _sys_fork
	.long _sys_read
	.long _sys_write
	.long _sys_open		/* 5 */
	.long _sys_close
	.long _sys_waitpid
	.long _sys_creat
	.long _sys_link
	.long _sys_unlink	/* 10 */
	.long _sys_execve
	.long _sys_chdir
	.long _sys_time
	.long _sys_mknod
	.long _sys_chmod	/* 15 */
	.long _sys_lchown
	.long _sys_ni_syscall			/* old break syscall holder */
	.long _sys_stat
	.long _sys_lseek
	.long _sys_getpid	/* 20 */
	.long _sys_mount
	.long _sys_oldumount
	.long _sys_setuid
	.long _sys_getuid
	.long _sys_stime	/* 25 */
	.long _sys_ptrace
	.long _sys_alarm
	.long _sys_fstat
	.long _sys_pause
	.long _sys_utime	/* 30 */
	.long _sys_ni_syscall			/* old stty syscall holder */
	.long _sys_ni_syscall			/* old gtty syscall holder */
	.long _sys_access
	.long _sys_nice
	.long _sys_ni_syscall	/* 35 */	/* old ftime syscall holder */
	.long _sys_sync
	.long _sys_kill
	.long _sys_rename
	.long _sys_mkdir
	.long _sys_rmdir	/* 40 */
	.long _sys_dup
	.long _sys_pipe
	.long _sys_times
	.long _sys_ni_syscall			/* old prof syscall holder */
	.long _sys_brk		/* 45 */
	.long _sys_setgid
	.long _sys_getgid
	.long _sys_signal
	.long _sys_geteuid
	.long _sys_getegid	/* 50 */
	.long _sys_acct
	.long _sys_umount			/* recycled never used phys() */
	.long _sys_ni_syscall			/* old lock syscall holder */
	.long _sys_ioctl
	.long _sys_fcntl	/* 55 */
	.long _sys_ni_syscall			/* old mpx syscall holder */
	.long _sys_setpgid
	.long _sys_ni_syscall			/* old ulimit syscall holder */
	.long _sys_ni_syscall			/* old _sys_olduname holder */
	.long _sys_umask	/* 60 */
	.long _sys_chroot
	.long _sys_ustat
	.long _sys_dup2
	.long _sys_getppid
	.long _sys_getpgrp	/* 65 */
	.long _sys_setsid
	.long _sys_sigaction
	.long _sys_sgetmask
	.long _sys_ssetmask
	.long _sys_setreuid	/* 70 */
	.long _sys_setregid
	.long _sys_sigsuspend
	.long _sys_sigpending
	.long _sys_sethostname
	.long _sys_setrlimit	/* 75 */
	.long _sys_old_getrlimit
	.long _sys_getrusage
	.long _sys_gettimeofday
	.long _sys_settimeofday
	.long _sys_getgroups	/* 80 */
	.long _sys_setgroups
	.long _sys_select			/* was old_select */
	.long _sys_symlink
	.long _sys_lstat
	.long _sys_readlink	/* 85 */
	.long _sys_uselib
	.long _sys_swapon
	.long _sys_reboot
	.long _sys_old_readdir
	.long _old_mmap		/* 90 */
	.long _sys_munmap
	.long _sys_truncate
	.long _sys_ftruncate
	.long _sys_fchmod
	.long _sys_fchown	/* 95 */
	.long _sys_getpriority
	.long _sys_setpriority
	.long _sys_ni_syscall			/* old profil syscall holder */
	.long _sys_statfs
	.long _sys_fstatfs	/* 100 */
	.long _sys_ni_syscall			/* _sys_ioperm in i386 */
	.long _sys_socketcall
	.long _sys_syslog
	.long _sys_setitimer
	.long _sys_getitimer	/* 105 */
	.long _sys_newstat
	.long _sys_newlstat
	.long _sys_newfstat
	.long _sys_ni_syscall			/* old _sys_uname holder */
	.long _sys_ni_syscall	/* 110 */	/* _sys_iopl in i386 */
	.long _sys_vhangup
	.long _sys_ni_syscall	/* old 'idle' syscall */
	.long _sys_ni_syscall			/* vm86 in i386 */
	.long _sys_wait4
	.long _sys_swapoff	/* 115 */
	.long _sys_sysinfo
	.long _sys_ipc
	.long _sys_fsync
	.long _sys_sigreturn
	.long _sys_clone	/* 120 */
	.long _sys_setdomainname
	.long _sys_newuname
	.long _sys_ni_syscall
	.long _sys_adjtimex
	.long _sys_mprotect	/* 125 */
	.long _sys_sigprocmask
	.long _sys_ni_syscall			/* old "create_module" */
	.long _sys_init_module
	.long _sys_delete_module
	.long _sys_ni_syscall   /* 130 */	/* old "get_kernel_syms" */
	.long _sys_quotactl
	.long _sys_getpgid
	.long _sys_fchdir
	.long _sys_bdflush
	.long _sys_sysfs	/* 135 */
	.long _sys_personality
	.long _sys_ni_syscall			/* reserved for afs_syscall */
	.long _sys_setfsuid
	.long _sys_setfsgid
	.long _sys_llseek	/* 140 */
	.long _sys_getdents
	.long _sys_select
	.long _sys_flock
	.long _sys_msync
	.long _sys_readv	/* 145 */
	.long _sys_writev
	.long _sys_getsid
	.long _sys_fdatasync
	.long _sys_sysctl
	.long _sys_mlock	/* 150 */
	.long _sys_munlock
	.long _sys_mlockall
	.long _sys_munlockall
	.long _sys_sched_setparam
	.long _sys_sched_getparam	/* 155 */
	.long _sys_sched_setscheduler
	.long _sys_sched_getscheduler
	.long _sys_sched_yield
	.long _sys_sched_get_priority_max
	.long _sys_sched_get_priority_min	/* 160 */
	.long _sys_sched_rr_get_interval
	.long _sys_nanosleep
	.long _sys_mremap
	.long _sys_setresuid
	.long _sys_getresuid	/* 165 */
	.long _sys_ni_syscall			/* old sys_query_module */
	.long _sys_poll
	.long _sys_nfsservctl
	.long _sys_setresgid
	.long _sys_getresgid	/* 170 */
	.long _sys_prctl
	.long _sys_rt_sigreturn
	.long _sys_rt_sigaction
	.long _sys_rt_sigprocmask
	.long _sys_rt_sigpending	/* 175 */
	.long _sys_rt_sigtimedwait
	.long _sys_rt_sigqueueinfo
	.long _sys_rt_sigsuspend
	.long _sys_pread64
	.long _sys_pwrite64	/* 180 */
	.long _sys_chown
	.long _sys_getcwd
	.long _sys_capget
	.long _sys_capset
	.long _sys_sigaltstack	/* 185 */
	.long _sys_sendfile
	.long _sys_ni_syscall			/* streams1 */
	.long _sys_ni_syscall			/* streams2 */
	.long _sys_vfork
	.long _sys_getrlimit	/* 190 */
	.long _sys_readahead
	.long _sys_mmap2
	.long _sys_truncate64
	.long _sys_ftruncate64
	.long _sys_stat64	/* 195 */
	.long _sys_lstat64
	.long _sys_fstat64
	.long _sys_ni_syscall			/* _sys_pciconfig_read */
	.long _sys_ni_syscall			/* _sys_pciconfig_write */
	.long _sys_ni_syscall	/* 200 */	/* _sys_pciconfig_iobase */
	.long _sys_ni_syscall			/* reserved - MacOnLinux */
	.long _sys_getdents64
	.long _sys_pivot_root
	.long _sys_fcntl64
	.long _sys_madvise	/* 205 */
	.long _sys_mincore
	.long _sys_gettid
	.long _sys_tkill
	.long _sys_setxattr
	.long _sys_lsetxattr	/* 210 */
	.long _sys_fsetxattr
	.long _sys_getxattr
	.long _sys_lgetxattr
	.long _sys_fgetxattr
	.long _sys_listxattr	/* 215 */
	.long _sys_llistxattr
	.long _sys_flistxattr
	.long _sys_removexattr
	.long _sys_lremovexattr
	.long _sys_fremovexattr	/* 220  */
	.long _sys_futex
	.long _sys_sched_setaffinity
	.long _sys_sched_getaffinity
	.long _sys_ni_syscall			/* reserved for sys_security */
	.long _sys_ni_syscall	/* 225 */	/* reserved for Tux */
	.long _sys_sendfile64
	.long _sys_io_setup
	.long _sys_io_destroy
	.long _sys_io_getevents
	.long _sys_io_submit	/* 230 */
	.long _sys_io_cancel

	.long _sys_set_tid_address
	.long _sys_fadvise64
	.long _sys_exit_group
	.long _sys_lookup_dcookie	/* 235 */
	.long _sys_epoll_create
	.long _sys_epoll_ctl
	.long _sys_epoll_wait
	.long _sys_remap_file_pages
	.long _sys_timer_create	/* 240 */
	.long _sys_timer_settime
	.long _sys_timer_gettime
	.long _sys_timer_getoverrun
	.long _sys_timer_delete
	.long _sys_clock_settime	/* 245 */
	.long _sys_clock_gettime
	.long _sys_clock_getres
	.long _sys_clock_nanosleep
	.long _sys_ni_syscall			/* _sys_swapcontext on powerpc */
	.long _sys_tgkill		/* 250 */
	.long _sys_utimes
	.long _sys_statfs64
	.long _sys_fstatfs64
	.long _sys_fadvise64_64
	.long _sys_ni_syscall	/* 255 */	/* supposed to be sys_rtas, whetever that is */
	.long _sys_ni_syscall			/* _sys_sys_debug_setcontext.. */
	.long _sys_ni_syscall			/* reserved for vserver */
	.long _sys_migrate_pages
	.long _sys_mbind
	.long _sys_get_mempolicy	/* 260 */
	.long _sys_set_mempolicy
	.long _sys_mq_open
	.long _sys_mq_unlink
	.long _sys_mq_timedsend
	.long _sys_mq_timedreceive	/* 265 */
	.long _sys_mq_notify
	.long _sys_mq_getsetattr
	.long _sys_kexec_load
	.long _sys_add_key
	.long _sys_request_key	/* 270 */
	.long _sys_keyctl
	.long _sys_waitid
	.long _sys_ioprio_set
	.long _sys_ioprio_get
	.long _sys_inotify_init	/* 275 */
	.long _sys_inotify_add_watch
	.long _sys_inotify_rm_watch
	.long _sys_spu_run
	.long _sys_spu_create
	.long _sys_ni_syscall 	/* 280 */	/* _sys_pselect6 but no TIF_RESTORE_SIGMASK */
	.long _sys_ni_syscall			/* _sys_ppoll but we need TIF_RESTORE_SIGMASK */
	.long _sys_unshare
	.long _sys_splice
	.long _sys_tee
	.long _sys_vmsplice	/* 285 */
	.long _sys_openat
	.long _sys_mkdirat
	.long _sys_mknodat
	.long _sys_fchownat
	.long _sys_futimesat	/* 290 */
	.long _sys_fstatat64
	.long _sys_unlinkat
	.long _sys_renameat
	.long _sys_linkat
	.long _sys_symlinkat	/* 295 */
	.long _sys_readlinkat
	.long _sys_fchmodat
	.long _sys_faccessat
	.long _sys_get_robust_list
	.long _sys_set_robust_list	/* 300 */
	.long _sys_move_pages
	.long _sys_getcpu
	.long _sys_epoll_pwait
	.long _sys_utimensat
	.long _sys_signalfd	/* 305 */
	.long _sys_timerfd_create
	.long _sys_eventfd
	.long _sys_sync_file_range2
	.long _sys_timerfd_settime
	.long _sys_timerfd_gettime /* 310 */

        /*
         * NOTE!! This doesn't have to be exact - we just have
         * to make sure we have _enough_ of the "sys_ni_syscall"
         * entries. Don't panic if you notice that this hasn't
         * been shrunk every time we add a new system call.
         */

	.rept NR_syscalls-(.-_sys_call_table)/4
		.long _sys_ni_syscall
	.endr

/* ============================================================[ EOF ]=== */
