/*
 *  linux/arch/ba/kernel/head.S
 *
 *  BA version
 *    author(s): Matjaz Breskvar (phoenix@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    initial port to BA architecture
 *  06. 12. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    major changes in handling exceptions
 *    physical addresses used only by code in head.S
 *    cleand exception handlers and moved them to entry.S
 */

#include <linux/linkage.h>
#include <linux/threads.h>
#include <linux/errno.h>
#include <asm/processor.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <asm/cache.h>
#include <asm/aeonR2_asm.h>
#include <asm/spr_defs.h>
#include <asm/board.h>
#include <asm/asm-offsets.h>
#include <asm/thread_info.h>
#include <asm/alternatives.h>

//#define CONFIG_AEONR2_BOOT

#ifdef CONFIG_AEONR2_BOOT
#define MIU_W(addr,val) \
	b.ori	r4,r3,(addr<<1)		;\
	b.ori	r5,r0,(val)		;\
	b.sw	0x0(r4),r5

#define MIU_INIT \
	MIU_W(0x1238, 0x0000)		;\
	MIU_W(0x1238, 0x0000)		;\
	MIU_W(0x1220, 0x229e)		;\
	MIU_W(0x1202, 0x0111)		;\
	MIU_W(0x1204, 0x0351)		;\
	MIU_W(0x1208, 0x33c8)		;\
	MIU_W(0x120a, 0x0252)		;\
	MIU_W(0x120c, 0x5634)		;\
	MIU_W(0x1206, 0x0010)		;\
	MIU_W(0x120e, 0x000e)		;\
	MIU_W(0x1240, 0x0001)		;\
	MIU_W(0x1260, 0x0001)		;\
	MIU_W(0x1280, 0x0001)		;\
	MIU_W(0x12a0, 0x0001)		;\
	MIU_W(0x1200, 0x0001)

#define AEON_CODE_BASE	0x0
#define AEON_DATA_BASE	0x0

#define BOOT_COPY_SPI_ADDR	0x0
#define BOOT_COPY_MIU_ADDR	0x0
#define BOOT_COPY_LENGTH	0x200000

#endif

/* some defines to ease BA assembly programming */
#include "ba_funcs.S"

#define DRAM_END (PAGE_OFFSET+CONFIG_AEONR2_MEMORY_SIZE)

#if defined(CONFIG_AEONR2_ICACHE_ENABLED) && defined(CONFIG_AEONR2_DCACHE_ENABLED)
#  define EXCEPTION_SR_CACHE  (SPR_SR_DCE | SPR_SR_ICE | SPR_SR_SM)
#elif defined(CONFIG_AEONR2_ICACHE_ENABLED) && !defined(CONFIG_AEONR2_DCACHE_ENABLED)
#  define EXCEPTION_SR_CACHE  (             SPR_SR_ICE | SPR_SR_SM)
#elif defined(CONFIG_AEONR2_DCACHE_ENABLED) && !defined(CONFIG_AEONR2_ICACHE_ENABLED)
#  define EXCEPTION_SR_CACHE  (SPR_SR_DCE              | SPR_SR_SM)
#else
#  define EXCEPTION_SR_CACHE  (                          SPR_SR_SM)
#endif

#ifdef CONFIG_MMU
#  define EXCEPTION_SR (EXCEPTION_SR_CACHE | SPR_SR_IME | SPR_SR_DME)
#else
#  define EXCEPTION_SR (EXCEPTION_SR_CACHE                          )
#endif

#undef	CONFIG_AEONR2_EXCEPTION_REGENERATE
//#define	CONFIG_AEONR2_EXCEPTION_REGENERATE 1

#undef	DEBUG_TLB_EXCEPTIONS
//#define	DEBUG_TLB_EXCEPTIONS 1


/* 2: enable debug printing of all writes to TLBs
 * 1: no printing, just placeholder instructions to retain same offests
 * 0: no printing and no placeholder instructions
 */
#define DEBUG_PRINT_TLB_WRITES 0
/* ============================================[ tmp store locations ]=== */

#define UART_PRINT_STORE_GPR3		b.sw	0x1000(r0),r3
#define UART_PRINT_RESTORE_GPR3		b.lwz	r3,0x1000(r0)
#define UART_PRINT_STORE_GPR4		b.sw	0x1004(r0),r4
#define UART_PRINT_RESTORE_GPR4		b.lwz	r4,0x1004(r0)
#define UART_PRINT_STORE_GPR5		b.sw	0x1008(r0),r5
#define UART_PRINT_RESTORE_GPR5		b.lwz	r5,0x1008(r0)
#define UART_PRINT_STORE_GPR6		b.sw	0x100c(r0),r6
#define UART_PRINT_RESTORE_GPR6		b.lwz	r6,0x100c(r0)
#define UART_PRINT_STORE_GPR7		b.sw	0x1010(r0),r7
#define UART_PRINT_RESTORE_GPR7		b.lwz	r7,0x1010(r0)
#define UART_PRINT_STORE_GPR8		b.sw	0x1014(r0),r8
#define UART_PRINT_RESTORE_GPR8		b.lwz	r8,0x1014(r0)
#define UART_PRINT_STORE_GPR9		b.sw	0x1018(r0),r9
#define UART_PRINT_RESTORE_GPR9		b.lwz	r9,0x1018(r0)
#define UART_PRINT_STORE_GPR10		b.sw	0x101c(r0),r10
#define UART_PRINT_RESTORE_GPR10	b.lwz	r10,0x101c(r0)
#define UART_PRINT_STORE_GPR11		b.sw	0x1020(r0),r11
#define UART_PRINT_RESTORE_GPR11	b.lwz	r11,0x1020(r0)
#define UART_PRINT_STORE_GPR12		b.sw	0x1024(r0),r12
#define UART_PRINT_RESTORE_GPR12	b.lwz	r12,0x1024(r0)
#define UART_PRINT_STORE_GPR13		b.sw	0x1028(r0),r13
#define UART_PRINT_RESTORE_GPR13	b.lwz	r13,0x1028(r0)
#define UART_PRINT_STORE_GPR14		b.sw	0x102c(r0),r14
#define UART_PRINT_RESTORE_GPR14	b.lwz	r14,0x102c(r0)
#define UART_PRINT_STORE_GPR15		b.sw	0x1030(r0),r15
#define UART_PRINT_RESTORE_GPR15	b.lwz	r15,0x1030(r0)
#define UART_PRINT_STORE_GPR16		b.sw	0x1030(r0),r16
#define UART_PRINT_RESTORE_GPR16	b.lwz	r16,0x1030(r0)
#define UART_PRINT_STORE_GPR17		b.sw	0x1034(r0),r17
#define UART_PRINT_RESTORE_GPR17	b.lwz	r17,0x1034(r0)
#define UART_PRINT_STORE_GPR18		b.sw	0x1038(r0),r18
#define UART_PRINT_RESTORE_GPR18	b.lwz	r18,0x1038(r0)

#define UART_CALL_STORE_GPR3		b.sw	0x1100(r0),r3
#define UART_CALL_RESTORE_GPR3		b.lwz	r3,0x1100(r0)
#define UART_CALL_STORE_GPR4		b.sw	0x1104(r0),r4
#define UART_CALL_RESTORE_GPR4		b.lwz	r4,0x1104(r0)
#define UART_CALL_STORE_GPR5		b.sw	0x1108(r0),r5
#define UART_CALL_RESTORE_GPR5		b.lwz	r5,0x1108(r0)
#define UART_CALL_STORE_GPR6		b.sw	0x110c(r0),r6
#define UART_CALL_RESTORE_GPR6		b.lwz	r6,0x110c(r0)
#define UART_CALL_STORE_GPR7		b.sw	0x1110(r0),r7
#define UART_CALL_RESTORE_GPR7		b.lwz	r7,0x1110(r0)
#define UART_CALL_STORE_GPR8		b.sw	0x1114(r0),r8
#define UART_CALL_RESTORE_GPR8		b.lwz	r8,0x1114(r0)
#define UART_CALL_STORE_GPR9		b.sw	0x1118(r0),r9
#define UART_CALL_RESTORE_GPR9		b.lwz	r9,0x1118(r0)
#define UART_CALL_STORE_GPR10		b.sw	0x111c(r0),r10
#define UART_CALL_RESTORE_GPR10		b.lwz	r10,0x111c(r0)
#define UART_CALL_STORE_GPR11		b.sw	0x1120(r0),r11
#define UART_CALL_RESTORE_GPR11		b.lwz	r11,0x1120(r0)
#define UART_CALL_STORE_GPR12		b.sw	0x1124(r0),r12
#define UART_CALL_RESTORE_GPR12		b.lwz	r12,0x1124(r0)
#define UART_CALL_STORE_GPR13		b.sw	0x1128(r0),r13
#define UART_CALL_RESTORE_GPR13		b.lwz	r13,0x1128(r0)
#define UART_CALL_STORE_GPR14		b.sw	0x112c(r0),r14
#define UART_CALL_RESTORE_GPR14		b.lwz	r14,0x112c(r0)
#define UART_CALL_STORE_GPR15		b.sw	0x1130(r0),r15
#define UART_CALL_RESTORE_GPR15		b.lwz	r15,0x1130(r0)
#define UART_CALL_STORE_GPR16		b.sw	0x1130(r0),r16
#define UART_CALL_RESTORE_GPR16		b.lwz	r16,0x1130(r0)
#define UART_CALL_STORE_GPR17		b.sw	0x1134(r0),r17
#define UART_CALL_RESTORE_GPR17		b.lwz	r17,0x1134(r0)
#define UART_CALL_STORE_GPR18		b.sw	0x1138(r0),r18
#define UART_CALL_RESTORE_GPR18		b.lwz	r18,0x1138(r0)

/*
 * emergency_print temporary stores
 */
#define EMERGENCY_PRINT_STORE_GPR4	b.sw	0x20(r0),r4
#define EMERGENCY_PRINT_LOAD_GPR4	b.lwz	r4,0x20(r0)

#define EMERGENCY_PRINT_STORE_GPR5	b.sw	0x24(r0),r5
#define EMERGENCY_PRINT_LOAD_GPR5	b.lwz	r5,0x24(r0)

#define EMERGENCY_PRINT_STORE_GPR6	b.sw	0x28(r0),r6
#define EMERGENCY_PRINT_LOAD_GPR6	b.lwz	r6,0x28(r0)

#define EMERGENCY_PRINT_STORE_GPR7	b.sw	0x2c(r0),r7
#define EMERGENCY_PRINT_LOAD_GPR7	b.lwz	r7,0x2c(r0)

#define EMERGENCY_PRINT_STORE_GPR8	b.sw	0x30(r0),r8
#define EMERGENCY_PRINT_LOAD_GPR8	b.lwz	r8,0x30(r0)

#define EMERGENCY_PRINT_STORE_GPR9	b.sw	0x34(r0),r9
#define EMERGENCY_PRINT_LOAD_GPR9	b.lwz	r9,0x34(r0)

/*
 * EXCEPTION_DEBUG temporary stores
 */
#define EXCEPTION_DEBUG_STORE_GPR3	b.sw	0x40(r0),r3
#define EXCEPTION_DEBUG_LOAD_GPR3	b.lwz	r3,0x40(r0)

#define EXCEPTION_DEBUG_STORE_GPR4	b.sw	0x44(r0),r4
#define EXCEPTION_DEBUG_LOAD_GPR4	b.lwz	r4,0x44(r0)

#define EXCEPTION_DEBUG_STORE_GPR9	b.sw	0x48(r0),r9
#define EXCEPTION_DEBUG_LOAD_GPR9	b.lwz	r9,0x48(r0)

#define EXCEPTION_DEBUG_STORE_F_GPR9	b.sw	0x4c(r0),r9
#define EXCEPTION_DEBUG_LOAD_F_GPR9	b.lwz	r9,0x4c(r0)

#define EXCEPTION_DEBUG_STORE_F_GPR31	b.sw	0x90(r0),r31
#define EXCEPTION_DEBUG_LOAD_F_GPR31	b.lwz	r31,0x90(r0)

/*
 * EXCEPTION_REGENERATE temporary stores
 */
#define E_REGEN_STORE_STATUS(reg)	b.sw	0x50(r0),reg
#define E_REGEN_LOAD_STATUS(reg)	b.lwz	reg,0x50(r0)

#define E_REGEN_STORE_GPR3		b.sw	0x54(r0),r3
#define E_REGEN_LOAD_GPR3		b.lwz	r3,0x54(r0)

#define E_REGEN_STORE_GPR4		b.sw	0x58(r0),r4
#define E_REGEN_LOAD_GPR4		b.lwz	r4,0x58(r0)

#define E_REGEN_STORE_GPR5		b.sw	0x5c(r0),r5
#define E_REGEN_LOAD_GPR5		b.lwz	r5,0x5c(r0)

#define E_REGEN_STORE_GPR6		b.sw	0x60(r0),r6
#define E_REGEN_LOAD_GPR6		b.lwz	r6,0x60(r0)

#define E_REGEN_STORE_GPR9		b.sw	0x14(r0),r9
#define E_REGEN_LOAD_GPR9		b.lwz	r9,0x14(r0)


/*
 * TLB miss handlers temorary stores
 */
#define EXCEPTION_STORE_GPR9		b.sw	0x10(r0),r9
#define EXCEPTION_LOAD_GPR9		b.lwz	r9,0x10(r0)

#define EXCEPTION_STORE_GPR2		b.sw	0x64(r0),r2
#define EXCEPTION_LOAD_GPR2		b.lwz	r2,0x64(r0)

#define EXCEPTION_STORE_GPR3		b.sw	0x68(r0),r3
#define EXCEPTION_LOAD_GPR3		b.lwz	r3,0x68(r0)
#define EXCEPTION_CLEAR_SAVED_GPR3	b.sw	0x68(r0),r0
#define EXCEPTION_T_STORE_GPR3		b.sw	0x6c(r0),r3
#define EXCEPTION_T_LOAD_GPR3		b.lwz	r3,0x6c(r0)

#define EXCEPTION_STORE_GPR4		b.sw	0x6c(r0),r4
#define EXCEPTION_LOAD_GPR4		b.lwz	r4,0x6c(r0)

#define EXCEPTION_STORE_GPR5		b.sw	0x70(r0),r5
#define EXCEPTION_LOAD_GPR5		b.lwz	r5,0x70(r0)

#define EXCEPTION_STORE_GPR6		b.sw	0x74(r0),r6
#define EXCEPTION_LOAD_GPR6		b.lwz	r6,0x74(r0)

#define EXCEPTION_STORE_GPR7		b.sw	0x78(r0),r7
#define EXCEPTION_LOAD_GPR7		b.lwz	r7,0x78(r0)

/*
 * EXCEPTION_HANDLE temporary stores
 */
#define EXCEPTION_T_STORE_GPR31		b.sw	0x7c(r0),r31
#define EXCEPTION_T_LOAD_GPR31(reg)	b.lwz	reg,0x7c(r0)

#define EXCEPTION_T_STORE_GPR29		b.sw	0x80(r0),r29
#define EXCEPTION_T_LOAD_GPR29(reg)	b.lwz	reg,0x80(r0)

#define EXCEPTION_T_STORE_SP		b.sw	0x84(r0),r1
#define EXCEPTION_T_LOAD_SP(reg)	b.lwz	reg,0x84(r0)

/*
 * UNHANDLED_EXCEPTION temporary stores
 */
#define UNH_EXCEPTION_T_STORE_GPR31	b.sw	0x88(r0),r31
#define UNH_EXCEPTION_T_LOAD_GPR31(reg)	b.lwz	reg,0x88(r0)

#define UNH_EXCEPTION_T_STORE_GPR2	b.sw	0x8c(r0),r2
#define UNH_EXCEPTION_T_LOAD_GPR2(reg)	b.lwz	reg,0x8c(r0)

#define UNH_EXCEPTION_T_STORE_SP	b.sw	0x90(r0),r1
#define UNH_EXCEPTION_T_LOAD_SP(reg)	b.lwz	reg,0x90(r0)

/* =========================================================[ macros ]=== */


#define LOAD_CURRENT_THREAD_INFO(reg,t1)			\
	LOAD_SYMBOL_2_GPR(reg,_current_thread_info_set)		;\
	tophys	(t1,reg)					;\
	b.lwz	reg,0(t1)

#define GET_CURRENT_THREAD_INFO(reg,ksp)			\
	b.srli	reg,ksp,PAGE_SHIFT				;\
	b.slli	reg,reg,PAGE_SHIFT

#define GET_CURRENT_PGD(reg,t1)					\
	LOAD_SYMBOL_2_GPR(reg,_current_pgd)			;\
	tophys	(t1,reg)					;\
	b.lwz	reg,0(t1)

#if (DEBUG_PRINT_TLB_WRITES==2)

#define DEBUG_BOOT_DTLB_WRITE_MR \
	EXCEPTION_DEBUG_STORE_F_GPR9 ;\
	C_ALT_L_EXT_JUMP(jal, _debug_boot_dtlb_write_mr)	;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_BOOT_DTLB_WRITE_TR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_boot_dtlb_write_tr)	;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_BOOT_ITLB_WRITE_MR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_boot_itlb_write_mr)	;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_BOOT_ITLB_WRITE_TR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_boot_itlb_write_tr)	;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_DTLB_WRITE_MR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_dtlb_write_mr)		;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_DTLB_WRITE_TR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_dtlb_write_tr)		;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_ITLB_WRITE_MR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_itlb_write_mr)		;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_ITLB_WRITE_TR \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_itlb_write_tr)		;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_EEAR_WRITE \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_eear_write)		;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#define DEBUG_EPCR_WRITE \
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	C_ALT_L_EXT_JUMP(jal, _debug_epcr_write)		;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#elif (DEBUG_PRINT_TLB_WRITES==1)

#  define DEBUG_BOOT_DTLB_WRITE_MR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_BOOT_DTLB_WRITE_TR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_BOOT_ITLB_WRITE_MR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_BOOT_ITLB_WRITE_TR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_DTLB_WRITE_MR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_DTLB_WRITE_TR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_ITLB_WRITE_MR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_ITLB_WRITE_TR \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_EEAR_WRITE \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#  define DEBUG_EPCR_WRITE \
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0 ;\
	b.nop 0x0

#else

#  define DEBUG_BOOT_DTLB_WRITE_MR
#  define DEBUG_BOOT_DTLB_WRITE_TR
#  define DEBUG_BOOT_ITLB_WRITE_MR
#  define DEBUG_BOOT_ITLB_WRITE_TR
#  define DEBUG_DTLB_WRITE_MR
#  define DEBUG_DTLB_WRITE_TR
#  define DEBUG_ITLB_WRITE_MR
#  define DEBUG_ITLB_WRITE_TR

#  define DEBUG_EEAR_WRITE
#  define DEBUG_EPCR_WRITE

#endif


#if defined(CONFIG_AEONR2_EXCEPTION_DEBUG) && defined(CONFIG_AEONR2_EXCEPTION_REGENERATE)

#  define EXCEPTION_DEBUG					\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.mfspr	r4,r0,SPR_NPC					;\
	b.jal	_exception_debug 				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_VALUE(value)				\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.addi	r4,r0,value					;\
	b.jal	_exception_debug				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_ER					\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.mfspr	r4,r0,SPR_NPC					;\
	b.jal	_exception_debug				;\
	b.jal	_exception_regenerate				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_VALUE_ER(value)			\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.addi	r4,r0,value					;\
	b.jal	_exception_debug				;\
	b.jal	_exception_regenerate				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#elif !defined(CONFIG_AEONR2_EXCEPTION_DEBUG) && defined(CONFIG_AEONR2_EXCEPTION_REGENERATE)

#  define EXCEPTION_DEBUG
#  define EXCEPTION_DEBUG_VALUE(value)

#  define EXCEPTION_DEBUG_ER					\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.jal	_exception_regenerate				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_VALUE_ER(value)			\
	EXCEPTION_DEBUG_STORE_GPR4				;\
   	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.jal	_exception_regenerate				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#elif defined(CONFIG_AEONR2_EXCEPTION_DEBUG) && !defined(CONFIG_AEONR2_EXCEPTION_REGENERATE)

#  define EXCEPTION_DEBUG					\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.mfspr  r4,r0,SPR_NPC					;\
	b.jal	_exception_debug 				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_VALUE(value)				\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.addi	r4,r0,value					;\
	b.jal	_exception_debug				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_ER					\
	EXCEPTION_DEBUG

#  define EXCEPTION_DEBUG_VALUE_ER(value)			\
	EXCEPTION_DEBUG_VALUE(value)

#else

#  define EXCEPTION_DEBUG
#  define EXCEPTION_DEBUG_VALUE(value)
#  define EXCEPTION_DEBUG_ER
#  define EXCEPTION_DEBUG_VALUE_ER(value)
#endif

#  define EXCEPTION_DEBUG_VALUE_ER_ENABLED(value)		\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.addi	r4,r0,value					;\
	b.jal	_exception_debug				;\
	b.jal	_exception_regenerate				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#  define EXCEPTION_DEBUG_VALUE_ENABLED(value)			\
	EXCEPTION_DEBUG_STORE_GPR4				;\
	EXCEPTION_DEBUG_STORE_F_GPR9				;\
	b.addi	r4,r0,value					;\
	b.jal	_exception_debug				;\
	EXCEPTION_DEBUG_LOAD_GPR4				;\
	EXCEPTION_DEBUG_LOAD_F_GPR9

#ifdef DEBUG_TLB_EXCEPTIONS
#  define DEBUG_TLB_PROBE(value)				\
	EXCEPTION_DEBUG_VALUE_ENABLED(value)
#else
#  define DEBUG_TLB_PROBE(value)
#endif


#define UNHANDLED_EXCEPTION_PUSH_PT_REGS			\
	/* PC, r1, r9 must be already stored */			;\
	b.sw	GPR2(r1),r2					;\
	b.sw	GPR3(r1),r3					;\
	b.sw	ORIG_GPR3(r1),r3				;\
	b.mfspr	r3,r0,SPR_ESR_BASE				;\
	b.sw	SR(r1),r3					;\
	b.sw	GPR4(r1),r4					;\
	b.sw	GPR5(r1),r5					;\
	b.sw	GPR6(r1),r6					;\
	b.sw	GPR7(r1),r7					;\
	b.sw	GPR8(r1),r8					;\
	b.sw	GPR10(r1),r10					;\
	b.sw	GPR11(r1),r11					;\
	b.sw	GPR12(r1),r12					;\
	b.sw	GPR13(r1),r13					;\
	b.sw	GPR14(r1),r14					;\
	b.sw	GPR15(r1),r15					;\
	b.sw	GPR16(r1),r16					;\
	b.sw	GPR17(r1),r17					;\
	b.sw	GPR18(r1),r18					;\
	b.sw	GPR19(r1),r19					;\
	b.sw	GPR20(r1),r20					;\
	b.sw	GPR21(r1),r21					;\
	b.sw	GPR22(r1),r22					;\
	b.sw	GPR23(r1),r23					;\
	b.sw	GPR24(r1),r24					;\
	b.sw	GPR25(r1),r25					;\
	b.sw	GPR26(r1),r26					;\
	b.sw	GPR27(r1),r27					;\
	b.sw	GPR28(r1),r28					;\
	b.sw	GPR29(r1),r29					;\
	b.sw	GPR30(r1),r30					;\
	b.sw	GPR31(r1),r31					;\
	b.sw	RESULT(r1),r0

#if defined(CONFIG_AEONR2_EXCEPTION_TRACE)
# define EXCEPTION_ADD_TO_TRACE					\
	EXCEPTION_DEBUG_STORE_GPR9				;\
	EXCEPTION_DEBUG_STORE_GPR3				;\
	b.mfspr	r3,r0,SPR_NPC					;\
	b.andi	r3,r3,0xf00					;\
	b.jal	exception_add_to_trace				;\
	EXCEPTION_DEBUG_LOAD_GPR9				;\
	EXCEPTION_DEBUG_LOAD_GPR3
#else
# define EXCEPTION_ADD_TO_TRACE
#endif

/*
 * DSCR: this is a common hook for handling exceptions. it will save
 *       the needed registers, set up stack and pointer to current
 *	 then jump to the handler while enabling MMU
 *
 * PRMS: handler	- a function to jump to. it has to save the
 *			remaining registers to kernel stack, call
 *			appropriate arch-independant exception handler
 *			and finaly jump to ret_from_except
 *
 * PREQ: unchanged state from the time exception happened
 *
 * POST: SAVED the following registers original value
 *	       to the new created exception frame pointed to by r1
 *
 *	 r1  - ksp	pointing to the new (exception) frame
 *	 r4  - EEAR     exception EA
 *	 r12 - syscall  0, since we didn't come from syscall
 *	 r13 - temp	it actually contains new SR, not needed anymore
 *	 r29 - current	pointing to current_thread_info struct
 *	 r31 - handler	address of the handler we'll jump to
 *
 *	 handler has to save remaining registers to the exception
 *	 ksp frame *before* tainting them!
 *
 * NOTE: this function is not reentrant per se. reentrancy is guaranteed
 *       by processor disabling all exceptions/interrupts when exception
 *	 accours.
 *
 * OPTM: no need to make it so wasteful to extract ksp when in user mode
 */
#define EXCEPTION_HANDLE(handler)				\
	EXCEPTION_DEBUG						;\
	EXCEPTION_T_STORE_GPR31					;\
	EXCEPTION_T_STORE_GPR29					;\
	EXCEPTION_T_STORE_SP					;\
	b.mfspr	r31,r0,SPR_ESR_BASE				;\
	b.andi	r31,r31,SPR_SR_SM				;\
	b.sfnei	r31,0 /* kernel_mode */				;\
	b.bf	2f /* kernel_mode */				;\
1: /* user_mode: */						;\
	LOAD_SYMBOL_2_GPR(r1,_current_thread_info_set)		;\
	tophys	(r31,r1)					;\
	/* load current_thread_info  */				;\
	b.lwz	r29,0(r31)					;\
	tophys	(r31,r29)					;\
	b.lwz	r1,(TI_KSP)(r31)				;\
	/* fall through */					;\
2: /* kernel_mode: */						;\
	EXCEPTION_ADD_TO_TRACE					;\
	/* create new stack frame, save only needed gprs */	;\
	/* r1: KSP, r29: current, r4: EEAR, r31: __pa(KSP) */	;\
	/* r12:	temp, syscall indicator, r13 temp */		;\
	b.addi	r1,r1,-(INT_FRAME_SIZE)				;\
	/* r1 is KSP, r31 is __pa(KSP) */			;\
	tophys	(r31,r1)					;\
	b.sw	GPR12(r31),r12					;\
	b.mfspr r12,r0,SPR_EPCR_BASE				;\
	b.sw	PC(r31),r12					;\
	b.mfspr	r12,r0,SPR_ESR_BASE				;\
	b.sw	SR(r31),r12					;\
	/* save r31 */						;\
	EXCEPTION_T_LOAD_GPR31(r12)				;\
	b.sw	GPR31(r31),r12					;\
	/* save r29 as was prior to exception */		;\
	EXCEPTION_T_LOAD_GPR29(r12)				;\
	b.sw	GPR29(r31),r12					;\
	/* save SP as was prior to exception */			;\
	EXCEPTION_T_LOAD_SP(r12)				;\
	b.sw	SP(r31),r12					;\
	b.sw	GPR13(r31),r13					;\
	/* save exception r4, set r4 = EA */			;\
	b.sw	GPR4(r31),r4					;\
	b.mfspr	r4,r0,SPR_EEAR_BASE				;\
	/* r12 == 1 if we come from syscall */			;\
	CLEAR_GPR(r12)						;\
	/* ----- turn on MMU ----- */				;\
	b.ori	r31,r0,(EXCEPTION_SR)				;\
	b.mtspr	r0,r31,SPR_ESR_BASE				;\
	/* r31:	EA address of handler */			;\
	LOAD_SYMBOL_2_GPR(r31,handler)				;\
	b.mtspr	r0,r31,SPR_EPCR_BASE				;\
	b.rfe

/*
 * this doesn't work
 *
 *
 * #ifdef CONFIG_JUMP_UPON_UNHANDLED_EXCEPTION
 * #define UNHANDLED_EXCEPTION(handler)				\
 *	b.ori	r3,r0,0x1					;\
 *	b.mtspr	r0,r3,SPR_SR					;\
 *	b.movhi	r3,hi(0xf0000100)				;\
 *	b.ori	r3,r3,lo(0xf0000100)				;\
 *	b.jr	r3						;\
 *
 * #endif
 */

/* DSCR: this is the same as EXCEPTION_HANDLE(), we are just
 *       a bit more carefull (if we have a SP or current pointer
 *       corruption) and set them up from 'current_set'
 *
 */
#define UNHANDLED_EXCEPTION(handler,vector)			\
	EXCEPTION_DEBUG						;\
	EXCEPTION_T_STORE_GPR31					;\
	EXCEPTION_T_STORE_GPR29					;\
	EXCEPTION_T_STORE_SP					;\
	/* temporary store r3, r9 into r31, r29 */		;\
	b.addi	r31,r3,0x0					;\
	b.addi	r29,r9,0x0					;\
	/* the string referenced by r3 must be low enough */	;\
	b.ori	r3,r0,lo(_string_unhandled_exception)		;\
	b.jal	_emergency_print				;\
	b.mfspr	r3,r0,SPR_NPC					;\
	b.andi	r3,r3,0x1f00					;\
	b.jal	_emergency_print_nr				;\
	/* the string referenced by r3 must be low enough */	;\
	b.ori	r3,r0,lo(_string_epc_prefix)			;\
	b.jal	_emergency_print				;\
	b.mfspr	r3,r0,SPR_EPCR_BASE				;\
	b.jal	_emergency_print_nr				;\
	b.ori	r3,r0,lo(_string_nl)				;\
	b.jal	_emergency_print				;\
	/* use special stack space just for unhandled except */ ;\
	LOAD_SYMBOL_2_GPR(r3,_unhandled_stack_top)		;\
	tophys	(r1,r3)						;\
	b.addi	r1,r1,-(INT_FRAME_SIZE)				;\
	/* save PC, r9, r1 to pt_regs struct */			;\
	b.mfspr	r3,r0,SPR_EPCR_BASE				;\
	b.sw	PC(r1),r3					;\
	b.sw	GPR9(r1),r29					;\
	EXCEPTION_T_LOAD_SP(r3)					;\
	b.sw	SP(r1),r3					;\
	/* restore r29, r31, unclobber r3 */			;\
	b.ori	r3,r31,0					;\
	EXCEPTION_T_LOAD_GPR29(r29)				;\
	EXCEPTION_T_LOAD_GPR31(r31)				;\
	/* save rest of registers */				;\
	b.jal	_unhandled_exception_push_pt_regs		;\
	/* jump to handler */					;\
	b.addi	r3,r1,0						;\
	b.mfspr	r4,r0,SPR_EEAR_BASE				;\
	b.mfspr	r5,r0,SPR_EPCR_BASE				;\
	b.addi	r6,r0,vector	                                ;\
	b.jal	handler

/* ===============================================[ page table masks ]=== */

/* bit 4 is used in hardware as write back cache bit. we never use this bit
 * explicitly, so we can reuse it as _PAGE_FILE bit and mask it out when
 * writing into hardware pte's
 */

#define DTLB_UP_CONVERT_MASK  0x3fa
#define ITLB_UP_CONVERT_MASK  0x3a

/* for SMP we'd have (this is a bit subtle, CC must be always set
 * for SMP, but since we have _PAGE_PRESENT bit always defined
 * we can just modify the mask)
 */
#define DTLB_SMP_CONVERT_MASK  0x3fb
#define ITLB_SMP_CONVERT_MASK  0x3b


/* =====================================================[ exceptions] === */

/* ---[ 0x100: RESET exception ]----------------------------------------- */
	.org 0x100

	b.ori	r3,r0,0x1
	b.mtspr	r0,r3,SPR_SR

#ifdef CONFIG_AEONR2
#if 0//AEONR2
wdt_init:
	b.jal	_wdt_init
#endif

uart_init:
	b.jal	_early_uart_init

#ifdef CONFIG_AEONR2_BOOT
miu_init:
	b.jal	_miu_init

spi_copy:
	b.jal	_spi_copy

remap:
	/* switch to SDRAM */
	b.movhi	r3,0xa000
	b.ori	r4,r3,(0x1e00<<1)
	b.ori	r5,r0,0x2
	b.sb	0x0(r4),r5
#endif // CONFIG_AEONR2_BOOT

#if 0//AEONR2
// set SPI base address
// This should be run or the CPU will hang
spi_init:
	b.jal	_spi_init

qmem_init:
	b.jal	_qmem_init
#endif

// Enable D-cache on Eris
// D-cache must be turned on or the CPU will hang
// This should be run before spi_init
enable_dc:
	b.jal	_dc_enable

// Enable I-cache on Eris
// Can be turned on or off
enable_ic:
	b.jal	_ic_enable

#if 0//AEONR2
// Enable JTAG function on Eris
enable_jtag:
	b.jal	_jtag_init
#endif

#endif // CONFIG_AEONR2
#if 1
	LOAD_SYMBOL_2_GPR(r4,_string_ok_booting)
	tophys	(r3,r4)
	b.jal	_emergency_print
#endif

jump_to_start:
	LOAD_SYMBOL_2_GPR(r4, _start)
	tophys	(r3,r4)				// physical address
	b.jr	r3

/* ---[ 0x200: BUS exception ]------------------------------------------- */
	.org 0x200
_dispatch_bus_fault:
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x200)

/* ---[ 0x300: Data Page Fault exception ]------------------------------- */
	.org 0x300
_dispatch_do_dpage_fault:
#if 1//AEONR2 MMU
	EXCEPTION_STORE_GPR2
	EXCEPTION_STORE_GPR4
	LOAD_SYMBOL_2_GPR(r2,_dtlb_miss_call_page_fault)
	tophys  (r4,r2)
	b.lwz	r2,0x0(r4)
	b.sfeqi	r2,0x1
	b.bnf	1f
	EXCEPTION_LOAD_GPR3
	b.sw	0x0(r4),r0
1:
	EXCEPTION_LOAD_GPR2
	EXCEPTION_LOAD_GPR4
#endif
	EXCEPTION_HANDLE(_data_page_fault_handler)

/* ---[ 0x400: Insn Page Fault exception ]------------------------------- */
	.org 0x400
_dispatch_do_ipage_fault:
#if 1//AEONR2 MMU
	EXCEPTION_STORE_GPR2
	EXCEPTION_STORE_GPR4
	LOAD_SYMBOL_2_GPR(r2,_itlb_miss_call_page_fault)
	tophys	(r4,r2)
	b.lwz	r2,0x0(r4)
	b.sfeqi	r2,0x1
	b.bnf	1f
	EXCEPTION_LOAD_GPR3
	b.sw	0x0(r4),r0
1:
	EXCEPTION_LOAD_GPR2
	EXCEPTION_LOAD_GPR4
#endif
#if 1
	EXCEPTION_HANDLE(_insn_page_fault_handler)
#else
	EXCEPTION_T_STORE_GPR31
	EXCEPTION_T_STORE_GPR29
	EXCEPTION_T_STORE_SP
	b.mfspr	r31,r0,SPR_ESR_BASE
	b.andi	r31,r31,SPR_SR_SM
	b.sfnei	r31,0 /* kernel_mode */
	b.bf	2f /* kernel_mode */
1: /* user_mode: */
	LOAD_SYMBOL_2_GPR(r1,_current_thread_info_set)
	tophys	(r31,r1)
	/* load current_thread_info */
	b.lwz	r29,0(r31)
	tophys	(r31,r29)
	b.lwz	r1,(TI_KSP)(r31)
	/* fall through */
2: /* kernel_mode: */
	/* create new stack frame, save only needed gprs */
	/* r1: KSP, r29: current, r4: EEAR, r31: __pa(KSP) */
	/* r12:	temp, syscall indicator, r13 temp */
	b.addi	r1,r1,-(INT_FRAME_SIZE)	
	/* r1 is KSP, r31 is __pa(KSP) */
	tophys	(r31,r1)
	b.sw	GPR12(r31),r12
	b.mfspr r12,r0,SPR_EPCR_BASE
	b.sw	PC(r31),r12
	b.mfspr	r12,r0,SPR_ESR_BASE
	b.sw	SR(r31),r12
	/* save r31 */
	EXCEPTION_T_LOAD_GPR31(r12)
	b.sw	GPR31(r31),r12
	/* save r29 as was prior to exception */
	EXCEPTION_T_LOAD_GPR29(r12)
	b.sw	GPR29(r31),r12
	/* save SP as was prior to exception */
	EXCEPTION_T_LOAD_SP(r12)
	b.sw	SP(r31),r12
	b.sw	GPR13(r31),r13
	/* save exception r4, set r4 = EA */
	b.sw	GPR4(r31),r4
	b.mfspr	r4,r0,SPR_EEAR_BASE
#if 1//AEONR2 MMU
	b.addi	r4,r4,0x4
	b.mfspr	r12,r0,80
	b.fl1	r12,r12
	b.sub	r4,r4,r12
	b.mtspr	r0,r4,SPR_EEAR_BASE
#endif
	/* r12 == 1 if we come from syscall */
	CLEAR_GPR(r12)
	/* ----- turn on MMU ----- */
	b.ori	r31,r0,(EXCEPTION_SR)
	b.mtspr	r0,r31,SPR_ESR_BASE
	/* r31:	EA address of handler */
	LOAD_SYMBOL_2_GPR(r31,_insn_page_fault_handler)
	b.mtspr	r0,r31,SPR_EPCR_BASE
	b.rfe
#endif

/* ---[ 0x500: Timer exception ]----------------------------------------- */
	.org 0x500
	EXCEPTION_HANDLE(_timer_handler)

/* ---[ 0x600: Aligment exception ]-------------------------------------- */
	.org 0x600
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x600)

/* ---[ 0x700: Illegal insn exception ]---------------------------------- */
	.org 0x700
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x700)

/* ---[ 0x800: External interrupt exception ]---------------------------- */
	.org 0x800
	EXCEPTION_HANDLE(_external_irq_handler)

/* ---[ 0x900: DTLB miss exception ]------------------------------------- */
	.org 0x900
	b.j	dtlb_miss_boot

	/* 0: dtlb_miss_boot, can not be relocated
	 * 1: dtlb_miss_mmu1, MMU1 miss handler
	 * 2: dtlb_miss_mmu2: MMU2 miss handler
	 */

.data
.align 4
ENTRY(_dtlb_miss_handlers)
	/* first entry  */
	.long	dtlb_miss_boot
	.long	dtlb_miss_boot_end
	/* second entry */
	.long	dtlb_miss_mmu2
	.long	dtlb_miss_mmu2_end
#if 1//AEONR2 MMU
ENTRY(_dtlb_miss_call_page_fault)
	.long	0x0
#endif
.previous


/* ---[ 0xa00: ITLB miss exception ]------------------------------------- */
	.org 0xa00
	b.j	itlb_miss_boot

	/* 0: dtlb_miss_boot, can not be relocated
	 * 1: dtlb_miss_mmu1, MMU1 miss handler
	 * 2: dtlb_miss_mmu2: MMU2 miss handler
	 */

.data
.align 4
ENTRY(_itlb_miss_handlers)
	/* first entry  */
	.long	itlb_miss_boot
	.long	itlb_miss_boot_end
	/* second entry */
	.long	itlb_miss_mmu2
	.long	itlb_miss_mmu2_end
#if 1//AEONR2 MMU
ENTRY(_itlb_miss_call_page_fault)
	.long	0x0
#endif
.previous


/* ---[ 0xb00: Range exception ]----------------------------------------- */
	.org 0xb00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0xb00)

/* ---[ 0xc00: Syscall exception ]--------------------------------------- */
	.org 0xc00
	EXCEPTION_HANDLE(_sys_call_handler)

/* ---[ 0xd00: Trap exception ]------------------------------------------ */
	.org 0xd00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0xd00)

/* ---[ 0xe00: Trap exception ]------------------------------------------ */
	.org 0xe00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0xe00)

/* ---[ 0xf00: Reserved exception ]-------------------------------------- */
	.org 0xf00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0xf00)

/* ---[ 0x1000: Reserved exception ]------------------------------------- */
	.org 0x1000
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1000)

/* ---[ 0x1100: Reserved exception ]------------------------------------- */
	.org 0x1100
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1100)

/* ---[ 0x1200: Reserved exception ]------------------------------------- */
	.org 0x1200
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1200)

/* ---[ 0x1300: Reserved exception ]------------------------------------- */
	.org 0x1300
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1300)

/* ---[ 0x1400: Reserved exception ]------------------------------------- */
	.org 0x1400
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1400)

/* ---[ 0x1500: Reserved exception ]------------------------------------- */
	.org 0x1500
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1500)

/* ---[ 0x1600: Reserved exception ]------------------------------------- */
	.org 0x1600
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1600)

/* ---[ 0x1700: Reserved exception ]------------------------------------- */
	.org 0x1700
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1700)

/* ---[ 0x1800: Reserved exception ]------------------------------------- */
	.org 0x1800
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1800)

/* ---[ 0x1900: Reserved exception ]------------------------------------- */
	.org 0x1900
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1900)

/* ---[ 0x1a00: Reserved exception ]------------------------------------- */
	.org 0x1a00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1a00)

/* ---[ 0x1b00: Reserved exception ]------------------------------------- */
	.org 0x1b00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1b00)

/* ---[ 0x1c00: Reserved exception ]------------------------------------- */
	.org 0x1c00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1c00)

/* ---[ 0x1d00: Reserved exception ]------------------------------------- */
	.org 0x1d00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1d00)

/* ---[ 0x1e00: Reserved exception ]------------------------------------- */
	.org 0x1e00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1e00)

/* ---[ 0x1f00: Reserved exception ]------------------------------------- */
	.org 0x1f00
	UNHANDLED_EXCEPTION(_unhandled_exception_handler, 0x1f00)

	.org 0x2000
/* ===================================================[ kernel start ]=== */

	.text
	.global _stext
_stext:

	.global _start
_start:
#ifndef CONFIG_AEONR2
	/*
	 * ensure a deterministic start
	 */
	b.ori	r3,r0,0x1
	b.mtspr	r0,r3,SPR_SR
#endif

	CLEAR_GPR(r1)
	CLEAR_GPR(r2)
	CLEAR_GPR(r3)
	CLEAR_GPR(r4)
	CLEAR_GPR(r5)
	CLEAR_GPR(r6)
	CLEAR_GPR(r7)
	CLEAR_GPR(r8)
	CLEAR_GPR(r9)
	CLEAR_GPR(r10)
	CLEAR_GPR(r11)
	CLEAR_GPR(r12)
	CLEAR_GPR(r13)
	CLEAR_GPR(r14)
	CLEAR_GPR(r15)
	CLEAR_GPR(r16)
	CLEAR_GPR(r17)
	CLEAR_GPR(r18)
	CLEAR_GPR(r19)
	CLEAR_GPR(r20)
	CLEAR_GPR(r21)
	CLEAR_GPR(r22)
	CLEAR_GPR(r23)
	CLEAR_GPR(r24)
	CLEAR_GPR(r25)
	CLEAR_GPR(r26)
	CLEAR_GPR(r27)
	CLEAR_GPR(r28)
	CLEAR_GPR(r29)
	CLEAR_GPR(r30)
	CLEAR_GPR(r31)

	/*
	 * set up initial ksp and current
	 */
	LOAD_SYMBOL_2_GPR(r1,_init_thread_union+0x2000)	// setup kernel stack
	LOAD_SYMBOL_2_GPR(r29,_init_thread_union)	// setup current
	tophys	(r31,r29)
	b.sw	TI_KSP(r31), r1

	/*
	 * .data contains initialized data,
	 * .bss contains uninitialized data - clear it up
	 */
clear_bss:
	LOAD_SYMBOL_2_GPR(r24, ___bss_start)
	LOAD_SYMBOL_2_GPR(r26, __end)
	tophys	(r28,r24)
	tophys	(r30,r26)
	CLEAR_GPR(r24)
	CLEAR_GPR(r26)
1:
	b.sw	(0)(r28),r0
	b.addi	r28,r28,4
	b.sfleu	r28,r30
	b.bf	1b

flush_tlb:
	/*
	 *  I N V A L I D A T E   T L B   e n t r i e s
	 */

#  define DTLB_SETS 64
	b.ori	r7,r0,DTLB_SETS
	b.ori	r8,r0,0x0
	b.ori	r27,r0,SPR_DTLBEIR
	b.ori	r28,r0,SPR_ITLBEIR
1:
	b.mtspr	r27,r8,0
	b.mtspr	r28,r8,0

	b.addi	r8,r8,0x2000      // PAGE_SIZE
	b.addi	r7,r7,-1
	b.sfnei	r7,0x0
	b.bf	1b

enable_mmu:
#ifdef CONFIG_MMU
	/*
	 * enable dmmu & immu
	 * SR[5] = 0, SR[6] = 0, 6th and 7th bit of SR set to 0
	 */
	b.mfspr	r28,r0,SPR_SR
	b.ori	r28,r28,(SPR_SR_DME | SPR_SR_IME)
	b.mtspr	r0,r28,SPR_ESR_BASE

	LOAD_SYMBOL_2_GPR(r28, enabled_mmu)
	b.mtspr r0,r28,SPR_EPCR_BASE
	b.rfe
#endif
enabled_mmu:

	// reset the simulation counters
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0

clear_regs:
	/*
	 * clear all GPRS to increase determinism
	 */
	CLEAR_GPR(r2)
	CLEAR_GPR(r3)
	CLEAR_GPR(r4)
	CLEAR_GPR(r5)
	CLEAR_GPR(r6)
	CLEAR_GPR(r7)
	CLEAR_GPR(r8)
	CLEAR_GPR(r9)
	CLEAR_GPR(r10)
	CLEAR_GPR(r11)
	CLEAR_GPR(r12)
	CLEAR_GPR(r13)
	CLEAR_GPR(r14)
	CLEAR_GPR(r15)
	CLEAR_GPR(r16)
	CLEAR_GPR(r17)
	CLEAR_GPR(r18)
	CLEAR_GPR(r19)
	CLEAR_GPR(r20)
	CLEAR_GPR(r21)
	CLEAR_GPR(r22)
	CLEAR_GPR(r23)
	CLEAR_GPR(r24)
	CLEAR_GPR(r25)
	CLEAR_GPR(r26)
	CLEAR_GPR(r27)
	CLEAR_GPR(r28)
	CLEAR_GPR(r30)
	CLEAR_GPR(r31)

jump_start_kernel:

	/*
	 * jump to kernel entry (start_kernel)
	 */
	LOAD_SYMBOL_2_GPR(r30, _start_kernel)
	b.jr	r30

ENTRY(_unhandled_exception_push_pt_regs)
	UNHANDLED_EXCEPTION_PUSH_PT_REGS
	b.jr	r9

	/* aligment here so we don't change memory offsets with
	 * memory controler defined
	 */
	.align 0x2000

#ifdef CONFIG_AEONR2
ENTRY(_ic_enable)
	/* Disable IC */
	b.mfspr r6,r0,SPR_SR
	LOAD_SYMBOL_2_GPR(r5,~SPR_SR_ICE)
	b.and   r5,r6,r5
	b.mtspr r0,r5,SPR_SR
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0

	/* Invalidate IC */
	b.addi  r6,r0,0
	b.addi  r5,r0,IC_SIZE
	b.addi	r7,r0,SPR_ICBIR
1:
	b.mtspr	r7,r6,0
	b.addi	r6,r6,IC_LINE
	b.sfleu	r6,r5
	b.bf	1b

	/* Enable IC */
	b.mfspr	r6,r0,SPR_SR
#ifdef CONFIG_AEONR2_ICACHE_ENABLED
	b.ori	r6,r6,SPR_SR_ICE
#else
	b.ori	r6,r6,0
#endif
	b.mtspr	r0,r6,SPR_SR
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0

	b.jr	r9

ENTRY(_dc_enable)
	/* Disable DC */
	b.mfspr r6,r0,SPR_SR
	LOAD_SYMBOL_2_GPR(r5,~SPR_SR_DCE)
	b.and   r5,r6,r5
	b.mtspr r0,r5,SPR_SR
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0

	/* Invalidate DC */
	b.addi  r5,r0,0
	b.addi  r6,r0,(DC_SIZE/DC_WAY)

1:
	b.invalidate_line 0(r5),0
	b.invalidate_line 0(r5),1
	b.invalidate_line 0(r5),2
	b.invalidate_line 0(r5),3
	b.addi	r5,r5,DC_LINE
	b.sfleu	r5,r6
	b.bf	1b

	/* Enable DC */
	b.mfspr r6,r0,SPR_SR
#ifdef CONFIG_AEONR2_DCACHE_ENABLED
	b.ori	r6,r6,SPR_SR_DCE
#else
	b.ori	r6,r6,0
#endif
	b.mtspr	r0,r6,SPR_SR
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0
	b.nop 0x0

	b.jr	r9

ENTRY(_wdt_init)
	b.movhi	r3,0xa000
	b.ori	r4,r3,(0x3c60<<1)
	b.ori	r5,r0,0xaa55
	b.sw	0x0(r4),r5

	// return
	b.jr	r9

#ifdef CONFIG_AEONR2_BOOT
ENTRY(_miu_init)
	b.movhi	r3,0xa000

	/* set sdr_base_i[25:0] = {rev3[9:8], rev1[15:8], rev2[15:0]} ;*/
	/* set rev3[1] */
	b.ori	r4,r3,(0xfe4<<1)
	b.ori	r5,r0,((AEON_CODE_BASE>>0)&0xffff)
	b.sw	0x0(r4),r5
	b.ori	r4,r3,(0xfe2<<1)
	b.ori	r5,r0,((AEON_CODE_BASE>>16)&0xffff)
	b.sw	0x0(r4),r5
	b.ori	r4,r3,(0xfe6<<1)
	b.ori	r5,r0,((AEON_CODE_BASE>>24)&0xffff)
	b.ori	r5,r5,0x6
	b.sw	0x0(r4),r5

	/* set sdr_base[31:0] */
	b.ori	r4,r3,(0xff4<<1)
	b.ori	r5,r0,((AEON_DATA_BASE>>0)&0xffff)
	b.sw	0x0(r4),r5
	b.ori	r4,r3,(0xff6<<1)
	b.ori	r5,r0,((AEON_DATA_BASE>>16)&0xffff)
	b.sw	0x0(r4),r5

	/* set mheg5_enable bit */
	b.ori	r4,r3,(0xff0<<1)
	b.lwz	r5,0x0(r4)
	b.ori	r5,r5,(0x1)
	b.sw	0x0(r4),r5

	MIU_INIT

	/* wait for miu init done */
	b.ori	r4,r3,(0x1200<<1)
1:
	b.lwz	r5,0x0(r4)
	b.andi	r5,r5,(0x8000)
	b.sfnei	r5,0x0
	b.bnf	1b

	// return
	b.jr	r9

ENTRY(_spi_copy)
	b.movhi	r3,0xa000

	/* set SPI_DMA source address */
	b.ori	r4,r3,(0x3ce0<<1)
	b.ori	r5,r0,((BOOT_COPY_SPI_ADDR>>0)&0xffff)
	b.sw	0x0(r4),r5
	b.ori	r4,r3,(0x3ce2<<1)
	b.ori	r5,r0,((BOOT_COPY_SPI_ADDR>>16)&0xffff)
	b.sw	0x0(r4),r5

	/* set SPI_DMA destination address */
	b.ori	r4,r3,(0x3ce4<<1)
	b.ori	r5,r0,((BOOT_COPY_MIU_ADDR>>0)&0xffff)
	b.sw	0x0(r4),r5
	b.ori	r4,r3,(0x3ce6<<1)
	b.ori	r5,r0,((BOOT_COPY_MIU_ADDR>>16)&0xffff)
	b.sw	0x0(r4),r5

	/* set SPI_DMA length */
	b.ori	r4,r3,(0x3ce8<<1)
	b.ori	r5,r0,((BOOT_COPY_LENGTH>>0)&0xffff)
	b.sw	0x0(r4),r5
	b.ori	r4,r3,(0x3cea<<1)
	b.ori	r5,r0,((BOOT_COPY_LENGTH>>16)&0xffff)
	b.sw	0x0(r4),r5

	/* trigger DMA */
	b.ori	r4,r3,(0x3cec<<1)
	b.ori	r5,r0,0x1
	b.sb	0x0(r4),r5

	/* wait for SPI_DMA done */
	b.ori	r4,r3,(0x3c20<<1)
1:
	b.lwz	r5,0x0(r4)
	b.andi	r5,r5,0x2
	b.sfnei	r5,0x2
	b.bf	1b

	// return
	b.jr	r9
#endif // CONFIG_AEONR2_BOOT

ENTRY(_spi_init)
	b.movhi	r3,0xa000
	b.ori	r4,r3,0x1ffc
	b.ori	r5,r0,0xa100
	b.sw	0x0(r4),r5

	// return
	b.jr	r9

ENTRY(_jtag_init)
	// JTAG setting
	b.movhi	r3,0xa000
	// Turn on BIT12 and BIT8 of 1E00
	b.ori	r4,r3,0x3c00
	b.lhz	r5,0x0(r4)
	b.andi	r5,r5,0xffff
	b.ori	r5,r5,0x1100
	b.sh	0x0(r4),r5

	// Turn on BIT0 of 1EAA
	b.ori	r4,r3,0x3d54
	b.lhz	r5,0x0(r4)
	b.andi	r5,r5,0xfffc
	b.ori	r5,r5,0x1
	b.sh	0x0(r4),r5

	// Turn off BIT7 of 1EA1
	b.ori	r4,r3,0x3d42
	b.lhz	r5,0x0(r4)
	b.andi	r5,r5,0xff7f
	b.sh	0x0(r4),r5

	// return
	b.jr	r9

ENTRY(_qmem_init)
	b.movhi	r3,0xa000
	b.ori	r4,r3,(0x0fec<<1)
	b.ori	r5,r0,0x0000
	b.sw	0x0(r4),r5

	b.movhi	r3,0xa000
	b.ori	r4,r3,(0x0fee<<1)
	b.ori	r5,r0,0xe000
	b.sw	0x0(r4),r5

	// return
	b.jr	r9
#endif // CONFIG_AEONR2


/* ===================================================[ tlb handlers ]=== */

#if defined __BA1__
#define JUMP_ABSOLUTE(reg,label) 		\
	b.movhi	reg,hi(0x40000000+label)	;\
	b.ori	reg,reg,lo(0x40000000+label)	;\
	b.jr	reg
#endif

#if defined __BA1__
#define JAL_ABSOLUTE(reg,label) 		\
	b.movhi	reg,hi(0x40000000+label)	;\
	b.ori	reg,reg,lo(label)		;\
	b.jalr	reg
#endif


/* ---[ dtlb miss handler, mmu2, ba1 ]----------------------------------- */

#ifdef __BA1__
dtlb_miss_mmu2:
	/* MMU2 TLB reload model handler. BA1 version supported by newer BA12 revisions
	 * and all later processors
	 *
	 * Addidionaly optimized, this is the fastest TLB reload handler
	 */
	//EXCEPTION_DEBUG_VALUE_ER(0x9f0)
	EXCEPTION_STORE_GPR2
	EXCEPTION_STORE_GPR3
	EXCEPTION_STORE_GPR4

#if defined(CONFIG_AEONR2_EXCEPTION_TRACE) || defined(CONFIG_AEONR2_TLB_STAT)
	EXCEPTION_DEBUG_STORE_GPR9
	EXCEPTION_STORE_GPR3
	EXCEPTION_STORE_GPR4
	EXCEPTION_STORE_GPR5
	JAL_ABSOLUTE(r5, dmmu_debug)
	EXCEPTION_LOAD_GPR3
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR5
	EXCEPTION_DEBUG_LOAD_GPR9
#endif

	// get EPC of the miss
	b.mfspr	r2,r0,SPR_EPCR_BASE

	// get EA of the miss
	b.mfspr	r2,r0,SPR_EEAR_BASE

	/*
	 * pte = (pte_t *)(current_pgd + pgd_index(daddr) << 2);
	 */
	b.srli	r2,r2,0x18
	b.slli	r2,r2,0x2		// this is because pointers are of size 4
	b.movhi	r3,_hi(_current_pgd+0x40000000)
	b.lwz	r3,_lo(_current_pgd+0x40000000)(r3)
	b.add	r2,r2,r3

	// pte
	b.movhi	r3,hi(0x40000000)
	b.add	r2,r3,r2
	b.lwz	r2,0(r2)
	b.sfeqi	r2,0
	b.bf	fast_d_pmd_none
	b.addi	r3,r0,0xffffe000	// this is needed because unused bits
					// in PTE are used as protection bits
					// could be changed in pgtables.h and removed here

	// pte present
	b.and	r2,r2,r3
	b.mfspr	r3,r0,SPR_EEAR_BASE
	b.srli	r3,r3,0xb
	b.andi	r3,r3,0x1ffc

	b.add	r2,r3,r2
	b.lwz	r2,0(r2)		// r2 holds PTE entry value
	// RAW

	// check if pte is present
	b.andi	r3,r2,1
	b.sfeqi	r3,0
	b.bf	fast_d_pte_not_present

#define VPN_MASK -0x00001000
#define DTLB_MR_MASK 0xfffff001
#define DTLB_TR_MASK 0xfffff330
	// speculatevly write to MR
	b.mfspr	r3,r0,SPR_EEAR_BASE
	b.ori	r3,r3,~(VPN_MASK)
	b.movhi	r4,hi(DTLB_MR_MASK)
	b.ori	r4,r4,lo(DTLB_MR_MASK)
	b.and	r3,r3,r4
	b.mtspr	r0,r3,SPR_DTLBMR
	DEBUG_DTLB_WRITE_MR

	b.addi	r3,r0,(0xffffe3fa)

#ifdef CONFIG_AEONR2_DCACHE_ENABLED

#else
	b.ori   r2,r2,SPR_DTLBTR_CI
#endif

	// apply the mask to PTE, result will be writen into TR
	b.and	r2,r2,r3
	b.mtspr	r0,r2,SPR_DTLBTR
	DEBUG_DTLB_WRITE_TR

	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR3
	EXCEPTION_CLEAR_SAVED_GPR3
	EXCEPTION_LOAD_GPR2
	b.rfe

fast_d_pmd_none:
fast_d_pte_not_present:
	LOAD_SYMBOL_2_GPR(r2,_dtlb_miss_call_page_fault)
	tophys	(r3,r2)
	b.ori	r2,r0,0x1
	b.sw	0x0(r3),r2
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR2
	/* an absolute jump to make this code position independant */
	// r3 will be restored in _dispatch_do_dpage_fault
	b.movhi	r3,hi(0x40000000+_dispatch_do_dpage_fault)
	b.ori	r3,r3,lo(0x40000000+_dispatch_do_dpage_fault)
	b.jr	r3
dtlb_miss_mmu2_end:
#endif /* __BA1__ */


/* ---[ itlb miss handler, mmu2, ba1 ]----------------------------------- */

#ifdef __BA1__
itlb_miss_mmu2:
	/* MMU2 TLB reload model handler. BA1 version supported by newer BA12 revisions
	 * and all later processors
	 *
	 * Addidionaly optimized, this is the fastest TLB reload handler
	 */
	//EXCEPTION_DEBUG_VALUE_ER(0xaf0)
	EXCEPTION_STORE_GPR2
	EXCEPTION_STORE_GPR3
	EXCEPTION_STORE_GPR4
	EXCEPTION_STORE_GPR5

#if defined(CONFIG_AEONR2_EXCEPTION_TRACE) || defined(CONFIG_AEONR2_TLB_STAT)
	EXCEPTION_DEBUG_STORE_GPR9
	EXCEPTION_STORE_GPR3
	EXCEPTION_STORE_GPR4
	EXCEPTION_STORE_GPR5
	JAL_ABSOLUTE(r5, immu_debug)
	EXCEPTION_LOAD_GPR3
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR5
	EXCEPTION_DEBUG_LOAD_GPR9
#endif
	// get EPC of the miss
	b.mfspr	r2,r0,SPR_EPCR_BASE

	// get EA of the miss
	b.mfspr	r2,r0,SPR_EEAR_BASE
#if 1//AEONR2 MMU
	b.addi	r2,r2,0x4
	b.mfspr	r3,r0,80
	b.fl1	r3,r3
	b.sub	r2,r2,r3
	b.mtspr	r0,r2,SPR_EEAR_BASE
#endif

	/*
	 * pte = (pte_t *)(current_pgd + pgd_index(daddr) << 2);
	 */
	b.srli	r2,r2,0x18
	b.slli	r2,r2,0x2		// this is because pointers are of size 4
	b.movhi	r3,_hi(_current_pgd+0x40000000)
	b.lwz	r3,_lo(_current_pgd+0x40000000)(r3)
	b.add	r2,r2,r3

	// pte
	b.movhi	r3,hi(0x40000000)
	b.add	r2,r3,r2
	b.lwz	r2,0(r2)
	b.sfeqi	r2,0
	b.bf	fast_i_pmd_none
	b.addi	r3,r0,0xffffe000	// this is needed because unused bits
					// in PTE are used as protection bits
					// could be changed in pgtables.h and removed here

	// pte present
	b.and	r2,r2,r3
	b.mfspr	r3,r0,SPR_EEAR_BASE
	b.srli	r3,r3,0xb
	b.andi	r3,r3,0x1ffc

	b.add	r2,r3,r2
	b.lwz	r2,0(r2)		// r2 holds PTE entry value
	// RAW

	// check if pte is present
	b.andi	r3,r2,1
	b.sfeqi	r3,0
	b.bf	fast_i_pte_not_present

#define VPN_MASK -0x00001000
#define ITLB_MR_MASK 0xfffff001
#define ITLB_TR_MASK 0xfffff050
	// speculatevly write to MR
	b.mfspr	r3,r0,SPR_EEAR_BASE
	b.ori	r3,r3,~(VPN_MASK)
	b.movhi	r4,hi(DTLB_MR_MASK)
	b.ori	r4,r4,lo(DTLB_MR_MASK)
	b.and	r3,r3,r4
	b.ori	r5,r0,SPR_ITLBMR
	b.mtspr	r5,r3,0
	DEBUG_ITLB_WRITE_MR

	b.andi	r3,r2,_PAGE_EXEC

	// get around crapy TLB entries definition
	b.sfeqi	r3,0
	b.bf	1f
	b.ori	r2,r2,0xc0		// sets SXE & UXE
1:
#ifdef CONFIG_AEONR2_ICACHE_ENABLED

#else
	b.ori	r2,r2,SPR_ITLBTR_CI
#endif
	// apply the mask to PTE and write it to TR,
	b.addi	r3,r0,(0xffffe000 | SPR_ITLBTR_CI | 0xc0)

	// apply the mask to PTE, result will be writen into TR
	b.and	r2,r2,r3

	// write to TR
	b.ori	r5,r0,SPR_ITLBTR
	b.mtspr	r5,r2,0
	DEBUG_ITLB_WRITE_TR

	EXCEPTION_LOAD_GPR5
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR3
	EXCEPTION_CLEAR_SAVED_GPR3
	EXCEPTION_LOAD_GPR2
	b.rfe

fast_i_pmd_none:
fast_i_pte_not_present:
	LOAD_SYMBOL_2_GPR(r2,_itlb_miss_call_page_fault)
	tophys	(r3,r2)
	b.ori	r2,r0,0x1
	b.sw	0x0(r3),r2
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR2
	/* an absolute jump to make this code position independant */
	// r3 will be restored in _dispatch_do_ipage_fault
	b.movhi	r3,hi(0x40000000+_dispatch_do_ipage_fault)
	b.ori	r3,r3,lo(0x40000000+_dispatch_do_ipage_fault)
	b.jr	r3
itlb_miss_mmu2_end:
#endif /* __BA1__ */


/* ==============================================[ boot tlb handlers ]=== */

/* ---[ boot dtlb miss handler ]----------------------------------------- */

/* mask for DTLB_MR register: - (0) sets V (valid) bit,
 *                            - (31-12) sets bits belonging to VPN (31-12)
 */
#define DTLB_MR_MASK 0xfffff001

/* mask for DTLB_TR register: - (2) sets CI (cache inhibit) bit,
 *                            - (4) sets A (access) bit,
 *                            - (5) sets D (dirty) bit,
 *                            - (8) sets SRE (superuser read) bit
 *                            - (9) sets SWE (superuser write) bit
 *                            - (31-12) sets bits belonging to VPN (31-12)
 */
#define DTLB_TR_MASK 0xfffff330

#define VPN_MASK -0x00001000
#define PPN_MASK -0x00001000

/*
 * a % 64 = a & MOD_64_MASK
 */
#define MOD_64_MASK 0x3f

dtlb_miss_boot:
	EXCEPTION_DEBUG_VALUE_ER(0x9b0)

	EXCEPTION_STORE_GPR2
	EXCEPTION_STORE_GPR3
	EXCEPTION_STORE_GPR4
	EXCEPTION_STORE_GPR5
	EXCEPTION_STORE_GPR6

	b.mfspr r6,r0,SPR_ESR_BASE
	b.andi  r6,r6,SPR_SR_SM		// are we in kernel mode ?
	b.sfnei	r6,0			// r6 == 0x1 --> SM
	b.bf	1f			// r6 == 0x1 --> SM
	b.j	exit_with_no_dtranslation
1:
#if defined(CONFIG_AEONR2_EXCEPTION_TRACE) || defined(CONFIG_AEONR2_TLB_STAT)
	EXCEPTION_DEBUG_STORE_GPR9
	b.jal	dmmu_debug
	EXCEPTION_DEBUG_LOAD_GPR9
#endif
	b.mfspr	r4,r0,SPR_EEAR_BASE		// get the offending EA

	/* do not do any translation if EA is above DRAM_END */
	LOAD_SYMBOL_2_GPR(r6,DRAM_END)		// exit if EA larger than DRAM_END
	b.sfgeu	r4,r6
	b.bf	exit_with_no_dtranslation
	CLEAR_GPR(r6)

	b.srli	r3,r4,0xd			// r3 <- r4 / 8192 sets are relative to page size (8Kb)
	b.andi	r2,r3,MOD_64_MASK		// r2 <- r3 % 64: ok this is SET
	b.or	r6,r0,r4			// r6 <- r4
	b.ori	r6,r6,~(VPN_MASK)		// r6 <- VPN :VPN .xfff - clear up lo(r6) to 0x**** *fff
	b.movhi	r5,hi(DTLB_MR_MASK)		// r5 <- ffff:0000.x000
	b.ori	r5,r5,lo(DTLB_MR_MASK)		// r5 <- ffff:1111.x001 - apply DTLB_MR_MASK
	b.and	r5,r5,r6			// r5 <- VPN :VPN .x001 - we have DTLBMR entry

	b.mtspr	r0,r5,SPR_DTLBMR		// set DTLBMR
	DEBUG_BOOT_DTLB_WRITE_MR

	/* set up DTLB with no translation for EA <= 0xbfffffff */
	LOAD_SYMBOL_2_GPR(r6,0xbfffffff)
	b.and	r3,r4,r4			// delay slot :: 24 <- r4 (if flag==1)
	b.sfgeu	r6,r4				// flag if r6 >= r4 (if 0xbfffffff >= EA)
	b.bf	1f				// flag if r6 >= r4 (if 0xbfffffff >= EA)
	tophys	(r3,r4)				// r3 <- PA, if (EA > 0xbfffffff)
1:
	b.ori	r3,r3,~(PPN_MASK)		// r3 <- PPN :PPN .xfff - clear up lo(r6) to 0x**** *fff
	b.movhi	r5,hi(DTLB_TR_MASK)		// r5 <- ffff:0000.x000
	b.ori	r5,r5,lo(DTLB_TR_MASK)		// r5 <- ffff:1111.x330 - apply DTLB_MR_MASK
	b.and	r5,r5,r3			// r5 <- PPN :PPN .x330 - we have DTLBTR entry
	/* page might be later marked as CI and if it
	 * would have been cached because of boot time
	 * TLB miss handler we might run into problems
	 */
	LOAD_SYMBOL_2_GPR(r6,0xbfffffff)
	b.sfgeu	r4,r6				// flag if (EA >= 0xbfffffff)
	b.bf	2f
	b.ori	r5,r5,SPR_DTLBTR_CI		// if (EA < 0xbfffffff)
2:
	b.mtspr r0,r5,SPR_DTLBTR		// set DTLBTR
	DEBUG_BOOT_DTLB_WRITE_TR

	EXCEPTION_LOAD_GPR6
	EXCEPTION_LOAD_GPR5
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR3
	EXCEPTION_CLEAR_SAVED_GPR3
	EXCEPTION_LOAD_GPR2
	b.rfe

/* This is a continuation of the boot dtlb handler */
exit_with_no_dtranslation:
	/* EA out of memory or not in supervisor mode */
	EXCEPTION_LOAD_GPR6
	EXCEPTION_LOAD_GPR5
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR3
	EXCEPTION_CLEAR_SAVED_GPR3
	EXCEPTION_LOAD_GPR2
	b.j	_dispatch_bus_fault

dtlb_miss_boot_end:

/* ---[ boot itlb miss handler ]----------------------------------------- */

#define VPN_MASK -0x00001000
#define PPN_MASK -0x00001000

/* mask for ITLB_MR register: - sets V (valid) bit,
 *                            - sets bits belonging to VPN (15-12)
 */
#define ITLB_MR_MASK 0xfffff001

/* mask for ITLB_TR register: - sets A (access) bit,
 *                            - sets SXE (superuser execute) bit
 *                            - sets bits belonging to VPN (15-12)
 */
#define ITLB_TR_MASK 0xfffff050

/*
 * a % 64 = a & MOD_64_MASK
 */
#define MOD_64_MASK 0x3f


itlb_miss_boot:
	EXCEPTION_STORE_GPR2
	EXCEPTION_STORE_GPR3
	EXCEPTION_STORE_GPR4
	EXCEPTION_STORE_GPR5
	EXCEPTION_STORE_GPR6
	EXCEPTION_STORE_GPR7

#if defined(CONFIG_AEONR2_EXCEPTION_TRACE) || defined(CONFIG_AEONR2_TLB_STAT)
	EXCEPTION_DEBUG_STORE_GPR9
	b.jal	immu_debug
	EXCEPTION_DEBUG_LOAD_GPR9
#endif
	b.mfspr	r6,r0,SPR_ESR_BASE
	b.andi	r6,r6,SPR_SR_SM		// are we in kernel mode ?
	b.sfnei	r6,0			// r6 == 0x1 --> SM
	b.bf	1f			// r6 == 0x1 --> SM
	C_ALT_L_EXT_JUMP(j, exit_with_no_itranslation)
1:
	b.mfspr	r4,r0,SPR_EEAR_BASE	// get the offending EA
#if 1//AEONR2 MMU
	b.addi	r4,r4,0x4
	b.mfspr	r3,r0,80
	b.fl1	r3,r3
	b.sub	r4,r4,r3
	b.mtspr	r0,r4,SPR_EEAR_BASE
#endif

	/* do not do any translation if EA is above DRAM_END */
	LOAD_SYMBOL_2_GPR(r6,DRAM_END)
	b.sfgeu	r4,r6
	b.bf	exit_with_no_itranslation

	CLEAR_GPR(r6)

	b.srli	r3,r4,0xd		// r3 <- r4 / 8192 sets are relative to page size (8Kb)
	b.andi	r2,r3,MOD_64_MASK	// r2 <- r3 % 64: ok this is SET
	b.or	r6,r0,r4		// r6 <- r4
	b.ori	r6,r6,~(VPN_MASK)	// r6 <- VPN :VPN .xfff - clear up lo(r6) to 0x**** *fff
	b.movhi	r5,hi(ITLB_MR_MASK)	// r5 <- ffff:0000.x000
	b.ori	r5,r5,lo(ITLB_MR_MASK)	// r5 <- ffff:1111.x001 - apply ITLB_MR_MASK
	b.and	r5,r5,r6		// r5 <- VPN :VPN .x001 - we have ITLBMR entry

	b.ori	r7,r0,SPR_ITLBMR
	b.mtspr r7,r5,0			// set ITLBMR
	DEBUG_BOOT_ITLB_WRITE_MR

	/*
	 * set up ITLB with no translation for EA <= 0x0fffffff
	 *
	 * we need this for head.S mapping (EA = PA). if we move all functions
	 * which run with mmu enabled into entry.S, we might be able to eliminate this.
	 *
	 */
	LOAD_SYMBOL_2_GPR(r6,0x0fffffff)
	b.and	r3,r4,r4		// delay slot :: 24 <- r4 (if flag==1)
	b.sfgeu	r6,r4			// goto out if r6 >= r4 (if 0xb0ffffff >= EA)
	b.bf	1f			// goto out if r6 >= r4 (if 0xb0ffffff >= EA)
	tophys	(r3,r4)			// r3 <- PA
1:
	b.ori	r3,r3,~(PPN_MASK)	// r3 <- PPN :PPN .xfff - clear up lo(r6) to 0x**** *fff
	b.movhi	r5,hi(ITLB_TR_MASK)	// r5 <- ffff:0000.x000
	b.ori	r5,r5,lo(ITLB_TR_MASK)	// r5 <- ffff:1111.x050 - apply ITLB_MR_MASK
	b.and	r5,r5,r3		// r5 <- PPN :PPN .x050 - we have ITLBTR entry

	b.ori	r7,r0,SPR_ITLBTR
        b.mtspr r7,r5,0			// set ITLBTR

	EXCEPTION_LOAD_GPR7
	EXCEPTION_LOAD_GPR6
	EXCEPTION_LOAD_GPR5
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR3
	EXCEPTION_CLEAR_SAVED_GPR3
	EXCEPTION_LOAD_GPR2
	b.rfe

/* This is a continuation of the boot itlb handler */
exit_with_no_itranslation:
	EXCEPTION_LOAD_GPR6
	EXCEPTION_LOAD_GPR5
	EXCEPTION_LOAD_GPR4
	EXCEPTION_LOAD_GPR3
	EXCEPTION_CLEAR_SAVED_GPR3
	EXCEPTION_LOAD_GPR2
	b.j	_dispatch_bus_fault

itlb_miss_boot_end:

/* =================================================[ debugging aids ]=== */

ENTRY(_debug_tng)
	b.addi	r1,r1,-16
	b.sw	0(r1),r3
	b.sw	4(r1),r4
	b.mfspr	r3,r0,SPR_SR
	b.sw	8(r1),r3
	b.mfspr	r3,r0,SPR_NPC
	b.sw	12(r1),r3

	b.ori	r3,r0,0x8001
	b.mtspr	r0,r3,SPR_ESR_BASE
	LOAD_SYMBOL_2_GPR(r3,nommu_entry)
	tophys	(r4,r3)
	b.mtspr	r0,r4,SPR_EPCR_BASE

	b.lwz	r3,0(r1)
	b.lwz	r4,4(r1)
	b.addi	r1,r1,8
	b.rfe

nommu_entry:
	EXCEPTION_T_STORE_GPR31
	EXCEPTION_T_STORE_GPR29
	EXCEPTION_T_STORE_SP
	/* temporary store r3, r9 into r1, r29 */
	b.addi	r1,r3,0x0
	b.addi	r29,r9,0x0
	/* the string referenced by r3 must be low enough */
	b.ori	r3,r0,lo(_string_ed)
	b.jal	_emergency_print
	b.mfspr	r3,r0,SPR_NPC
	b.andi	r3,r3,0x1f00
	b.jal	_emergency_print_nr
	/* the string referenced by r3 must be low enough */
	b.ori	r3,r0,lo(_string_epc_prefix)
	b.jal	_emergency_print
	b.mfspr	r3,r0,SPR_EPCR_BASE
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	/* end of printing */
	b.addi	r3,r1,0x0
	b.addi	r9,r29,0x0
	/* extract current, ksp from current_set */
	LOAD_SYMBOL_2_GPR(r1,_unhandled_stack_top)
	/* create new stack frame, save only needed gprs */
	/* r1: KSP, r29: current, r31: __pa(KSP) */
	/* r12:	temp, syscall indicator, r13 temp */
	b.addi  r1,r1,-(INT_FRAME_SIZE)
	/* r1 is KSP, r31 is __pa(KSP) */
	tophys	(r31,r1)
	b.sw	GPR12(r31),r12
	b.mfspr	r12,r0,SPR_EPCR_BASE
	b.sw	PC(r31),r12
	b.mfspr	r12,r0,SPR_ESR_BASE
	b.sw	SR(r31),r12
	/* save r31 */
	EXCEPTION_T_LOAD_GPR31(r12)
	b.sw	GPR31(r31),r12
	/* save r29 as was prior to exception */
	EXCEPTION_T_LOAD_GPR29(r12)
	b.sw	GPR29(r31),r12
	/* save SP as was prior to exception */
	EXCEPTION_T_LOAD_SP(r12)
	b.sw	SP(r31),r12
	b.sw	GPR13(r31),r13
	/* --> */
	/* save exception r4, set r4 = EA */
	b.sw	GPR4(r31),r4
	b.mfspr r4,r0,SPR_EEAR_BASE
	/* r31 is __pa(r1), r1 <- r31 */
	b.addi	r1,r31,0x0
	/* r1, r29, EPCR, ESR a	already saved */
	b.sw	GPR2(r1),r2
	b.sw	GPR3(r1),r3
	b.sw	ORIG_GPR3(r1),r3
	b.sw	GPR5(r1),r5
	b.sw	GPR6(r1),r6
	b.sw	GPR7(r1),r7
	b.sw	GPR8(r1),r8
	b.sw	GPR9(r1),r9
	/* r29 already saved */
	b.sw	GPR11(r1),r11
	/* r12,r13 already saved */
	b.sw	GPR14(r1),r14
	b.sw	GPR15(r1),r15
	b.sw	GPR16(r1),r16
	b.sw	GPR17(r1),r17
	b.sw	GPR18(r1),r18
	b.sw	GPR19(r1),r19
	b.sw	GPR20(r1),r20
	b.sw	GPR21(r1),r21
	b.sw	GPR22(r1),r22
	b.sw	GPR23(r1),r23
	b.sw	GPR24(r1),r24
	b.sw	GPR25(r1),r25
	b.sw	GPR26(r1),r26
	b.sw	GPR27(r1),r27
	b.sw	GPR28(r1),r28
	b.sw	GPR29(r1),r29
	b.sw	GPR30(r1),r30
	/* r31 already saved */
	b.sw	RESULT(r1),r0
	b.addi	r3,r1,0
	/* r4 is exception EA */
	b.ori	r5,r0,0xdead
	b.jal	_nommu_dump_state


/* ========================================[ standard debugging aids ]=== */


#ifdef CONFIG_AEONR2_EXCEPTION_TRACE
ENTRY(exception_add_to_trace)
	EXCEPTION_DEBUG_STORE_GPR4
	EXCEPTION_DEBUG_STORE_F_GPR9
	EXCEPTION_DEBUG_STORE_F_GPR31

	LOAD_SYMBOL_2_GPR(r4,_except_stack_indx)
	tophys	(r9,r4)
	b.lwz	r4,0(r9)
	b.addi	r4,r4,1
	b.ori	r31,r0,CONFIG_AEONR2_EXCEPTION_TRACE_DEPTH
	b.bltu	r4,r31,1f
	b.addi	r4,r0,0

1:	b.sw	0(r9),r4

	b.slli	r4,r4,4

	LOAD_SYMBOL_2_GPR(r9,_except_stack)
	b.add	r9,r9,r4

	tophys	(r4,r9)

	b.sw	0(r4),r3
	b.mfspr	r9,r0,SPR_EPCR_BASE
	b.sw	4(r4),r9
	b.mfspr	r9,r0,SPR_ESR_BASE
	b.sw	8(r4),r9
	b.mfspr	r9,r0,SPR_EEAR_BASE
	b.sw	12(r4),r9

	/* Add the exception to the per-thread exception stack */
	tophys	(r9,r29)

	b.lwz	r4,TI_EXCEPT_TRACE_POS(r9)
	b.addi	r4,r4,1
	b.ori	r9,r0,CONFIG_AEONR2_EXCEPTION_TRACE_DEPTH
	b.bltu	r4,r9,1f
	b.addi	r4,r0,0

1:	tophys	(r9,r29)

	b.sw	TI_EXCEPT_TRACE_POS(r9),r4

	b.slli	r4,r4,4

	b.addi	r9,r9,TI_EXCEPT_TRACE
	b.add	r4,r9,r4

	b.sw	0(r4),r3
	b.mfspr	r9,r0,SPR_EPCR_BASE
	b.sw	4(r4),r9
	b.mfspr	r9,r0,SPR_ESR_BASE
	b.sw	8(r4),r9
	b.mfspr	r9,r0,SPR_EEAR_BASE
	b.sw	12(r4),r9

	EXCEPTION_DEBUG_LOAD_GPR4
	EXCEPTION_DEBUG_LOAD_F_GPR9
	EXCEPTION_DEBUG_LOAD_F_GPR31

	b.jr	r9
#endif

#if defined(CONFIG_AEONR2_TLB_STAT)
#define INC_TLB_STAT(user, kernel, g_user, g_kernel) \
	b.mfspr r3,r0,SPR_EEAR_BASE ;\
	b.movhi	r2,hi(0xc0000000) ;\
	b.ori	r4,r3,0 ;\
	tophys	(r3,r29) ;\
	b.sfgeu	r4,r2 ;\
	b.bf	1f ;\
/* user space missed */ ;\
	b.addi	r3,r3,user ;\
	LOAD_SYMBOL_2_GPR(r4,g_user) ;\
	b.j	2f ;\
1: ;\
/* kernel space missed */ ;\
	b.addi	r3,r3,kernel ;\
	LOAD_SYMBOL_2_GPR(r4,g_kernel) ;\
2: ;\
	b.lwz	r2,0(r3) ;\
	b.addi	r2,r2,1 ;\
	b.sw	0(r3),r2 ;\
	tophys	(r3,r4) ;\
	b.lwz	r2,0(r3) ;\
	b.addi	r2,r2,1 ;\
	b.sw	0(r3),r2
#else
#define INC_TLB_STAT(user, kernel, g_user, g_kernel)
#endif

#if defined(CONFIG_AEONR2_EXCEPTION_TRACE)
#define EXCEPT_TRACE(x) \
	b.ori	r2,r9,0 ;\
	b.ori	r3,r0,x ;\
	b.jal	exception_add_to_trace ;\
	b.ori	r9,r2,0
#else
#define EXCEPT_TRACE(x)
#endif

#if defined(CONFIG_AEONR2_EXCEPTION_TRACE) || defined(CONFIG_AEONR2_TLB_STAT)
immu_debug:
	EXCEPTION_T_STORE_GPR29
	EXCEPTION_DEBUG_STORE_GPR3
	b.mfspr r3,r0,SPR_ESR_BASE
	b.andi  r3,r3,SPR_SR_SM
	b.sfnei	r3,0 /* kernel_mode */
	b.bf	2f /* kernel_mode */
1: /* user_mode:   */
	LOAD_SYMBOL_2_GPR(r2,_current_thread_info_set)
	tophys  (r29,r2)
	/* load current_thread_info  */
	b.lwz   r29,0(r29)
	/* fall through */
2: /* kernel_mode: */

	EXCEPT_TRACE(0xa00)
	INC_TLB_STAT(TI_U_ITLB_MISS, TI_K_ITLB_MISS, _g_u_itlb_miss, _g_k_itlb_miss)

	EXCEPTION_DEBUG_LOAD_GPR3
	EXCEPTION_T_LOAD_GPR29(r29)

	b.jr	r9

dmmu_debug:
	EXCEPTION_T_STORE_GPR29
	b.mfspr r3,r0,SPR_ESR_BASE
	b.andi  r3,r3,SPR_SR_SM
	b.sfnei	r3,0 /* kernel_mode */
	b.bf	2f /* kernel_mode */
1: /* user_mode:   */
	LOAD_SYMBOL_2_GPR(r2,_current_thread_info_set)
	tophys  (r29,r2)
	/* load current_thread_info  */
	b.lwz   r29,0(r29)
	/* fall through */
2: /* kernel_mode: */

	EXCEPT_TRACE(0x900)
	INC_TLB_STAT(TI_U_DTLB_MISS, TI_K_DTLB_MISS, _g_u_dtlb_miss, _g_k_dtlb_miss)

	EXCEPTION_DEBUG_LOAD_GPR3
	EXCEPTION_T_LOAD_GPR29(r29)

	b.jr	r9
#endif

#if (DEBUG_PRINT_TLB_WRITES>0)
_debug_boot_dtlb_write_mr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_boot_prefix_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_dtlbmr_write)
	b.jal	_emergency_print
	b.ori	r3,r2,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_boot_dtlb_write_tr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_boot_prefix)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_dtlbtr_write)
	b.jal	_emergency_print
	b.ori	r3,r2,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_boot_itlb_write_mr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_boot_prefix_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_itlbmr_write)
	b.jal	_emergency_print
	b.ori	r3,r2,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_boot_itlb_write_tr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_boot_prefix)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_itlbtr_write)
	b.jal	_emergency_print
	b.ori	r3,r2,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_dtlb_write_mr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_prefix_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_dtlbmr_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r4,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_dtlb_write_tr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_prefix_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_dtlbtr_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r4,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_itlb_write_mr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_prefix_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_itlbmr_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r4,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_itlb_write_tr:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_debug_prefix_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_itlbtr_write)
	b.jal	_emergency_print
	b.ori	r3,r5,0
	b.jal	_emergency_print_nr
	b.ori	r3,r0,lo(_string_debug_tlb_val_write)
	b.jal	_emergency_print
	b.ori	r3,r4,0
	b.jal	_emergency_print_nr_full
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_epcr_write:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_epcr)
	b.jal	_emergency_print
	b.ori	r3,r4,0
	b.jal	_emergency_print_nr_full
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9

_debug_eear_write:
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR9
	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print
	b.ori	r3,r0,lo(_string_debug_eear)
	b.jal	_emergency_print
	b.ori	r3,r4,0
	b.jal	_emergency_print_nr_full
	EXCEPTION_DEBUG_LOAD_GPR9
	EXCEPTION_DEBUG_LOAD_GPR3
	b.jr	r9
#endif


// this HAS to be atomic. also disable interrupts beetwen to entries
// may use gpr4
ENTRY(_exception_regenerate)
	E_REGEN_LOAD_STATUS(r4)		// were we in this exception yet
	b.sfnei	r4,0x0
	b.bf	1f

0:					// first entry into this exception
					// patch the ESR, reenter exception

	b.mfspr r4,r0,SPR_ESR_BASE	// get original ESR
	E_REGEN_STORE_STATUS(r4)	// save original ESR

	SPR_DISABLE_LO_BITS(SPR_ESR_BASE,
		(SPR_SR_ICE | SPR_SR_DCE | SPR_SR_IEE | SPR_SR_TEE),r4)
	SPR_DISABLE_LO_BITS(SPR_SR,
		(SPR_SR_ICE | SPR_SR_DCE | SPR_SR_IEE | SPR_SR_TEE),r4)

	EXCEPTION_DEBUG_LOAD_GPR4
	EXCEPTION_DEBUG_LOAD_F_GPR9

	b.rfe				// this will regenerate exception

1:					// second entry into this exception
					// patch ESR, invalidate the caches

	// we already loaded previous ESR into r4
	b.mtspr r0,r4,SPR_ESR_BASE

	b.ori	r4,r0,0x0		// next time is fresh entry
	E_REGEN_STORE_STATUS(r4)	// prepare for next exception

	E_REGEN_STORE_GPR3
	E_REGEN_STORE_GPR4
	E_REGEN_STORE_GPR5
	E_REGEN_STORE_GPR6
	E_REGEN_STORE_GPR9
	b.jal	_dc_enable
	b.jal	_ic_enable
	E_REGEN_LOAD_GPR3
	E_REGEN_LOAD_GPR4
	E_REGEN_LOAD_GPR5
	E_REGEN_LOAD_GPR6
	E_REGEN_LOAD_GPR9

	b.jr	r9

ENTRY(_exception_debug)
	EXCEPTION_DEBUG_STORE_GPR3
	EXCEPTION_DEBUG_STORE_GPR4
	EXCEPTION_DEBUG_STORE_GPR9

	// disable tick timer
	b.ori	r4,r0,SPR_TTMR
	b.mfspr	r3,r4,0
	LOAD_SYMBOL_2_GPR(r9,~(0x30000003))
	b.and	r3,r3,r9
	b.ori	r4,r0,SPR_TTMR
	b.mtspr	r4,r3,0

	b.ori	r3,r0,lo(_string_debug_exception)
	b.jal	_emergency_print
	b.andi	r3,r4,0x1ff0
	b.jal	_emergency_print_nr
	/* the string referenced by r3 must be low enough */
	b.ori	r3,r0,lo(_string_epc_prefix)
	b.jal	_emergency_print

	b.mfspr	r3,r0,SPR_EPCR_BASE
	b.jal	_emergency_print_nr

	b.ori	r3,r0,lo(_string_debug_esr_prefix)
	b.jal	_emergency_print

	b.mfspr	r3,r0,SPR_ESR_BASE
	b.jal	_emergency_print_nr

	b.ori	r3,r0,lo(_string_debug_eear_prefix)
	b.jal	_emergency_print

	b.mfspr	r3,r0,SPR_EEAR_BASE
	b.jal	_emergency_print_nr

	b.ori	r3,r0,lo(_string_nl)
	b.jal	_emergency_print

	// enable tick timer
	b.ori	r4,r0,SPR_TTMR
	b.mfspr	r3,r4,0
	LOAD_SYMBOL_2_GPR(r9,0x20000001)
	b.or	r3,r3,r9
	b.ori	r4,r0,SPR_TTMR
	b.mtspr	r4,r3,0

	EXCEPTION_DEBUG_LOAD_GPR4
	EXCEPTION_DEBUG_LOAD_GPR3
	EXCEPTION_DEBUG_LOAD_GPR9
	b.jr	r9

/*
 * DSCR: prints just one character (from r3)
 *
 * PRMS: r3             - holds the character
 *
 * PREQ: UART at UART_BASE_ADD has to be initialized
 *
 * POST: it clobbers r3, r4, r5, r6 like a normal function
 *       should not be used when clobbering is not allowed
 *
 * NOTE: this function is ment for post crash reporting
 *       and thus we do not disable it when CONFIG_AEONR2_NO_UART_INIT
 *       is defined
 */
ENTRY(_emergency_print_ch)
//#ifndef CONFIG_AEONR2_NO_UART_INIT
	b.movhi	r4,hi(UART_BASE_ADD)

	b.addi	r6,r0,0x20
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x20
	b.sfne	r5,r6
	b.bf	1b
	b.sb	0(r4),r3

	b.addi	r6,r0,0x60
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x60
	b.sfne	r5,r6
	b.bf	1b
//#endif
	b.jr	r9


/*
 * DSCR: prints a string referenced by r3. if it's address is in
 *       virtual range (0xcxxxxxxx) it converts it to physical
 *
 * PRMS: r3		- address of the first character of null
 *			terminated string to be printed
 *
 * PREQ: UART at UART_BASE_ADD has to be initialized
 *
 * POST: caller should be aware that function does by itself doesn't
 *       clobber any register.
 */
ENTRY(_emergency_print)
#ifndef CONFIG_AEONR2_NO_UART_INIT
	EMERGENCY_PRINT_STORE_GPR4
	EMERGENCY_PRINT_STORE_GPR5
	EMERGENCY_PRINT_STORE_GPR6
	EMERGENCY_PRINT_STORE_GPR7

	b.srli	r4,r3,28
	b.sfnei	r4,0xc
	b.bf	2f
	b.ori	r4,r3,0
	tophys  (r3,r4)

2:
	b.lbz	r7,0(r3)
	b.sfeqi	r7,0
	b.bf	9f
// putc:
	b.movhi	r4,hi(UART_BASE_ADD)

	b.addi	r6,r0,0x20
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x20
	b.sfne	r5,r6
	b.bf	1b
	b.sb	0(r4),r7

	b.addi	r6,r0,0x60
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x60
	b.sfne	r5,r6
	b.bf	1b
	/* next character */
	b.addi	r3,r3,0x1
	b.j	2b

9:
	EMERGENCY_PRINT_LOAD_GPR7
	EMERGENCY_PRINT_LOAD_GPR6
	EMERGENCY_PRINT_LOAD_GPR5
	EMERGENCY_PRINT_LOAD_GPR4
#endif
	b.jr	r9

/* removes leading zeros */
ENTRY(_emergency_print_nr)
#ifndef CONFIG_AEONR2_NO_UART_INIT
	EMERGENCY_PRINT_STORE_GPR4
	EMERGENCY_PRINT_STORE_GPR5
	EMERGENCY_PRINT_STORE_GPR6
	EMERGENCY_PRINT_STORE_GPR7
	EMERGENCY_PRINT_STORE_GPR8

	b.addi	r8,r0,32		// shift register

1:	/* remove leading zeros */
	b.addi	r8,r8,-0x4
	b.srl	r7,r3,r8
	b.andi	r7,r7,0xf

	/* don't skip the last zero if number == 0x0 */
	b.sfeqi	r8,4
	b.bf	2f
	b.sfeqi	r7,0
	b.bf	1b

2:
	b.srl	r7,r3,r8

	b.andi	r7,r7,0xf
#if 1//AEONR2
	b.sflesi	r8,-1
#else
	b.sfltsi	r8,0
#endif
	b.bf	9f

	b.sfleui r7,9
	b.bf	8f
	b.addi	r7,r7,0x27
8:
	b.addi	r7,r7,0x30
// putc:
	b.movhi	r4,hi(UART_BASE_ADD)

	b.addi	r6,r0,0x20
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x20
	b.sfne	r5,r6
	b.bf	1b
	b.sb	0(r4),r7

	b.addi	r6,r0,0x60
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x60
	b.sfne	r5,r6
	b.bf	1b
	/* next character */
	b.addi	r8,r8,-0x4
	b.j	2b

9:
	EMERGENCY_PRINT_LOAD_GPR8
	EMERGENCY_PRINT_LOAD_GPR7
	EMERGENCY_PRINT_LOAD_GPR6
	EMERGENCY_PRINT_LOAD_GPR5
	EMERGENCY_PRINT_LOAD_GPR4
#endif
	b.jr	r9

/* does not remove leading zeros */
ENTRY(_emergency_print_nr_full)
#ifndef CONFIG_AEONR2_NO_UART_INIT
	EMERGENCY_PRINT_STORE_GPR4
	EMERGENCY_PRINT_STORE_GPR5
	EMERGENCY_PRINT_STORE_GPR6
	EMERGENCY_PRINT_STORE_GPR7
	EMERGENCY_PRINT_STORE_GPR8

	b.addi	r8,r0,28

2:
	b.srl	r7,r3,r8

	b.andi	r7,r7,0xf
#if 1//AEONR2
	b.sflesi	r8,-1
#else
	b.sfltsi	r8,0
#endif
	b.bf	9f
	b.sfleui r7,9
	b.bf	8f
	b.addi	r7,r7,0x27

8:
	b.addi	r7,r7,0x30
// putc:
	b.movhi	r4,hi(UART_BASE_ADD)

	b.addi	r6,r0,0x20
	1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x20
	b.sfne	r5,r6
	b.bf	1b
	b.sb	0(r4),r7

	b.addi	r6,r0,0x60
1:
	b.lbz	r5,5(r4)
	b.andi	r5,r5,0x60
	b.sfne	r5,r6
	b.bf	1b

	/* next character */
	b.addi	r8,r8,-0x4
	b.j	2b

9:
	EMERGENCY_PRINT_LOAD_GPR8
	EMERGENCY_PRINT_LOAD_GPR7
	EMERGENCY_PRINT_LOAD_GPR6
	EMERGENCY_PRINT_LOAD_GPR5
	EMERGENCY_PRINT_LOAD_GPR4
#endif
	b.jr	r9

#ifdef CONFIG_AEONR2
ENTRY(_early_uart_init)
	b.movhi	r3,hi(UART_BASE_ADD)

	b.addi	r4,r0,0x7
	b.sb	0x2(r3),r4

	b.addi	r4,r0,0x0
	b.sb	0x1(r3),r4

	b.addi	r4,r0,0x3
	b.sb	0x3(r3),r4

	b.lbz	r5,3(r3)
	b.ori	r4,r5,0x80
	b.sb	0x3(r3),r4
	b.addi	r4,r0,((UART_DIVISOR>>8) & 0x000000ff)
	b.sb	UART_DLM(r3),r4
	b.addi  r4,r0,((UART_DIVISOR) & 0x000000ff)
	b.sb	UART_DLL(r3),r4
	b.sb	0x3(r3),r5

#if 1 // UART OUTPUT TEST
	b.addi	r4,r0,'\n'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'\r'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'u'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'a'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'r'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'t'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,' '
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'i'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'n'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'i'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'t'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,' '
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'d'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'o'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'n'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'e'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'\n'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
	b.addi	r4,r0,'\r'
	b.sb	0x0(r3),r4
	b.nop 0x0
	b.nop 0x0
#endif

	b.jr	r9

#else // !CONFIG_AEONR2
ENTRY(_early_uart_init)
#ifndef CONFIG_AEONR2_NO_UART_INIT
	b.movhi	r3,hi(UART_BASE_ADD)

	b.addi	r4,r0,0x7
	b.sb	0x2(r3),r4

	b.addi	r4,r0,0x0
	b.sb	0x1(r3),r4

	b.addi	r4,r0,0x3
	b.sb	0x3(r3),r4

	b.lbz	r5,3(r3)
	b.ori	r4,r5,0x80
	b.sb	0x3(r3),r4
	b.addi	r4,r0,((UART_DEVISOR>>8) & 0x000000ff)
	b.sb	UART_DLM(r3),r4
	b.addi	r4,r0,((UART_DEVISOR) & 0x000000ff)
	b.sb	UART_DLL(r3),r4
	b.sb	0x3(r3),r5
#endif /* CONFIG_AEONR2_NO_UART_INIT */

	b.jr	r9
#endif // CONFIG_AEONR2

ENTRY(_uart_putc)
	UART_PRINT_STORE_GPR18
	// save SR and disable interrupt
	b.mfspr	r18,r0,SPR_SR
	b.sw	0x1040(r0),r18
	b.andi	r18,r18,lo(~(SPR_SR_IEE | SPR_SR_TEE))
	b.mtspr	r0,r18,SPR_SR

	UART_PRINT_STORE_GPR4
	UART_PRINT_STORE_GPR5
	UART_PRINT_STORE_GPR6
	UART_PRINT_STORE_GPR7

	b.movhi r4,hi(UART_BASE_ADD)

	b.or	r7,r0,r3

1:
	b.addi  r6,r0,0x20
2:	b.lbz   r5,5(r4)
	b.andi  r5,r5,0x20
	b.sfne	r5,r6
	b.bf	2b

	b.sb    0(r4),r7

	b.addi  r6,r0,0x60
3:	b.lbz   r5,5(r4)
	b.andi  r5,r5,0x60
	b.sfne	r5,r6
	b.bf	3b

	b.sfnei	r7,0xa
	b.bf	4f
	b.ori	r7,r0,0xd
	b.j	1b

4:
	UART_PRINT_RESTORE_GPR7
	UART_PRINT_RESTORE_GPR6
	UART_PRINT_RESTORE_GPR5
	UART_PRINT_RESTORE_GPR4

	// restore SR
	b.lwz	r18,0x1040(r0)
	b.mtspr	r0,r18,SPR_SR
	UART_PRINT_RESTORE_GPR18

	// return to the caller
	b.jr	r9

ENTRY(_uart__putc)
	b.movhi r4,hi(UART_BASE_ADD)

	b.or	r7,r0,r3

1:
	b.addi  r6,r0,0x20
2:	b.lbz   r5,5(r4)
	b.andi  r5,r5,0x20
	b.sfne	r5,r6
	b.bf	2b

	b.sb    0(r4),r7

	b.addi  r6,r0,0x60
3:	b.lbz   r5,5(r4)
	b.andi  r5,r5,0x60
	b.sfne	r5,r6
	b.bf	3b

	b.sfnei	r7,0xa
	b.bf	4f
	b.ori	r7,r0,0xd
	b.j	1b

4:
	// return to the caller
	b.jr	r9

ENTRY(_uart_puthex)
	UART_PRINT_STORE_GPR18
	// save SR and disable interrupt
	b.mfspr	r18,r0,SPR_SR
	b.sw	0x1040(r0),r18
	b.andi	r18,r18,lo(~(SPR_SR_IEE | SPR_SR_TEE))
	b.mtspr	r0,r18,SPR_SR

	UART_PRINT_STORE_GPR4
	UART_PRINT_STORE_GPR5
	UART_PRINT_STORE_GPR6
	UART_PRINT_STORE_GPR7
	UART_PRINT_STORE_GPR8
	UART_PRINT_STORE_GPR9
	UART_PRINT_STORE_GPR10

	b.or	r8,r0,r3
	b.ori	r10,r0,28

	b.ori	r3,r0,'0'
	b.jal	_uart__putc
	b.ori	r3,r0,'X'
	b.jal	_uart__putc

1:
	b.srl	r3,r8,r10
	b.andi	r3,r3,0xf
#if 1//AEONR2
	b.sfleui	r3,0x9
#else
	b.sfltui	r3,0xa
#endif
	b.bf	2f
	b.addi	r3,r3,0x7
2:
	b.addi	r3,r3,'0'
	b.jal	_uart__putc
	b.sfeqi	r10,0x0
	b.bf	3f
	b.addi	r10,r10,-4
	b.j	1b
3:
	UART_PRINT_RESTORE_GPR10
	UART_PRINT_RESTORE_GPR9
	UART_PRINT_RESTORE_GPR8
	UART_PRINT_RESTORE_GPR7
	UART_PRINT_RESTORE_GPR6
	UART_PRINT_RESTORE_GPR5
	UART_PRINT_RESTORE_GPR4

	// restore SR
	b.lwz	r18,0x1040(r0)
	b.mtspr	r0,r18,SPR_SR
	UART_PRINT_RESTORE_GPR18

	// return to the caller
	b.jr	r9

ENTRY(_uart_puts)
	UART_PRINT_STORE_GPR18
	// save SR and disable interrupt
	b.mfspr	r18,r0,SPR_SR
	b.sw	0x1040(r0),r18
	b.andi	r18,r18,lo(~(SPR_SR_IEE | SPR_SR_TEE))
	b.mtspr	r0,r18,SPR_SR

	UART_PRINT_STORE_GPR4
	UART_PRINT_STORE_GPR5
	UART_PRINT_STORE_GPR6
	UART_PRINT_STORE_GPR7

	b.movhi r4,hi(UART_BASE_ADD)

1:
	b.lbz	r7,0x0(r3)
	b.sfeqi	r7,0x0
	b.bf	6f

2:
	b.addi  r6,r0,0x20
3:	b.lbz   r5,5(r4)
	b.andi  r5,r5,0x20
	b.sfne	r5,r6
	b.bf	3b

	b.sb    0(r4),r7

	b.addi  r6,r0,0x60
4:	b.lbz   r5,5(r4)
	b.andi  r5,r5,0x60
	b.sfne	r5,r6
	b.bf	4b

	b.sfnei	r7,0xa
	b.bf	5f
	b.ori	r7,r0,0xd
	b.j	2b

5:
	/* next character */
	b.addi	r3,r3,0x1
	b.j	1b

6:
	UART_PRINT_RESTORE_GPR7
	UART_PRINT_RESTORE_GPR6
	UART_PRINT_RESTORE_GPR5
	UART_PRINT_RESTORE_GPR4

	// restore SR
	b.lwz	r18,0x1040(r0)
	b.mtspr	r0,r18,SPR_SR
	UART_PRINT_RESTORE_GPR18

	// return to the caller
	b.jr	r9

_string_copying_linux:
	.string "\n\n\n\n\n\rCopying Linux... "

_string_ok_booting:
	.string "Ok, booting the kernel.\n\r"

_string_unhandled_exception:
	.string "\n\rRunarunaround: Unhandled exception 0x"

_string_epc_prefix:
	.string ": EPC=0x"

_string_nl:
	.string "\n\r"

_string_debug_exception:
	.string "\n\r\tDebug: exception 0x"

_string_debug_dtlbmr_write:
	.string "\tdTLBmr[0x"

_string_debug_dtlbtr_write:
	.string "\tdTLBtr[0x"

_string_debug_itlbmr_write:
	.string "iTLBmr[0x"

_string_debug_itlbtr_write:
	.string "iTLBtr[0x"

_string_debug_boot_itlbtr_write:
	.string "iTLBtr[0x"

_string_debug_boot_prefix_nl:
	.string "\n\r\t\tboot "

_string_debug_boot_prefix:
	.string "\t\tboot "

_string_debug_prefix_nl:
	.string "\n\r\t\t"

_string_debug_prefix:
	.string "\t\t"

_string_debug_eear:
	.string "\t\tEEAR: 0x"

_string_debug_epcr:
	.string "\t\tEPCR: 0x"

_string_debug_tlb_val_write:
	.string "] <- 0x"

_string_debug_esr_prefix:
	.string ", ESR=0x"

_string_debug_eear_prefix:
	.string ", EEAR=0x"

_string_dcache_clean:
	.string "dcache cleaning\n\r"

_string_icache_clean:
	.string "icache cleaning\n\r"

_string_ic_invalidate:
	.string "\n\r\tic invalidate:  "

_string_ic_enable:
	.string "\n\ricache enabling\n\r"

_string_dc_invalidate:
	.string "\n\r\tdc invalidate:  "

_string_dc_enable:
	.string "\n\rdcache enabling\n\r"

_string_pbjds:
	.string "[jd]"

_string_ed:
	.string "\n\r[debug] "

	.global	_string_esr_irq_bug
_string_esr_irq_bug:
	.string "\n\rESR external interrupt bug, for details look into entry.S\n\r"



/* ========================================[ page aligned structures ]=== */

/*
 * .data section should be page aligned
 *	(look into arch/ba/kernel/vmlinux.lds)
 */

	.data
	.align 8192
	.global _swapper_pg_dir
_swapper_pg_dir:
	.space 8192

	.global	_unhandled_stack
_unhandled_stack:
	.space	8192
_unhandled_stack_top:

/* ============================================================[ EOF ]=== */
