/*
 *  linux/arch/ba/kernel/idle.c
 *
 *  BA version
 *    author(s): Matjaz Breskvar (phoenix@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  derived from cris, i386, m68k, ppc, sh ports.
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    initial port to BA architecture
 *  16. 07. 2007: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    CONFIG_NO_HZ support (ripped code from arm port)
 *
 * Idle daemon for ba.  Idle daemon will handle any action
 * that needs to be taken when the system becomes idle.
 *
 * Based on:
 * arch/ppc/kernel/idle.c
 * Written by Cort Dougan (cort@cs.nmt.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/tick.h>

#include <asm/pgtable.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/processor.h>
#include <asm/mmu.h>
#include <asm/cache.h>

#define cpu_do_idle() do { } while (0)

/*
 * This is our default idle handler.  We need to disable
 * interrupts here to ensure we don't miss a wakeup call.
 */
static void default_idle(void)
{
	local_irq_disable();
	if (!need_resched()) {
//		timer_dyn_reprogram();
		cpu_do_idle();
	}
	local_irq_enable();
}

/*
 * The idle thread.  We try to conserve power, while trying to keep
 * overall latency low.  The architecture specific idle is passed
 * a value to indicate the level of "idleness" of the system.
 */
void cpu_idle(void)
{
	while (1) {
		void (*idle)(void) = default_idle;

		/* idle start */
		tick_nohz_stop_sched_tick(1);
		while (!need_resched())
			idle();

		/* idle end */
		tick_nohz_restart_sched_tick();
		preempt_enable_no_resched();
		schedule();
		preempt_disable();
	}
}

