/*
 *  linux/arch/ba/kernel/irq.c
 *
 *  BA version
 *    author(s): Matjaz Breskvar (phoenix@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  derived from cris, i386, m68k, ppc, sh ports.
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    initial port to BA architecture
 *
 */

#include <asm/uaccess.h>
#include <linux/module.h>
#include <linux/seq_file.h>
#include <linux/interrupt.h>
#include <linux/kernel_stat.h>
#include <linux/notifier.h>
#include <linux/cpu.h>
#include <linux/delay.h>
#include <linux/mm.h>
#include <asm/aeonR2-hf.h>

static void pic_unmask_irq(unsigned int irq)
{
	/* Enable in the IMR */
	mtspr(SPR_PICMR, mfspr(SPR_PICMR) | (0x00000001L << irq));
}

static void pic_mask_irq(unsigned int irq)
{
	/* Disable in the IMR */
	mtspr(SPR_PICMR, mfspr(SPR_PICMR) & ~(0x00000001L << irq));
}

static void pic_ack_irq(unsigned int irq)
{
	/* Hmm.. I just noticed that this has a race condition .. Oh well, edge
	 * trigered interrupts are apparently not used anymore so it shouldn't
	 * matter.. */
	mtspr(SPR_PICSR, mfspr(SPR_PICSR) & ~(1UL << irq));
}

static void pic_init(void)
{
	/* turn off all interrupts */
	mtspr(SPR_PICMR, 0);
}

void ack_bad_irq(unsigned int irq)
{
	printk("unexpected IRQ trap at vector %02x\n", irq);
	pic_ack_irq(irq);
}

int show_interrupts(struct seq_file *p, void *v)
{
	phx_warn("NOT implemented yet");
#if 0
	int i = *(loff_t *) v;
	struct irqaction * action;
	unsigned long flags;
	
	if (i < NR_IRQS) {
                local_irq_save(flags);
		action = irq_action[i];
                if (!action) 
                        goto skip;
		seq_printf(p, "%2d: %10u %c %s",
			   i, kstat_cpu(0).irqs[i],
			   (action->flags & SA_INTERRUPT) ? '+' : ' ',
			   action->name);
                for (action = action->next; action; action = action->next) {
                        seq_printf(p, ",%s %s",
				   (action->flags & SA_INTERRUPT) ? " +" : "",
				   action->name);
                }
		seq_putc(p, '\n');
	skip:
                local_irq_restore(flags);
        }
#endif
        return 0;
}

asmlinkage void do_IRQ(struct pt_regs *regs)
{
	struct pt_regs *old_regs = set_irq_regs(regs);
	int irq = ffs(mfspr(SPR_PICSR) & 0xfffffffc);
	struct irq_desc *desc;

	irq_enter();

	check_stack(NULL, __FILE__, __FUNCTION__, __LINE__);

	while(irq > 0) {
		irq--;
		desc = irq_desc + irq;
		handle_level_irq(irq, desc);
		irq = ffs(mfspr(SPR_PICSR) & 0xfffffffc);
	}
	
	irq_exit();
	set_irq_regs(old_regs);
}

static struct irq_chip pic_chip = {
	.name	= "interrupts",
	.ack	= pic_ack_irq,
	.mask	= pic_mask_irq,
	.unmask	= pic_unmask_irq,
};

void __init init_IRQ(void)
{
	
	int i;

	pic_init();

	for(i = 0; i < NR_IRQS; i++)
		set_irq_chip_and_handler(i, &pic_chip, handle_level_irq);
}

