/*
 *  linux/arch/ba/kernel/setup.c
 *
 *  BA version
 *    author(s): Matjaz Breskvar (matjaz.breskvar@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  derived from cris, i386, m68k, ppc, sh ports.
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (matjaz.breskvar@beyondsemi.com)
 *    initial port to BA architecture
 *
 */

/*
 * This file handles the architecture-dependent parts of initialization
 */

#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/console.h>
#include <linux/init.h>
#include <linux/bootmem.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/pfn.h>
#include <linux/proc_fs.h>

#include <asm/board.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/smp.h>
#include <asm/pgtable.h>
#include <asm/types.h>
#include <asm/setup.h>
#include <asm/io.h>

/*
 * Debugging stuff
 */

int __phx_mmu__ = 0;
int __phx_warn__ = 0;
int __phx_debug__ = 0;
int __phx_signal__ = 0;

#ifdef CONFIG_AEONR2_TLB_STAT
unsigned long g_k_itlb_miss = 0;
unsigned long g_k_dtlb_miss = 0;
unsigned long g_u_itlb_miss = 0;
unsigned long g_u_dtlb_miss = 0;
#endif

/*
 * Setup options
 */

extern int root_mountflags;
extern char _stext, _etext, _edata, _end;

#ifdef CONFIG_BLK_DEV_INITRD
extern unsigned long initrd_start, initrd_end;
extern char __initrd_start, __initrd_end;
extern char __initramfs_start, __initramfs_end;
#endif

unsigned long fb_mem_start;

#define _STR(x)	#x
#define STR(x)	_STR(x)

#ifdef CONFIG_AEONR2
static char command_line[COMMAND_LINE_SIZE] = "root=/dev/ram console=uart,io," STR(UART_BASE_ADD) ",38400n8 console=ttyS0";
#else
static char command_line[COMMAND_LINE_SIZE] = "root=/dev/ram console=uart,mmio," STR(UART_BASE_ADD) ",38400n8 console=ttyS0";
#endif
extern const unsigned long text_start, edata; /* set by the linker script */

static unsigned long __init setup_memory(void)
{
	unsigned long bootmap_size, start_pfn, max_low_pfn;

#ifndef CONFIG_FB_OC_SHMEM_SIZE
#define CONFIG_FB_OC_SHMEM_SIZE 0
#endif /* CONFIG_FB_OC_SHMEM_SIZE */


	/* min_low_pfn points to the start of DRAM, start_pfn points
	 * to the first DRAM pages after the kernel, and max_low_pfn
	 * to the end of DRAM. Partial pages are not useful, so round it
	 * down.
	 */
	start_pfn   = PFN_UP(__pa(&_end));
	max_low_pfn = PFN_DOWN(CONFIG_AEONR2_MEMORY_SIZE
			       -CONFIG_FB_OC_SHMEM_SIZE
			       -CONFIG_AEONR2_RESERVED_MEM_SIZE);
	min_low_pfn = PAGE_OFFSET >> PAGE_SHIFT;

#undef CONFIG_FB_OC_SHMEM_SIZE

	/*
	 * set the beginning of frame buffer
	 */
	fb_mem_start = PFN_PHYS(max_low_pfn);

	/*
	 * initialize the boot-time allocator (with low memory only)
	 */
	bootmap_size = init_bootmem(start_pfn, max_low_pfn);
	free_bootmem(PFN_PHYS(start_pfn), PFN_PHYS(max_low_pfn - start_pfn));
	reserve_bootmem(PFN_PHYS(start_pfn), bootmap_size, BOOTMEM_DEFAULT);

	return(max_low_pfn);
}

static inline unsigned long extract_value_bits(unsigned long reg,
					       short bit_nr, short width)
{
	return((reg >> bit_nr) & (0 << width));
}

static inline unsigned long extract_value(unsigned long reg,
					  unsigned long mask)
{
	while (!(mask & 0x1)) {
		reg  = reg  >> 1;
		mask = mask >> 1;
	}
	return(mask & reg);
}

void __init detect_timer(void)
{
	unsigned long cfg;

	cfg = mfspr(SPR_TTMR);
	printk("period 0x%x\n", SPR_TTMR & SPR_TTMR_PERIOD);
}

void __init detect_unit_config(unsigned long upr, unsigned long mask,
			       char *text, void (*func)(void))
{
        if (text != NULL)
		printk("%s", text);

	if ( upr & mask ) {
		if (func != NULL)
			func();
		else
			printk("present\n");
	}
	else
		printk("N/A\n");
}


void __init detect_soc_generic(unsigned long upr)
{
	detect_unit_config(upr, SPR_UPR_DCP,  "  dCACHE: ", NULL);
	detect_unit_config(upr, SPR_UPR_ICP,  "  iCACHE: ", NULL);
	detect_unit_config(upr, SPR_UPR_DMP,  "  dMMU\t: ", NULL);
	detect_unit_config(upr, SPR_UPR_IMP,  "  iMMU\t: ", NULL);
	detect_unit_config(upr, SPR_UPR_DUP,  "  debug : ", NULL);
	detect_unit_config(upr, SPR_UPR_PCUP, "  PerfC : ", NULL);
	detect_unit_config(upr, SPR_UPR_PMP,  "  PM    : ", NULL);
	detect_unit_config(upr, SPR_UPR_PICP, "  PIC   : ", NULL);
	detect_unit_config(upr, SPR_UPR_TTP,  "  TIMER : ", detect_timer);
	detect_unit_config(upr, SPR_UPR_CUST, "  CUs   : ", NULL);

/* add amount configured memory
 */
}

void __init detect_soc_rev_0(unsigned long upr)
{

	printk("  WARNING, using default values !\n"
               "           (some early revision 0 processors don't have unit present register\n"
	       "            populated with all avaliable units)\n");

	printk("  dCACHE: assumed %4d Kb size, %2d bytes/line, %d way(s)",
	       CONFIG_AEONR2_DC_SIZE, CONFIG_AEONR2_DC_LINE, 1);
	detect_unit_config(upr, SPR_UPR_DCP,  ", detected: ", NULL);

	printk("  iCACHE: assumed %4d Kb size, %2d bytes/line, %d way(s)",
	       CONFIG_AEONR2_IC_SIZE, CONFIG_AEONR2_IC_LINE, 1);
	detect_unit_config(upr, SPR_UPR_ICP,  ", detected: ", NULL);

	printk("  dMMU\t: assumed %4d entries, %d way(s)",
	       CONFIG_AEONR2_DTLB_ENTRIES, 1);
	detect_unit_config(upr, SPR_UPR_DMP,  ", detected: ", NULL);

	printk("  iMMU\t: assumed %4d entries, %d way(s)",
	       CONFIG_AEONR2_ITLB_ENTRIES, 1);
	detect_unit_config(upr, SPR_UPR_IMP,  ", detected: ", NULL);
	detect_unit_config(upr, SPR_UPR_DUP,  "  debug : unknown (guess yes), detected: ", NULL);
	detect_unit_config(upr, SPR_UPR_PCUP, "  PerfC : unknown (guess no ), detected: ", NULL);
	detect_unit_config(upr, SPR_UPR_PMP,  "  PM    : unknown (guess yes), detected: ", NULL);
	detect_unit_config(upr, SPR_UPR_PICP, "  PIC   : unknown (guess yes), detected: ", NULL);
	detect_unit_config(upr, SPR_UPR_TTP,  "  TIMER : unknown (guess yes), detected: ", NULL);
	detect_unit_config(upr, SPR_UPR_CUST, "  CUs   : unknown (guess no ), detected: ", NULL);

/* add amount configured memory
 */
}

void __init detect_soc(void)
{
	unsigned long upr, cfg, version, revision;

	upr = mfspr(SPR_UPR);
	if (upr &  SPR_UPR_UP)
		printk("Detecting Processor units:\n");
	else {
		printk("Unit Present Register not avaliable\n");
		return;
	}

	cfg = mfspr(SPR_VR);
	version=extract_value(cfg, SPR_VR_VER);
	revision=extract_value(cfg, SPR_VR_REV);

#ifndef CONFIG_AEONR2_ANONYMOUS
	printk("  CPU\t: BA%lx, revision %lx, @%d MHz, %s\n",
	       version>>16, revision,
	       CONFIG_AEONR2_SYS_CLK,
	       (upr & (unsigned long)SPR_UPR_SRP) ?
	       "with shadow registers" : "with no shadow registers");

	if ((version == 0x120000) & (revision == 0)) {
		detect_soc_rev_0(upr);
	} else {
		detect_soc_generic(upr);
	}

#ifdef CONFIG_AEONR2_ICACHE_ENABLED
	printk("  iCACHE: enabled \n");
#else
	printk("  iCACHE: disabled\n");
#endif

#ifdef CONFIG_AEONR2_DCACHE_ENABLED
	printk("  dCACHE: enabled \n");
#else
	printk("  dCACHE: disabled\n");
#endif

#endif /* CONFIG_AEONR2_ANONYMOUS */
}


void __init setup_arch(char **cmdline_p)
{
	/* detect System on Chip parameters */
	//detect_soc();

	/* process 1's initial memory region is the kernel code/data */
	init_mm.start_code = (unsigned long) &_stext;
	init_mm.end_code =   (unsigned long) &_etext;
	init_mm.end_data =   (unsigned long) &_edata;
	init_mm.brk =        (unsigned long) &_end;

#ifdef CONFIG_BLK_DEV_INITRD
	initrd_start = 0;//(unsigned long)&__initrd_start;
	initrd_end = 0;//(unsigned long)&__initrd_end;
#endif

        /* setup bootmem allocator */
	setup_memory();

	/* paging_init() sets up the MMU and marks all pages as reserved */
	paging_init();

#ifdef CONFIG_SERIAL_8250_CONSOLE
	setup_early_serial8250_console(command_line);
#endif

#if defined(CONFIG_VT) && defined(CONFIG_DUMMY_CONSOLE)
	if(!conswitchp)
        	conswitchp = &dummy_con;
#endif

	*cmdline_p = command_line;

	/* Save command line copy for /proc/cmdline */
	memcpy(boot_command_line, command_line, COMMAND_LINE_SIZE);
	boot_command_line[COMMAND_LINE_SIZE-1] = '\0';

#ifndef CONFIG_AEONR2_ANONYMOUS
#ifdef  CONFIG_MMU
	printk("Linux BA port 2003-2007 Beyond Semiconductor <matjaz.breskvar@beyondsemi.com>\n");
	printk("Linux AeonR2 port 2008-2009 MStar Semiconductor\n");
#else  /* CONFIG_MMU */
	printk("uClinux BA port 2003-2007 Beyond Semiconductor <matjaz.breskvar@beyondsemi.com>\n");
	printk("uClinux AeonR2 port 2008-2009 MStar Semiconductor\n");
#endif /* CONFIG_MMU */
#endif /* CONFIG_AEONR2_ANONYMOUS */
}

static int show_cpuinfo(struct seq_file *m, void *v)
{
	unsigned long cfg, version, revision;

	cfg = mfspr(SPR_VR);
	version=extract_value(cfg, SPR_VR_VER);
	revision=extract_value(cfg, SPR_VR_REV);

#ifndef CONFIG_AEONR2_ANONYMOUS
	return seq_printf(m,
			  "cpu\t\t: Beyond Architecture (BA)\n"
			  "cpu revision\t: %lx\n"
			  "cpu model\t: BA%lx\n"
			  "cpu MHz\t\t: %d\n"
			  "icache size\t: %d kB\n"
			  "dcache size\t: %d kB\n"
			  "immu\t\t: %s\n"
			  "dmmu\t\t: %s\n"
			  "fpu\t\t: %s\n"
			  "bogomips\t: %lu.%02lu\n",

			  revision,
			  version>>16,
			  CONFIG_AEONR2_SYS_CLK,
			  8,
			  8,
			  "64 entries, 1 way",
			  "64 entries, 1 way",
			  "no",
			  (loops_per_jiffy * HZ) / 500000,
			  ((loops_per_jiffy * HZ) / 5000) % 100);
#else /* CONFIG_AEONR2_ANONYMOUS */
	return seq_printf(m,
			  "cpu revision\t: %lx\n"
			  "cpu MHz\t\t: %d\n"
			  "icache size\t: %d kB\n"
			  "dcache size\t: %d kB\n"
			  "immu\t\t: %s\n"
			  "dmmu\t\t: %s\n"

			  revision,
			  CONFIG_AEONR2_SYS_CLK,
			  8,
			  8,
			  "64 entries, 1 way",
			  "64 entries, 1 way");
#endif /* CONFIG_AEONR2_ANONYMOUS */
}

static void *c_start(struct seq_file *m, loff_t *pos)
{
	/* We only got one CPU... */
	return *pos < 1 ? (void *)1 : NULL;
}

static void *c_next(struct seq_file *m, void *v, loff_t *pos)
{
	++*pos;
	return NULL;
}

static void c_stop(struct seq_file *m, void *v)
{
}

struct seq_operations cpuinfo_op = {
	start:  c_start,
	next:   c_next,
	stop:   c_stop,
	show:   show_cpuinfo,
};

#ifdef CONFIG_AEONR2_TLB_STAT
static int proc_g_tlb_miss_show(struct seq_file *m, void *v)
{
	seq_printf(m,
	           "k-ITLB misses: %lu\n"
	           "k-DTLB misses: %lu\n"
	           "u-ITLB misses: %lu\n"
	           "u-DTLB misses: %lu\n",
	           g_k_itlb_miss,
	           g_k_dtlb_miss,
	           g_u_itlb_miss,
	           g_u_dtlb_miss);

	return 0;
}

static int proc_g_tlb_miss_open(struct inode *inode, struct file *file)
{
	return single_open(file, proc_g_tlb_miss_show, NULL);
}

static const struct file_operations proc_g_tlb_miss_operations = {
	.open		= proc_g_tlb_miss_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= single_release,
};

static int __init proc_g_tlb_miss_init(void)
{
	struct proc_dir_entry *e;

	e = create_proc_entry("tlb_miss", 0, NULL);
	if (e)
		e->proc_fops = &proc_g_tlb_miss_operations;

	return 0;
}

__initcall(proc_g_tlb_miss_init);
#endif

