/*
 *  linux/arch/ba/kernel/time.c
 *
 *  BA version
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  Copied/hacked from:
 *
 *  linux/arch/m68knommu/kernel/time.c
 *
 *  Copyright (C) 1991, 1992, 1995  Linus Torvalds
 *
 * This file contains the m68k-specific time handling details.
 * Most of the stuff is located in the machine specific files.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/profile.h>
#include <linux/mm.h>
#include <linux/time.h>
#include <linux/timex.h>
#include <linux/interrupt.h>

#include <asm/segment.h>
#include <asm/io.h>
#include <asm/aeonR2-hf.h>
#include <asm/irq.h>

/* 
 * timer_interrupt() needs to keep up the real-time clock,
 * as well as call the "do_timer()" routine every clocktick
 */
void do_timer_interrupt(struct pt_regs *regs)
{
	/* may need to kick the hardware timer */
#ifndef CONFIG_AEONR2_TICK_ON_INSN_COUNT
	mtspr(SPR_TTMR, SPR_TTMR_IE | SPR_TTMR_RT |
	      (SYS_TICK_PER & SPR_TTMR_PERIOD));
#else
        mtspr(SPR_TTUR, SPR_TTUR_ICTM);
        mtspr(SPR_TTCR, 0x0);
	/* assume avarage ticks per cycles is 4
	 * hence the SYS_TICK_PER>>2
	 */
	mtspr(SPR_TTMR, SPR_TTMR_IE | SPR_TTMR_RT |
	      ((SYS_TICK_PER>>2) & SPR_TTMR_PERIOD));
#endif	                
	
	do_timer(1);
	
#ifndef CONFIG_SMP
	update_process_times(user_mode(regs));
#endif
	profile_tick(CPU_PROFILING);
}


/*
 * This get called at every clock tick...
 *
 */
irqreturn_t timer_interrupt(struct pt_regs * regs)
{
	struct pt_regs *old_regs = set_irq_regs(regs);

  	check_stack(regs, __FILE__, __FUNCTION__, __LINE__);
	
        /*
         * Here we are in the timer irq handler. We just have irqs locally
         * disabled but we don't know if the timer_bh is running on the other
         * CPU. We need to avoid to SMP race with it. NOTE: we don' t need
         * the irq version of write_lock because as just said we have irq
         * locally disabled. -arca
         */
        //write_seqlock(&xtime_lock);
        do_timer_interrupt(regs);
        //write_sequnlock(&xtime_lock);
 
	set_irq_regs(old_regs);

        return IRQ_HANDLED;
}

/*
 * Scheduler clock - returns current time in nanosec units.
 */
unsigned long long sched_clock(void)
{
	return (jiffies_64 - INITIAL_JIFFIES) * (1000000000 / HZ);
}

void __init time_init(void)
{
	/* Maybe oneday when we have an RTC, we'll have to set the current
	 * time here.. */

	/* Set counter period, enable timer and interrupt */
#ifndef CONFIG_AEONR2_TICK_ON_INSN_COUNT
	mtspr(SPR_TTMR, SPR_TTMR_IE | SPR_TTMR_RT | (SYS_TICK_PER & SPR_TTMR_PERIOD));
#else
	mtspr(SPR_TTUR, SPR_TTUR_ICTM);
        mtspr(SPR_TTCR, 0x0);
        mtspr(SPR_TTMR, SPR_TTMR_IE | SPR_TTMR_RT | (4000 & SPR_TTMR_PERIOD));
#endif
}
