#include <linux/serial_core.h>
#include <linux/serial_reg.h>
#include <linux/serial.h>
#include <asm/io.h>
#include <asm/serial.h>

#define UART_REGSHIFT	0
#define UART_BASE	    UART_BASE_ADD

static inline unsigned int serial_in(int offset)
{
	return readb(UART_BASE + (offset << UART_REGSHIFT));
}

static inline void serial_out(int offset, int value)
{
	writeb(value, UART_BASE + (offset << UART_REGSHIFT));
}

void uart_wait_for_xmit(void)
{
	unsigned int status;

	for (;;) {
		status = serial_in(UART_LSR);
		if ((status & (UART_LSR_TEMT | UART_LSR_THRE)) == (UART_LSR_TEMT | UART_LSR_THRE)) {
			return;
		}
	}
}

void uart_output_c_raw(int c)
{
	uart_wait_for_xmit();
	serial_out(UART_TX, c);
	if (c == '\n') {
		uart_wait_for_xmit();
		serial_out(UART_TX, '\r');
	}
}

void uart_output_c(int c)
{
	unsigned long flags;
	unsigned long ier;

	local_irq_save(flags);
	ier = serial_in(UART_IER);
	serial_out(UART_IER, 0);
	uart_wait_for_xmit();
	serial_out(UART_TX, c);
	if (c == '\n') {
		uart_wait_for_xmit();
		serial_out(UART_TX, '\r');
	}
	serial_out(UART_IER, ier);
	local_irq_restore(flags);
}

void uart_output_hex(unsigned long val)
{
	unsigned long flags;
	unsigned long ier;
	int c;
	int shift = 28;

	local_irq_save(flags);
	ier = serial_in(UART_IER);
	serial_out(UART_IER, 0);
	uart_wait_for_xmit();
	serial_out(UART_TX, '0');
	uart_wait_for_xmit();
	serial_out(UART_TX, 'X');
	do {
		c = (val >> shift) & 0xf;
		if (c >= 10) {
			c += 7;
		}
		c += '0';
		uart_wait_for_xmit();
		serial_out(UART_TX, c);
		shift -= 4;
	} while (shift >= 0);
	serial_out(UART_IER, ier);
	local_irq_restore(flags);
}

void uart_output_s(const char *s)
{
	unsigned long flags;
	unsigned long ier;

	local_irq_save(flags);
	ier = serial_in(UART_IER);
	serial_out(UART_IER, 0);
	while (*s) {
		uart_output_c_raw(*s);
		s++;
	}
	serial_out(UART_IER, ier);
	local_irq_restore(flags);
}

