/* ld script to make Linux/BA kernel
 * Authors: Matjaz Breskvar <phoenix@beyondsemi.com>
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 */

/* __PHX__ :: TODO 
 *		- clean up __offset & stuff
 *		- change all 8192 aligment to PAGE !!!
 *		- recheck if all aligments are really needed
 *		- make ramsize an option (maybe config option) or suck it of
 * hardware and calculate dram_end acordingly
 */


#ifndef CONFIG_MMU
#define LOAD_BASE	0x00000000
#else
#define LOAD_BASE	0xc0000000
#ifdef CONFIG_AEONR2_FLASH_BOOT
#  define LOAD_OFFSET	-0x30000000
#else
#  define LOAD_OFFSET	0xc0000000
#endif
#endif


#define VMLINUX_SYMBOL(_sym_) _##_sym_

#include <asm-generic/vmlinux.lds.h>

OUTPUT_ARCH(aeon:aeonR2)

#ifdef CONFIG_AEONR2_BIG_ENDIAN
_jiffies = _jiffies_64 + 4;
OUTPUT_FORMAT("elf32-bigaeon", "elf32-bigaeon", "elf32-bigaeon")
#else
_jiffies = _jiffies_64;
OUTPUT_FORMAT("elf32-littleaeon", "elf32-littleaeon", "elf32-littleaeon")
#endif
SECTIONS
{
        /* Read-only sections, merged into text segment: */
        . = LOAD_BASE ; 
        _dram_start = .;

        __s_protected_core = .;
	
        __stext = .;
        .text			: AT(ADDR(.text) - LOAD_OFFSET)
        {
		__text = .;
		_text = .; /* Should only be __text, but scripts/kallsyms.c is
		            * broken wrt. symbol prefixes. */
		TEXT_TEXT
		SCHED_TEXT
		LOCK_TEXT
		KPROBES_TEXT
		*(.text.*)
		*(.fixup)
		*(.gnu.warning)
		*(.text.__*)
		FILL (0);
		. = ALIGN (8192);
        }
        __etext = .;

        RODATA
	
	. = ALIGN(8192); 
        __e_protected_core = .;

        . = ALIGN(4);
        .data    		: AT(ADDR(.data) - LOAD_OFFSET)
        {
        	*(.data)
		DATA_DATA
		CONSTRUCTORS
        }

	/* writeable */

	__ex_table		: AT(ADDR(__ex_table) - LOAD_OFFSET)
	{
		___start___ex_table = .;
		*(__ex_table)
		___stop___ex_table = .;
	}

        __bug_table		: AT(ADDR(__bug_table) - LOAD_OFFSET) 
	{
                ___start___bug_table = .;
                *(___bug_table)
                ___stop___bug_table = .;
        }


	. = ALIGN(8192);
	__nosave_begin = .;
	.data_nosave : { *(.data.nosave) }
  	. = ALIGN(8192);
	__nosave_end = .;

	. = ALIGN(32);
	.data.cacheline_aligned	: AT(ADDR(.data.cacheline_aligned) - LOAD_OFFSET)
	{ 
		*(.data.cacheline_aligned) 
	}
	
        __edata  =  .;

       /* init_task and stack, must be aligned */
        . = ALIGN(8192);      
        .data.init_task		: AT(ADDR(.data.init_task) - LOAD_OFFSET) 
        { 
          *(.data.init_task) 
        }

	/* Alternate stuff .. */
	. = ALIGN(4);
	.c_alt			: AT(ADDR(.c_alt) - LOAD_OFFSET)
	{
		*(.c_alt.*)
	}
	. = ALIGN(4);
	.c_alts			: AT(ADDR(.c_alts) - LOAD_OFFSET)
	{
		*(.c_alts)
	}
	___start___alt_fixups = . ;
	.c_alt_fixups		: AT(ADDR(.c_alt_fixups) - LOAD_OFFSET)
	{
		*(.c_alt_fixups)
	}
	___stop___alt_fixups = . ;
 
        /* Init code and data */
        . = ALIGN(8192);
        ___init_begin = .;
        .init.text		: AT(ADDR(.init.text) - LOAD_OFFSET)
	{
		__sinittext = .;
		*(.init.text.1)
                *(.init.text)
                __einittext = .;
        }

        .init.data		: AT(ADDR(.init.data) - LOAD_OFFSET)
	{ 
		INIT_DATA
	}

	/* Setup calls */
        . = ALIGN(16);
        ___setup_start = .;
        .init.setup		: AT(ADDR(.init.setup) - LOAD_OFFSET) 
	{ 
		*(.init.setup) 
	}
        ___setup_end = .;

	
        ___start___param = .;
        __param			: AT(ADDR(__param) - LOAD_OFFSET) 
	{ 
		*(__param) 
	}
        ___stop___param = .;


	
        .initcall.init		: AT(ADDR(.initcall.init) - LOAD_OFFSET)
	{
                ___initcall_start = .;
		INITCALLS
                ___initcall_end = .;
        }

        .con_initcall.init	: AT(ADDR(.con_initcall.init) - LOAD_OFFSET) 
	{
                ___con_initcall_start = .;
                *(.con_initcall.init)
                ___con_initcall_end = .;
        }
	
        SECURITY_INIT

	. = ALIGN(4);
	
	/* .exit.text is discarded at runtime, not link time,
           to deal with references from __bug_table */
	.exit.text : AT(ADDR(.exit.text) - LOAD_OFFSET) { *(.exit.text) }
	.exit.data : AT(ADDR(.exit.data) - LOAD_OFFSET) { *(.exit.data) }

#if defined(CONFIG_BLK_DEV_INITRD)
	. = ALIGN(8192); 
	.init.ramfs		: AT(ADDR(.init.ramfs) - LOAD_OFFSET)
	{
                ___initramfs_start = .;
                *(.init.ramfs)
                ___initramfs_end = .;
                /* We fill to the next page, so we can discard all init
                   pages without needing to consider what payload might be
                   appended to the kernel image.  */
                FILL (0);
                . = ALIGN (8192);
        }
#endif
        ___init_end = .;
	
	. = ALIGN(8192); 
	.initrd			: AT(ADDR(.initrd) - LOAD_OFFSET)
	{
		___initrd_start = .; 
		*(.initrd)
		FILL (0);
                . = ALIGN (8192);
		___initrd_end = .; 
	}

        ___vmlinux_end = .;            /* last address of the physical file */
/*      ___init_end = .;
*/
        ___data_end = . ;              /* Move to _edata ? */
        ___bss_start = .;              /* BSS */
        .sbss           : AT(ADDR(.sbss) - LOAD_OFFSET)
        {
                *(.sbss)
                /* *(.scommon) */	/* aeon toolchain do not need scommon */
        }

        .bss			: AT(ADDR(.bss) - LOAD_OFFSET)
	{
                *(COMMON)
                *(.bss)
        }
        ___bss_stop = .;

        . =  ALIGN (0x20);
        __end = .;

	/* Sections to be discarded */
	/DISCARD/ : {
        *(.exitcall.exit)
        }

	STABS_DEBUG

	DWARF_DEBUG

	NOTES
}
