/*
 *  linux/arch/ba/lib/memcpy.c
 *
 *  Chunk copy optimizations by Matjaz Breskvar (phoenix@beyondsemi.com).
 *
 *  changes:
 *  17. 10. 2007: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    Chunk compy optimizations.
 *
 */

#include <linux/string.h>
#include <linux/module.h>

#include <asm/spr_defs.h>
#include <linux/linkage.h>

void *memcpy(void *to, const void *from, size_t size)
{
	unsigned char *src = (unsigned char *)from;
	unsigned char *dst = (unsigned char *)to;
	unsigned long t1, t2, t3, t4;
	unsigned char c1, c2;

	if (size < 8) {

		switch (size) {

		case 7:
			*(dst+6) = *(src+6);
		case 6:
			*(dst+5) = *(src+5);
		case 5:
			*(dst+4) = *(src+4);
		case 4:
			*(dst+3) = *(src+3);
		case 3:
			*(dst+2) = *(src+2);
		case 2:
			*(dst+1) = *(src+1);
		case 1:
			*(dst+0) = *(src+0);
		case 0:
			return to;
		}
	}

	if (!(((unsigned long)src ^ (unsigned long)dst) & 0x3)) {
		
	
		if ((unsigned long)dst & 0x1) {
			//	printk("aligning x1 %d\n", size);
			*dst = *src;
			dst  += 1;
			src  += 1;
			size -= 1;
		}
		
		if ((unsigned long)dst & 0x2) {
			//	printk("aligning x2 %d\n", size);
			*(unsigned short*)dst = *(unsigned short*)src;
			dst  += 2;
			src  += 2;		
			size -= 2;
		}


//		printk ("aligned %d\n", size);

		switch (size)
		{

		default:
			while (size >= 32) {
				size -= 32;
				
				t1 = *((unsigned long*)(src + 12));
				t2 = *((unsigned long*)(src +  8));
				t3 = *((unsigned long*)(src +  4));
				t4 = *((unsigned long*)(src +  0));
				*((unsigned long*)(dst + 12)) = t1;
				*((unsigned long*)(dst +  8)) = t2;
				*((unsigned long*)(dst +  4)) = t3;
				*((unsigned long*)(dst +  0)) = t4;
				t1 = *((unsigned long*)(src + 28));
				t2 = *((unsigned long*)(src + 24));
				t3 = *((unsigned long*)(src + 20));
				t4 = *((unsigned long*)(src + 16));
				*((unsigned long*)(dst + 28)) = t1;
				*((unsigned long*)(dst + 24)) = t2;
				*((unsigned long*)(dst + 20)) = t3;
				*((unsigned long*)(dst + 16)) = t4;
				
				src += 32;
				dst += 32;
			}
		case 31:
		case 30:
		case 29:
		case 28:
		case 27:
		case 26:
		case 25:
		case 24:
		case 23:
		case 22:
		case 21:
		case 20:
		case 19:
		case 18:
		case 17:
		case 16:

			if (size >= 16) {
				size -= 16;
				
				t1 = *((unsigned long*)(src + 12));
				t2 = *((unsigned long*)(src +  8));
				t3 = *((unsigned long*)(src +  4));
				t4 = *((unsigned long*)(src +  0));
				*((unsigned long*)(dst + 12)) = t1;
				*((unsigned long*)(dst +  8)) = t2;
				*((unsigned long*)(dst +  4)) = t3;
				*((unsigned long*)(dst +  0)) = t4;
				
				src += 16;
				dst += 16;
			}
			
		case 15:
		case 14:
		case 13:
		case 12:
		case 11:
		case 10:
		case 9:
		case 8:
			
			if (size >= 8) {
				size -= 8;
				
				t1 = *((unsigned long*)(src +  4));
				t2 = *((unsigned long*)(src +  0));
				*((unsigned long*)(dst +  4)) = t1;
				*((unsigned long*)(dst +  0)) = t2;
				
				src += 8;
				dst += 8;
			}

		case 7:
		case 6:
		case 5:
		case 4:
			
			if (size >= 4) {
				size -= 4;
				
				*((unsigned long*)dst) = *((unsigned long*)src);
				
				dst  += 4;
				src  += 4;
			}
			
		case 3:
		case 2:
			
			if (size >= 2) {
				size -= 2;
				
				*((unsigned short*)dst) = *((unsigned short*)src);
				
				dst  += 2;
				src  += 2;
			}
			
		case 1:
			if (size >= 1) {
				// size -= 1;
				*dst = *src;
				
				//src++;
				//dst++;
			}
			return to;
			
		case 0:
			return to;
			
		}
		return to;
	}

	/* we know that destination is word aligned, use that */
	while (size >= 4) {
	
		c1 = *(src + 0);
		c2 = *(src + 1);
		*(dst + 0) = c1;
		*(dst + 1) = c2;
		c1 = *(src + 2);
		c2 = *(src + 3);
		*(dst + 2) = c1;
		*(dst + 3) = c2;
		
		dst  += 4;
		src  += 4;
		size -= 4;
	}

	while (size--) {
		*dst = *src;
		
		src++;
		dst++;
	}
	
	return to;
}
