/*
 *  linux/arch/ba/lib/memset.c
 *
 *  Chunk optimizations by Matjaz Breskvar (phoenix@beyondsemi.com).
 *
 *  changes:
 *  17. 10. 2007: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    Chunk copy optimizations.
 *
 */

#include <linux/string.h>
#include <linux/module.h>

#include <asm/spr_defs.h>
#include <linux/linkage.h>

#undef memset

void *__memset_aligned32(void *s, unsigned long cccc, size_t count)
{
	unsigned char *dst = (unsigned char *)s;

	switch (count)
	{
	default:
		while (count >= 32) {
			count -= 32;
			*((unsigned long*)(dst +  0)) = cccc;
			*((unsigned long*)(dst +  4)) = cccc;
			*((unsigned long*)(dst +  8)) = cccc;
			*((unsigned long*)(dst + 12)) = cccc;
			*((unsigned long*)(dst + 16)) = cccc;
			*((unsigned long*)(dst + 20)) = cccc;
			*((unsigned long*)(dst + 24)) = cccc;
			*((unsigned long*)(dst + 28)) = cccc;
			dst += 32;
		}
	case 31:
	case 30:
	case 29:
	case 28:
	case 27:
	case 26:
	case 25:
	case 24:
	case 23:
	case 22:
	case 21:
	case 20:
	case 19:
	case 18:
	case 17:
	case 16:
		if (count >= 16) {
			count -= 16;
			*((unsigned long*)(dst +  0)) = cccc;
			*((unsigned long*)(dst +  4)) = cccc;
			*((unsigned long*)(dst +  8)) = cccc;
			*((unsigned long*)(dst + 12)) = cccc;
			dst += 16;
		}
	case 15:
	case 14:
	case 13:
	case 12:
	case 11:
	case 10:
	case 9:
	case 8:
		if (count >= 8) {
			count -= 8;
			*((unsigned long*)(dst +  0)) = cccc;
			*((unsigned long*)(dst +  4)) = cccc;
			dst += 8;
		}
	case 7:
	case 6:
	case 5:
	case 4:
		if (count >= 4) {
			count -= 4;
			*((unsigned long*)(dst +  0)) = cccc;
			dst  += 4;
		}
	case 3:
	case 2:
		if (count >= 2) {
			count -= 2;
			*((unsigned short*)dst) = (unsigned short)cccc;
			dst  += 2;
		}
	case 1:
		if (count >= 1) {
			// count -= 1;
			*dst = (unsigned char)cccc;
			//dst++;
		}
		return s;
		
	case 0:
		return s;
		
	}
	return s;	
}


void *__memset_unaligned(void *s, unsigned long cccc, size_t count)
{
	char *dst = (char *)s;
	
	if (count >= 1 && (unsigned long)dst & 0x1) {
		*dst = cccc;
		dst  += 1;
		count -= 1;
	}
		
	if (count >= 2 && ((unsigned long)dst & 0x2)) {
		*((unsigned short*)dst) = (unsigned short)cccc;
		dst  += 2;
		count -= 2;
	}

	return(__memset_aligned32(dst, cccc, count));
}

void *__memset_generic(void *s, int c, size_t count)
{
	return(__memset_unaligned(s, 0x01010101UL*(unsigned char)c, count));
}

void *memset(void *s, int c, size_t count)
{
	return __memset_generic(s, c, count);
}

EXPORT_SYMBOL(memset);
