/*
 *  linux/arch/ba/lib/usercopy.c
 *
 *  Original pure assembly version by Matjaz Breskvar (phoenix@beyondsemi.com).
 *  Translated to inline assembly and optimised by Gyorgy Jeney (nog@beyondsemi.com).
 *  Chunk copy optimizations by Matjaz Breskvar (phoenix@beyondsemi.com).
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    initial port to BA architecture
 *  26. 07. 2006: Gyorgy Jeney (nog@beyondsemi.com)
 *    Optimised and translated to inline assembly.
 *  27. 03. 2007: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    Chunk compy optimizations.
 *
 */


#include <linux/linkage.h>
#include <asm/errno.h>
#include <asm/uaccess.h>

/* NOTE: return's size remaining */
static inline unsigned long __asm_copy_from_user_32(void *to, const void *from, unsigned long size)
{
	unsigned long tmp1, tmp2;

	asm volatile (
		"1:	b.lwz	%0,0(%2)\n"
		"2:	b.lwz	%1,4(%2)\n"
		"	b.sw	0(%3),%0\n"
		"	b.sw	4(%3),%1\n"
		"3:	b.lwz	%0,8(%2)\n"
		"4:	b.lwz	%1,12(%2)\n"
		"	b.sw	8(%3),%0\n"
		"	b.sw	12(%3),%1\n"
		"5:	b.lwz	%0,16(%2)\n"
		"6:	b.lwz	%1,20(%2)\n"
		"	b.sw	16(%3),%0\n"
		"	b.sw	20(%3),%1\n"
		"7:	b.lwz	%0,24(%2)\n"
		"8:	b.lwz	%1,28(%2)\n"
		"	b.sw	24(%3),%0\n"
		"	b.sw	28(%3),%1\n"
		"	b.addi	%0,%4,-32\n"
		"9:\n"
		
		".section .fixup, \"ax\"\n"
		"98:\n"
		"	b.sw	24(%3),%0\n"
		"	b.addi	%0,%4,-28\n"
		"	b.j	9b\n"
		"97:\n"
		"	b.addi	%0,%4,-24\n"
		"	b.j	9b\n"
		"96:\n"
		"	b.sw	16(%3),%0\n"
		"	b.addi	%0,%4,-20\n"
		"	b.j	9b\n"
		"95:\n"
		"	b.addi	%0,%4,-16\n"
		"	b.j	9b\n"
		"94:\n"
		"	b.sw	8(%3),%0\n"
		"	b.addi	%0,%4,-12\n"
		"	b.j	9b\n"
		"93:\n"
		"	b.addi	%0,%4,-8\n"
		"	b.j	9b\n"
		"92:\n"
		"	b.sw	0(%3),%0\n"
		"	b.addi	%0,%4,-4\n"
		"       b.j	9b\n"
		"91:\n"
		"	b.addi	%0,%4,0\n"
		"       b.j	9b\n"
		"	.previous\n"
		
		"	.section __ex_table, \"a\"\n"
		"		.long 1b, 91b\n"
		"		.long 2b, 92b\n"
		"		.long 3b, 93b\n"
		"		.long 4b, 94b\n"
		"		.long 5b, 95b\n"
		"		.long 6b, 96b\n"
		"		.long 7b, 97b\n"
		"		.long 8b, 98b\n"
		"	.previous\n"

		: "=&r"(tmp1), "=&r"(tmp2)
		: "r"(from), "r"(to), "r"(size));
		return(tmp1);
}

/* NOTE: returns size remaining */
static inline unsigned long __asm_copy_to_user_32(void *to, const void *from, unsigned long size)
{
	unsigned long tmp1, tmp2;

	asm volatile (
		"	b.lwz	%0,0(%2)\n"
		"	b.lwz	%1,4(%2)\n"
		"1:	b.sw	0(%3),%0\n"
		"2:	b.sw	4(%3),%1\n"
		"	b.lwz	%0,8(%2)\n"
		"	b.lwz	%1,12(%2)\n"
		"3:	b.sw	8(%3),%0\n"
		"4:	b.sw	12(%3),%1\n"
		"	b.lwz	%0,16(%2)\n"
		"	b.lwz	%1,20(%2)\n"
		"5:	b.sw	16(%3),%0\n"
		"6:	b.sw	20(%3),%1\n"
		"	b.lwz	%0,24(%2)\n"
		"	b.lwz	%1,28(%2)\n"
		"7:	b.sw	24(%3),%0\n"
		"8:	b.sw	28(%3),%1\n"
		"	b.addi	%0,%4,-32\n"
		"9:\n"
		
		".section .fixup, \"ax\"\n"
		"98:\n"
		"	b.addi	%0,%4,-28\n"
		"	b.j	9b\n"
		"97:\n"
		"	b.addi	%0,%4,-24\n"
		"	b.j	9b\n"
		"96:\n"
		"	b.addi	%0,%4,-20\n"
		"	b.j	9b\n"
		"95:\n"
		"	b.addi	%0,%4,-16\n"
		"	b.j	9b\n"
		"94:\n"
		"	b.addi	%0,%4,-12\n"
		"	b.j	9b\n"
		"93:\n"
		"	b.addi	%0,%4,-8\n"
		"	b.j	9b\n"
		"92:\n"
		"	b.addi	%0,%4,-4\n"
		"       b.j	9b\n"
		"91:\n"
		"	b.addi	%0,%4,0\n"
		"       b.j	9b\n"
		"	.previous\n"


		"	.section __ex_table, \"a\"\n"
		"		.long 1b, 91b\n"
		"		.long 2b, 92b\n"
		"		.long 3b, 93b\n"
		"		.long 4b, 94b\n"
		"		.long 5b, 95b\n"
		"		.long 6b, 96b\n"
		"		.long 7b, 97b\n"
		"		.long 8b, 98b\n"
		"	.previous\n"
		
		: "=&r"(tmp1), "=&r"(tmp2)
		: "r"(from), "r"(to), "r"(size));
		return(tmp1);
}

/* NOTE: it returns number of bytes NOT copied !!! */
unsigned long __copy_from_user(void *to, const void *from, unsigned long size)
{
	unsigned long tmp, s;

		
	if (!(((unsigned long)to & 0x3) || ((unsigned long)from & 0x3))) {

		while (size >= 32) {
			size -= 32;

			s = __asm_copy_from_user_32(to, from, size+32);
			if (s != size) {
				return(s);
			}
			
			to   += 32;
			from += 32;
		}
	}
	
	if (!size)
		return 0;

	asm volatile (
		"2:	b.lbz	%0,0(%2)\n"
		"	b.sb	0(%3),%0\n"
		"	b.addi	%1,%1,-1\n"
		"	b.addi	%2,%2,1\n"
		"	b.addi	%3,%3,1\n"
		"	b.sfgtu	%1,r0\n"
		"	b.bf	2b\n"
		"3:\n"
		"	.section __ex_table, \"a\"\n"
		"		.long 2b, 3b\n"
		"	.previous\n"

		: "=&r"(tmp), "+r"(size), "+r"(from), "+r"(to));

	return size;
}

/* NOTE: it returns number of bytes NOT copied !!! */
unsigned long __copy_to_user(void *to, const void *from, unsigned long size)
{
	unsigned long tmp, s;


	if (!(((unsigned long)to & 0x3) || ((unsigned long)from & 0x3))) {

		while (size >= 32) {
			size -= 32;

			s = __asm_copy_to_user_32(to, from, size+32);
			if (s != size) {
				return(s);
			}
			
			to   += 32;
			from += 32;
		}
	}
	
	if(!size)
		return 0;

	asm volatile (
		"1:	b.lbz	%0,0(%2)\n"
		"2:	b.sb	0(%3),%0\n"
		"	b.addi	%1,%1,-1\n"
		"	b.addi	%2,%2,1\n"
		"	b.addi	%3,%3,1\n"
		"	b.sfgtu	%1,r0\n"
		"	b.bf	1b\n"
		"3:\n"
		"	.section __ex_table, \"a\"\n"
		"		.long 2b, 3b\n"
		"	.previous\n"

		: "=&r"(tmp), "+r"(size), "+r"(from), "+r"(to));

	return size;
}

/* NOTE: it returns number of bytes NOT cleared !!! */
unsigned long __clear_user(void *addr, unsigned long size)
{
	if(!size)
		return 0;

	asm volatile (
		"2:	b.sb	0(%1),r0\n"
		"	b.addi	%0,%0,-1\n"
		"	b.addi	%1,%1,1\n"
		"	b.sfgtu	%0,r0\n"
		"	b.bf	2b\n"
		"3:\n"
		"	.section __ex_table, \"a\"\n"
		"		.long 2b, 3b\n"
		"	.previous\n"

		: "+r"(size), "+r"(addr));

	return size;
}

int __strncpy_from_user(char *to, const char *from, long size)
{
	unsigned long tmp;
	int ret = 0;

	if(!size)
		return 0;

	asm volatile (
		"2:	b.lbz	%0,0(%1)\n"
		"	b.sb	0(%2),%0\n"
		"	b.sfeq	%0,r0\n"
		"	b.bf	3f\n"
		"	b.addi	%3,%3,1\n"
		"	b.addi	%2,%2,1\n"
		"	b.addi	%1,%1,1\n"
		"	b.sfne	%3,%4\n"
		"	b.bf	2b\n"

		"3:\n"
		"	.section .fixup, \"ax\"\n"
		"99:\n"
		"	b.addi	%3,r0,%5\n"
		"	b.j	3b\n"
		"	.previous\n"

		"	.section __ex_table, \"a\"\n"
		"		.long 2b, 99b\n"
		"	.previous\n"

		: "=&r"(tmp), "+r"(from), "+r"(to), "+r"(ret)
		: "r"(size), "i"(-EFAULT));

	return ret;
}

int __strnlen_user(const char *str, long len, unsigned long top)
{
	unsigned long tmp;
	int ret = 0;
	unsigned long stop;

	if(top < (unsigned long)str + len)
		stop = top;
	else
		stop = (unsigned long)str + len;

	asm volatile (
		"1:	b.addi	%2,%2,1\n"
		"	b.sfeq	%1,%3\n"
		"	b.bf	3f\n"

		"2:	b.lbz	%0,0(%1)\n"
		"	b.addi	%1,%1,1\n"
		"	b.sfne	%0,r0\n"
		"	b.bf	1b\n"

		"3:\n"
		"	.section .fixup, \"ax\"\n"
		"99:\n"
		"	b.addi	%2,r0,0\n"
		"	b.j	3b\n"
		"	.previous\n"

		"	.section __ex_table, \"a\"\n"
		"		.long 2b, 99b\n"
		"	.previous\n"

		: "=&r"(tmp), "+&r"(str), "+&r"(ret)
		: "r"(stop));

	if(top < (unsigned long)str + len)
		if((unsigned long)str > top && tmp)
			return 0;

	return ret;
}

