/*
 *  linux/arch/ba/mm/init.c
 *
 *  BA version
 *    author(s): Matjaz Breskvar (phoenix@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  derived from cris, i386, m68k, ppc, sh ports.
 *
 *  changes:
 *  18. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    initial port to BA architecture
 *
 *  22. 11. 2003: Matjaz Breskvar (phoenix@beyondsemi.com)
 *    cleanups, identical mapping for serial console
 */
 
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/types.h>
#include <linux/ptrace.h>
#include <linux/mman.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/smp.h>
#include <linux/bootmem.h>
#include <linux/init.h>
#include <linux/delay.h>
#ifdef CONFIG_BLK_DEV_INITRD
#include <linux/blkdev.h>          /* for initrd_* */
#endif

#include <asm/system.h>
#include <asm/segment.h>
#include <asm/pgalloc.h>
#include <asm/pgtable.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/tlb.h>
#include <asm/mmu_context.h>
#include <asm/aeonR2-hf.h>
#include <asm/kmap_types.h>
#include <asm/fixmap.h>
#include <asm/alternatives.h>

#include <asm/board.h>

typedef struct {
	unsigned int beg;
	unsigned int end;
} tMissHandler;

extern tMissHandler dtlb_miss_handlers[];
extern tMissHandler itlb_miss_handlers[];

extern void __switch_pic_alternate(unsigned long, unsigned long, unsigned long, int);

int mem_init_done = 0;

DEFINE_PER_CPU(struct mmu_gather, mmu_gathers);

unsigned long empty_zero_page;

#define static noinline


void show_mem(void)
{
   
	int i,free = 0,total = 0,cached = 0, reserved = 0, nonshared = 0;
	int shared = 0;
	
	printk("\nMem-info:\n");
	show_free_areas();
	printk("Free swap:       %6ldkB\n",nr_swap_pages<<(PAGE_SHIFT-10));
	i = max_mapnr;
	while (i-- > 0) 
	{
		total++;
		if (PageReserved(mem_map+i))
			reserved++;
		else if (PageSwapCache(mem_map+i))
			cached++;
		else if (!page_count(mem_map+i))
			free++;
		else if (page_count(mem_map+i) == 1)
			nonshared++;
		else
			shared += page_count(mem_map+i) - 1;
	}
	
	printk("%d pages of RAM\n",total);
	printk("%d free pages\n",free);
	printk("%d reserved pages\n",reserved);
	printk("%d pages nonshared\n",nonshared);
	printk("%d pages shared\n",shared);
	printk("%d pages swap cached\n",cached);
}

static pgprot_t __init set_tlb_cache_inhibit(unsigned long vaddr, pgprot_t page_protection)
{
#ifdef CONFIG_AEONR2_TLB_RANDOM_CI
#include <linux/random.h>

	/* only randomize CI bit for pages with valid page
	 * protection (!0) and thus without _PAGE_CI already set
	 */
	if (!((pgprot_val(page_protection) & _PAGE_CI)) &&
	    pgprot_val(page_protection)) {
		
		char random = 0;
		get_random_bytes(&random, 1);
		
		if ((random & 0x1) && 
		    pgprot_val(page_protection)) {
			page_protection =  __pgprot(pgprot_val(page_protection)
						    |_PAGE_CI);
			
		}
	}
	return(page_protection);
#else
	return(page_protection);
#endif
}

static int __init populate_pgd(pgd_t *pgd_base, unsigned long beg_addr, unsigned long end_addr,
			       unsigned long offset, pgprot_t page_protection)
{
	int i,j;
	unsigned long vaddr = beg_addr;
	pgd_t *pgd;
	pte_t *pte;
	
	if ((i = pgd_index(beg_addr)) >= PTRS_PER_PGD) {
		printk("too much memory requested to be mapped\n");
		BUG();
	}

	pgd = pgd_base + i;

	for ( ; i <= pgd_index(end_addr) ; pgd++) {

		if (pgd_val(*pgd) == pgd_val(__pgd(0))) {
			/* alloc one page containing PTE's */
			pte = (pte_t *)alloc_bootmem_pages(PAGE_SIZE);
		} else {
			/* adding PTE entries into already existing PGD */
			pte = (pte_t *)__va(pgd_val(*pgd));
		}

		j = (vaddr >> PAGE_SHIFT) % PTRS_PER_PTE;
		for ( ; j < PTRS_PER_PTE ; j++) {

			pte[j] = mk_pte_phys(vaddr - offset, 
					     set_tlb_cache_inhibit(vaddr,page_protection));
			vaddr += PAGE_SIZE;
			if (vaddr >= end_addr)
				break;
		}

		/* link page containing PTE's to PGD */
		*pgd = __pgd(__pa((unsigned int)pte));
		i++;
	} 

	return(0);
}

void __init fixrange_init (unsigned long start, unsigned long end, pgd_t *pgd_base)
{
	populate_pgd(pgd_base, start, end, KERNELBASE, __pgprot(0));	
}

void __init zone_sizes_init(void)
{
	unsigned long zones_size[MAX_NR_ZONES] = {0, };
	unsigned int max_dma, low;

	max_dma = virt_to_phys((char *)MAX_DMA_ADDRESS) >> PAGE_SHIFT;
	low = max_low_pfn;

	if (low < max_dma)
		zones_size[ZONE_DMA] = low;
	else {
		zones_size[ZONE_DMA] = max_dma;
		zones_size[ZONE_NORMAL] = low - max_dma;
	}
	free_area_init(zones_size);
}

void __init identical_mapping(unsigned long start, unsigned long size,
				     unsigned long page_attrs)
{

	printk("Setting up identical mapping (0x%08lx - 0x%08lx)\n",
	       start, start + size);

	page_attrs |= _PAGE_ALL | _PAGE_SRE | _PAGE_SWE |
		      _PAGE_SHARED | _PAGE_DIRTY | _PAGE_EXEC;

	populate_pgd(swapper_pg_dir, start, start+size, 0, __pgprot(page_attrs));
}



void __init paging_init(void)
{
	extern void tlb_init(void);

	int i;
	unsigned long addr, end;

	/* allocate and clear zero page */
	empty_zero_page = (unsigned long) alloc_bootmem_pages(PAGE_SIZE);
	memset((void*)empty_zero_page, 0, PAGE_SIZE);

	//printk("Setting up paging and PTEs.\n");

	/* clear out the init_mm.pgd that will contain the kernel's mappings */

	for(i = 0; i < PTRS_PER_PGD; i++)
		swapper_pg_dir[i] = __pgd(0);
	
	/* make sure the current pgd table points to something sane
	 * (even if it is most probably not used until the next 
	 *  switch_mm)
	 */
         current_pgd = init_mm.pgd;
 
         /* initialise the TLB (tlb.c) */
         tlb_init();

	/* we've cleared all PGD entries up to KERNELBASE so we can
	 * continue at KERNELBASE, mapping the kernel
	 */
	end = (unsigned long)__va(max_low_pfn*PAGE_SIZE);

#ifdef CONFIG_GUARD_PROTECTED_CORE
	{
	        extern char _e_protected_core;
		unsigned long ro_end;

		ro_end = PAGE_MASK&(unsigned long)&(_e_protected_core);

		printk("write protecting ro sections (0x%08lx - 0x%08lx)\n", 
	               PAGE_OFFSET + PAGE_SIZE, ro_end);

		/* map the write protected part */		
		populate_pgd(swapper_pg_dir, PAGE_OFFSET + PAGE_SIZE, ro_end, PAGE_OFFSET,
			     __pgprot(_PAGE_ALL | _PAGE_SRE |_PAGE_SHARED | _PAGE_DIRTY | _PAGE_EXEC));

		/* map the rest of kernel */
		populate_pgd(swapper_pg_dir, ro_end, end,
			     PAGE_OFFSET,  PAGE_KERNEL);

	}
#else  /* CONFIG_GUARD_PROTECTED_CORE */
	populate_pgd(swapper_pg_dir, PAGE_OFFSET + PAGE_SIZE, end,
		     PAGE_OFFSET,  PAGE_KERNEL);
#endif /* CONFIG_GUARD_PROTECTED_CORE */

#ifdef CONFIG_AEONR2
	identical_mapping(UART_BASE_ADD, 0x10000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
	identical_mapping(0xa0000000, 0x10000, _PAGE_CI |
			  _PAGE_URE | _PAGE_UWE);
#else
	identical_mapping(0x90000000, 0x10000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
	identical_mapping(0x93000000, 0x2000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
	identical_mapping(0x97000000, 0x2000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
	identical_mapping(0x99000000, 0x1000000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
	identical_mapping(0xf0000000, 0x100000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
	identical_mapping(0x1e50000, 0x150000, _PAGE_CI | 
			  _PAGE_URE | _PAGE_UWE);
#endif

	zone_sizes_init();

	/*
	 * Fixed mappings, only the page table structure has to be
	 * created - mappings will be set by set_fixmap():
	 */
	addr = __fix_to_virt(__end_of_fixed_addresses - 1) & PMD_MASK;
	fixrange_init(addr, 0xffffffff, swapper_pg_dir);

#ifdef CONFIG_MMU
	{
		/*
		 * enable EA translations via PT mechanism
		 */
	
		int len = dtlb_miss_handlers[1].end - dtlb_miss_handlers[1].beg;
		BUG_ON(len > 0x100);

		__switch_pic_alternate(0xc0000900,
				       dtlb_miss_handlers[1].beg,
				       len,
				       1);

		len = itlb_miss_handlers[1].end - itlb_miss_handlers[1].beg;
		BUG_ON(len > 0x100);

		__switch_pic_alternate(0xc0000a00,
				       itlb_miss_handlers[1].beg,
				       len,
				       1);
	}
#endif
}

/* References to section boundaries */

extern char _stext, _etext, _edata, __bss_start, _end;
extern char __init_begin, __init_end;

int __init free_pages_init(void)
{
	int reservedpages, pfn;
	
	/* this will put all low memory onto the freelists */
	totalram_pages = free_all_bootmem();
	
	reservedpages = 0;
	for (pfn = 0; pfn < max_low_pfn; pfn++) {
		/*
		 * Only count reserved RAM pages
		 */
		if (PageReserved(pfn_to_page(pfn)))
			reservedpages++;
	}

	return reservedpages;
}

void __init set_max_mapnr_init(void)
{
        max_mapnr = num_physpages = max_low_pfn;
}

void __init mem_init(void)
{
	int codesize, reservedpages, datasize, initsize;

	phx_warn("mem_map %p", mem_map);
	if (!mem_map)
		BUG();
	
	set_max_mapnr_init();

	high_memory = (void *) __va(max_low_pfn * PAGE_SIZE);

	reservedpages = free_pages_init();
	
	codesize =  (unsigned long) &_etext - (unsigned long) &_stext;
	datasize =  (unsigned long) &_edata - (unsigned long) &_etext;
	initsize =  (unsigned long) &__init_end - (unsigned long) &__init_begin;

	printk(KERN_INFO "Memory: %luk/%luk available (%dk kernel code, %dk reserved, %dk data, %dk init, %ldk highmem)\n",
		(unsigned long) nr_free_pages() << (PAGE_SHIFT-10),
		max_mapnr << (PAGE_SHIFT-10),
		codesize >> 10,
		reservedpages << (PAGE_SHIFT-10),
		datasize >> 10,
		initsize >> 10,
		(unsigned long) (0 << (PAGE_SHIFT-10))
	       );

	mem_init_done = 1;
	return;
}


/*
 * Associate a virtual page frame with a given physical page frame 
 * and protection flags for that frame.
 */ 
void set_pte_pfn(unsigned long vaddr, unsigned long pfn, pgprot_t flags)
{
	pgd_t *pgd;
	pud_t *pud;
	pmd_t *pmd;
	pte_t *pte;

	pgd = swapper_pg_dir + pgd_index(vaddr);
	if (pgd_none(*pgd)) {
		BUG();
		return;
	}

	pud = pud_offset(pgd, vaddr);
	if (pud_none(*pud)) {
		BUG();
		return;
	}

	pmd = pmd_offset(pud, vaddr);
	if (pmd_none(*pmd)) {
		BUG();
		return;
	}

	pte = pte_offset_kernel(pmd, vaddr);
	/* <pfn,flags> stored as-is, to permit clearing entries */
	set_pte(pte, pfn_pte(pfn, flags));
}

void __set_fixmap (enum fixed_addresses idx, unsigned long phys, pgprot_t flags)
{
	unsigned long address = __fix_to_virt(idx);

	if (idx >= __end_of_fixed_addresses) {
		BUG();
		return;
	}
	set_pte_pfn(address, phys >> PAGE_SHIFT, flags);
	flush_tlb_all();
}

void free_init_pages(char *what, unsigned long begin, unsigned long end)
{
	unsigned long addr;

	for (addr = begin; addr < end; addr += PAGE_SIZE) {
		ClearPageReserved(virt_to_page(addr));
		init_page_count(virt_to_page(addr));
		memset((void *)addr, POISON_FREE_INITMEM, PAGE_SIZE);
		free_page(addr);
		totalram_pages++;
	}
	printk(KERN_INFO "Freeing %s: %luk freed\n", what, (end - begin) >> 10);
}

void free_initmem(void)
{
	free_init_pages("unused kernel memory",
			(unsigned long)(&__init_begin),
			(unsigned long)(&__init_end));
}

#ifdef CONFIG_BLK_DEV_INITRD
void free_initrd_mem(unsigned long start, unsigned long end)
{
	free_init_pages("initrd memory", start, end);
}
#endif
