/*
 *  linux/arch/ba/oprofile/init.c
 *
 *  BA version
 *    author(s): Gyorgy Jeney (nog@beyondsemi.com)
 *
 *  For more information about BA processors, licensing and
 *  design services you may contact Beyond Semiconductor at
 *  sales@beyondsemi.com or visit website http://www.beyondsemi.com.
 *
 *  derived from cris, i386, m68k, ppc, sh ports.
 *
 *  changes:
 *  04. 11. 2007: Gyorgy Jeney (nog@beyondsemi.com)
 *    initial port to the BA architecture
 *
 *  based on: linux/arch/i386/oprofile/init.c
 *  Copyright 2002 OProfile authors
 *  Author: John Levon
 *  Author: David Smith
 */

#include <linux/oprofile.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <asm/ptrace.h>
#include <asm/uaccess.h>
#include <asm/abi.h>

static unsigned long dump_kernel_backtrace(unsigned long fp)
{
	unsigned long addr;
	unsigned long next_fp;

	addr = *(unsigned long *)(fp - 4);
	oprofile_add_trace(addr);
	next_fp = *(unsigned long *)(fp - 8);
	if(fp > next_fp)
		return 0;

	return next_fp;
}

static unsigned long dump_user_backtrace(unsigned long fp)
{
	unsigned long bufhead[2];

	/* Also check accessibility of one struct frame_head beyond */
	if(!access_ok(VERIFY_READ, (void *)(fp - sizeof(bufhead)),
	              sizeof(bufhead)))
		return 0;
	if(__copy_from_user_inatomic(bufhead, (void *)(fp - sizeof(bufhead)),
	                             sizeof(bufhead)))
		return 0;

	oprofile_add_trace(bufhead[1]);

	/* frame pointers should strictly progress back up the stack
	 * (towards higher addresses) */
	if(fp >= bufhead[0])
		return 0;

	return bufhead[0];
}

#ifdef CONFIG_FRAME_POINTER
static int valid_kernel_stack(unsigned long fp, struct pt_regs * regs)
{
	unsigned long stack = (unsigned long)regs;
	unsigned long stack_base = (stack & ~(THREAD_SIZE - 1)) + THREAD_SIZE;

	return fp > stack && fp < stack_base;
}
#else
/* without fp, it's just junk */
static int valid_kernel_stack(unsigned long fp, struct pt_regs * regs)
{
	return 0;
}
#endif


void ba_backtrace(struct pt_regs * const regs, unsigned int depth)
{
	unsigned long fp;

	fp = regs->gprs[REG_FP_PT];

	if(!user_mode(regs)) {
		while(depth-- && valid_kernel_stack(fp, regs))
			fp = dump_kernel_backtrace(fp);
		return;
	}

	while(depth-- && fp)
		fp = dump_user_backtrace(fp);
}

