////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2007 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    mdrv_iic.c
/// @brief  IIC Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
//#include "MsCommon.h"
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>
#include <linux/timer.h>
#include <asm/io.h>

#include "mst_devid.h"

#include "mdrv_iic.h"
#include "mhal_iic_reg.h"
#include "mhal_iic.h"

//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define IIC_DBG_ENABLE			0

#if IIC_DBG_ENABLE
#define IIC_DBG(_f)				(_f)
#else
#define IIC_DBG(_f)
#endif

#define IIC_PRINT(fmt, args...)	//printk("[IIC][%05d] " fmt, __LINE__, ## args)

// for SW I2C
#define I2C_CH_MAX              8//3

#define _INPUT 					1
#define _OUTPUT					0
#define _HIGH  					1
#define _LOW   					0
#define SWIIC_READ 				0
#define SWIIC_WRITE				1
#define I2C_CHECK_PIN_DUMMY		200//100
#define I2C_ACKNOWLEDGE    		_LOW
#define I2C_NON_ACKNOWLEDGE		_HIGH
#define I2C_ACCESS_DUMMY_TIME	5//3

#define SWIIC_SCL_PIN(chNum, x) \
        ( MHal_GPIO_REG(_I2CBus[chNum].SclOenReg) = (x == _INPUT) ? \
         (MHal_GPIO_REG(_I2CBus[chNum].SclOenReg) |  _I2CBus[chNum].SclOenBit) : \
         (MHal_GPIO_REG(_I2CBus[chNum].SclOenReg) & ~_I2CBus[chNum].SclOenBit) )

#define SWIIC_SDA_PIN(chNum, x) \
        ( MHal_GPIO_REG(_I2CBus[chNum].SdaOenReg) = (x == _INPUT) ? \
         (MHal_GPIO_REG(_I2CBus[chNum].SdaOenReg) |  _I2CBus[chNum].SdaOenBit) : \
         (MHal_GPIO_REG(_I2CBus[chNum].SdaOenReg) & ~_I2CBus[chNum].SdaOenBit) )

#define SWIIC_SCL_OUT(chNum, x) \
        ( MHal_GPIO_REG(_I2CBus[chNum].SclOutReg) = (x == _HIGH) ? \
         (MHal_GPIO_REG(_I2CBus[chNum].SclOutReg) |  _I2CBus[chNum].SclOutBit) : \
         (MHal_GPIO_REG(_I2CBus[chNum].SclOutReg) & ~_I2CBus[chNum].SclOutBit) )

#define SWIIC_SDA_OUT(chNum, x) \
        ( MHal_GPIO_REG(_I2CBus[chNum].SdaOutReg) = (x == _HIGH) ? \
         (MHal_GPIO_REG(_I2CBus[chNum].SdaOutReg) |  _I2CBus[chNum].SdaOutBit) : \
         (MHal_GPIO_REG(_I2CBus[chNum].SdaOutReg) & ~_I2CBus[chNum].SdaOutBit) )

#define GET_SWIIC_SCL(chNum) \
        ( (MHal_GPIO_REG(_I2CBus[chNum].SclInReg) & _I2CBus[chNum].SclInBit) ? _HIGH : _LOW )

#define GET_SWIIC_SDA(chNum) \
        ( (MHal_GPIO_REG(_I2CBus[chNum].SdaInReg) & _I2CBus[chNum].SdaInBit) ? _HIGH : _LOW )

#define _SDA_HIGH(chNum)	SWIIC_SDA_PIN(chNum, _INPUT)
#define _SDA_LOW(chNum) 	do { SWIIC_SDA_OUT(chNum, _LOW); SWIIC_SDA_PIN(chNum, _OUTPUT); } while(0)

#define _SCL_HIGH(chNum)	SWIIC_SCL_PIN(chNum, _INPUT)
#define _SCL_LOW(chNum) 	do { SWIIC_SCL_OUT(chNum, _LOW); SWIIC_SCL_PIN(chNum, _OUTPUT); } while(0)

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
#if defined(CONFIG_MSTAR_TITANIA4)
IIC_Bus_t _I2CBus[] =
{
///*CH2*/{REG_DDCR_CK_OEN, BIT2, REG_DDCR_CK_OUT, BIT2, REG_DDCR_CK_IN, BIT2, \
//     REG_DDCR_DA_OEN, BIT1, REG_DDCR_DA_OUT, BIT1, REG_DDCR_DA_IN, BIT1, 2},

/*CH2*/{REG_DDCR_DA_OEN, BIT1, REG_DDCR_DA_OUT, BIT1, REG_DDCR_DA_IN, BIT1,\
     REG_DDCR_CK_OEN, BIT2, REG_DDCR_CK_OUT, BIT2, REG_DDCR_CK_IN, BIT2, 2},

/*CH3*/{REG_I2S_OUT_SD3_OEN, BIT3, REG_I2S_OUT_SD3_OUT, BIT3, REG_I2S_OUT_SD3_IN, BIT3, \
     REG_I2S_OUT_SD2_OEN, BIT2, REG_I2S_OUT_SD2_OUT, BIT2, REG_I2S_OUT_SD2_IN, BIT2, 2},

/*CH4*/{REG_I2S_OUT_SD1_OEN, BIT1, REG_I2S_OUT_SD1_OUT, BIT1, REG_I2S_OUT_SD1_IN, BIT1, \
     REG_SPDIF_IN_OEN, BIT3, REG_SPDIF_IN_OUT, BIT3, REG_SPDIF_IN_IN, BIT3, 2},


/*CH5*/{REG_TGPIO0_OEN, BIT5, REG_TGPIO0_OUT, BIT5, REG_TGPIO0_IN, BIT5, \
    REG_TGPIO1_OEN, BIT6, REG_TGPIO1_OUT, BIT6, REG_TGPIO1_IN, BIT6, 2},

/*CH6*/{REG_TGPIO2_OEN, BIT2, REG_TGPIO2_OUT, BIT2, REG_TGPIO2_IN, BIT2, \
    REG_TGPIO3_OEN, BIT3, REG_TGPIO3_OUT, BIT3, REG_TGPIO3_IN, BIT3, 2},


/*CH7*/{REG_I2S_IN_WS_OEN, BIT0, REG_I2S_IN_WS_OUT, BIT0, REG_I2S_IN_WS_IN, BIT0, \
     REG_I2S_IN_BCK_OEN, BIT1, REG_I2S_IN_BCK_OUT, BIT1, REG_I2S_IN_BCK_IN, BIT1, 2},

/*CH8 : RGB - EDID */
    {REG_DDCA_CK_OEN, BIT1, REG_DDCA_CK_OUT, BIT2, REG_DDCA_CK_IN, BIT0, \
     REG_DDCA_DA_OEN, BIT5, REG_DDCA_DA_OUT, BIT6, REG_DDCA_DA_IN, BIT4, 2},

/*CH9 : HDMI port A - EDID */
    {REG_DDCDA_CK_OEN, BIT1, REG_DDCDA_CK_OUT, BIT2, REG_DDCDA_CK_IN, BIT0, \
     REG_DDCDA_DA_OEN, BIT5, REG_DDCDA_DA_OUT, BIT6, REG_DDCDA_DA_IN, BIT4, 2},
/*CH10 : HDMI port B - EDID */
    {REG_DDCDB_CK_OEN, BIT1, REG_DDCDB_CK_OUT, BIT2, REG_DDCDB_CK_IN, BIT0, \
     REG_DDCDB_DA_OEN, BIT5, REG_DDCDB_DA_OUT, BIT6, REG_DDCDB_DA_IN, BIT4, 2},
/*CH11 : HDMI port C - EDID */
    {REG_DDCDC_CK_OEN, BIT1, REG_DDCDC_CK_OUT, BIT2, REG_DDCDC_CK_IN, BIT0, \
     REG_DDCDC_DA_OEN, BIT5, REG_DDCDC_DA_OUT, BIT6, REG_DDCDC_DA_IN, BIT4, 2},

#if 0
/*CH12 : HDMI port C - EDID */
    {REG_DDCDD_CK_OEN, BIT1, REG_DDCDD_CK_OUT, BIT2, REG_DDCDD_CK_IN, BIT0, \
     REG_DDCDD_DA_OEN, BIT5, REG_DDCDD_DA_OUT, BIT6, REG_DDCDD_DA_IN, BIT4, 5},
#endif

/*CH12 : HDMI port D - EDID */
    {REG_DDCDD_CK_OEN, BIT1, REG_DDCDD_CK_OUT, BIT2, REG_DDCDD_CK_IN, BIT0, \
     REG_DDCDD_DA_OEN, BIT5, REG_DDCDD_DA_OUT, BIT6, REG_DDCDD_DA_IN, BIT4, 2},


#if 0  //2
/*CH13 : NEC */
    {REG_GPIO24_OEN, BIT0, REG_GPIO24_OUT, BIT0, REG_GPIO24_IN, BIT0, \
     REG_GPIO25_OEN, BIT1, REG_GPIO25_OUT, BIT1, REG_GPIO25_IN, BIT1, 5},
#endif

};

#else
IIC_Bus_t _I2CBus[I2C_CH_MAX] =
{
    //channel 0
    {REG_DDCR_CK2_OEN, BIT1, REG_DDCR_CK2_OUT, BIT1, REG_DDCR_CK2_IN, BIT1, \
     REG_DDCR_DA2_OEN, BIT0, REG_DDCR_DA2_OUT, BIT0, REG_DDCR_DA2_IN, BIT0, 5},
    //channel 1
    {REG_LCK_OEN, BIT2, REG_LCK_OUT, BIT3, REG_LCK_IN, BIT3, \
     REG_LDE_OEN, BIT3, REG_LDE_OUT, BIT2, REG_LDE_IN, BIT2, 5},
    //channel 2
    {REG_I2S_IN_SD_OEN, BIT2, REG_I2S_IN_SD_OUT, BIT2, REG_I2S_IN_SD_IN, BIT2, \
     REG_I2S_IN_BCK_OEN, BIT1, REG_I2S_IN_BCK_OUT, BIT1, REG_I2S_IN_BCK_IN, BIT1, 5},
    //channel 3
    {REG_DDCROM_CK_OEN, BIT1, REG_DDCROM_CK_OUT, BIT2, REG_DDCROM_CK_IN, BIT0, \
     REG_DDCROM_DA_OEN, BIT5, REG_DDCROM_DA_OUT, BIT6, REG_DDCROM_DA_IN, BIT4, 5},
    //channel 4
    {REG_DDCA_CK_OEN, BIT1, REG_DDCA_CK_OUT, BIT2, REG_DDCA_CK_IN, BIT0, \
     REG_DDCA_DA_OEN, BIT5, REG_DDCA_DA_OUT, BIT6, REG_DDCA_DA_IN, BIT4, 5},
    //channel 5
    {REG_DDCDA_CK_OEN, BIT1, REG_DDCDA_CK_OUT, BIT2, REG_DDCDA_CK_IN, BIT0, \
     REG_DDCDA_DA_OEN, BIT5, REG_DDCDA_DA_OUT, BIT6, REG_DDCDA_DA_IN, BIT4, 5},
    //channel 6
    {REG_DDCDB_CK_OEN, BIT1, REG_DDCDB_CK_OUT, BIT2, REG_DDCDB_CK_IN, BIT0, \
     REG_DDCDB_DA_OEN, BIT5, REG_DDCDB_DA_OUT, BIT6, REG_DDCDB_DA_IN, BIT4, 5},
    //channel 7
    {REG_DDCDC_CK_OEN, BIT1, REG_DDCDC_CK_OUT, BIT2, REG_DDCDC_CK_IN, BIT0, \
     REG_DDCDC_DA_OEN, BIT5, REG_DDCDC_DA_OUT, BIT6, REG_DDCDC_DA_IN, BIT4, 5},
};
#endif
static U16 _u16SwIicDly[I2C_CH_MAX];
#if 0//PIU Timer solution
static U16 u16TimerLastCount;
#endif

//--------------------------------------------------------------------------------------------------
// Forward declaration
//--------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
#define NEW_DELAY_FORMULA   1//alex_tung@@@
#define EXTRA_DELAY_CYCLE   1//alex_tung@@@
#if (NEW_DELAY_FORMULA)
#define FACTOR_DLY    69000UL
#define FACTOR_ADJ    11040UL
static U32 u32CpuSpeedMHz;
static U32 u32AdjParam;
static U16 _u16SwIicDly[I2C_CH_MAX];
#define DELAY_CYCLES(SpeedKHz)  ((FACTOR_DLY/(SpeedKHz))-((130+u32AdjParam)-((SpeedKHz)/u32AdjParam))+((1<<((440-SpeedKHz)/40))))
#endif


void MDrv_SW_IIC_SetSpeed(U8 u8ChIIC, U8 u8Speed)
{
    if(u8ChIIC>=I2C_CH_MAX)
        return;


#if (NEW_DELAY_FORMULA)
    switch(u8ChIIC)
    {
        case 2:
            _u16SwIicDly[u8ChIIC]=400;//KHz
            break;
        case 3:
        case 4:
        case 5:
            _u16SwIicDly[u8ChIIC]=100;//KHz
            break;
        case 6:
            _u16SwIicDly[u8ChIIC]=50;//KHz
            break;
        default:
            _u16SwIicDly[u8ChIIC]=20;//KHz
            break;
    }
#endif


}

void MDrv_SW_IIC_Delay(U8 u8ChIIC)
{
#if (NEW_DELAY_FORMULA)
    __asm__ __volatile__ (
        ".set noreorder\n"
        "1:"
        "\tbgtz %0,1b\n"
        "\taddiu %0,%0,-1\n"
        : : "r" (DELAY_CYCLES(_u16SwIicDly[u8ChIIC])));
#endif
}

#if 0//PIU Timer solution
#define REG_TIMER_MAX1      (0x1E50<<2)
#define REG_TIMER_CNT1      (0x1E52<<2)
#define REG_TIMER_CTL10     (0x1E54<<2)     //[0]: capture, [1]: clear
#define REG_TIMER_CTL11     ((0x1E54<<2)+1) //[1:0]=2'b11: Enable, [1:0]=2'b00: Disable
#define TIMER_EN            (BIT1|BIT0)
#define TIMER_CLR           (BIT1)
#define TIMER_CAP           (BIT0)
#define PIU_XTAL_CLOCK_KHZ  12000UL
#define MCU_CLOCK_KHZ       216000UL
void MDrv_SW_IIC_Delay(U8 u8ChIIC)
{
    static U16 u16TimerExpCount;
    register U16 u16TimerTmpCount;
    register int i;

    //u16TimerExpCount = (PIU_XTAL_CLOCK_KHZ/800) - 8;//100;
    //printk("[IIC]: u16TimerExpCount=%d\n", u16TimerExpCount);
    //MHal_PIU_REG(REG_TIMER_CTL10) &= ~TIMER_CAP;
    while(1)
    {
        MHal_PIU_REG(REG_TIMER_CTL10) = TIMER_CAP;

        __asm__ __volatile__ (
            ".set noreorder\n"
            "1:"
            "\tbgtz %0,1b\n"
            "\taddiu %0,%0,-1\n"
            : : "r" ((MCU_CLOCK_KHZ / PIU_XTAL_CLOCK_KHZ) * 2));

        u16TimerTmpCount = (*(volatile U16*)(REG_MIPS_BASE + (REG_TIMER_CNT1)));//MHal_PIU_REG(REG_TIMER_CNT1);
        //if(u16TimerTmpCount==u16TimerLastCount)
        //    continue;

        if(u16TimerTmpCount>=((PIU_XTAL_CLOCK_KHZ/800) - 8))
        {
            MHal_PIU_REG(REG_TIMER_CTL10) = TIMER_CLR;
            //u16TimerLastCount = u16TimerTmpCount;
            break;
        }
    }
}
#endif

#if 0
void MDrv_SW_IIC_Delay(U8 u8ChIIC)
{
    U8 u8I;
    U8 u8Loop = 5;

    if(u8ChIIC>=I2C_CH_MAX)
        return;

    u8Loop = _u16SwIicDly[u8ChIIC];
    if(u8Loop<2)
        u8Loop = 2;
    //for fast mode
    if (u8Loop == 2)
        for (u8I=0;u8I<17;u8I++)
            ;
    else
    //for normal and slow mode
    {
        u8Loop = (u8Loop - 2) * 2;
        while (u8Loop-- > 0)
        {
         udelay(1);
        }
    }
}
#endif

void MDrv_SW_IIC_SCL(U8 u8ChIIC, U8 u8Data)
{
    if(u8ChIIC>=I2C_CH_MAX)
        return;

    if ( u8Data == _HIGH )
    {
        SWIIC_SCL_PIN(u8ChIIC, _INPUT);  //set to input
    }
    else
    {
        SWIIC_SCL_OUT(u8ChIIC, _LOW);
        SWIIC_SCL_PIN(u8ChIIC, _OUTPUT);
    }
}

void MDrv_SW_IIC_SDA(U8 u8ChIIC, U8 u8Data)
{
    if(u8ChIIC>=I2C_CH_MAX)
        return;

    if ( u8Data == _HIGH )
    {
        SWIIC_SDA_PIN(u8ChIIC, _INPUT);  //set to input
    }
    else
    {
        SWIIC_SDA_OUT(u8ChIIC, _LOW);
        SWIIC_SDA_PIN(u8ChIIC, _OUTPUT); //set to output
    }
}

void MDrv_SW_IIC_SCL_Chk(U8 u8ChIIC, B16 bSet)
{
    U8 u8Dummy;       // loop dummy

    if(u8ChIIC>=I2C_CH_MAX)
        return;

    if (bSet == _HIGH)  // if set pin high
    {
        SWIIC_SCL_PIN(u8ChIIC, _INPUT);
        u8Dummy = I2C_CHECK_PIN_DUMMY; // initialize dummy
        while ((GET_SWIIC_SCL(u8ChIIC) == _LOW) && (u8Dummy--)) ; // check SCL pull high
    }
    else
    {
        MDrv_SW_IIC_SCL(u8ChIIC, _LOW);    // set SCL pin
        SWIIC_SCL_PIN(u8ChIIC, _OUTPUT);
    }
}

void MDrv_SW_IIC_SDA_Chk(U8 u8ChIIC, B16 bSet)
{
    U8 u8Dummy;       // loop dummy

    if(u8ChIIC>=I2C_CH_MAX)
        return;

    if (bSet == _HIGH)  // if set pin high
    {
        SWIIC_SDA_PIN(u8ChIIC, _INPUT);
        u8Dummy = I2C_CHECK_PIN_DUMMY; // initialize dummy
        while ((GET_SWIIC_SDA(u8ChIIC) == _LOW) && (u8Dummy--)) ; // check SDA pull high
    }
    else
    {
        MDrv_SW_IIC_SDA(u8ChIIC, _LOW);    // set SDA pin
        SWIIC_SDA_PIN(u8ChIIC, _OUTPUT);
    }
}

//-------------------------------------------------------------------------------------------------
// SW I2C: start signal.
// <comment>
//  SCL ________
//              \_________
//  SDA _____
//           \____________
//
// Return value: None
//-------------------------------------------------------------------------------------------------
B16 MDrv_SW_IIC_Start(U8 u8ChIIC)
{
    B16 bStatus = TRUE;    // success status

    if(u8ChIIC>=I2C_CH_MAX)
        return FALSE;

    #if 0//PIU Timer solution
    //MHal_PIU_REG(REG_TIMER_CTL10) = TIMER_CLR;
    #endif

    MDrv_SW_IIC_SDA_Chk(u8ChIIC, _HIGH);
    MDrv_SW_IIC_Delay(u8ChIIC);

    MDrv_SW_IIC_SCL_Chk(u8ChIIC, _HIGH);
    MDrv_SW_IIC_Delay(u8ChIIC);

    // check pin error
	SWIIC_SCL_PIN(u8ChIIC, _INPUT);
 	SWIIC_SDA_PIN(u8ChIIC, _INPUT);


    if ((GET_SWIIC_SCL(u8ChIIC) == _LOW) || (GET_SWIIC_SDA(u8ChIIC) == _LOW))
    {
   		SWIIC_SCL_PIN(u8ChIIC, _OUTPUT);
   		SWIIC_SDA_PIN(u8ChIIC, _OUTPUT);
        bStatus = FALSE;
    }
    else // success
    {
        MDrv_SW_IIC_SDA(u8ChIIC, _LOW);
        MDrv_SW_IIC_Delay(u8ChIIC);
        MDrv_SW_IIC_SCL(u8ChIIC, _LOW);
    }

    return bStatus;	 //vain
}

//-------------------------------------------------------------------------------------------------
// SW I2C: stop signal.
// <comment>
//              ____________
//  SCL _______/
//                 _________
//  SDA __________/
//-------------------------------------------------------------------------------------------------
void MDrv_SW_IIC_Stop(U8 u8ChIIC)
{
    if(u8ChIIC>=I2C_CH_MAX)
        return;

	_SCL_LOW(u8ChIIC);

    MDrv_SW_IIC_Delay(u8ChIIC);
    _SDA_LOW(u8ChIIC);

    MDrv_SW_IIC_Delay(u8ChIIC);
    _SCL_HIGH(u8ChIIC);
    MDrv_SW_IIC_Delay(u8ChIIC);
    _SDA_HIGH(u8ChIIC);
    MDrv_SW_IIC_Delay(u8ChIIC);
}

//-------------------------------------------------------------------------------------------------
///SW I2C: Send 1 bytes data
///@param u8data \b IN: 1 byte data to send
//-------------------------------------------------------------------------------------------------
B16 MDrv_IIC_SendByte(U8 u8ChIIC, U8 u8data)   // Be used int IIC_SendByte
{
    U8      u8Mask = 0x80;
    B16     bAck; // acknowledge bit

    if(u8ChIIC>=I2C_CH_MAX)
        return I2C_NON_ACKNOWLEDGE;

    while ( u8Mask )
    {
        if (u8data & u8Mask)
        {
            MDrv_SW_IIC_SDA_Chk(u8ChIIC, _HIGH);
        }
        else
        {
            MDrv_SW_IIC_SDA_Chk(u8ChIIC, _LOW);
        }

        MDrv_SW_IIC_Delay(u8ChIIC);
        MDrv_SW_IIC_SCL_Chk(u8ChIIC, _HIGH); // clock
        MDrv_SW_IIC_Delay(u8ChIIC);
        MDrv_SW_IIC_SCL(u8ChIIC, _LOW);

        //MDrv_SW_IIC_Delay(u8ChIIC);

        u8Mask >>= 1; // next
    }

    // recieve acknowledge
    SWIIC_SDA_PIN(u8ChIIC, _INPUT);
    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_SCL_Chk(u8ChIIC, _HIGH);

    MDrv_SW_IIC_Delay(u8ChIIC);
    bAck = GET_SWIIC_SDA(u8ChIIC); // recieve acknowlege
//	SWIIC_SDA(u8ChIIC, bAck);     //for I2c waveform sharp
//	SWIIC_SDA_PIN(u8ChIIC, _OUTPUT);
    MDrv_SW_IIC_SCL(u8ChIIC, _LOW);

    MDrv_SW_IIC_Delay(u8ChIIC);

    MDrv_SW_IIC_SDA(u8ChIIC, bAck);     //for I2c waveform sharp
    SWIIC_SDA_PIN(u8ChIIC, _OUTPUT);

    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_Delay(u8ChIIC);

    return (bAck);
}

//-------------------------------------------------------------------------------------------------
///SW I2C: Send 1 bytes data, this function will retry 5 times until success.
///@param u8data \b IN: 1 byte data to send
///@return BOOLEAN:
///- TRUE: Success
///- FALSE: Fail
//-------------------------------------------------------------------------------------------------
B16 MDrv_SW_IIC_SendByte(U8 u8ChIIC, U8 u8data)
{
	U8 u8I;

    if(u8ChIIC>=I2C_CH_MAX)
        return FALSE;

	for(u8I=0;u8I<5;u8I++)
	{
	    if (MDrv_IIC_SendByte(u8ChIIC, u8data) == I2C_ACKNOWLEDGE)
		    return TRUE;
	}

	//printk("IIC write byte 0x%02x fail!!\n", u8data);
	return FALSE;
}

//-------------------------------------------------------------------------------------------------
// SW I2C: access start.
//
// Arguments: u8SlaveAdr - slave address
//            u8Trans - I2C_TRANS_WRITE/I2C_TRANS_READ
//-------------------------------------------------------------------------------------------------
B16 MDrv_SW_IIC_AccessStart(U8 u8ChIIC, U8 u8SlaveAdr, U8 u8Trans)
{
    U8 u8Dummy; // loop dummy

    if(u8ChIIC>=I2C_CH_MAX)
        return FALSE;

    if (u8Trans == SWIIC_READ) // check i2c read or write
    {
        u8SlaveAdr = u8SlaveAdr | 0x01; // read
    }
    else
    {
        u8SlaveAdr = u8SlaveAdr & 0xfe; // write
    }

    u8Dummy = I2C_ACCESS_DUMMY_TIME;

    while (u8Dummy--)
    {
        if ( MDrv_SW_IIC_Start(u8ChIIC) == FALSE)
        {
            printk("MDrv_SW_IIC_Start=>Failed\n");
            continue;
        }

        if ( MDrv_SW_IIC_SendByte(u8ChIIC, u8SlaveAdr) == TRUE )   //I2C_ACKNOWLEDGE) // check acknowledge
        {
            return TRUE;
        }

        MDrv_SW_IIC_Stop(u8ChIIC);
    }

    return FALSE;
}

//-------------------------------------------------------------------------------------------------
///SW I2C: Get 1 bytes data
///@param u16Ack  \b IN: acknowledge
///@return U8:    \b OUT: get data from the device
///- TRUE: Success
///- FALSE: Fail
//-------------------------------------------------------------------------------------------------
U8 MDrv_SW_IIC_GetByte (U8 u8ChIIC, U16 u16Ack)
{
    U8 u8Receive = 0;
    U8 u8Mask = 0x80;

    if(u8ChIIC>=I2C_CH_MAX)
        return 0;

    SWIIC_SDA_PIN(u8ChIIC, _INPUT);		  //SWIIC_SDA_PIN

    while ( u8Mask )
    {
        MDrv_SW_IIC_Delay(u8ChIIC);
        MDrv_SW_IIC_SCL_Chk(u8ChIIC, _HIGH);		//SWIIC_SCL_Chk
        MDrv_SW_IIC_Delay(u8ChIIC);

        if (GET_SWIIC_SDA(u8ChIIC) == _HIGH)	  //GET_SWIIC_SDA
        {
            u8Receive |= u8Mask;
        }
        u8Mask >>= 1; // next

        MDrv_SW_IIC_SCL(u8ChIIC, _LOW);	  //SWIIC_SCL
        //SWIIC_Delay();

    }
    if (u16Ack)
    {
        // acknowledge
        MDrv_SW_IIC_SDA_Chk(u8ChIIC, I2C_ACKNOWLEDGE);	  //SWIIC_SDA_Chk
    }
    else
    {
        // non-acknowledge
        MDrv_SW_IIC_SDA_Chk(u8ChIIC, I2C_NON_ACKNOWLEDGE);
    }
    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_SCL_Chk(u8ChIIC, _HIGH);	//SWIIC_SCL_Chk
    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_SCL(u8ChIIC, _LOW);		 //SWIIC_SCL
    MDrv_SW_IIC_Delay(u8ChIIC);


    MDrv_SW_IIC_Delay(u8ChIIC);
    MDrv_SW_IIC_Delay(u8ChIIC);

    return u8Receive;
}

//-------------------------------------------------------------------------------------------------
///SW I2C: Write bytes, be able to write 1 byte or several bytes to several register offsets in same slave address.
///@param u8SlaveID \b IN: Slave ID (Address)
///@param u8AddrCnt \b IN:  register NO to write, this parameter is the NO of register offsets in pu8addr buffer,
///it should be 0 when *pu8Addr = NULL.
///@param *pu8Addr \b IN: pointer to a buffer containing target register offsets to write
///@param u32BufLen \b IN: Data length (in byte) to write
///@param *pu8Buf \b IN: pointer to the data buffer for write
///@return BOOLEAN:
///- TRUE: Success
///- FALSE: Fail
//-------------------------------------------------------------------------------------------------
S32 MDrv_SW_IIC_Write(U8 u8ChIIC, U8 u8SlaveID, U8 u8AddrCnt, U8* pu8Addr, U32 u32BufLen, U8* pu8Buf)
{
    U8  u8Dummy; // loop dummy
    S32 s32RetCountIIC;

    if(u8ChIIC>=I2C_CH_MAX)
        return 0;

    u8Dummy = 1;  //I2C_ACCESS_DUMMY_TIME;
    s32RetCountIIC = u32BufLen;

    while (u8Dummy--)
    {
        if (MDrv_SW_IIC_AccessStart(u8ChIIC, u8SlaveID, SWIIC_WRITE) == FALSE)
        {
            s32RetCountIIC = -2;
            goto SW_IIC_Write_End;
        }

		while( u8AddrCnt )
		{
			  u8AddrCnt--;
			  if ( MDrv_SW_IIC_SendByte(u8ChIIC, *pu8Addr) == FALSE )
              {
			    s32RetCountIIC = -3;
                goto SW_IIC_Write_End;
              }
			  pu8Addr++;
		}
        while (u32BufLen) // loop of writting data
        {
            u32BufLen-- ;
            //IIC_SendByte(*pu8Buf); // send byte
            if ( MDrv_SW_IIC_SendByte(u8ChIIC, *pu8Buf) == FALSE )
            {
			    s32RetCountIIC = -4;
                goto SW_IIC_Write_End;
            }
            pu8Buf++; // next byte pointer
        }

        break;
    }

SW_IIC_Write_End:
    MDrv_SW_IIC_Stop(u8ChIIC);

    return s32RetCountIIC;
}


//-------------------------------------------------------------------------------------------------
///SW I2C: Read bytes, be able to read 1 byte or several bytes from several register offsets in same slave address.
///@param u8SlaveID \b IN: Slave ID (Address)
///@param u8AddrCnt \b IN:  register NO to read, this parameter is the NO of register offsets in pu8addr buffer,
///it should be 0 when *paddr = NULL.
///@param *pu8Addr \b IN: pointer to a buffer containing target register offsets to read
///@param u32BufLen \b IN: Data length (in byte) to read
///@param *pu8Buf \b IN: pointer to retun data buffer.
///@return BOOLEAN:
///- TRUE: Success
///- FALSE: Fail
//-------------------------------------------------------------------------------------------------
S32 MDrv_SW_IIC_Read(U8 u8ChIIC, U8 u8SlaveID, U8 u8AddrCnt, U8* pu8Addr, U32 u32BufLen, U8* pu8Buf)
{
    U8  u8Dummy; // loop dummy
    S32 s32RetCountIIC;

    if(u8ChIIC>=I2C_CH_MAX)
        return 0;

    u8Dummy = I2C_ACCESS_DUMMY_TIME;
    s32RetCountIIC = u32BufLen;

    while (u8Dummy--)
    {
        /*	20080812:
        	fixed:	Don't make a I2C's write start condition when u8AddrCnt == 0
        */
        if( u8AddrCnt > 0 )
        {
            if (MDrv_SW_IIC_AccessStart(u8ChIIC, u8SlaveID, SWIIC_WRITE) == FALSE)
            {
                s32RetCountIIC = -2;
                goto SW_IIC_Read_End;
            }

            while( u8AddrCnt )
            {
                u8AddrCnt--;
                if (MDrv_SW_IIC_SendByte(u8ChIIC, *pu8Addr) == FALSE)
                {
                    s32RetCountIIC = -3;
                    goto SW_IIC_Read_End;
                }
                pu8Addr++;
            }
        }

        if (MDrv_SW_IIC_AccessStart(u8ChIIC, u8SlaveID, SWIIC_READ) == FALSE)
        {
            s32RetCountIIC = -4;
            goto SW_IIC_Read_End;
        }

        while (u32BufLen--) // loop to burst read
        {
            *pu8Buf = MDrv_SW_IIC_GetByte(u8ChIIC, u32BufLen); // receive byte

            pu8Buf++; // next byte pointer
        }

        break;
    }

SW_IIC_Read_End:
    MDrv_SW_IIC_Stop(u8ChIIC);

    return s32RetCountIIC;
}



//-------------------------------------------------------------------------------------------------
/// IIC master initialization
/// @return None
/// @note   Hardware IIC. Called only once at system initialization
//-------------------------------------------------------------------------------------------------
#if defined(CONFIG_MSTAR_TITANIA4)

void MDrv_IIC_Init(void)
{
    MHal_IIC_Init();

    //set all pads(except SPI) as input
    MHal_GPIO_REG(REG_ALL_PAD_IN) &= ~BIT7;
#if 0
    MHal_GPIO_REG(REG_DDCR_CK_SET1)  &= ~(BIT6|BIT5|BIT4);
    MHal_GPIO_REG(REG_DDCR_CK_SET2) &= ~(BIT1|BIT0);
    MHal_GPIO_REG(REG_DDCR_CK_SET3) |= (BIT7);

    MHal_GPIO_REG(REG_TEST_MODE)  &= ~(BIT2|BIT1|BIT0);
    MHal_GPIO_REG(REG_TEST_MODE)  &= ~(BIT6|BIT5|BIT4);

    MHal_GPIO_REG(REG_I2S_IN_WS_SET2) &= ~(BIT5|BIT4);
    MHal_GPIO_REG(REG_I2S_IN_WS_SET3) |= (BIT5);
    MHal_GPIO_REG(REG_I2S_IN_WS_SET3) &=~ (BIT6);

    MHal_GPIO_REG(REG_I2S_IN_BCK_SET2) |= (BIT5);

    MHal_GPIO_REG(REG_SPDIF_IN_SET) |= BIT6;

    //MHal_GPIO_REG(REG_I2S_OUT_SD1_SET) |= (BIT6);
    //MHal_GPIO_REG(REG_I2S_OUT_SD2_SET) |= (BIT6);
    //MHal_GPIO_REG(REG_I2S_OUT_SD3_SET) |= (BIT6);

    MHal_GPIO_REG(REG_TGPIO0_SET1) &= ~(BIT7|BIT6);
    MHal_GPIO_REG(REG_TGPIO0_SET2) &= ~(BIT6);;
    MHal_GPIO_REG(REG_TGPIO1_SET) &= ~(BIT7|BIT6);

    MHal_GPIO_REG(REG_TGPIO2_SET2) &= ~(BIT1|BIT0);
    MHal_GPIO_REG(REG_TGPIO3_SET2) &= ~(BIT1|BIT0);

    MHal_GPIO_REG(REG_DDCDA_CK_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDA_DA_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDB_DA_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDB_CK_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDC_CK_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDC_DA_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDD_CK_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_DDCDD_DA_SET) &= ~BIT7;
    MHal_GPIO_REG(REG_LHSYNC2_SET) &= ~(BIT7|BIT6);
    MHal_GPIO_REG(REG_SAR3_OEN) |= BIT3;
#endif
#if (NEW_DELAY_FORMULA)
{
    extern unsigned int Chip_Query_MIPS_CLK(void);
    U8 chidx;

    u32CpuSpeedMHz = Chip_Query_MIPS_CLK()/1000000UL;
    u32AdjParam = FACTOR_ADJ/u32CpuSpeedMHz;
    printk("@@@@@@ u32CpuSpeedMHz= %d MHz\n",u32CpuSpeedMHz);
    printk("@@@@@@ u32AdjParam= %d MHz\n",u32AdjParam);
    for(chidx=0;chidx<I2C_CH_MAX;chidx++)
    {
        _u16SwIicDly[chidx] = 100; //100KHz
    }
}
#endif
}
#else
void MDrv_IIC_Init(void)
{
    int chidx;

    MHal_IIC_Init();

    //set all pads(except SPI) as input
    MHal_GPIO_REG(REG_ALL_PAD_IN) &= ~BIT7;

    //for channel 0 set
    //REG_DDCR_CK2_SET
    //REG_DDCR_DA2_SET
    //for channel 1 set
    MHal_GPIO_REG(REG_LCK_SET) |= BIT3;
    MHal_GPIO_REG(REG_LDE_SET) |= BIT2;
    //for channel 2 set
    MHal_GPIO_REG(REG_I2S_IN_SD_SET) |= BIT4;
    MHal_GPIO_REG(REG_I2S_IN_BCK_SET) |= BIT4;
    //for channel 3 set
    MHal_GPIO_REG(REG_DDCROM_CK_SET) |= BIT7;
    MHal_GPIO_REG(REG_DDCROM_DA_SET) |= BIT7;
    //for channel 4 set
    MHal_GPIO_REG(REG_DDCA_CK_SET) |= BIT7;
    MHal_GPIO_REG(REG_DDCA_DA_SET) |= BIT7;
    //for channel 5 set
    MHal_GPIO_REG(REG_DDCDA_CK_SET) |= BIT7;
    MHal_GPIO_REG(REG_DDCDA_DA_SET) |= BIT7;
    //for channel 6 set
    MHal_GPIO_REG(REG_DDCDB_CK_SET) |= BIT7;
    MHal_GPIO_REG(REG_DDCDB_DA_SET) |= BIT7;
    //for channel 7 set
    MHal_GPIO_REG(REG_DDCDC_CK_SET) |= BIT7;
    MHal_GPIO_REG(REG_DDCDC_DA_SET) |= BIT7;

    for(chidx=0;chidx<I2C_CH_MAX;chidx++)
    {
        _u16SwIicDly[chidx] = 100; //100KHz //_I2CBus[chidx].DefDelay;
    }
    #if 0//PIU Timer solution
    MHal_PIU_REG(REG_TIMER_MAX1)=0;
    MHal_PIU_REG(REG_TIMER_MAX1+1)=0;

    //enable PIU timer1 for I2C speed control
    MHal_PIU_REG(REG_TIMER_CTL11) = TIMER_EN;
    u16TimerLastCount=MHal_PIU_REG(REG_TIMER_CNT1);
    #endif
}
#endif
//-------------------------------------------------------------------------------------------------
/// IIC clock selection
/// @param  u8ClockIIC            \b IN:  clock selection
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_HW_IIC_Clock_Select(U8 u8ClockIIC)
{
    MHal_IIC_Clock_Select(u8ClockIIC);
}

//-------------------------------------------------------------------------------------------------
/// Write data to an IIC device
/// @param  u8SlaveIdIIC            \b IN:  device slave ID
/// @param  u8AddrSizeIIC           \b IN:  address length in bytes
/// @param  pu8AddrIIC              \b IN:  pointer to the start address of the device
/// @param  u32BufSizeIIC           \b IN:  number of bytes to be written
/// @param  pu8BufIIC               \b IN:  pointer to write data buffer
/// @return TRUE(succeed), FALSE(fail)
/// @note   Not allowed in interrupt context
//-------------------------------------------------------------------------------------------------
S32 MDrv_HW_IIC_Write(U8 u8SlaveIdIIC, U8 u8AddrSizeIIC, U8 *pu8AddrIIC, U32 u32BufSizeIIC, U8 *pu8BufIIC)
{
    U32     u32I;
    U8      u8I;
    B16     bReturnIIC = TRUE;
    S32     s32RetCountIIC = -1;
    MHal_IIC_Start();

    if (FALSE == MHal_IIC_SendByte(u8SlaveIdIIC&0xFE))
    {
        bReturnIIC = FALSE;
        s32RetCountIIC = -2;
        goto HW_IIC_Write_End;
    }

    for (u8I = 0; u8I < u8AddrSizeIIC; u8I++)
    {
        if (FALSE == MHal_IIC_SendByte(pu8AddrIIC[u8I]))
        {
            bReturnIIC = FALSE;
            s32RetCountIIC = -3;
            goto HW_IIC_Write_End;
        }
    }

    for (u32I = 0; u32I < u32BufSizeIIC; u32I++)
    {
        if (FALSE == MHal_IIC_SendByte(pu8BufIIC[u32I]))
        {
            bReturnIIC = FALSE;
            s32RetCountIIC = -4;
            goto HW_IIC_Write_End;
        }
    }

    s32RetCountIIC = u32BufSizeIIC;

HW_IIC_Write_End:
    MHal_IIC_Stop();

    IIC_DBG(printk("MDrv_IIC_Write() --> s32RetCountIIC=%d \n", s32RetCountIIC));
    return s32RetCountIIC;
}


//-------------------------------------------------------------------------------------------------
/// Read data from an IIC device
/// @param  u8SlaveIdIIC            \b IN:  device slave ID
/// @param  u8AddrSizeIIC           \b IN:  address length in bytes
/// @param  pu8AddrIIC              \b IN:  ptr to the start address inside the device
/// @param  u32BufSizeIIC           \b IN:  number of bytes to be read
/// @param  pu8BufIIC               \b OUT: pointer to read data buffer
/// @return TRUE : succeed
/// @return FALSE : fail
/// @note   Not allowed in interrupt context
//-------------------------------------------------------------------------------------------------
S32 MDrv_HW_IIC_Read(U8 u8SlaveIdIIC, U8 u8AddrSizeIIC, U8 *pu8AddrIIC, U32 u32BufSizeIIC, U8 *pu8BufIIC)
{
    U32     u32I;
    U8      u8I;
    B16     bReturnIIC = TRUE;
    S32     s32RetCountIIC = -1;

    MHal_IIC_Start();

    if (FALSE == MHal_IIC_SendByte(u8SlaveIdIIC&0xFE))
    {
        bReturnIIC = FALSE;
        s32RetCountIIC = -2;
        goto HW_IIC_Read_End;
    }

    for (u8I = 0; u8I < u8AddrSizeIIC; u8I++)
    {
        if (FALSE == MHal_IIC_SendByte(pu8AddrIIC[u8I]))
        {
            bReturnIIC = FALSE;
            s32RetCountIIC = -3;
            goto HW_IIC_Read_End;
        }
    }

    MHal_IIC_Start();

	udelay(100);

    if (FALSE == MHal_IIC_SendByte(u8SlaveIdIIC|0x1))
    {
        bReturnIIC = FALSE;
        s32RetCountIIC = -4;
        goto HW_IIC_Read_End;
    }

    for (u32I = 0; u32I < u32BufSizeIIC; u32I++)
    {
        if (u32I == (u32BufSizeIIC-1))
        {
            MHal_IIC_NoAck();
        }

        if (FALSE == MHal_IIC_GetByte(&pu8BufIIC[u32I]))
        {
            bReturnIIC = FALSE;
            s32RetCountIIC = -5;
            goto HW_IIC_Read_End;
        }
    }

    s32RetCountIIC = u32BufSizeIIC;

HW_IIC_Read_End:
    MHal_IIC_Stop();

    IIC_DBG(printk("MDrv_IIC_Read() --> s32RetCountIIC=%d \n", s32RetCountIIC));
    return s32RetCountIIC;
}
