/******************************************************************************
 *-----------------------------------------------------------------------------
 *
 *  Copyright (c) 2008 MStar Semiconductor, Inc.  All rights reserved.
 *
 *-----------------------------------------------------------------------------
 * FILE NAME      columbus_mci.h
 *
 * DESCRIPTION    Columbus LinuxBSP MCI driver.
 *                MultiMedia Card Interface (MCI) registers
 *                Base on FCIE (Flash Card Interface Engine) datasheet
 *
 * AUTHOR         Ray Hsu           Initial Version
 *                Evan Chang        Update MStar code convention
 *                                  Resolved mounting problem
                   William Wang Update the pioneer configuration
 *
 * HISTORY        2008/05/06        Initial Version
 *                2008/07/18        Update IRQ assignment & code convention
 *                2008/07/22        Resolved mounting problem
 *                2008/10/02         Update the pioneer configuration
 *****************************************************************************/


#ifndef COLUMBUS_MCI_H
#define COLUMBUS_MCI_H

/* Get Register Address Macro Functions */
//#define REG(Reg_Addr)       (*(volatile u32*)(Reg_Addr))
#define GET_REG_ADDR(x, y)  (x+(y)*4)

#define readl(addr) (*(volatile unsigned int *) (addr))
#define writel(b,addr) ((*(volatile unsigned int *) (addr)) = (b))

#define INREG32(a)          readl(a)  //columbus_readl(a)
#define OUTREG32(a, v)      writel((u32)v, a)  //columbus_writel((u32)v,a)
#define SETREG32(a, v)      OUTREG32(a, INREG32(a)|(v))
#define CLRREG32(a, v)      OUTREG32(a, INREG32(a)&~(v))

#define fcie_readw(addr)        (*(volatile unsigned short *) (addr))
#define fcie_writew(val, addr)  ((*(volatile unsigned short *) (addr)) = (u16)(val))

/* Columbus Register Address Base */
#define FCIE_TASKLET        /* Defer tasks in IRQ, so IRQ can be finished faster */

#if defined(CONFIG_ARCH_PIONEER)
#define RIU_BASE            0xA0000000
#define CHIPTOP_BASE        GET_REG_ADDR(RIU_BASE, 0x0F00)   /* 0xA0003C00 */
#define PM_BASE             GET_REG_ADDR(RIU_BASE, 0x1A00)   /* 0xA0006800 */
// winder
#define REG_CLK_NFIE_FCIE_CKGEN   GET_REG_ADDR(CHIPTOP_BASE, 0x1A) /* 0xA0003C68 */
#define REG_CLK_NFIE_FCIE   GET_REG_ADDR(CHIPTOP_BASE, 0x24) /* 0xA0003C90 */
#define REG_GPIO_C_PU       GET_REG_ADDR(CHIPTOP_BASE, 0x3C)   // 0xA0003CF0 D0~D8
#define REG_GPIO_C_PD       GET_REG_ADDR(CHIPTOP_BASE, 0x3D)   // 0xA0003CF4 D0~D8
#define REG_GPIO_C_DRV      GET_REG_ADDR(CHIPTOP_BASE, 0x3E)   // 0xA0003CF8 D0~D5
#define REG_PM_IRQ_MASK     GET_REG_ADDR(PM_BASE, 0x57) /* A000695C */
#define FCIE0_BASE          GET_REG_ADDR(RIU_BASE, 0x1000)   /* 0xA0004000 */
#define FCIE1_BASE          GET_REG_ADDR(RIU_BASE, 0x1080)   /* 0xA0004200 */
#define FCIE2_BASE          GET_REG_ADDR(RIU_BASE, 0x1100)   /* 0xA0004400 */
#define FCIE3_BASE          GET_REG_ADDR(RIU_BASE, 0x1180)   /* 0xA0004600 */
#define INT_BASE            GET_REG_ADDR(RIU_BASE, 0x0280)  /*0xA0000A00*/
#elif defined(CONFIG_ARCH_COLUMBUS)
#define RIU_BASE            0xA0000000
#define CHIPTOP_BASE        GET_REG_ADDR(RIU_BASE, 0x0F00)   /* 0xA0003C00 */
#define REG_CLK_NFIE_FCIE   GET_REG_ADDR(CHIPTOP_BASE, 0x25) /* 0xA0003C94 */
#define FCIE0_BASE          GET_REG_ADDR(RIU_BASE, 0x1000)   /* 0xA0004000 */
#define FCIE1_BASE          GET_REG_ADDR(RIU_BASE, 0x1080)   /* 0xA0004200 */
#define FCIE2_BASE          GET_REG_ADDR(RIU_BASE, 0x1100)   /* 0xA0004400 */
#define FCIE3_BASE          GET_REG_ADDR(RIU_BASE, 0x1180)   /* 0xA0004600 */
#define INT_BASE            GET_REG_ADDR(RIU_BASE, 0x1580)  /* 0xA0005600 */
#elif defined(CONFIG_MSTAR_URANUS4)
#define RIU_BASE            0xBF200000
#define CHIPTOP_BASE        GET_REG_ADDR(RIU_BASE, 0x0F00)   /* 0xBF203C00 */
//#define REG_GPIO_C_PU       GET_REG_ADDR(CHIPTOP_BASE, 0x3C)   // 0xA0003CF0 D0~D8
//#define REG_GPIO_C_PD       GET_REG_ADDR(CHIPTOP_BASE, 0x3D)   // 0xA0003CF4 D0~D8
//#define REG_GPIO_C_DRV      GET_REG_ADDR(CHIPTOP_BASE, 0x3E)   // 0xA0003CF8 D0~D5

#define CLKGEN_BASE         GET_REG_ADDR(RIU_BASE, 0x0580)   /*  */
#define REG_CLK_NFIE_FCIE   GET_REG_ADDR(CLKGEN_BASE, 0x64)  /*  */
#define REG_CLK_NFIE_SSC    GET_REG_ADDR(CLKGEN_BASE, 0x65)

//#define PM_BASE             GET_REG_ADDR(RIU_BASE, 0x1A00)   /* 0xA0006800 */
//#define REG_PM_IRQ_MASK     GET_REG_ADDR(PM_BASE, 0x57)     /* A000695C */

#define FCIE0_BASE          GET_REG_ADDR(RIU_BASE, 0x8980)   /* 0xBF222600 */
#define FCIE1_BASE          GET_REG_ADDR(RIU_BASE, 0x89E0)   /* 0xBF222780 */
#define FCIE2_BASE          GET_REG_ADDR(RIU_BASE, 0x8A00)   /* 0xBF222800 */
#define FCIE3_BASE          GET_REG_ADDR(RIU_BASE, 0x8A80)   /* 0xBF22A000 */
//#define INT_BASE          GET_REG_ADDR(RIU_BASE, 0x0280)   /* 0xA0000A00 */

/******************************************************************************
* MStar CHIPTOP Register Address
******************************************************************************/
#define CHIPTOP_SDIO_MODE   GET_REG_ADDR(CHIPTOP_BASE, 0x06)
#define PAD_SDIO_MODE_CLR   (3<<4)
#define PAD_SDIO_MODE_ACT   (1<<5)

#define PADTOP1_BASE        GET_REG_ADDR(RIU_BASE, 0x1300)
#define PADTOP1_DM_GPIO     GET_REG_ADDR(PADTOP1_BASE, 0x06)
#define DM_GPIO_OUT         (1<<4)
#define DM_GPIO_OEN         (1<<5)

#else
#error "Unknown Machine Type "
#endif

#define CLK_43MHz   (43000000)
#define CLK_36MHz   (36000000)
#define CLK_27MHz   (27000000)
#define CLK_43MHz   (43000000)
#define CLK_21_5MHz (43000000)
#define CLK_300KHz  (300000)

#if defined(CONFIG_ARCH_PIONEER)
// winder
#define BIT_FCIE_CLK_SSC_CLRALL    0x7FF
#define BIT_FCIE_CLK_SSC_27M       0x9
#define BIT_FCIE_CLK_SSC_18M       0xD
#define BIT_FCIE_CLK_SSC_13_5M     0x11
#define BIT_FCIE_CLK_SSC_5_4M      0x29
#define BIT_FCIE_CLK_SSC_750K      0x121
#define BIT_FCIE_CLK_SSC_300K      0x2D1

#define BIT_FCIE_CLK_72M           (0xA<<2)
#define BIT_FCIE_CLK_54M           (0x9<<2)
#define BIT_FCIE_CLK_43M           (0x8<<2)
#define BIT_FCIE_CLK_36M           (0x7<<2)
#define BIT_FCIE_CLK_27M           (0x6<<2)
#define BIT_FCIE_CLK_22_8M         (0x5<<2)
#define BIT_FCIE_CLK_18M           (0x4<<2)
#define BIT_FCIE_CLK_13_5M         (0x3<<2)
#define BIT_FCIE_CLK_5_4M          (0x2<<2)
#define BIT_FCIE_CLK_750K          (0x1<<2) // actually it is 1.3MHz in Pioneer

#define BIT_FCIE_CLK_DEFAULT       BIT_FCIE_CLK_SSC_18M

// winder
#define BIT_FCIE_PAD_WPZCLKCMDDAT  0x7F
#define BIT_FCIE_PAD_WPZCMDDAT     0x5F
#define BIT_FCIE_PAD_CLKCMDDAT     0x3F
#define BIT_FCIE_PAD_STRONGUP      (0x3F<<9)

#endif /* if defined(CONFIG_ARCH_PIONEER) */


#if defined(CONFIG_MSTAR_URANUS4)

#define BIT_FCIE_CLK_SSC           (0xC<<2)
#define BIT_FCIE_CLK_86M           (0xB<<2)
#define BIT_FCIE_CLK_72M           (0xA<<2)
#define BIT_FCIE_CLK_54M           (0x9<<2)
#define BIT_FCIE_CLK_43M           (0x8<<2)
#define BIT_FCIE_CLK_32M           (0x7<<2)
#define BIT_FCIE_CLK_27M           (0x6<<2)
#define BIT_FCIE_CLK_22_8M         (0x3<<2)//(0x5<<2)
#define BIT_FCIE_CLK_18M           (0x4<<2)
#define BIT_FCIE_CLK_13_5M         (0x3<<2)
#define BIT_FCIE_CLK_5_4M          (0x2<<2)
#define BIT_FCIE_CLK_375K          (0x1<<2)

#endif /* if defined(CONFIG_MSTAR_URANUS4) */

/* Select High or Low trigger, active low */
//#define INT_TRIG_LEVEL_REG_LW_ADDR  GET_REG_ADDR(INT_BASE, 0x14)  /* 0xA0005650 */
//#define INT_TRIG_LEVEL_REG_HI_ADDR  GET_REG_ADDR(INT_BASE, 0x15)  /* 0xA0005654 */
//#define INT_FCIE_TRIGGER      (0x01 << 15)    /* Bit 15, use LW_ADDR */

/* Mask for IRQ, active low */
//#define INT_MASK_REG_LW_ADDR    GET_REG_ADDR(INT_BASE, 0x0C)  /* 0xA0005630 */
//#define INT_MASK_REG_HI_ADDR    GET_REG_ADDR(INT_BASE, 0x0D)  /* 0xA0005634 */
//#define INT_FCIE_MASK          (0x01 << 15)    /* Bit 15, use LW_ADDR */
/******************************************************************************
* MStar FCIE Register Address
******************************************************************************/

#define FCIE_MIE_EVENT    0x00  //00h
#define MMA_DATA_END      (1 << 0) /* MMA data transcation complete event */
#define SD_CMD_END        (1 << 1) /* SD/MMC card command and response transcation complete event */
#define SD_DATA_END       (1 << 2) /* SD/MMC data transcation complete event */
#define MS_DATA_END       (1 << 3) /* Memstick data transcation complete event */
#define SDIO_INT          (1 << 7) /* SDIO interrupt event */
#define CARD_DMA_END      (1 << 11)
#define SD_ALL_INT        (SDIO_INT|MMA_DATA_END|SD_CMD_END|SD_DATA_END|CARD_DMA_END)
#define MS_ALL_INT           (MMA_DATA_END|MS_DATA_END)
/*---------------------------------------------------------------------------*/
#define FCIE_MIE_INT_EN   0x04  //01h
#define MMA_DATA_ENDE     (1 << 0) /* MMA_DATA_END interrupt enable*/
#define SD_CMD_ENDE       (1 << 1) /* SD_CMD_END interrupt enable */
#define SD_DATA_ENDE      (1 << 2) /* SD_DATA_END interrupt enable */
#define MS_DATA_ENDE      (1 << 3)
#define SDIO_INTE         (1 << 7) /* SDIO_INT interrupt enable */
#define CARD_DMA_END_EN   (1 << 11)
/*---------------------------------------------------------------------------*/
#define FCIE_MMA_PRI_REG  0x08  //02h
#define MMA_R_PRIORITY    (1 << 0) /* MIU read request priority. 0:low priority, 1:high priroity*/
#define MMA_W_PRIORITY    (1 << 1) /* MIU write request priority. 0:low priority, 1:high priroity*/
#define JOB_RW_DIR        (1 << 2) /* Specify DMA cycle is Read or Write.*/
#define FIFO_CLKRDY       (1 << 5)
//0:Read from card(Data write to DRAM)
//1:Write to card(Data read from DRAM)
/*---------------------------------------------------------------------------*/
#define FCIE_MIU_DMA1     0x0C     /* DMA address: addr16~addr23*/  //03h
/*---------------------------------------------------------------------------*/
#define FCIE_MIU_DMA0     0x10     /* DMA address: addr00~addr15*/  //04h
/*---------------------------------------------------------------------------*/
#define FCIE_CARD_EVENT   0x14  //05h
#define SD_STS_CHG        (1 << 0) /* SD/MMC Card plug-in or remove status */
#define MS_STS_CHG        (1 << 1) /* MemoryStick Card plug-in or remove status */
#define SD_PWR_OC_CHG     (1 << 5) /* SD/MMC Card over current status */
#define SD_SDIO_CHG       (1 << 7) /* SDIO Card plug-in or remove status 1 */
#define SD_SDIO2_CHG      (1 << 8) /* SDIO Card plug-in or remove status 2 */
#define SD_CARD_EVENTS    (SD_STS_CHG|SD_PWR_OC_CHG|SD_SDIO_CHG|SD_SDIO2_CHG)
/*---------------------------------------------------------------------------*/
#define FCIE_CARD_INT_EN  0x18  //06h
#define SD_STS_EN         (1 << 0) /* SD card status change interrupt enable */
#define MS_STS_EN         (1 << 1) /* SD card status change interrupt enable */
#define SD_PWR_OC_EN      (1 << 5) /* SD card pwr over current detection status change interrupt enable */
#define SD_CARD_DET_SRC   (1 << 7) /* SD Card detect select.0:SD_CDZ, 1:SD_DAT3*/
#define SDIO_STS_EN       (1 << 8) /* SDIO card status change interrupt enable 1*/
#define SDIO2_STS_EN      (1 << 11) /* SDIO card status change interrupt enable 2*/
/*---------------------------------------------------------------------------*/
#define FCIE_CARD_DET     0x1C  //07h
#define SD_DET_N          (1 << 0) /* SD/MMC card Detection*/
#define MS_DET_N          (1 << 1) /* MS card Detection*/
#define SD_OCDET_STS      (1 << 5) /* SD/MMC Card PWR over current detection status */
#define SDIO_DET_N        (1 << 7) /* SDIO Card detection 1 */
#define SDIO2_DET_N       (1 << 9) /* SDIO Card detection 2 */
/*---------------------------------------------------------------------------*/
#define FCIE_CARD_PWR     0x20  //08h
#define SD_PWR_ON         (1 << 0) /* SD/MMC/MS/MSPro/SM/XD pwr. O:ON, 1:OFF*/
#define SD_PWR_PAD_OEN    (1 << 2) /* SD card power pad control. 0:Output, 1:Input*/
/*---------------------------------------------------------------------------*/
#define FCIE_INTR_TEST    0x24     /* For Force interrupt test*/    //09h
/*---------------------------------------------------------------------------*/
#define FCIE_MIE_PATH_CTL 0x28  //0Ah
#define MMA_ENABLE        (1 << 0) /* MIU DMA enable*/
#define SD_EN             (1 << 1) /* SD / MMC card interface enable*/
#define  MS_EN             (1 << 2) /* MemoryStick card interface enable*/
/*---------------------------------------------------------------------------*/
#define FCIE_JOB_BL_CNT   0x2C     /* Total block count for this job*/  //0Bh
/*---------------------------------------------------------------------------*/
#define FCIE_TR_BL_CNT    0x30     /* Real time number of sectors remained  //0Ch
to be transferred*/
/*---------------------------------------------------------------------------*/
#define FCIE_RSP_SIZE     0x34     /* The expected respone size(byte count).  //0Dh
h01=1 byte, h40=64 bytes*/
/*---------------------------------------------------------------------------*/
#define FCIE_CMD_SIZE     0x38     /* The command transfer size(byte count).
                h01=1 byte, h40=64 bytes*/  //0Eh
/*---------------------------------------------------------------------------*/
#define FCIE_CARD_WD_CNT  0x3C     /* Expect data word count that transferred
                                through CIF FIFO. 0x00 represents 256 words.*/  //0Fh
/*---------------------------------------------------------------------------*/
#define FCIE_SD_MODE      0x40  //10h
#define SD_CLK_EN         (1 << 0) /* SD MIF output clock enable*/
#define SD_DAT_LINE       (3 << 1)
#define SD_DAT_LINE0      (1 << 1) /* Use DAT3-0 line*/
#define SD_DAT_LINE1      (1 << 2) /* Use DAT7-0 line*/
#define SDDRL             (1 << 3) /* Firmware write 1 to SD interface drive low*/
#define SD_CS_EN          (1 << 4) /* Set to enable clock auto-stop featuer, which will stop
                                CLK between read blocks when Data FIFO is FULL. 1:enable*/
#define SD_DEST           (1 << 5) /* SD/MMC data transfer destination. 0:Data FIFO. 1:CIF FIFO.*/
#define SD_DATSYNC        (1 << 6) /* Synchornize data bus, for SD1.1 specification*/
#define MMC_BUS_TEST      (1 << 7) /* Test MMC bus type through CIF Data FIFO*/
#define SDIO_RDWAIT       (1 << 8) /* When read block,and Data FIFO busy. Hardware will drive SD_DAT1 to low
                                to inform card controller that host is busy. High active*/
#define SDIO_SD_BUS_SW    (1 << 9) /* SDIO interface and SD/MMC card interface select.
                                Set"1" to select SDIO. Default SD/MMC*/
#define SDIO_PORT_SEL     (1 << 12)
/*---------------------------------------------------------------------------*/
#define FCIE_SD_CTL       0x44  //11h
#define SD_RSPR2_EN       (1 << 0) /* SD/MMC receive command response for R2 type*/
#define SD_RSP_EN         (1 << 1) /* SD/MMC receive command response enable*/
#define SD_CMD_EN         (1 << 2) /* SD/MMC transmit command enable*/
#define SD_DTRX_EN        (1 << 3) /* SD/MMC data transmit/receive enable*/
#define SD_DTRX_DIR       (1 << 4) /* SD/MMC data transmit direction.
                                0:Read from card. 1:Write to card*/
#define SDIO_INT_MOD0     (1 << 8) /* 00:continuous interrupt dectect.
                                01:CMD12 or IO Abort command interrupt detect.*/
#define SDIO_INT_MOD1     (1 << 9) /* 10:single block read/write interrupt detect.
                                11:multi blocks read/write interrupt detect.*/
#define SDIO_DET_ON       (1 << 10) /* SDIO interrupt detect function switch,Active High.*/
/*---------------------------------------------------------------------------*/
#define FCIE_SD_STS       0x48  //12h
#define SD_DAT_CERR       (1 << 0) /* Received data phase CRC error event*/
#define SD_DAT_STSERR     (1 << 1) /* Transmitted data phase:Flash program error*/
#define SD_DAT_STSNEG     (1 << 2) /* Transmitted data phase:Transmission error*/
#define SD_DAT_NORSP      (1 << 3) /* Transmitted CMD phase: Time out=64 clocks,
                                there is no response on CMD line.*/
#define SD_CMDRSP_CERR    (1 << 4) /* Response CRC error*/
#define SD_WR_PRO_N       (1 << 5) /* Used for SD card Write protected*/
#define SD_STS_ERRORS     (SD_DAT_CERR|SD_DAT_STSERR|SD_DAT_STSNEG|SD_DAT_NORSP|SD_CMDRSP_CERR)
/*---------------------------------------------------------------------------*/
#define FCIE_MS_MODE      0x4C  //13h
#define MS_DAT_LINE0      (1 << 0)    /* Use 4 bits mode */
/*---------------------------------------------------------------------------*/
#define FCIE_MS_CTL       0x50  //14h
#define MS_REGTRX_EN      (1 << 0)
#define MS_DTRX_EN        (1 << 1)
#define MS_BUS_DIR        (1 << 2)
#define MS_BURST          (1 << 3)
#define MS_DEST           (1 << 4)
/*---------------------------------------------------------------------------*/
#define FCIE_MS_STS       0x54  //15h
#define  MS_TOUT          (1 << 4) /* Handshaking time out*/
#define  MS_CRCERR        (1 << 5) /* Received data phase CRC error event*/
#define MS_STS_ERRORS     (MS_TOUT|MS_CRCERR)
/*---------------------------------------------------------------------------*/
#define SDIO_BLK_SIZE     0x6C  //1B
#define  SDIO_BLK_512     (1 << 9)
#define  SDIO_BLK_MOD_EN  (1 << 15) /* SDIO block mode enable */
/*---------------------------------------------------------------------------*/
#define SDIO_MIU_DMA0     0x70  //1C
/*---------------------------------------------------------------------------*/
#define SDIO_MIU_DMA1     0x74  //1D
/*---------------------------------------------------------------------------*/
#define FCIE_SDIO_STS     0x78  //1Eh
#define  SDIO2_D0          (1 << 8)
#define  SDIO2_D1          (1 << 9)
#define  SDIO2_D2          (1 << 10)
#define  SDIO2_D3          (1 << 11)
/*---------------------------------------------------------------------------*/
#define FCIE_DMA_OFFSET   0xB8  // 2Eh
/*---------------------------------------------------------------------------*/
#define FCIE_TEST_MODE    0xC0  //30h
#define FCIE_SOFT_RST     (1 << 12) /* FCIE module soft reset, low active*/
#define ENDIAN_SEL        (1 << 13) /* Low:little endian. High:big endian*/
/*---------------------------------------------------------------------------*/


/******************************************************************************
* Low level type for this driver
******************************************************************************/
struct fcie_ms
{
    struct platform_device  *pdev;
    struct timer_list       timer;
    struct memstick_request *req;
    struct tasklet_struct   notify;
    spinlock_t              lock;

    unsigned int        block_pos;
    unsigned long       timeout_jiffies;
    unsigned char       cmd_flags;
    unsigned char       io_pos;
    unsigned int        io_word[2];

    dma_addr_t     physical_address;
#ifdef FCIE_TASKLET
    struct tasklet_struct   pio_tasklet;
#endif

    int         irq;
    void __iomem        *regs;
    unsigned char       present:1;
};

struct msfm_ms_host
{
    struct memstick_host    *ms;
    struct memstick_request *request;

    void __iomem *baseaddr;
    s32 irq;
    s32 present;

    struct clk *mci_clk;

    /* Flag indicating when the command has been sent.          */
    /* This is used to work out whether or not to send the stop */
    u8  cmd_flags;
    u32 flags;
    /* flag for current bus settings */
    u32 bus_mode;

    /* DMA buffer used for transmitting */
    u32 *rx_buffer;
    u32 *tx_buffer;
    dma_addr_t physical_address;
    u32 total_length;

    /* Latest in the scatterlist that has been enabled for transfer, but not freed */
    s32 in_use_index;

    /* Latest in the scatterlist that has been enabled for transfer */
    s32 transfer_index;

    volatile u32 pio_sgptr;
    volatile u32 pio_words;
    volatile u32 *pio_ptr;
    s32 dodma;

#ifdef FCIE_TASKLET
    struct tasklet_struct   notify;
#endif

};  /* struct mstar_mci_host*/

struct mstar_mci_host
{
    struct mmc_host     *mmc;
    struct mmc_command  *cmd;
    struct mmc_request  *request;

    void __iomem *baseaddr;
    s32 irq;
    s32 present;

    struct clk *mci_clk;

    u16 sd_clk;
    u16 sd_mod;

    /* Flag indicating when the command has been sent.          */
    /* This is used to work out whether or not to send the stop */
    u32 flags;
    /* flag for current bus settings */
    u32 bus_mode;

    /* DMA buffer used for transmitting */
    u32 *rx_buffer;
    u32 *tx_buffer;
    dma_addr_t physical_address;
    u32 total_length;

    /* Latest in the scatterlist that has been enabled for transfer, but not freed */
    s32 in_use_index;

    /* Latest in the scatterlist that has been enabled for transfer */
    s32 transfer_index;

    volatile u32 pio_sgptr;
    volatile u32 pio_words;
    volatile u32 *pio_ptr;
    s32 dodma;

#ifdef FCIE_TASKLET
    struct tasklet_struct   pio_tasklet;
#endif

    struct semaphore sem;     /* Mutual exclusion semaphore     */
    struct cdev cdev;         /* Char device structure */
    wait_queue_head_t event_q; /* Wait event queue. Now, only SD card insert/remove event. */
    int card_chg; /* flag for card status change event. */
    struct list_head queue;
};  /* struct mstar_mci_host*/

#ifndef MSTAR_MCI_MAJOR
#define MSTAR_MCI_MAJOR 0   /* dynamic major by default */
#endif

#ifndef MSTAR_MCI_NR_DEVS
#define MSTAR_MCI_NR_DEVS 1    /* columbus_udc0 */
#endif

#ifndef BIT0
#define BIT0    0x0001
#endif
#ifndef BIT1
#define BIT1    0x0002
#endif
#ifndef BIT2
#define BIT2    0x0004
#endif
#ifndef BIT3
#define BIT3    0x0008
#endif
#ifndef BIT4
#define BIT4    0x0010
#endif
#ifndef BIT5
#define BIT5    0x0020
#endif
#ifndef BIT6
#define BIT6    0x0040
#endif
#ifndef BIT7
#define BIT7    0x0080
#endif
#ifndef BIT8
#define BIT8    0x0100
#endif
#ifndef BIT9
#define BIT9    0x0200
#endif
#ifndef BIT10
#define BIT10   0x0400
#endif
#ifndef BIT11
#define BIT11   0x0800
#endif
#ifndef BIT12
#define BIT12   0x1000
#endif
#ifndef BIT13
#define BIT13   0x2000
#endif
#ifndef BIT14
#define BIT14   0x4000
#endif
#ifndef BIT15
#define BIT15   0x8000
#endif

#endif
