///////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// @file   mdrv_gflip_io.c
// @brief  GFlip KMD Driver Interface
// @author MStar Semiconductor Inc.
//////////////////////////////////////////////////////////////////////////////////////////////////

//=============================================================================
// Include Files
//=============================================================================
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/fs.h>
#include <asm/io.h>
#include <asm/string.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>

//drver header files
#include "mdrv_mstypes.h"
#include "chip_int.h"
#include "mdrv_gflip.h"
#include "mhal_gflip.h"
#include "mdrv_gflip_interrupt.h"
#include "mdrv_gflip_io.h"
#include "mst_devid.h"

//=============================================================================
// Local Defines
//=============================================================================
#define     MDRV_GFLIP_DEVICE_COUNT           1
#define     MDRV_GFLIP_NAME                           "GFLIP"

#define MAX_FILE_HANDLE_SUPPRT  64

//-------------------------------------------------------------------------------------------------
// Macros
//-------------------------------------------------------------------------------------------------
#define     GFLIPIO_DEBUG_ENABLE  //
#ifdef      GFLIPIO_DEBUG_ENABLE
#define     GFLIPIO_KDBG(_fmt, _args...)        printk(KERN_WARNING _fmt, ## _args)
#define     GFLIPIO_ASSERT(_con)   do {\
                                                            if (!(_con)) {\
                                                                printk(KERN_CRIT "BUG at %s:%d assert(%s)\n",\
                                                                                                    __FILE__, __LINE__, #_con);\
                                                                BUG();\
                                                            }\
                                                          } while (0)
#else
#define     GFLIPIO_KDBG(fmt, args...)
#define     GFLIPIO_ASSERT(arg)
#endif

//--------------------------------------------------------------------------------------------------
// IOCtrl functions declaration
//--------------------------------------------------------------------------------------------------
static int _MDrv_GFLIPIO_Open (struct inode *inode, struct file *filp);
static int _MDrv_GFLIPIO_Release(struct inode *inode, struct file *filp);
static int _MDrv_GFLIPIO_IOCtl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
MSYSTEM_STATIC int _MDrv_GFLIPIO_ModuleInit(void);
MSYSTEM_STATIC void _MDrv_GFLIPIO_ModuleExit(void);

//=============================================================================
// Local Variables: Device handler
//=============================================================================
typedef struct
{
    int s32Major;
    int s32Minor;
    int refCnt;
    struct cdev cdev;
    struct file_operations fops;
}GFLIP_DEV;

static GFLIP_DEV _devGFLIP =
{
    .s32Major = MDRV_MAJOR_GFLIP,
    .s32Minor = MDRV_MINOR_GFLIP,
    .refCnt = 0,
    .cdev =
    {
        .kobj = {.name= MDRV_NAME_GFLIP, },
        .owner = THIS_MODULE,
    },
    .fops =
    {
        .open = _MDrv_GFLIPIO_Open,
        .release = _MDrv_GFLIPIO_Release,
        .ioctl = _MDrv_GFLIPIO_IOCtl,
    }
};

struct
{
    struct file* filp;
    MS_U32    u32GOPIdx;
}_filpGopIdxGFLIP[MAX_FILE_HANDLE_SUPPRT];
MS_U16  g_u16GOPRefCnt[MAX_GOP_SUPPORT];
//=============================================================================
// Local Function Prototypes
//=============================================================================
static int _MDrv_GFLIPIO_IOC_Init(struct file *filp, unsigned long arg);
//static int _MDrv_GFLIPIO_IOC_DeInit(struct file *filp, unsigned long arg);
static int _MDrv_GFLIPIO_IOC_SetFlipInfo(struct file *filp, unsigned long arg);
static int _MDrv_GFLIPIO_IOC_SetInputSigStatus(struct file *filp, unsigned long arg);
static MS_U32 _MDrv_GFLIP_RegisterInterrupt(void);
static MS_U32 _MDrv_GFLIP_DeRegisterInterrupt(void);

//-------------------------------------------------------------------------------------------------
// IOCtrl Driver functions
//-------------------------------------------------------------------------------------------------
int _MDrv_GFLIPIO_IOC_Init(struct file *filp, unsigned long arg)
{
    MS_U32 u32GopIdx;
    MS_U16 u16Idx, u16AllocIdx=MAX_FILE_HANDLE_SUPPRT;

    if(__get_user(u32GopIdx, (MS_U32 __user *)arg))
    {
        return -EFAULT;
    }

    if(MAX_GOP_SUPPORT<= u32GopIdx)
    {
        return -EFAULT;
    }

    for(u16Idx=0; u16Idx<MAX_FILE_HANDLE_SUPPRT; u16Idx++)
    {
        if(_filpGopIdxGFLIP[u16Idx].filp==filp && _filpGopIdxGFLIP[u16Idx].u32GOPIdx==u32GopIdx)
        {//already init:
            return 0;
        }

        if(u16AllocIdx==MAX_FILE_HANDLE_SUPPRT && NULL==_filpGopIdxGFLIP[u16Idx].filp)
        {
            u16AllocIdx= u16Idx;
        }
    }

    if(MAX_FILE_HANDLE_SUPPRT<=u16AllocIdx)
    {
        return -EFAULT;
    }

    _filpGopIdxGFLIP[u16AllocIdx].filp = filp;
    _filpGopIdxGFLIP[u16AllocIdx].u32GOPIdx = u32GopIdx;
    //GFLIPIO_ASSERT(g_u16GOPRefCnt[u32GopIdx] >= 0);

    if(++g_u16GOPRefCnt[u32GopIdx] > 1)
    {
         return 0;
    }

    return MDrv_GFLIP_Init(u32GopIdx) ? 0 : (-EFAULT);
}

/********************************************************
*** Do not export this IO DDI now ****************************
int _MDrv_GFLIPIO_IOC_DeInit(struct file *filp, unsigned long arg)
{
    MS_U32 u32GopIdx;

    if (__get_user(u32GopIdx, (MS_U32 __user *)arg))
    {
        return EFAULT;
    }

    return MDrv_GFLIP_DeInit(u32GopIdx);
}
*/

int _MDrv_GFLIPIO_IOC_SetFlipInfo(struct file *filp, unsigned long arg)
{
    MS_GFLIP_INFO stFlipInfo;

    if(copy_from_user(&stFlipInfo, (MS_GFLIP_INFO __user *)arg, sizeof(MS_GFLIP_INFO)))
    {
        return EFAULT;
    }

    if (MDrv_GFLIP_SetFlipInfo(stFlipInfo.u32GopIdx, stFlipInfo.u32GwinIdx, stFlipInfo.u32Addr,stFlipInfo.u32TagId,&stFlipInfo.u32QEntry,&stFlipInfo.u32Result))
    {
        __put_user(stFlipInfo.u32QEntry, &(((MS_GFLIP_INFO __user *)arg)->u32QEntry));
        __put_user(stFlipInfo.u32Result, &(((MS_GFLIP_INFO __user *)arg)->u32Result));
	    return 0;
    }
    else
    {
        // The return value is not so appropriate.
        return EFAULT;
    }
}

int _MDrv_GFLIPIO_IOC_SetInputSigStatus(struct file *filp, unsigned long arg)
{
    MS_BOOL bHasSignal;

    if (__get_user(bHasSignal, (MS_BOOL __user *)arg))
    {
        return EFAULT;
    }

    if(bHasSignal)
    {
        MDrv_GFLIP_RestoreFromVsyncLimitation();
    }

    return 0;
}

int _MDrv_GFLIPIO_IOC_GetDWinIntInfo(struct file *filp, unsigned long arg)
{
    MS_GFLIP_DWININT_INFO stDWinIntInfo;

    if(copy_from_user(&stDWinIntInfo, (MS_GFLIP_DWININT_INFO __user *)arg, sizeof(MS_GFLIP_DWININT_INFO)))
    {
        return EFAULT;
    }

    if (MDrv_GFLIP_GetDWinIntInfo(&stDWinIntInfo.gflipDWinIntInfo, stDWinIntInfo.bResetDWinIntInfo))
    {
        __put_user(stDWinIntInfo.gflipDWinIntInfo.u8DWinIntInfo, &(((MS_GFLIP_DWININT_INFO __user *)arg)->gflipDWinIntInfo.u8DWinIntInfo));
	    return 0;
    }
    else
    {
        // The return value is not so appropriate.
        return EFAULT;
    }
}

MS_U32 _MDrv_GFLIP_RegisterInterrupt(void)
{
   if(0 != (request_irq(E_IRQ_GOP, MDrv_GFLIPINT_IntHandler, SA_INTERRUPT, "gop", NULL)))
    {
        GFLIPIO_KDBG("[GFLIP] Fail to request IRQ:%d\n", E_IRQ_GOP);
        return EFAULT;
    }

    return 0;
}

MS_U32 _MDrv_GFLIP_DeRegisterInterrupt(void)
{
    GFLIPIO_KDBG("[GFLIP]  dismiss IRQ:%d\n", E_IRQ_GOP);
    free_irq(E_IRQ_GOP,NULL);

    return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
// IOCtrl Driver interface functions
//-------------------------------------------------------------------------------------------------
int _MDrv_GFLIPIO_Open(struct inode *inode, struct file *filp)
{
    GFLIPIO_KDBG("[GFLIP] GFLIP DRIVER OPEN\n");

    GFLIPIO_ASSERT(_devGFLIP.refCnt>=0);
    _devGFLIP.refCnt++;
    return 0;
}

int _MDrv_GFLIPIO_Release(struct inode *inode, struct file *filp)
{
    MS_U32 u32Idx;

    GFLIPIO_KDBG("[GFLIP] GFLIP DRIVER CLOSE\n");

    GFLIPIO_ASSERT(_devGFLIP.refCnt>0);

    for(u32Idx=0; u32Idx<MAX_FILE_HANDLE_SUPPRT; u32Idx++)
    {
        if(_filpGopIdxGFLIP[u32Idx].filp != filp)
        {
            continue;
        }

        GFLIPIO_ASSERT(g_u16GOPRefCnt[_filpGopIdxGFLIP[u32Idx].u32GOPIdx] > 0);

        if(0 == --g_u16GOPRefCnt[_filpGopIdxGFLIP[u32Idx].u32GOPIdx])
        {
                MDrv_GFLIP_DeInit(_filpGopIdxGFLIP[u32Idx].u32GOPIdx);
        }

        _filpGopIdxGFLIP[u32Idx].filp = NULL;
     }

    return 0;
}

int _MDrv_GFLIPIO_IOCtl(struct inode *inode, struct file *filp, U32 u32Cmd, unsigned long u32Arg)
{
    int err = 0;
    int retval = 0;
    if(_devGFLIP.refCnt <= 0)
    {
        return -EFAULT;
    }
    /* check u32Cmd valid */
    if (MDRV_GFLIP_IOC_MAGIC!= _IOC_TYPE(u32Cmd))
    {
        return -ENOTTY;
    }

    if (_IOC_NR(u32Cmd) >= MDRV_GFLIP_IOC_MAX_NR)
    {
        GFLIPIO_KDBG("[GFLIP] IOCtl NR Error!!! (Cmd=%x)\n",u32Cmd);
        return -ENOTTY;
    }

    /* verify Access */
    if (_IOC_DIR(u32Cmd) & _IOC_READ)
    {
        err = !access_ok(VERIFY_WRITE, (void __user *)u32Arg, _IOC_SIZE(u32Cmd));
    }
    else if (_IOC_DIR(u32Cmd) & _IOC_WRITE)
    {
        err =  !access_ok(VERIFY_READ, (void __user *)u32Arg, _IOC_SIZE(u32Cmd));
    }
    if (err)
    {
        return -EFAULT;
    }

    /* handle u32Cmd */
    switch(u32Cmd)
    {
        case MDRV_GFLIP_IOC_INIT:
            retval = _MDrv_GFLIPIO_IOC_Init(filp, u32Arg);
            break;
        //case MDRV_GFLIP_IOC_DEINIT:
            //retval = _MDrv_GFLIPIO_IOC_DeInit(filp, u32Arg);
            //break;
        case MDRV_GFLIP_IOC_SETFLIPINFO:
            retval = _MDrv_GFLIPIO_IOC_SetFlipInfo(filp, u32Arg);
            break;
        case MDRV_GFLIP_IOC_SETINPUTSIGSTATUS:
            retval = _MDrv_GFLIPIO_IOC_SetInputSigStatus(filp, u32Arg);
            break;
        case MDRV_GFLIP_IOC_GETDWININTINFO:
            retval = _MDrv_GFLIPIO_IOC_GetDWinIntInfo(filp, u32Arg);
            break;
        default:  /* redundant, as cmd was checked against MAXNR */
            GFLIPIO_KDBG("[GFLIP] ERROR IOCtl number %x\n ",u32Cmd);
            return -ENOTTY;
    }

    return retval;
}


//-------------------------------------------------------------------------------------------------
// Module functions
//-------------------------------------------------------------------------------------------------
int _MDrv_GFLIPIO_ModuleInit(void)
{
    int s32Ret;
    dev_t  dev;

    memset(_filpGopIdxGFLIP, 0, sizeof(_filpGopIdxGFLIP));
    memset(g_u16GOPRefCnt, 0, sizeof(g_u16GOPRefCnt));

    if(_devGFLIP.s32Major)
    {
        dev = MKDEV(_devGFLIP.s32Major, _devGFLIP.s32Minor);
        s32Ret = register_chrdev_region(dev, MDRV_GFLIP_DEVICE_COUNT, MDRV_GFLIP_NAME);
    }
    else
    {
        s32Ret = alloc_chrdev_region(&dev, _devGFLIP.s32Minor, MDRV_GFLIP_DEVICE_COUNT, MDRV_GFLIP_NAME);
        _devGFLIP.s32Major = MAJOR(dev);
    }

    if (0 > s32Ret)
    {
        GFLIPIO_KDBG("[GFLIP] Unable to get major %d\n", _devGFLIP.s32Major);
        return s32Ret;
    }

    cdev_init(&_devGFLIP.cdev, &_devGFLIP.fops);
    if (0 != (s32Ret= cdev_add(&_devGFLIP.cdev, dev, MDRV_GFLIP_DEVICE_COUNT)))
    {
        GFLIPIO_KDBG("[GFLIP] Unable add a character device\n");
        unregister_chrdev_region(dev, MDRV_GFLIP_DEVICE_COUNT);
        return s32Ret;
    }

    /* initial the whole GFLIP Driver */

    if(EFAULT == _MDrv_GFLIP_RegisterInterrupt())
    {
        GFLIPIO_KDBG("[GFLIP] Startup GFLIP Driver Failed! %d\n", _devGFLIP.s32Major);
        cdev_del(&_devGFLIP.cdev);
        unregister_chrdev_region(dev, MDRV_GFLIP_DEVICE_COUNT);
	 return -ENOMEM;
    }

    return 0;
}


void _MDrv_GFLIPIO_ModuleExit(void)
{
    /*de-initial the who GFLIPDriver */
    _MDrv_GFLIP_DeRegisterInterrupt();

    cdev_del(&_devGFLIP.cdev);
    unregister_chrdev_region(MKDEV(_devGFLIP.s32Major, _devGFLIP.s32Minor), MDRV_GFLIP_DEVICE_COUNT);
}

#if defined(CONFIG_MSTAR_MSYSTEM) || defined(CONFIG_MSTAR_MSYSTEM_MODULE)
#else//#if defined(CONFIG_MSTAR_MSYSTEM) || defined(CONFIG_MSTAR_MSYSTEM_MODULE)
module_init(_MDrv_GFLIPIO_ModuleInit);
module_exit(_MDrv_GFLIPIO_ModuleExit);

MODULE_AUTHOR("MSTAR");
MODULE_DESCRIPTION("GFLIP ioctrl driver");
MODULE_LICENSE("GPL");
#endif//#if defined(CONFIG_MSTAR_MSYSTEM) || defined(CONFIG_MSTAR_MSYSTEM_MODULE)
