////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2007 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    mdrv_gpio.c
/// @brief  GPIO Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
//#include "MsCommon.h"
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>
#include <linux/timer.h>
#include <asm/io.h>

#include "mst_devid.h"

#include "mdrv_gpio.h"
#include "mhal_gpio_reg.h"
#include "mhal_gpio.h"

//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------



//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------



//--------------------------------------------------------------------------------------------------
// Forward declaration
//--------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
extern void MDrv_MICOM_Init(void);
extern U8 MDrv_MICOM_RegRead( U16 u16Addr );
extern B16 MDrv_MICOM_RegWrite(U16 u16Addr, U8 u8Data);

//-------------------------------------------------------------------------------------------------
/// GPIO chiptop initialization
/// @return None
/// @note   Called only once at system initialization
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Init(void)
{
    MHal_GPIO_Init();
#ifndef DEBUG_MICOM_MODULE
//    MDrv_MICOM_Init();
#endif
	msleep(150);
#if 1 
/* 
	HDMI
*/
#ifndef DEBUG_MICOM_MODULE
	MDrv_MICOM_RegWrite(0x106A, 2);// 1);
#endif
#endif
}
EXPORT_SYMBOL(MDrv_GPIO_Init);

//-------------------------------------------------------------------------------------------------
/// select one pad to set
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Pad_Set(U8 u8IndexGPIO)
{
#ifndef DEBUG_MICOM_MODULE
    U8  u8RegTemp;
#endif
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_Set();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_Set();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_Set();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_Set();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_Set();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_Set();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_Set();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_Set();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_Set();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_Set();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_Set();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_Set();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_Set();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_Set();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_Set();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_Set();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_Set();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_Set();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_Set();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_Set();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_Set();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_Set();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_Set();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_Set();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_Set();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_Set();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_Set();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_Set();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_Set();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_Set();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_Set();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_Set();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_Set();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_Set();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_Set();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_Set();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_Set();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_Set();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_Set();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_Set();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_Set();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_Set();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_Set();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_Set();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_Set();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_Set();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_Set();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_Set();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_Set();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_Set();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_Set();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_Set();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_Set();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_Set();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_Set();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_Set();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_Set();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_Set();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_Set();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_Set();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_Set();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_Set();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_Set();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_Set();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_Set();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_Set();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_Set();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_Set();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_Set();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_Set();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_Set();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_Set();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_Set();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_Set();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_Set();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_Set();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_Set();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_Set();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_Set();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_Set();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_Set();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_Set();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_Set();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_Set();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_Set();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_Set();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_Set();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_Set();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_Set();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_Set();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_Set();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_Set();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_Set();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_Set();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_Set();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_Set();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_Set();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_Set();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_Set();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_Set();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_Set();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_Set();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_Set();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_Set();
            break;
#ifndef DEBUG_MICOM_MODULE
        case PAD_GPIO_SAR0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR0_SET);
            u8RegTemp &= ~0x01;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR0_SET,u8RegTemp);
            break;
        case PAD_GPIO_SAR1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR1_SET);
            u8RegTemp &= ~0x02;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR1_SET,u8RegTemp);
            break;
        case PAD_GPIO_SAR2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR2_SET);
            u8RegTemp &= ~0x04;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR2_SET,u8RegTemp);
            break;
        case PAD_GPIO_SAR3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR3_SET);
            u8RegTemp &= ~0x08;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR3_SET,u8RegTemp);
            break;
#endif	/* #ifndef DEBUG_MICOM_MODULE */
        case PAD_TCON0:
            MHal_GPIO_TCON0_Set();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_Set();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_Set();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_Set();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_Set();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_Set();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_Set();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_Set();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_Set();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_Set();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_Set();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_Set();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_Set();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_Set();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_Set();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_Set();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_Set();
            break;
        default:
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Pad_Set);

//-------------------------------------------------------------------------------------------------
/// enable output for selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Pad_Oen(U8 u8IndexGPIO)
{
#ifndef DEBUG_MICOM_MODULE
    U8  u8RegTemp;
#endif
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_Oen();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_Oen();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_Oen();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_Oen();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_Oen();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_Oen();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_Oen();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_Oen();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_Oen();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_Oen();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_Oen();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_Oen();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_Oen();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_Oen();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_Oen();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_Oen();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_Oen();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_Oen();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_Oen();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_Oen();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_Oen();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_Oen();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_Oen();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_Oen();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_Oen();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_Oen();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_Oen();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_Oen();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_Oen();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_Oen();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_Oen();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_Oen();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_Oen();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_Oen();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_Oen();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_Oen();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_Oen();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_Oen();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_Oen();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_Oen();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_Oen();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_Oen();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_Oen();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_Oen();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_Oen();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_Oen();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_Oen();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_Oen();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_Oen();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_Oen();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_Oen();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_Oen();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_Oen();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_Oen();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_Oen();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_Oen();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_Oen();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_Oen();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_Oen();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_Oen();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_Oen();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_Oen();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_Oen();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_Oen();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_Oen();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_Oen();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_Oen();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_Oen();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_Oen();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_Oen();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_Oen();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_Oen();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_Oen();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_Oen();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_Oen();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_Oen();
            break;
        case PAD_DDCR_DA2:
            MHal_GPIO_DDCR_DA2_Oen();
            break;
        case PAD_DDCR_CK2:
            MHal_GPIO_DDCR_CK2_Oen();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_Oen();
            break;
        case PAD_LVSYNC2:
            MHal_GPIO_LVSYNC2_Oen();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_Oen();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_Oen();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_Oen();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_Oen();
            break;
        case PAD_GPIO12:
            MHal_GPIO_GPIO12_Oen();
            break;
        case PAD_GPIO13:
            MHal_GPIO_GPIO13_Oen();
            break;
        case PAD_GPIO14:
            MHal_GPIO_GPIO14_Oen();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_Oen();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_Oen();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_Oen();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_Oen();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_Oen();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_Oen();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_Oen();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_Oen();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_Oen();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_Oen();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_Oen();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_Oen();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_Oen();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_Oen();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_Oen();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_Oen();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_Oen();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_Oen();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_Oen();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_Oen();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_Oen();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_Oen();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_Oen();
            break;
#ifndef DEBUG_MICOM_MODULE
        case PAD_GPIO_PM0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM0_OEN);
            u8RegTemp &= 0xfe;
            MDrv_MICOM_RegWrite(REG_GPIO_PM0_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM1_OEN);
            u8RegTemp &= 0xfd;
            MDrv_MICOM_RegWrite(REG_GPIO_PM1_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM2_OEN);
            u8RegTemp &= 0xfb;
            MDrv_MICOM_RegWrite(REG_GPIO_PM2_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM3_OEN);
            u8RegTemp &= 0xf7;
            MDrv_MICOM_RegWrite(REG_GPIO_PM3_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM4:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM4_OEN);
            u8RegTemp &= 0xef;
            MDrv_MICOM_RegWrite(REG_GPIO_PM4_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM5:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM5_OEN);
            u8RegTemp &= 0xdf;
            MDrv_MICOM_RegWrite(REG_GPIO_PM5_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM6:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM6_OEN);
            u8RegTemp &= 0xbf;
            MDrv_MICOM_RegWrite(REG_GPIO_PM6_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM7:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM7_OEN);
            u8RegTemp &= 0x7f;
            MDrv_MICOM_RegWrite(REG_GPIO_PM7_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM8:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM8_OEN);
            u8RegTemp &= 0xfe;
            MDrv_MICOM_RegWrite(REG_GPIO_PM8_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM9:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM9_OEN);
            u8RegTemp &= 0xfd;
            MDrv_MICOM_RegWrite(REG_GPIO_PM9_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM10:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM10_OEN);
            u8RegTemp &= 0xfb;
            MDrv_MICOM_RegWrite(REG_GPIO_PM10_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR0_OEN);
            u8RegTemp &= 0xef;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR0_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR1_OEN);
            u8RegTemp &= 0xdf;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR1_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR2_OEN);
            u8RegTemp &= 0xbf;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR2_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR3_OEN);
            u8RegTemp &= 0x7f;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR3_OEN,u8RegTemp);
            break;
#endif	/* #ifndef DEBUG_MICOM_MODULE */
        case PAD_TCON0:
            MHal_GPIO_TCON0_Oen();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_Oen();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_Oen();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_Oen();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_Oen();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_Oen();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_Oen();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_Oen();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_Oen();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_Oen();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_Oen();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_Oen();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_Oen();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_Oen();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_Oen();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_Oen();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_Oen();
            break;
        case PAD_HOTPLUGB:
            MHal_GPIO_HOTPLUGB_Oen();
            break;
        case PAD_HOTPLUGA:
            MHal_GPIO_HOTPLUGA_Oen();
            break;
        case PAD_HOTPLUGC:
            MHal_GPIO_HOTPLUGC_Oen();
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Pad_Oen);

//-------------------------------------------------------------------------------------------------
/// disable output for selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Pad_Odn(U8 u8IndexGPIO)
{
#ifndef DEBUG_MICOM_MODULE
    U8  u8RegTemp;
#endif
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_Odn();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_Odn();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_Odn();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_Odn();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_Odn();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_Odn();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_Odn();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_Odn();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_Odn();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_Odn();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_Odn();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_Odn();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_Odn();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_Odn();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_Odn();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_Odn();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_Odn();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_Odn();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_Odn();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_Odn();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_Odn();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_Odn();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_Odn();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_Odn();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_Odn();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_Odn();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_Odn();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_Odn();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_Odn();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_Odn();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_Odn();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_Odn();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_Odn();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_Odn();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_Odn();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_Odn();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_Odn();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_Odn();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_Odn();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_Odn();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_Odn();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_Odn();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_Odn();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_Odn();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_Odn();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_Odn();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_Odn();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_Odn();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_Odn();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_Odn();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_Odn();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_Odn();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_Odn();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_Odn();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_Odn();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_Odn();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_Odn();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_Odn();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_Odn();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_Odn();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_Odn();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_Odn();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_Odn();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_Odn();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_Odn();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_Odn();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_Odn();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_Odn();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_Odn();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_Odn();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_Odn();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_Odn();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_Odn();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_Odn();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_Odn();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_Odn();
            break;
        case PAD_DDCR_DA2:
            MHal_GPIO_DDCR_DA2_Odn();
            break;
        case PAD_DDCR_CK2:
            MHal_GPIO_DDCR_CK2_Odn();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_Odn();
            break;
        case PAD_LVSYNC2:
            MHal_GPIO_LVSYNC2_Odn();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_Odn();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_Odn();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_Odn();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_Odn();
            break;
        case PAD_GPIO12:
            MHal_GPIO_GPIO12_Odn();
            break;
        case PAD_GPIO13:
            MHal_GPIO_GPIO13_Odn();
            break;
        case PAD_GPIO14:
            MHal_GPIO_GPIO14_Odn();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_Odn();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_Odn();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_Odn();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_Odn();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_Odn();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_Odn();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_Odn();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_Odn();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_Odn();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_Odn();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_Odn();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_Odn();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_Odn();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_Odn();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_Odn();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_Odn();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_Odn();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_Odn();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_Odn();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_Odn();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_Odn();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_Odn();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_Odn();
            break;
#ifndef DEBUG_MICOM_MODULE
        case PAD_GPIO_PM0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM0_OEN);
            u8RegTemp |= 0x01;
            MDrv_MICOM_RegWrite(REG_GPIO_PM0_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM1_OEN);
            u8RegTemp |= 0x02;
            MDrv_MICOM_RegWrite(REG_GPIO_PM1_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM2_OEN);
            u8RegTemp |= 0x04;
            MDrv_MICOM_RegWrite(REG_GPIO_PM2_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM3_OEN);
            u8RegTemp |= 0x08;
            MDrv_MICOM_RegWrite(REG_GPIO_PM3_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM4:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM4_OEN);
            u8RegTemp |= 0x10;
            MDrv_MICOM_RegWrite(REG_GPIO_PM4_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM5:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM5_OEN);
            u8RegTemp |= 0x20;
            MDrv_MICOM_RegWrite(REG_GPIO_PM5_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM6:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM6_OEN);
            u8RegTemp |= 0x40;
            MDrv_MICOM_RegWrite(REG_GPIO_PM6_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM7:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM7_OEN);
            u8RegTemp |= 0x80;
            MDrv_MICOM_RegWrite(REG_GPIO_PM7_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM8:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM8_OEN);
            u8RegTemp |= 0x01;
            MDrv_MICOM_RegWrite(REG_GPIO_PM8_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM9:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM9_OEN);
            u8RegTemp |= 0x02;
            MDrv_MICOM_RegWrite(REG_GPIO_PM9_OEN,u8RegTemp);
            break;
        case PAD_GPIO_PM10:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM10_OEN);
            u8RegTemp |= 0x10;
            MDrv_MICOM_RegWrite(REG_GPIO_PM10_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR0_OEN);
            u8RegTemp |= 0x10;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR0_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR1_OEN);
            u8RegTemp |= 0x20;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR1_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR2_OEN);
            u8RegTemp |= 0x40;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR2_OEN,u8RegTemp);
            break;
        case PAD_GPIO_SAR3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR3_OEN);
            u8RegTemp |= 0x80;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR3_OEN,u8RegTemp);
            break;
#endif	/* #ifndef DEBUG_MICOM_MODULE */
        case PAD_TCON0:
            MHal_GPIO_TCON0_Odn();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_Odn();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_Odn();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_Odn();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_Odn();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_Odn();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_Odn();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_Odn();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_Odn();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_Odn();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_Odn();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_Odn();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_Odn();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_Odn();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_Odn();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_Odn();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_Odn();
            break;
        case PAD_HOTPLUGB:
            MHal_GPIO_HOTPLUGB_Odn();
            break;
        case PAD_HOTPLUGA:
            MHal_GPIO_HOTPLUGA_Odn();
            break;
        case PAD_HOTPLUGC:
            MHal_GPIO_HOTPLUGC_Odn();
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Pad_Odn);

//-------------------------------------------------------------------------------------------------
/// read data from selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
U8 MDrv_GPIO_Pad_Read(U8 u8IndexGPIO)
{
#ifndef DEBUG_MICOM_MODULE
    U8  u8RegTemp;
#endif
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            return MHal_GPIO_UART_RX2_In();
            break;
        case PAD_UART_TX2:
            return MHal_GPIO_UART_TX2_In();
            break;
        case PAD_TS0_D0:
            return MHal_GPIO_TS0_D0_In();
            break;
        case PAD_TS0_D1:
            return MHal_GPIO_TS0_D1_In();
            break;
        case PAD_TS0_D2:
            return MHal_GPIO_TS0_D2_In();
            break;
        case PAD_TS0_D3:
            return MHal_GPIO_TS0_D3_In();
            break;
        case PAD_TS0_D4:
            return MHal_GPIO_TS0_D4_In();
            break;
        case PAD_TS0_D5:
            return MHal_GPIO_TS0_D5_In();
            break;
        case PAD_TS0_D6:
            return MHal_GPIO_TS0_D6_In();
            break;
        case PAD_TS0_D7:
            return MHal_GPIO_TS0_D7_In();
            break;
        case PAD_TS0_VLD:
            return MHal_GPIO_TS0_VLD_In();
            break;
        case PAD_TS0_SYNC:
            return MHal_GPIO_TS0_SYNC_In();
            break;
        case PAD_TS0_CLK:
            return MHal_GPIO_TS0_CLK_In();
            break;
        case PAD_PF_AD15:
            return MHal_GPIO_PF_AD15_In();
            break;
        case PAD_PF_CE0Z:
            return MHal_GPIO_PF_CE0Z_In();
            break;
        case PAD_PF_CE1Z:
            return MHal_GPIO_PF_CE1Z_In();
            break;
        case PAD_PF_OEZ:
            return MHal_GPIO_PF_OEZ_In();
            break;
        case PAD_PF_WEZ:
            return MHal_GPIO_PF_WEZ_In();
            break;
        case PAD_PF_ALE:
            return MHal_GPIO_PF_ALE_In();
            break;
        case PAD_F_RBZ:
            return MHal_GPIO_F_RBZ_In();
            break;
        case PAD_PCM_D3:
            return MHal_GPIO_PCM_D3_In();
            break;
        case PAD_PCM_D4:
            return MHal_GPIO_PCM_D4_In();
            break;
        case PAD_PCM_D5:
            return MHal_GPIO_PCM_D5_In();
            break;
        case PAD_PCM_D6:
            return MHal_GPIO_PCM_D6_In();
            break;
        case PAD_PCM_D7:
            return MHal_GPIO_PCM_D7_In();
            break;
        case PAD_PCM_CE_N:
            return MHal_GPIO_PCM_CE_N_In();
            break;
        case PAD_PCM_A10:
            return MHal_GPIO_PCM_A10_In();
            break;
        case PAD_PCM_OE_N:
            return MHal_GPIO_PCM_OE_N_In();
            break;
        case PAD_PCM_A11:
            return MHal_GPIO_PCM_A11_In();
            break;
        case PAD_PCM_IORD_N:
            return MHal_GPIO_PCM_IORD_N_In();
            break;
        case PAD_PCM_A9:
            return MHal_GPIO_PCM_A9_In();
            break;
        case PAD_PCM_IOWR_N:
            return MHal_GPIO_PCM_IOWR_N_In();
            break;
        case PAD_PCM_A8:
            return MHal_GPIO_PCM_A8_In();
            break;
        case PAD_PCM_A13:
            return MHal_GPIO_PCM_A13_In();
            break;
        case PAD_PCM_A14:
            return MHal_GPIO_PCM_A14_In();
            break;
        case PAD_PCM_IRQA_N:
            return MHal_GPIO_PCM_IRQA_N_In();
            break;
        case PAD_PCM_A12:
            return MHal_GPIO_PCM_A12_In();
            break;
        case PAD_PCM_A7:
            return MHal_GPIO_PCM_A7_In();
            break;
        case PAD_PCM_A6:
            return MHal_GPIO_PCM_A6_In();
            break;
        case PAD_PCM_A5:
            return MHal_GPIO_PCM_A5_In();
            break;
        case PAD_PCM_WAIT_N:
            return MHal_GPIO_PCM_WAIT_N_In();
            break;
        case PAD_PCM_A4:
            return MHal_GPIO_PCM_A4_In();
            break;
        case PAD_PCM2_CE_N:
            return MHal_GPIO_PCM2_CE_N_In();
            break;
        case PAD_PCM2_IRQA_N:
            return MHal_GPIO_PCM2_IRQA_N_In();
            break;
        case PAD_PCM2_WAIT_N:
            return MHal_GPIO_PCM2_WAIT_N_In();
            break;
        case PAD_PWM0:
            return MHal_GPIO_PWM0_In();
            break;
        case PAD_PWM1:
            return MHal_GPIO_PWM1_In();
            break;
        case PAD_PWM2:
            return MHal_GPIO_PWM2_In();
            break;
        case PAD_PWM3:
            return MHal_GPIO_PWM3_In();
            break;
        case PAD_DDCR_DA:
            return MHal_GPIO_DDCR_DA_In();
            break;
        case PAD_DDCR_CK:
            return MHal_GPIO_DDCR_CK_In();
            break;
        case PAD_PCM_A3:
            return MHal_GPIO_PCM_A3_In();
            break;
        case PAD_PCM_A2:
            return MHal_GPIO_PCM_A2_In();
            break;
        case PAD_PCM_REG_N:
            return MHal_GPIO_PCM_REG_N_In();
            break;
        case PAD_PCM_A1:
            return MHal_GPIO_PCM_A1_In();
            break;
        case PAD_PCM_A0:
            return MHal_GPIO_PCM_A0_In();
            break;
        case PAD_PCM_D0:
            return MHal_GPIO_PCM_D0_In();
            break;
        case PAD_PCM_D1:
            return MHal_GPIO_PCM_D1_In();
            break;
        case PAD_PCM_D2:
            return MHal_GPIO_PCM_D2_In();
            break;
        case PAD_PCM_RESET:
            return MHal_GPIO_PCM_RESET_In();
            break;
        case PAD_PCM2_RESET:
            return MHal_GPIO_PCM2_RESET_In();
            break;
        case PAD_PCM_CD_N:
            return MHal_GPIO_PCM_CD_N_In();
            break;
        case PAD_PCM2_CD_N:
            return MHal_GPIO_PCM2_CD_N_In();
            break;
        case PAD_TS1_D0:
            return MHal_GPIO_TS1_D0_In();
            break;
        case PAD_TS1_VLD:
            return MHal_GPIO_TS1_VLD_In();
            break;
        case PAD_TS1_SYNC:
            return MHal_GPIO_TS1_SYNC_In();
            break;
        case PAD_TS1_CLK:
            return MHal_GPIO_TS1_CLK_In();
            break;
        case PAD_I2S_IN_WS:
            return MHal_GPIO_I2S_IN_WS_In();
            break;
        case PAD_I2S_IN_BCK:
            return MHal_GPIO_I2S_IN_BCK_In();
            break;
        case PAD_I2S_IN_SD:
            return MHal_GPIO_I2S_IN_SD_In();
            break;
        case PAD_SPDIF_IN:
            return MHal_GPIO_SPDIF_IN_In();
            break;
        case PAD_I2S_OUT_MCK:
            return MHal_GPIO_I2S_OUT_MCK_In();
            break;
        case PAD_I2S_OUT_WS:
            return MHal_GPIO_I2S_OUT_WS_In();
            break;
        case PAD_I2S_OUT_BCK:
            return MHal_GPIO_I2S_OUT_BCK_In();
            break;
        case PAD_I2S_OUT_SD:
            return MHal_GPIO_I2S_OUT_SD_In();
            break;
        case PAD_SPDIF_OUT:
            return MHal_GPIO_SPDIF_OUT_In();
            break;
        case PAD_DDCR_DA2:
            return MHal_GPIO_DDCR_DA2_In();
            break;
        case PAD_DDCR_CK2:
            return MHal_GPIO_DDCR_CK2_In();
            break;
        case PAD_LHSYNC2:
            return MHal_GPIO_LHSYNC2_In();
            break;
        case PAD_LVSYNC2:
            return MHal_GPIO_LVSYNC2_In();
            break;
        case PAD_SPI_CK:
            return MHal_GPIO_SPI_CK_In();
            break;
        case PAD_SPI_DI:
            return MHal_GPIO_SPI_DI_In();
            break;
        case PAD_SPI_CZ:
            return MHal_GPIO_SPI_CZ_In();
            break;
        case PAD_SPI_DO:
            return MHal_GPIO_SPI_DO_In();
            break;
        case PAD_GPIO12:
            return MHal_GPIO_GPIO12_In();
            break;
        case PAD_GPIO13:
            return MHal_GPIO_GPIO13_In();
            break;
        case PAD_GPIO14:
            return MHal_GPIO_GPIO14_In();
            break;
        case PAD_GPIO15:
            return MHal_GPIO_GPIO15_In();
            break;
        case PAD_ET_CRS:
            return MHal_GPIO_ET_CRS_In();
            break;
        case PAD_ET_COL:
            return MHal_GPIO_ET_COL_In();
            break;
        case PAD_ET_TXD3:
            return MHal_GPIO_ET_TXD3_In();
            break;
        case PAD_ET_TXD2:
            return MHal_GPIO_ET_TXD2_In();
            break;
        case PAD_ET_TXD1:
            return MHal_GPIO_ET_TXD1_In();
            break;
        case PAD_ET_TXD0:
            return MHal_GPIO_ET_TXD0_In();
            break;
        case PAD_ET_TX_EN:
            return MHal_GPIO_ET_TX_EN_In();
            break;
        case PAD_ET_TX_CLK:
            return MHal_GPIO_ET_TX_CLK_In();
            break;
        case PAD_ET_TXER:
            return MHal_GPIO_ET_TXER_In();
            break;
        case PAD_ET_RXER:
            return MHal_GPIO_ET_RXER_In();
            break;
        case PAD_ET_RX_CLK:
            return MHal_GPIO_ET_RX_CLK_In();
            break;
        case PAD_ET_RX_DV:
            return MHal_GPIO_ET_RX_DV_In();
            break;
        case PAD_ET_RXD0:
            return MHal_GPIO_ET_RXD0_In();
            break;
        case PAD_ET_RXD1:
            return MHal_GPIO_ET_RXD1_In();
            break;
        case PAD_ET_RXD2:
            return MHal_GPIO_ET_RXD2_In();
            break;
        case PAD_ET_RXD3:
            return MHal_GPIO_ET_RXD3_In();
            break;
        case PAD_ET_MDC:
            return MHal_GPIO_ET_MDC_In();
            break;
        case PAD_ET_MDIO:
            return MHal_GPIO_ET_MDIO_In();
            break;
        case PAD_LCK:
            return MHal_GPIO_LCK_In();
            break;
        case PAD_LDE:
            return MHal_GPIO_LDE_In();
            break;
        case PAD_LHSYNC:
            return MHal_GPIO_LHSYNC_In();
            break;
        case PAD_LVSYNC:
            return MHal_GPIO_LVSYNC_In();
            break;
#ifndef DEBUG_MICOM_MODULE
        case PAD_GPIO_PM0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM0_IN);
            u8RegTemp &= 0x01;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM1_IN);
            u8RegTemp &= 0x02;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM2_IN);
            u8RegTemp &= 0x04;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM3_IN);
            u8RegTemp &= 0x08;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM4:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM4_IN);
            u8RegTemp &= 0x10;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM5:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM5_IN);
            u8RegTemp &= 0x20;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM6:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM6_IN);
            u8RegTemp &= 0x40;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM7:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM7_IN);
            u8RegTemp &= 0x80;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM8:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM8_IN);
            u8RegTemp &= 0x01;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM9:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM9_IN);
            u8RegTemp &= 0x02;
            return (u8RegTemp);
            break;
        case PAD_GPIO_PM10:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM10_IN);
            u8RegTemp &= 0x04;
            return (u8RegTemp);
            break;
        case PAD_GPIO_SAR0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR0_IN);
            u8RegTemp &= 0x01;
            return (u8RegTemp);
            break;
        case PAD_GPIO_SAR1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR1_IN);
            u8RegTemp &= 0x02;
            return (u8RegTemp);
            break;
        case PAD_GPIO_SAR2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR2_IN);
            u8RegTemp &= 0x04;
            return (u8RegTemp);
            break;
        case PAD_GPIO_SAR3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR3_IN);
            u8RegTemp &= 0x08;
            return (u8RegTemp);
            break;
#endif	/* #ifndef DEBUG_MICOM_MODULE */
        case PAD_TCON0:
            return MHal_GPIO_TCON0_In();
            break;
        case PAD_TCON1:
            return MHal_GPIO_TCON1_In();
            break;
        case PAD_TCON2:
            return MHal_GPIO_TCON2_In();
            break;
        case PAD_TCON3:
            return MHal_GPIO_TCON3_In();
            break;
        case PAD_TCON4:
            return MHal_GPIO_TCON4_In();
            break;
        case PAD_TCON5:
            return MHal_GPIO_TCON5_In();
            break;
        case PAD_TCON6:
            return MHal_GPIO_TCON6_In();
            break;
        case PAD_TCON7:
            return MHal_GPIO_TCON7_In();
            break;
        case PAD_TCON8:
            return MHal_GPIO_TCON8_In();
            break;
        case PAD_TCON9:
            return MHal_GPIO_TCON9_In();
            break;
        case PAD_TCON10:
            return MHal_GPIO_TCON10_In();
            break;
        case PAD_TCON11:
            return MHal_GPIO_TCON11_In();
            break;
        case PAD_TCON12:
            return MHal_GPIO_TCON12_In();
            break;
        case PAD_TCON13:
            return MHal_GPIO_TCON13_In();
            break;
        case PAD_TCON14:
            return MHal_GPIO_TCON14_In();
            break;
        case PAD_TCON15:
            return MHal_GPIO_TCON15_In();
            break;
        case PAD_TCON16:
            return MHal_GPIO_TCON16_In();
            break;
        case PAD_HOTPLUGB:
            return MHal_GPIO_HOTPLUGB_In();
            break;
        case PAD_HOTPLUGA:
            return MHal_GPIO_HOTPLUGA_In();
            break;
        case PAD_HOTPLUGC:
            return MHal_GPIO_HOTPLUGC_In();
            break;
        default:
            return 255;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Pad_Read);

//-------------------------------------------------------------------------------------------------
/// output pull high for selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Pull_High(U8 u8IndexGPIO)
{
#ifndef DEBUG_MICOM_MODULE
    U8  u8RegTemp;
#endif
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_High();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_High();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_High();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_High();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_High();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_High();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_High();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_High();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_High();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_High();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_High();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_High();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_High();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_High();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_High();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_High();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_High();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_High();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_High();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_High();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_High();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_High();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_High();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_High();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_High();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_High();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_High();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_High();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_High();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_High();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_High();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_High();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_High();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_High();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_High();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_High();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_High();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_High();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_High();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_High();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_High();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_High();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_High();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_High();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_High();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_High();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_High();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_High();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_High();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_High();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_High();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_High();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_High();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_High();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_High();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_High();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_High();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_High();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_High();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_High();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_High();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_High();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_High();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_High();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_High();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_High();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_High();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_High();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_High();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_High();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_High();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_High();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_High();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_High();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_High();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_High();
            break;
        case PAD_DDCR_DA2:
            MHal_GPIO_DDCR_DA2_High();
            break;
        case PAD_DDCR_CK2:
            MHal_GPIO_DDCR_CK2_High();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_High();
            break;
        case PAD_LVSYNC2:
            MHal_GPIO_LVSYNC2_High();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_High();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_High();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_High();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_High();
            break;
        case PAD_GPIO12:
            MHal_GPIO_GPIO12_High();
            break;
        case PAD_GPIO13:
            MHal_GPIO_GPIO13_High();
            break;
        case PAD_GPIO14:
            MHal_GPIO_GPIO14_High();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_High();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_High();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_High();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_High();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_High();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_High();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_High();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_High();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_High();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_High();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_High();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_High();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_High();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_High();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_High();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_High();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_High();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_High();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_High();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_High();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_High();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_High();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_High();
            break;
#ifndef DEBUG_MICOM_MODULE
        case PAD_GPIO_PM0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM0_OUT);
            u8RegTemp |= 0x01;
            MDrv_MICOM_RegWrite(REG_GPIO_PM0_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM1_OUT);
            u8RegTemp |= 0x02;
            MDrv_MICOM_RegWrite(REG_GPIO_PM1_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM2_OUT);
            u8RegTemp |= 0x04;
            MDrv_MICOM_RegWrite(REG_GPIO_PM2_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM3_OUT);
            u8RegTemp |= 0x08;
            MDrv_MICOM_RegWrite(REG_GPIO_PM3_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM4:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM4_OUT);
            u8RegTemp |= 0x10;
            MDrv_MICOM_RegWrite(REG_GPIO_PM4_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM5:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM5_OUT);
            u8RegTemp |= 0x20;
            MDrv_MICOM_RegWrite(REG_GPIO_PM5_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM6:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM6_OUT);
            u8RegTemp |= 0x40;
            MDrv_MICOM_RegWrite(REG_GPIO_PM6_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM7:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM7_OUT);
            u8RegTemp |= 0x80;
            MDrv_MICOM_RegWrite(REG_GPIO_PM7_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM8:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM8_OUT);
            u8RegTemp |= 0x01;
            MDrv_MICOM_RegWrite(REG_GPIO_PM8_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM9:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM9_OUT);
            u8RegTemp |= 0x02;
            MDrv_MICOM_RegWrite(REG_GPIO_PM9_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM10:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM10_OUT);
            u8RegTemp |= 0x04;
            MDrv_MICOM_RegWrite(REG_GPIO_PM10_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR0_OUT);
            u8RegTemp |= 0x01;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR0_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR1_OUT);
            u8RegTemp |= 0x02;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR1_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR2_OUT);
            u8RegTemp |= 0x04;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR2_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR3_OUT);
            u8RegTemp |= 0x08;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR3_OUT,u8RegTemp);
            break;
#endif	/* #ifndef DEBUG_MICOM_MODULE */
        case PAD_TCON0:
            MHal_GPIO_TCON0_High();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_High();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_High();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_High();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_High();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_High();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_High();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_High();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_High();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_High();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_High();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_High();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_High();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_High();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_High();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_High();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_High();
            break;
        case PAD_HOTPLUGA:
            MHal_GPIO_HOTPLUGA_High();
            break;
        case PAD_HOTPLUGB:
            MHal_GPIO_HOTPLUGB_High();
            break;
        case PAD_HOTPLUGC:
            MHal_GPIO_HOTPLUGC_High();
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Pull_High);

//-------------------------------------------------------------------------------------------------
/// output pull low for selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Pull_Low(U8 u8IndexGPIO)
{
#ifndef DEBUG_MICOM_MODULE
    U8  u8RegTemp;
#endif
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_Low();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_Low();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_Low();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_Low();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_Low();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_Low();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_Low();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_Low();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_Low();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_Low();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_Low();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_Low();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_Low();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_Low();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_Low();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_Low();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_Low();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_Low();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_Low();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_Low();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_Low();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_Low();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_Low();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_Low();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_Low();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_Low();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_Low();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_Low();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_Low();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_Low();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_Low();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_Low();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_Low();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_Low();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_Low();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_Low();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_Low();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_Low();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_Low();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_Low();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_Low();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_Low();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_Low();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_Low();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_Low();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_Low();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_Low();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_Low();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_Low();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_Low();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_Low();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_Low();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_Low();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_Low();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_Low();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_Low();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_Low();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_Low();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_Low();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_Low();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_Low();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_Low();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_Low();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_Low();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_Low();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_Low();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_Low();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_Low();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_Low();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_Low();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_Low();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_Low();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_Low();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_Low();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_Low();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_Low();
            break;
        case PAD_DDCR_DA2:
            MHal_GPIO_DDCR_DA2_Low();
            break;
        case PAD_DDCR_CK2:
            MHal_GPIO_DDCR_CK2_Low();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_Low();
            break;
        case PAD_LVSYNC2:
            MHal_GPIO_LVSYNC2_Low();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_Low();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_Low();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_Low();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_Low();
            break;
        case PAD_GPIO12:
            MHal_GPIO_GPIO12_Low();
            break;
        case PAD_GPIO13:
            MHal_GPIO_GPIO13_Low();
            break;
        case PAD_GPIO14:
            MHal_GPIO_GPIO14_Low();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_Low();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_Low();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_Low();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_Low();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_Low();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_Low();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_Low();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_Low();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_Low();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_Low();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_Low();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_Low();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_Low();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_Low();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_Low();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_Low();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_Low();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_Low();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_Low();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_Low();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_Low();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_Low();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_Low();
            break;
#ifndef DEBUG_MICOM_MODULE
        case PAD_GPIO_PM0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM0_OUT);
            u8RegTemp &= 0xfe;
            MDrv_MICOM_RegWrite(REG_GPIO_PM0_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM1_OUT);
            u8RegTemp &= 0xfd;
            MDrv_MICOM_RegWrite(REG_GPIO_PM1_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM2_OUT);
            u8RegTemp &= 0xfb;
            MDrv_MICOM_RegWrite(REG_GPIO_PM2_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM3_OUT);
            u8RegTemp &= 0xf7;
            MDrv_MICOM_RegWrite(REG_GPIO_PM3_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM4:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM4_OUT);
            u8RegTemp &= 0xef;
            MDrv_MICOM_RegWrite(REG_GPIO_PM0_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM5:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM5_OUT);
            u8RegTemp &= 0xdf;
            MDrv_MICOM_RegWrite(REG_GPIO_PM5_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM6:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM6_OUT);
            u8RegTemp &= 0xbf;
            MDrv_MICOM_RegWrite(REG_GPIO_PM6_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM7:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM7_OUT);
            u8RegTemp &= 0x7f;
            MDrv_MICOM_RegWrite(REG_GPIO_PM7_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM8:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM8_OUT);
            u8RegTemp &= 0xfe;
            MDrv_MICOM_RegWrite(REG_GPIO_PM8_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM9:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM9_OUT);
            u8RegTemp &= 0xfd;
            MDrv_MICOM_RegWrite(REG_GPIO_PM9_OUT,u8RegTemp);
            break;
        case PAD_GPIO_PM10:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_PM10_OUT);
            u8RegTemp &= 0xfb;
            MDrv_MICOM_RegWrite(REG_GPIO_PM10_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR0:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR0_OUT);
            u8RegTemp &= 0xfe;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR0_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR1:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR1_OUT);
            u8RegTemp &= 0xfd;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR1_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR2:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR2_OUT);
            u8RegTemp &= 0xfb;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR2_OUT,u8RegTemp);
            break;
        case PAD_GPIO_SAR3:
            u8RegTemp = MDrv_MICOM_RegRead(REG_GPIO_SAR3_OUT);
            u8RegTemp &= 0xf7;
            MDrv_MICOM_RegWrite(REG_GPIO_SAR3_OUT,u8RegTemp);
            break;
#endif	/* #ifndef DEBUG_MICOM_MODULE */
        case PAD_TCON0:
            MHal_GPIO_TCON0_Low();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_Low();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_Low();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_Low();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_Low();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_Low();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_Low();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_Low();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_Low();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_Low();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_Low();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_Low();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_Low();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_Low();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_Low();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_Low();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_Low();
            break;
        case PAD_HOTPLUGB:
            MHal_GPIO_HOTPLUGB_Low();
            break;
        case PAD_HOTPLUGA:
            MHal_GPIO_HOTPLUGA_Low();
            break;
        case PAD_HOTPLUGC:
            MHal_GPIO_HOTPLUGC_Low();
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Pull_Low);

//-------------------------------------------------------------------------------------------------
/// output pull high for selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Set_High(U8 u8IndexGPIO)
{
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_Set();
            MHal_GPIO_UART_RX2_Oen();
            MHal_GPIO_UART_RX2_High();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_Set();
            MHal_GPIO_UART_TX2_Oen();
            MHal_GPIO_UART_TX2_High();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_Set();
            MHal_GPIO_TS0_D0_Oen();
            MHal_GPIO_TS0_D0_High();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_Set();
            MHal_GPIO_TS0_D1_Oen();
            MHal_GPIO_TS0_D1_High();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_Set();
            MHal_GPIO_TS0_D2_Oen();
            MHal_GPIO_TS0_D2_High();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_Set();
            MHal_GPIO_TS0_D3_Oen();
            MHal_GPIO_TS0_D3_High();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_Set();
            MHal_GPIO_TS0_D4_Oen();
            MHal_GPIO_TS0_D4_High();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_Set();
            MHal_GPIO_TS0_D5_Oen();
            MHal_GPIO_TS0_D5_High();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_Set();
            MHal_GPIO_TS0_D6_Oen();
            MHal_GPIO_TS0_D6_High();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_Set();
            MHal_GPIO_TS0_D7_Oen();
            MHal_GPIO_TS0_D7_High();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_Set();
            MHal_GPIO_TS0_VLD_Oen();
            MHal_GPIO_TS0_VLD_High();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_Set();
            MHal_GPIO_TS0_SYNC_Oen();
            MHal_GPIO_TS0_SYNC_High();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_Set();
            MHal_GPIO_TS0_CLK_Oen();
            MHal_GPIO_TS0_CLK_High();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_Set();
            MHal_GPIO_PF_AD15_Oen();
            MHal_GPIO_PF_AD15_High();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_Set();
            MHal_GPIO_PF_CE0Z_Oen();
            MHal_GPIO_PF_CE0Z_High();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_Set();
            MHal_GPIO_PF_CE1Z_Oen();
            MHal_GPIO_PF_CE1Z_High();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_Set();
            MHal_GPIO_PF_OEZ_Oen();
            MHal_GPIO_PF_OEZ_High();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_Set();
            MHal_GPIO_PF_WEZ_Oen();
            MHal_GPIO_PF_WEZ_High();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_Set();
            MHal_GPIO_PF_ALE_Oen();
            MHal_GPIO_PF_ALE_High();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_Set();
            MHal_GPIO_F_RBZ_Oen();
            MHal_GPIO_F_RBZ_High();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_Set();
            MHal_GPIO_PCM_D3_Oen();
            MHal_GPIO_PCM_D3_High();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_Set();
            MHal_GPIO_PCM_D4_Oen();
            MHal_GPIO_PCM_D4_High();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_Set();
            MHal_GPIO_PCM_D5_Oen();
            MHal_GPIO_PCM_D5_High();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_Set();
            MHal_GPIO_PCM_D6_Oen();
            MHal_GPIO_PCM_D6_High();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_Set();
            MHal_GPIO_PCM_D7_Oen();
            MHal_GPIO_PCM_D7_High();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_Set();
            MHal_GPIO_PCM_CE_N_Oen();
            MHal_GPIO_PCM_CE_N_High();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_Set();
            MHal_GPIO_PCM_A10_Oen();
            MHal_GPIO_PCM_A10_High();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_Set();
            MHal_GPIO_PCM_OE_N_Oen();
            MHal_GPIO_PCM_OE_N_High();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_Set();
            MHal_GPIO_PCM_A11_Oen();
            MHal_GPIO_PCM_A11_High();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_Set();
            MHal_GPIO_PCM_IORD_N_Oen();
            MHal_GPIO_PCM_IORD_N_High();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_Set();
            MHal_GPIO_PCM_A9_Oen();
            MHal_GPIO_PCM_A9_High();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_Set();
            MHal_GPIO_PCM_IOWR_N_Oen();
            MHal_GPIO_PCM_IOWR_N_High();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_Set();
            MHal_GPIO_PCM_A8_Oen();
            MHal_GPIO_PCM_A8_High();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_Set();
            MHal_GPIO_PCM_A13_Oen();
            MHal_GPIO_PCM_A13_High();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_Set();
            MHal_GPIO_PCM_A14_Oen();
            MHal_GPIO_PCM_A14_High();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_Set();
            MHal_GPIO_PCM_IRQA_N_Oen();
            MHal_GPIO_PCM_IRQA_N_High();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_Set();
            MHal_GPIO_PCM_A12_Oen();
            MHal_GPIO_PCM_A12_High();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_Set();
            MHal_GPIO_PCM_A7_Oen();
            MHal_GPIO_PCM_A7_High();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_Set();
            MHal_GPIO_PCM_A6_Oen();
            MHal_GPIO_PCM_A6_High();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_Set();
            MHal_GPIO_PCM_A5_Oen();
            MHal_GPIO_PCM_A5_High();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_Set();
            MHal_GPIO_PCM_WAIT_N_Oen();
            MHal_GPIO_PCM_WAIT_N_High();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_Set();
            MHal_GPIO_PCM_A4_Oen();
            MHal_GPIO_PCM_A4_High();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_Set();
            MHal_GPIO_PCM2_CE_N_Oen();
            MHal_GPIO_PCM2_CE_N_High();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_Set();
            MHal_GPIO_PCM2_IRQA_N_Oen();
            MHal_GPIO_PCM2_IRQA_N_High();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_Set();
            MHal_GPIO_PCM2_WAIT_N_Oen();
            MHal_GPIO_PCM2_WAIT_N_High();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_Set();
            MHal_GPIO_PWM0_Oen();
            MHal_GPIO_PWM0_High();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_Set();
            MHal_GPIO_PWM1_Oen();
            MHal_GPIO_PWM1_High();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_Set();
            MHal_GPIO_PWM2_Oen();
            MHal_GPIO_PWM2_High();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_Set();
            MHal_GPIO_PWM3_Oen();
            MHal_GPIO_PWM3_High();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_Set();
            MHal_GPIO_DDCR_DA_Oen();
            MHal_GPIO_DDCR_DA_High();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_Set();
            MHal_GPIO_DDCR_CK_Oen();
            MHal_GPIO_DDCR_CK_High();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_Set();
            MHal_GPIO_PCM_A3_Oen();
            MHal_GPIO_PCM_A3_High();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_Set();
            MHal_GPIO_PCM_A2_Oen();
            MHal_GPIO_PCM_A2_High();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_Set();
            MHal_GPIO_PCM_REG_N_Oen();
            MHal_GPIO_PCM_REG_N_High();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_Set();
            MHal_GPIO_PCM_A1_Oen();
            MHal_GPIO_PCM_A1_High();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_Set();
            MHal_GPIO_PCM_A0_Oen();
            MHal_GPIO_PCM_A0_High();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_Set();
            MHal_GPIO_PCM_D0_Oen();
            MHal_GPIO_PCM_D0_High();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_Set();
            MHal_GPIO_PCM_D1_Oen();
            MHal_GPIO_PCM_D1_High();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_Set();
            MHal_GPIO_PCM_D2_Oen();
            MHal_GPIO_PCM_D2_High();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_Set();
            MHal_GPIO_PCM_RESET_Oen();
            MHal_GPIO_PCM_RESET_High();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_Set();
            MHal_GPIO_PCM2_RESET_Oen();
            MHal_GPIO_PCM2_RESET_High();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_Set();
            MHal_GPIO_PCM_CD_N_Oen();
            MHal_GPIO_PCM_CD_N_High();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_Set();
            MHal_GPIO_PCM2_CD_N_Oen();
            MHal_GPIO_PCM2_CD_N_High();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_Set();
            MHal_GPIO_TS1_D0_Oen();
            MHal_GPIO_TS1_D0_High();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_Set();
            MHal_GPIO_TS1_VLD_Oen();
            MHal_GPIO_TS1_VLD_High();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_Set();
            MHal_GPIO_TS1_SYNC_Oen();
            MHal_GPIO_TS1_SYNC_High();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_Set();
            MHal_GPIO_TS1_CLK_Oen();
            MHal_GPIO_TS1_CLK_High();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_Set();
            MHal_GPIO_I2S_IN_WS_Oen();
            MHal_GPIO_I2S_IN_WS_High();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_Set();
            MHal_GPIO_I2S_IN_BCK_Oen();
            MHal_GPIO_I2S_IN_BCK_High();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_Set();
            MHal_GPIO_I2S_IN_SD_Oen();
            MHal_GPIO_I2S_IN_SD_High();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_Set();
            MHal_GPIO_SPDIF_IN_Oen();
            MHal_GPIO_SPDIF_IN_High();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_Set();
            MHal_GPIO_I2S_OUT_MCK_Oen();
            MHal_GPIO_I2S_OUT_MCK_High();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_Set();
            MHal_GPIO_I2S_OUT_WS_Oen();
            MHal_GPIO_I2S_OUT_WS_High();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_Set();
            MHal_GPIO_I2S_OUT_BCK_Oen();
            MHal_GPIO_I2S_OUT_BCK_High();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_Set();
            MHal_GPIO_I2S_OUT_SD_Oen();
            MHal_GPIO_I2S_OUT_SD_High();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_Set();
            MHal_GPIO_SPDIF_OUT_Oen();
            MHal_GPIO_SPDIF_OUT_High();
            break;
        case PAD_DDCR_DA2:
            MHal_GPIO_DDCR_DA2_Oen();
            MHal_GPIO_DDCR_DA2_High();
            break;
        case PAD_DDCR_CK2:
            MHal_GPIO_DDCR_CK2_Oen();
            MHal_GPIO_DDCR_CK2_High();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_Set();
            MHal_GPIO_LHSYNC2_Oen();
            MHal_GPIO_LHSYNC2_High();
            break;
        case PAD_LVSYNC2:
            MHal_GPIO_LVSYNC2_Oen();
            MHal_GPIO_LVSYNC2_High();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_Set();
            MHal_GPIO_SPI_CK_Oen();
            MHal_GPIO_SPI_CK_High();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_Set();
            MHal_GPIO_SPI_DI_Oen();
            MHal_GPIO_SPI_DI_High();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_Set();
            MHal_GPIO_SPI_CZ_Oen();
            MHal_GPIO_SPI_CZ_High();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_Set();
            MHal_GPIO_SPI_DO_Oen();
            MHal_GPIO_SPI_DO_High();
            break;
        case PAD_GPIO12:
            MHal_GPIO_GPIO12_Oen();
            MHal_GPIO_GPIO12_High();
            break;
        case PAD_GPIO13:
            MHal_GPIO_GPIO13_Oen();
            MHal_GPIO_GPIO13_High();
            break;
        case PAD_GPIO14:
            MHal_GPIO_GPIO14_Oen();
            MHal_GPIO_GPIO14_High();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_Set();
            MHal_GPIO_GPIO15_Oen();
            MHal_GPIO_GPIO15_High();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_Set();
            MHal_GPIO_ET_CRS_Oen();
            MHal_GPIO_ET_CRS_High();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_Set();
            MHal_GPIO_ET_COL_Oen();
            MHal_GPIO_ET_COL_High();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_Set();
            MHal_GPIO_ET_TXD3_Oen();
            MHal_GPIO_ET_TXD3_High();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_Set();
            MHal_GPIO_ET_TXD2_Oen();
            MHal_GPIO_ET_TXD2_High();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_Set();
            MHal_GPIO_ET_TXD1_Oen();
            MHal_GPIO_ET_TXD1_High();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_Set();
            MHal_GPIO_ET_TXD0_Oen();
            MHal_GPIO_ET_TXD0_High();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_Set();
            MHal_GPIO_ET_TX_EN_Oen();
            MHal_GPIO_ET_TX_EN_High();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_Set();
            MHal_GPIO_ET_TX_CLK_Oen();
            MHal_GPIO_ET_TX_CLK_High();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_Set();
            MHal_GPIO_ET_TXER_Oen();
            MHal_GPIO_ET_TXER_High();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_Set();
            MHal_GPIO_ET_RXER_Oen();
            MHal_GPIO_ET_RXER_High();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_Set();
            MHal_GPIO_ET_RX_CLK_Oen();
            MHal_GPIO_ET_RX_CLK_High();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_Set();
            MHal_GPIO_ET_RX_DV_Oen();
            MHal_GPIO_ET_RX_DV_High();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_Set();
            MHal_GPIO_ET_RXD0_Oen();
            MHal_GPIO_ET_RXD0_High();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_Set();
            MHal_GPIO_ET_RXD1_Oen();
            MHal_GPIO_ET_RXD1_High();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_Set();
            MHal_GPIO_ET_RXD2_Oen();
            MHal_GPIO_ET_RXD2_High();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_Set();
            MHal_GPIO_ET_RXD3_Oen();
            MHal_GPIO_ET_RXD3_High();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_Set();
            MHal_GPIO_ET_MDC_Oen();
            MHal_GPIO_ET_MDC_High();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_Set();
            MHal_GPIO_ET_MDIO_Oen();
            MHal_GPIO_ET_MDIO_High();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_Set();
            MHal_GPIO_LCK_Oen();
            MHal_GPIO_LCK_High();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_Set();
            MHal_GPIO_LDE_Oen();
            MHal_GPIO_LDE_High();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_Set();
            MHal_GPIO_LHSYNC_Oen();
            MHal_GPIO_LHSYNC_High();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_Set();
            MHal_GPIO_LVSYNC_Oen();
            MHal_GPIO_LVSYNC_High();
            break;
        case PAD_GPIO_PM0:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM0);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM0);
            break;
        case PAD_GPIO_PM1:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM1);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM1);
            break;
        case PAD_GPIO_PM2:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM2);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM2);
            break;
        case PAD_GPIO_PM3:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM3);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM3);
            break;
        case PAD_GPIO_PM4:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM4);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM4);
            break;
        case PAD_GPIO_PM5:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM5);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM5);
            break;
        case PAD_GPIO_PM6:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM6);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM6);
            break;
        case PAD_GPIO_PM7:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM7);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM7);
            break;
        case PAD_GPIO_PM8:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM8);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM8);
            break;
        case PAD_GPIO_PM9:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM9);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM9);
            break;
        case PAD_GPIO_PM10:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM10);
            MDrv_GPIO_Pull_High(PAD_GPIO_PM10);
            break;
        case PAD_GPIO_SAR0:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR0);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR0);
            MDrv_GPIO_Pull_High(PAD_GPIO_SAR0);
            break;
        case PAD_GPIO_SAR1:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR1);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR1);
            MDrv_GPIO_Pull_High(PAD_GPIO_SAR1);
            break;
        case PAD_GPIO_SAR2:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR2);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR2);
            MDrv_GPIO_Pull_High(PAD_GPIO_SAR2);
            break;
        case PAD_GPIO_SAR3:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR3);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR3);
            MDrv_GPIO_Pull_High(PAD_GPIO_SAR3);
            break;
        case PAD_TCON0:
            MHal_GPIO_TCON0_Set();
            MHal_GPIO_TCON0_Oen();
            MHal_GPIO_TCON0_High();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_Set();
            MHal_GPIO_TCON1_Oen();
            MHal_GPIO_TCON1_High();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_Set();
            MHal_GPIO_TCON2_Oen();
            MHal_GPIO_TCON2_High();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_Set();
            MHal_GPIO_TCON3_Oen();
            MHal_GPIO_TCON3_High();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_Set();
            MHal_GPIO_TCON4_Oen();
            MHal_GPIO_TCON4_High();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_Set();
            MHal_GPIO_TCON5_Oen();
            MHal_GPIO_TCON5_High();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_Set();
            MHal_GPIO_TCON6_Oen();
            MHal_GPIO_TCON6_High();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_Set();
            MHal_GPIO_TCON7_Oen();
            MHal_GPIO_TCON7_High();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_Set();
            MHal_GPIO_TCON8_Oen();
            MHal_GPIO_TCON8_High();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_Set();
            MHal_GPIO_TCON9_Oen();
            MHal_GPIO_TCON9_High();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_Set();
            MHal_GPIO_TCON10_Oen();
            MHal_GPIO_TCON10_High();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_Set();
            MHal_GPIO_TCON11_Oen();
            MHal_GPIO_TCON11_High();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_Set();
            MHal_GPIO_TCON12_Oen();
            MHal_GPIO_TCON12_High();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_Set();
            MHal_GPIO_TCON13_Oen();
            MHal_GPIO_TCON13_High();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_Set();
            MHal_GPIO_TCON14_Oen();
            MHal_GPIO_TCON14_High();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_Set();
            MHal_GPIO_TCON15_Oen();
            MHal_GPIO_TCON15_High();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_Set();
            MHal_GPIO_TCON16_Oen();
            MHal_GPIO_TCON16_High();
            break;
        case PAD_HOTPLUGB:
            MDrv_GPIO_Pad_Oen(PAD_HOTPLUGB);
            MDrv_GPIO_Pull_High(PAD_HOTPLUGB);
            break;
        case PAD_HOTPLUGA:
            MDrv_GPIO_Pad_Oen(PAD_HOTPLUGA);
            MDrv_GPIO_Pull_High(PAD_HOTPLUGA);
            break;
        case PAD_HOTPLUGC:
            MDrv_GPIO_Pad_Oen(PAD_HOTPLUGC);
            MDrv_GPIO_Pull_High(PAD_HOTPLUGC);
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Set_High);

//-------------------------------------------------------------------------------------------------
/// output pull low for selected one pad
/// @param  u8IndexGPIO              \b IN:  pad index
/// @return None
/// @note
//-------------------------------------------------------------------------------------------------
void MDrv_GPIO_Set_Low(U8 u8IndexGPIO)
{
    switch(u8IndexGPIO)
    {
        case PAD_UART_RX2:
            MHal_GPIO_UART_RX2_Set();
            MHal_GPIO_UART_RX2_Oen();
            MHal_GPIO_UART_RX2_Low();
            break;
        case PAD_UART_TX2:
            MHal_GPIO_UART_TX2_Set();
            MHal_GPIO_UART_TX2_Oen();
            MHal_GPIO_UART_TX2_Low();
            break;
        case PAD_TS0_D0:
            MHal_GPIO_TS0_D0_Set();
            MHal_GPIO_TS0_D0_Oen();
            MHal_GPIO_TS0_D0_Low();
            break;
        case PAD_TS0_D1:
            MHal_GPIO_TS0_D1_Set();
            MHal_GPIO_TS0_D1_Oen();
            MHal_GPIO_TS0_D1_Low();
            break;
        case PAD_TS0_D2:
            MHal_GPIO_TS0_D2_Set();
            MHal_GPIO_TS0_D2_Oen();
            MHal_GPIO_TS0_D2_Low();
            break;
        case PAD_TS0_D3:
            MHal_GPIO_TS0_D3_Set();
            MHal_GPIO_TS0_D3_Oen();
            MHal_GPIO_TS0_D3_Low();
            break;
        case PAD_TS0_D4:
            MHal_GPIO_TS0_D4_Set();
            MHal_GPIO_TS0_D4_Oen();
            MHal_GPIO_TS0_D4_Low();
            break;
        case PAD_TS0_D5:
            MHal_GPIO_TS0_D5_Set();
            MHal_GPIO_TS0_D5_Oen();
            MHal_GPIO_TS0_D5_Low();
            break;
        case PAD_TS0_D6:
            MHal_GPIO_TS0_D6_Set();
            MHal_GPIO_TS0_D6_Oen();
            MHal_GPIO_TS0_D6_Low();
            break;
        case PAD_TS0_D7:
            MHal_GPIO_TS0_D7_Set();
            MHal_GPIO_TS0_D7_Oen();
            MHal_GPIO_TS0_D7_Low();
            break;
        case PAD_TS0_VLD:
            MHal_GPIO_TS0_VLD_Set();
            MHal_GPIO_TS0_VLD_Oen();
            MHal_GPIO_TS0_VLD_Low();
            break;
        case PAD_TS0_SYNC:
            MHal_GPIO_TS0_SYNC_Set();
            MHal_GPIO_TS0_SYNC_Oen();
            MHal_GPIO_TS0_SYNC_Low();
            break;
        case PAD_TS0_CLK:
            MHal_GPIO_TS0_CLK_Set();
            MHal_GPIO_TS0_CLK_Oen();
            MHal_GPIO_TS0_CLK_Low();
            break;
        case PAD_PF_AD15:
            MHal_GPIO_PF_AD15_Set();
            MHal_GPIO_PF_AD15_Oen();
            MHal_GPIO_PF_AD15_Low();
            break;
        case PAD_PF_CE0Z:
            MHal_GPIO_PF_CE0Z_Set();
            MHal_GPIO_PF_CE0Z_Oen();
            MHal_GPIO_PF_CE0Z_Low();
            break;
        case PAD_PF_CE1Z:
            MHal_GPIO_PF_CE1Z_Set();
            MHal_GPIO_PF_CE1Z_Oen();
            MHal_GPIO_PF_CE1Z_Low();
            break;
        case PAD_PF_OEZ:
            MHal_GPIO_PF_OEZ_Set();
            MHal_GPIO_PF_OEZ_Oen();
            MHal_GPIO_PF_OEZ_Low();
            break;
        case PAD_PF_WEZ:
            MHal_GPIO_PF_WEZ_Set();
            MHal_GPIO_PF_WEZ_Oen();
            MHal_GPIO_PF_WEZ_Low();
            break;
        case PAD_PF_ALE:
            MHal_GPIO_PF_ALE_Set();
            MHal_GPIO_PF_ALE_Oen();
            MHal_GPIO_PF_ALE_Low();
            break;
        case PAD_F_RBZ:
            MHal_GPIO_F_RBZ_Set();
            MHal_GPIO_F_RBZ_Oen();
            MHal_GPIO_F_RBZ_Low();
            break;
        case PAD_PCM_D3:
            MHal_GPIO_PCM_D3_Set();
            MHal_GPIO_PCM_D3_Oen();
            MHal_GPIO_PCM_D3_Low();
            break;
        case PAD_PCM_D4:
            MHal_GPIO_PCM_D4_Set();
            MHal_GPIO_PCM_D4_Oen();
            MHal_GPIO_PCM_D4_Low();
            break;
        case PAD_PCM_D5:
            MHal_GPIO_PCM_D5_Set();
            MHal_GPIO_PCM_D5_Oen();
            MHal_GPIO_PCM_D5_Low();
            break;
        case PAD_PCM_D6:
            MHal_GPIO_PCM_D6_Set();
            MHal_GPIO_PCM_D6_Oen();
            MHal_GPIO_PCM_D6_Low();
            break;
        case PAD_PCM_D7:
            MHal_GPIO_PCM_D7_Set();
            MHal_GPIO_PCM_D7_Oen();
            MHal_GPIO_PCM_D7_Low();
            break;
        case PAD_PCM_CE_N:
            MHal_GPIO_PCM_CE_N_Set();
            MHal_GPIO_PCM_CE_N_Oen();
            MHal_GPIO_PCM_CE_N_Low();
            break;
        case PAD_PCM_A10:
            MHal_GPIO_PCM_A10_Set();
            MHal_GPIO_PCM_A10_Oen();
            MHal_GPIO_PCM_A10_Low();
            break;
        case PAD_PCM_OE_N:
            MHal_GPIO_PCM_OE_N_Set();
            MHal_GPIO_PCM_OE_N_Oen();
            MHal_GPIO_PCM_OE_N_Low();
            break;
        case PAD_PCM_A11:
            MHal_GPIO_PCM_A11_Set();
            MHal_GPIO_PCM_A11_Oen();
            MHal_GPIO_PCM_A11_Low();
            break;
        case PAD_PCM_IORD_N:
            MHal_GPIO_PCM_IORD_N_Set();
            MHal_GPIO_PCM_IORD_N_Oen();
            MHal_GPIO_PCM_IORD_N_Low();
            break;
        case PAD_PCM_A9:
            MHal_GPIO_PCM_A9_Set();
            MHal_GPIO_PCM_A9_Oen();
            MHal_GPIO_PCM_A9_Low();
            break;
        case PAD_PCM_IOWR_N:
            MHal_GPIO_PCM_IOWR_N_Set();
            MHal_GPIO_PCM_IOWR_N_Oen();
            MHal_GPIO_PCM_IOWR_N_Low();
            break;
        case PAD_PCM_A8:
            MHal_GPIO_PCM_A8_Set();
            MHal_GPIO_PCM_A8_Oen();
            MHal_GPIO_PCM_A8_Low();
            break;
        case PAD_PCM_A13:
            MHal_GPIO_PCM_A13_Set();
            MHal_GPIO_PCM_A13_Oen();
            MHal_GPIO_PCM_A13_Low();
            break;
        case PAD_PCM_A14:
            MHal_GPIO_PCM_A14_Set();
            MHal_GPIO_PCM_A14_Oen();
            MHal_GPIO_PCM_A14_Low();
            break;
        case PAD_PCM_IRQA_N:
            MHal_GPIO_PCM_IRQA_N_Set();
            MHal_GPIO_PCM_IRQA_N_Oen();
            MHal_GPIO_PCM_IRQA_N_Low();
            break;
        case PAD_PCM_A12:
            MHal_GPIO_PCM_A12_Set();
            MHal_GPIO_PCM_A12_Oen();
            MHal_GPIO_PCM_A12_Low();
            break;
        case PAD_PCM_A7:
            MHal_GPIO_PCM_A7_Set();
            MHal_GPIO_PCM_A7_Oen();
            MHal_GPIO_PCM_A7_Low();
            break;
        case PAD_PCM_A6:
            MHal_GPIO_PCM_A6_Set();
            MHal_GPIO_PCM_A6_Oen();
            MHal_GPIO_PCM_A6_Low();
            break;
        case PAD_PCM_A5:
            MHal_GPIO_PCM_A5_Set();
            MHal_GPIO_PCM_A5_Oen();
            MHal_GPIO_PCM_A5_Low();
            break;
        case PAD_PCM_WAIT_N:
            MHal_GPIO_PCM_WAIT_N_Set();
            MHal_GPIO_PCM_WAIT_N_Oen();
            MHal_GPIO_PCM_WAIT_N_Low();
            break;
        case PAD_PCM_A4:
            MHal_GPIO_PCM_A4_Set();
            MHal_GPIO_PCM_A4_Oen();
            MHal_GPIO_PCM_A4_Low();
            break;
        case PAD_PCM2_CE_N:
            MHal_GPIO_PCM2_CE_N_Set();
            MHal_GPIO_PCM2_CE_N_Oen();
            MHal_GPIO_PCM2_CE_N_Low();
            break;
        case PAD_PCM2_IRQA_N:
            MHal_GPIO_PCM2_IRQA_N_Set();
            MHal_GPIO_PCM2_IRQA_N_Oen();
            MHal_GPIO_PCM2_IRQA_N_Low();
            break;
        case PAD_PCM2_WAIT_N:
            MHal_GPIO_PCM2_WAIT_N_Set();
            MHal_GPIO_PCM2_WAIT_N_Oen();
            MHal_GPIO_PCM2_WAIT_N_Low();
            break;
        case PAD_PWM0:
            MHal_GPIO_PWM0_Set();
            MHal_GPIO_PWM0_Oen();
            MHal_GPIO_PWM0_Low();
            break;
        case PAD_PWM1:
            MHal_GPIO_PWM1_Set();
            MHal_GPIO_PWM1_Oen();
            MHal_GPIO_PWM1_Low();
            break;
        case PAD_PWM2:
            MHal_GPIO_PWM2_Set();
            MHal_GPIO_PWM2_Oen();
            MHal_GPIO_PWM2_Low();
            break;
        case PAD_PWM3:
            MHal_GPIO_PWM3_Set();
            MHal_GPIO_PWM3_Oen();
            MHal_GPIO_PWM3_Low();
            break;
        case PAD_DDCR_DA:
            MHal_GPIO_DDCR_DA_Set();
            MHal_GPIO_DDCR_DA_Oen();
            MHal_GPIO_DDCR_DA_Low();
            break;
        case PAD_DDCR_CK:
            MHal_GPIO_DDCR_CK_Set();
            MHal_GPIO_DDCR_CK_Oen();
            MHal_GPIO_DDCR_CK_Low();
            break;
        case PAD_PCM_A3:
            MHal_GPIO_PCM_A3_Set();
            MHal_GPIO_PCM_A3_Oen();
            MHal_GPIO_PCM_A3_Low();
            break;
        case PAD_PCM_A2:
            MHal_GPIO_PCM_A2_Set();
            MHal_GPIO_PCM_A2_Oen();
            MHal_GPIO_PCM_A2_Low();
            break;
        case PAD_PCM_REG_N:
            MHal_GPIO_PCM_REG_N_Set();
            MHal_GPIO_PCM_REG_N_Oen();
            MHal_GPIO_PCM_REG_N_Low();
            break;
        case PAD_PCM_A1:
            MHal_GPIO_PCM_A1_Set();
            MHal_GPIO_PCM_A1_Oen();
            MHal_GPIO_PCM_A1_Low();
            break;
        case PAD_PCM_A0:
            MHal_GPIO_PCM_A0_Set();
            MHal_GPIO_PCM_A0_Oen();
            MHal_GPIO_PCM_A0_Low();
            break;
        case PAD_PCM_D0:
            MHal_GPIO_PCM_D0_Set();
            MHal_GPIO_PCM_D0_Oen();
            MHal_GPIO_PCM_D0_Low();
            break;
        case PAD_PCM_D1:
            MHal_GPIO_PCM_D1_Set();
            MHal_GPIO_PCM_D1_Oen();
            MHal_GPIO_PCM_D1_Low();
            break;
        case PAD_PCM_D2:
            MHal_GPIO_PCM_D2_Set();
            MHal_GPIO_PCM_D2_Oen();
            MHal_GPIO_PCM_D2_Low();
            break;
        case PAD_PCM_RESET:
            MHal_GPIO_PCM_RESET_Set();
            MHal_GPIO_PCM_RESET_Oen();
            MHal_GPIO_PCM_RESET_Low();
            break;
        case PAD_PCM2_RESET:
            MHal_GPIO_PCM2_RESET_Set();
            MHal_GPIO_PCM2_RESET_Oen();
            MHal_GPIO_PCM2_RESET_Low();
            break;
        case PAD_PCM_CD_N:
            MHal_GPIO_PCM_CD_N_Set();
            MHal_GPIO_PCM_CD_N_Oen();
            MHal_GPIO_PCM_CD_N_Low();
            break;
        case PAD_PCM2_CD_N:
            MHal_GPIO_PCM2_CD_N_Set();
            MHal_GPIO_PCM2_CD_N_Oen();
            MHal_GPIO_PCM2_CD_N_Low();
            break;
        case PAD_TS1_D0:
            MHal_GPIO_TS1_D0_Set();
            MHal_GPIO_TS1_D0_Oen();
            MHal_GPIO_TS1_D0_Low();
            break;
        case PAD_TS1_VLD:
            MHal_GPIO_TS1_VLD_Set();
            MHal_GPIO_TS1_VLD_Oen();
            MHal_GPIO_TS1_VLD_Low();
            break;
        case PAD_TS1_SYNC:
            MHal_GPIO_TS1_SYNC_Set();
            MHal_GPIO_TS1_SYNC_Oen();
            MHal_GPIO_TS1_SYNC_Low();
            break;
        case PAD_TS1_CLK:
            MHal_GPIO_TS1_CLK_Set();
            MHal_GPIO_TS1_CLK_Oen();
            MHal_GPIO_TS1_CLK_Low();
            break;
        case PAD_I2S_IN_WS:
            MHal_GPIO_I2S_IN_WS_Set();
            MHal_GPIO_I2S_IN_WS_Oen();
            MHal_GPIO_I2S_IN_WS_Low();
            break;
        case PAD_I2S_IN_BCK:
            MHal_GPIO_I2S_IN_BCK_Set();
            MHal_GPIO_I2S_IN_BCK_Oen();
            MHal_GPIO_I2S_IN_BCK_Low();
            break;
        case PAD_I2S_IN_SD:
            MHal_GPIO_I2S_IN_SD_Set();
            MHal_GPIO_I2S_IN_SD_Oen();
            MHal_GPIO_I2S_IN_SD_Low();
            break;
        case PAD_SPDIF_IN:
            MHal_GPIO_SPDIF_IN_Set();
            MHal_GPIO_SPDIF_IN_Oen();
            MHal_GPIO_SPDIF_IN_Low();
            break;
        case PAD_I2S_OUT_MCK:
            MHal_GPIO_I2S_OUT_MCK_Set();
            MHal_GPIO_I2S_OUT_MCK_Oen();
            MHal_GPIO_I2S_OUT_MCK_Low();
            break;
        case PAD_I2S_OUT_WS:
            MHal_GPIO_I2S_OUT_WS_Set();
            MHal_GPIO_I2S_OUT_WS_Oen();
            MHal_GPIO_I2S_OUT_WS_Low();
            break;
        case PAD_I2S_OUT_BCK:
            MHal_GPIO_I2S_OUT_BCK_Set();
            MHal_GPIO_I2S_OUT_BCK_Oen();
            MHal_GPIO_I2S_OUT_BCK_Low();
            break;
        case PAD_I2S_OUT_SD:
            MHal_GPIO_I2S_OUT_SD_Set();
            MHal_GPIO_I2S_OUT_SD_Oen();
            MHal_GPIO_I2S_OUT_SD_Low();
            break;
        case PAD_SPDIF_OUT:
            MHal_GPIO_SPDIF_OUT_Set();
            MHal_GPIO_SPDIF_OUT_Oen();
            MHal_GPIO_SPDIF_OUT_Low();
            break;
        case PAD_DDCR_DA2:
            MHal_GPIO_DDCR_DA2_Oen();
            MHal_GPIO_DDCR_DA2_Low();
            break;
        case PAD_DDCR_CK2:
            MHal_GPIO_DDCR_CK2_Oen();
            MHal_GPIO_DDCR_CK2_Low();
            break;
        case PAD_LHSYNC2:
            MHal_GPIO_LHSYNC2_Set();
            MHal_GPIO_LHSYNC2_Oen();
            MHal_GPIO_LHSYNC2_Low();
            break;
        case PAD_LVSYNC2:
            MHal_GPIO_LVSYNC2_Oen();
            MHal_GPIO_LVSYNC2_Low();
            break;
        case PAD_SPI_CK:
            MHal_GPIO_SPI_CK_Set();
            MHal_GPIO_SPI_CK_Oen();
            MHal_GPIO_SPI_CK_Low();
            break;
        case PAD_SPI_DI:
            MHal_GPIO_SPI_DI_Set();
            MHal_GPIO_SPI_DI_Oen();
            MHal_GPIO_SPI_DI_Low();
            break;
        case PAD_SPI_CZ:
            MHal_GPIO_SPI_CZ_Set();
            MHal_GPIO_SPI_CZ_Oen();
            MHal_GPIO_SPI_CZ_Low();
            break;
        case PAD_SPI_DO:
            MHal_GPIO_SPI_DO_Set();
            MHal_GPIO_SPI_DO_Oen();
            MHal_GPIO_SPI_DO_Low();
            break;
        case PAD_GPIO12:
            MHal_GPIO_GPIO12_Oen();
            MHal_GPIO_GPIO12_Low();
            break;
        case PAD_GPIO13:
            MHal_GPIO_GPIO13_Oen();
            MHal_GPIO_GPIO13_Low();
            break;
        case PAD_GPIO14:
            MHal_GPIO_GPIO14_Oen();
            MHal_GPIO_GPIO14_Low();
            break;
        case PAD_GPIO15:
            MHal_GPIO_GPIO15_Set();
            MHal_GPIO_GPIO15_Oen();
            MHal_GPIO_GPIO15_Low();
            break;
        case PAD_ET_CRS:
            MHal_GPIO_ET_CRS_Set();
            MHal_GPIO_ET_CRS_Oen();
            MHal_GPIO_ET_CRS_Low();
            break;
        case PAD_ET_COL:
            MHal_GPIO_ET_COL_Set();
            MHal_GPIO_ET_COL_Oen();
            MHal_GPIO_ET_COL_Low();
            break;
        case PAD_ET_TXD3:
            MHal_GPIO_ET_TXD3_Set();
            MHal_GPIO_ET_TXD3_Oen();
            MHal_GPIO_ET_TXD3_Low();
            break;
        case PAD_ET_TXD2:
            MHal_GPIO_ET_TXD2_Set();
            MHal_GPIO_ET_TXD2_Oen();
            MHal_GPIO_ET_TXD2_Low();
            break;
        case PAD_ET_TXD1:
            MHal_GPIO_ET_TXD1_Set();
            MHal_GPIO_ET_TXD1_Oen();
            MHal_GPIO_ET_TXD1_Low();
            break;
        case PAD_ET_TXD0:
            MHal_GPIO_ET_TXD0_Set();
            MHal_GPIO_ET_TXD0_Oen();
            MHal_GPIO_ET_TXD0_Low();
            break;
        case PAD_ET_TX_EN:
            MHal_GPIO_ET_TX_EN_Set();
            MHal_GPIO_ET_TX_EN_Oen();
            MHal_GPIO_ET_TX_EN_Low();
            break;
        case PAD_ET_TX_CLK:
            MHal_GPIO_ET_TX_CLK_Set();
            MHal_GPIO_ET_TX_CLK_Oen();
            MHal_GPIO_ET_TX_CLK_Low();
            break;
        case PAD_ET_TXER:
            MHal_GPIO_ET_TXER_Set();
            MHal_GPIO_ET_TXER_Oen();
            MHal_GPIO_ET_TXER_Low();
            break;
        case PAD_ET_RXER:
            MHal_GPIO_ET_RXER_Set();
            MHal_GPIO_ET_RXER_Oen();
            MHal_GPIO_ET_RXER_Low();
            break;
        case PAD_ET_RX_CLK:
            MHal_GPIO_ET_RX_CLK_Set();
            MHal_GPIO_ET_RX_CLK_Oen();
            MHal_GPIO_ET_RX_CLK_Low();
            break;
        case PAD_ET_RX_DV:
            MHal_GPIO_ET_RX_DV_Set();
            MHal_GPIO_ET_RX_DV_Oen();
            MHal_GPIO_ET_RX_DV_Low();
            break;
        case PAD_ET_RXD0:
            MHal_GPIO_ET_RXD0_Set();
            MHal_GPIO_ET_RXD0_Oen();
            MHal_GPIO_ET_RXD0_Low();
            break;
        case PAD_ET_RXD1:
            MHal_GPIO_ET_RXD1_Set();
            MHal_GPIO_ET_RXD1_Oen();
            MHal_GPIO_ET_RXD1_Low();
            break;
        case PAD_ET_RXD2:
            MHal_GPIO_ET_RXD2_Set();
            MHal_GPIO_ET_RXD2_Oen();
            MHal_GPIO_ET_RXD2_Low();
            break;
        case PAD_ET_RXD3:
            MHal_GPIO_ET_RXD3_Set();
            MHal_GPIO_ET_RXD3_Oen();
            MHal_GPIO_ET_RXD3_Low();
            break;
        case PAD_ET_MDC:
            MHal_GPIO_ET_MDC_Set();
            MHal_GPIO_ET_MDC_Oen();
            MHal_GPIO_ET_MDC_Low();
            break;
        case PAD_ET_MDIO:
            MHal_GPIO_ET_MDIO_Set();
            MHal_GPIO_ET_MDIO_Oen();
            MHal_GPIO_ET_MDIO_Low();
            break;
        case PAD_LCK:
            MHal_GPIO_LCK_Set();
            MHal_GPIO_LCK_Oen();
            MHal_GPIO_LCK_Low();
            break;
        case PAD_LDE:
            MHal_GPIO_LDE_Set();
            MHal_GPIO_LDE_Oen();
            MHal_GPIO_LDE_Low();
            break;
        case PAD_LHSYNC:
            MHal_GPIO_LHSYNC_Set();
            MHal_GPIO_LHSYNC_Oen();
            MHal_GPIO_LHSYNC_Low();
            break;
        case PAD_LVSYNC:
            MHal_GPIO_LVSYNC_Set();
            MHal_GPIO_LVSYNC_Oen();
            MHal_GPIO_LVSYNC_Low();
            break;
        case PAD_GPIO_PM0:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM0);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM0);
            break;
        case PAD_GPIO_PM1:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM1);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM1);
            break;
        case PAD_GPIO_PM2:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM2);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM2);
            break;
        case PAD_GPIO_PM3:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM3);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM3);
            break;
        case PAD_GPIO_PM4:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM4);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM4);
            break;
        case PAD_GPIO_PM5:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM5);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM5);
            break;
        case PAD_GPIO_PM6:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM6);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM6);
            break;
        case PAD_GPIO_PM7:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM7);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM7);
            break;
        case PAD_GPIO_PM8:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM8);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM8);
            break;
        case PAD_GPIO_PM9:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM9);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM9);
            break;
        case PAD_GPIO_PM10:
            MDrv_GPIO_Pad_Oen(PAD_GPIO_PM10);
            MDrv_GPIO_Pull_Low(PAD_GPIO_PM10);
            break;
        case PAD_GPIO_SAR0:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR0);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR0);
            MDrv_GPIO_Pull_Low(PAD_GPIO_SAR0);
            break;
        case PAD_GPIO_SAR1:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR1);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR1);
            MDrv_GPIO_Pull_Low(PAD_GPIO_SAR1);
            break;
        case PAD_GPIO_SAR2:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR2);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR2);
            MDrv_GPIO_Pull_Low(PAD_GPIO_SAR2);
            break;
        case PAD_GPIO_SAR3:
            MDrv_GPIO_Pad_Set(PAD_GPIO_SAR3);
            MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR3);
            MDrv_GPIO_Pull_Low(PAD_GPIO_SAR3);
            break;
        case PAD_TCON0:
            MHal_GPIO_TCON0_Set();
            MHal_GPIO_TCON0_Oen();
            MHal_GPIO_TCON0_Low();
            break;
        case PAD_TCON1:
            MHal_GPIO_TCON1_Set();
            MHal_GPIO_TCON1_Oen();
            MHal_GPIO_TCON1_Low();
            break;
        case PAD_TCON2:
            MHal_GPIO_TCON2_Set();
            MHal_GPIO_TCON2_Oen();
            MHal_GPIO_TCON2_Low();
            break;
        case PAD_TCON3:
            MHal_GPIO_TCON3_Set();
            MHal_GPIO_TCON3_Oen();
            MHal_GPIO_TCON3_Low();
            break;
        case PAD_TCON4:
            MHal_GPIO_TCON4_Set();
            MHal_GPIO_TCON4_Oen();
            MHal_GPIO_TCON4_Low();
            break;
        case PAD_TCON5:
            MHal_GPIO_TCON5_Set();
            MHal_GPIO_TCON5_Oen();
            MHal_GPIO_TCON5_Low();
            break;
        case PAD_TCON6:
            MHal_GPIO_TCON6_Set();
            MHal_GPIO_TCON6_Oen();
            MHal_GPIO_TCON6_Low();
            break;
        case PAD_TCON7:
            MHal_GPIO_TCON7_Set();
            MHal_GPIO_TCON7_Oen();
            MHal_GPIO_TCON7_Low();
            break;
        case PAD_TCON8:
            MHal_GPIO_TCON8_Set();
            MHal_GPIO_TCON8_Oen();
            MHal_GPIO_TCON8_Low();
            break;
        case PAD_TCON9:
            MHal_GPIO_TCON9_Set();
            MHal_GPIO_TCON9_Oen();
            MHal_GPIO_TCON9_Low();
            break;
        case PAD_TCON10:
            MHal_GPIO_TCON10_Set();
            MHal_GPIO_TCON10_Oen();
            MHal_GPIO_TCON10_Low();
            break;
        case PAD_TCON11:
            MHal_GPIO_TCON11_Set();
            MHal_GPIO_TCON11_Oen();
            MHal_GPIO_TCON11_Low();
            break;
        case PAD_TCON12:
            MHal_GPIO_TCON12_Set();
            MHal_GPIO_TCON12_Oen();
            MHal_GPIO_TCON12_Low();
            break;
        case PAD_TCON13:
            MHal_GPIO_TCON13_Set();
            MHal_GPIO_TCON13_Oen();
            MHal_GPIO_TCON13_Low();
            break;
        case PAD_TCON14:
            MHal_GPIO_TCON14_Set();
            MHal_GPIO_TCON14_Oen();
            MHal_GPIO_TCON14_Low();
            break;
        case PAD_TCON15:
            MHal_GPIO_TCON15_Set();
            MHal_GPIO_TCON15_Oen();
            MHal_GPIO_TCON15_Low();
            break;
        case PAD_TCON16:
            MHal_GPIO_TCON16_Set();
            MHal_GPIO_TCON16_Oen();
            MHal_GPIO_TCON16_Low();
            break;
        case PAD_HOTPLUGB:
            MDrv_GPIO_Pad_Oen(PAD_HOTPLUGB);
            MDrv_GPIO_Pull_Low(PAD_HOTPLUGB);
            break;
        case PAD_HOTPLUGA:
            MDrv_GPIO_Pad_Oen(PAD_HOTPLUGA);
            MDrv_GPIO_Pull_Low(PAD_HOTPLUGA);
            break;
        case PAD_HOTPLUGC:
            MDrv_GPIO_Pad_Oen(PAD_HOTPLUGC);
            MDrv_GPIO_Pull_Low(PAD_HOTPLUGC);
            break;
    }
}
EXPORT_SYMBOL(MDrv_GPIO_Set_Low);
