////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2007 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    mdrv_gpio_io.c
/// @brief  GPIO Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
//#include "MsCommon.h"
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <linux/semaphore.h>

#include "mst_devid.h"

#include "mdrv_gpio_io.h"
#include "mhal_gpio_reg.h"
#include "mhal_gpio.h"
#include "mdrv_gpio.h"

//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------
//#define CONFIG_MSTAR_TITANIA_BD_T2_CUS03_VBRD_BOARD_DVB_1 1
//#define CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_ATSC_1 1
//#define CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_DVB_1 1
//#define CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MIBRD_BOARD_ATSC_1 1

#if defined(CONFIG_MSTAR_TITANIA_BD_S5_CUS03_DEMO_BOARD_1)
#define CUS03_S5_BOARD_GPIO
#elif defined(CONFIG_MSTAR_TITANIA_BD_S6_CUS03_DEMO_BOARD_ATSC_1)
#define CUS03_S6_ATSC_BOARD_GPIO
#elif defined(CONFIG_MSTAR_TITANIA_BD_S6_CUS03_DEMO_BOARD_DVB_1)
#define CUS03_S6_DVB_BOARD_GPIO
#elif defined(CONFIG_MSTAR_TITANIA_BD_MST064C_D01A_S)
#define MSTAR_BD_MST064C_GPIO
#endif

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define GPIO_DBG_ENABLE              0

#if GPIO_DBG_ENABLE
#define GPIO_DBG(_f)                 (_f)
#else
#define GPIO_DBG(_f)
#endif

#if 0
#define LINE_DBG()                  printf("GPIO %d\n", __LINE__)
#else
#define LINE_DBG()
#endif

#define GPIO_PRINT(fmt, args...)        //printk("[GPIO][%05d] " fmt, __LINE__, ## args)

typedef struct
{
    S32                          s32MajorGPIO;
    S32                          s32MinorGPIO;
    struct cdev                 cDevice;
    struct file_operations      GPIOFop;
    struct fasync_struct        *async_queue; /* asynchronous readers */
} GPIO_ModHandle_t;


#define MOD_GPIO_DEVICE_COUNT         1
#define MOD_GPIO_NAME                 "ModGPIO"


//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
DECLARE_MUTEX(PfModeSem);


//--------------------------------------------------------------------------------------------------
// Forward declaration
//--------------------------------------------------------------------------------------------------
static int                      _MDrv_GPIO_Open (struct inode *inode, struct file *filp);
static int                      _MDrv_GPIO_Release(struct inode *inode, struct file *filp);
static ssize_t                  _MDrv_GPIO_Read(struct file *filp, char __user *buf, size_t count, loff_t *f_pos);
static ssize_t                  _MDrv_GPIO_Write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos);
static unsigned int             _MDrv_GPIO_Poll(struct file *filp, poll_table *wait);
static int                      _MDrv_GPIO_Ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
static int                      _MDrv_GPIO_Fasync(int fd, struct file *filp, int mode);

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------

static GPIO_ModHandle_t GPIODev=
{
    .s32MajorGPIO = MDRV_MAJOR_GPIO,
    .s32MinorGPIO = MDRV_MINOR_GPIO,
    .cDevice =
    {
        .kobj = {.name= MOD_GPIO_NAME, },
        .owner = THIS_MODULE,
    },
    .GPIOFop =
    {
        .open =     _MDrv_GPIO_Open,
        .release =  _MDrv_GPIO_Release,
        .read =     _MDrv_GPIO_Read,
        .write =    _MDrv_GPIO_Write,
        .poll =     _MDrv_GPIO_Poll,
        .ioctl =    _MDrv_GPIO_Ioctl,
        .fasync =   _MDrv_GPIO_Fasync,
    },
};

//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------

static int _MDrv_GPIO_Open (struct inode *inode, struct file *filp)
{
	GPIO_ModHandle_t *dev;

    GPIO_PRINT("%s is invoked\n", __FUNCTION__);

    dev = container_of(inode->i_cdev, GPIO_ModHandle_t, cDevice);
	filp->private_data = dev;

    return 0;
}

static int _MDrv_GPIO_Release(struct inode *inode, struct file *filp)
{
    GPIO_PRINT("%s is invoked\n", __FUNCTION__);
    return 0;
}

static ssize_t _MDrv_GPIO_Read(struct file *filp, char __user *buf, size_t count, loff_t *f_pos)
{
    GPIO_PRINT("%s is invoked\n", __FUNCTION__);
    return 0;
}

static ssize_t _MDrv_GPIO_Write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos)
{
    GPIO_PRINT("%s is invoked\n", __FUNCTION__);
    return 0;
}

static unsigned int _MDrv_GPIO_Poll(struct file *filp, poll_table *wait)
{
    GPIO_PRINT("%s is invoked\n", __FUNCTION__);
    return 0;
}


static int _MDrv_GPIO_Fasync(int fd, struct file *filp, int mode)
{
    GPIO_PRINT("%s is invoked\n", __FUNCTION__);

	return 0;//fasync_helper(fd, filp, mode, &IICDev.async_queue);
}

static int _MDrv_GPIO_Ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
    S32 s32Err= 0;
  	U8 u8Temp;

    GPIO_PRINT("%s is invoked\n", __FUNCTION__);

    /*
     * extract the type and number bitfields, and don't decode
     * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
     */
    if ((GPIO_IOC_MAGIC!= _IOC_TYPE(cmd)) || (_IOC_NR(cmd)> GPIO_IOC_MAXNR))
    {
        return -ENOTTY;
    }

    /*
     * the direction is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. `Type' is user-oriented, while
     * access_ok is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
    {
        s32Err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
    }
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
    {
        s32Err =  !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
    }
    if (s32Err)
    {
        return -EFAULT;
    }

    switch(cmd)
    {
        //the cases of this section set to initialize
        case MDRV_GPIO_INIT:
            GPIO_PRINT("ioctl: MDRV_GPIO_INIT\n");
            MDrv_GPIO_Init();
            break;
        case MDRV_GPIO_SET:
            GPIO_PRINT("ioctl: MDRV_GPIO_SET\n");
            if (copy_from_user(&u8Temp, (U8 *) arg, sizeof(U8)))
 	            return -EFAULT;
            MDrv_GPIO_Pad_Set(u8Temp);
            break;
        case MDRV_GPIO_OEN:
            GPIO_PRINT("ioctl: MDRV_GPIO_OEN\n");
            if (copy_from_user(&u8Temp, (U8 *) arg, sizeof(U8)))
 	            return -EFAULT;
            MDrv_GPIO_Pad_Oen(u8Temp);
            break;
        case MDRV_GPIO_ODN:
            GPIO_PRINT("ioctl: MDRV_GPIO_ODN\n");
            if (copy_from_user(&u8Temp, (U8 *) arg, sizeof(U8)))
 	            return -EFAULT;
            MDrv_GPIO_Pad_Odn(u8Temp);
            break;
        case MDRV_GPIO_READ:
            GPIO_PRINT("ioctl: MDRV_GPIO_READ\n");
            if (copy_from_user(&u8Temp, (U8 *) arg, sizeof(U8)))
 	            return -EFAULT;
            u8Temp=MDrv_GPIO_Pad_Read(u8Temp);
            if (copy_to_user((U8 *) arg, &u8Temp, sizeof(U8)))
                return -EFAULT;
            break;
        case MDRV_GPIO_PULL_HIGH:
            GPIO_PRINT("ioctl: MDRV_GPIO_PULL_HIGH\n");
            if (copy_from_user(&u8Temp, (U8 *) arg, sizeof(U8)))
 	            return -EFAULT;
            MDrv_GPIO_Pull_High(u8Temp);
            break;
        case MDRV_GPIO_PULL_LOW:
            GPIO_PRINT("ioctl: MDRV_GPIO_PULL_LOW\n");
            if (copy_from_user(&u8Temp, (U8 *) arg, sizeof(U8)))
 	            return -EFAULT;
            MDrv_GPIO_Pull_Low(u8Temp);
            break;


        default:
            GPIO_PRINT("ioctl: unknown command\n");
            return -ENOTTY;
    }

    return 0;
}

static int __init mod_gpio_init(void)
{
    S32         s32Ret;
    dev_t       dev;

    GPIO_PRINT("%s is invoked\n", __FUNCTION__);

    if (GPIODev.s32MajorGPIO)
    {
        dev = MKDEV(GPIODev.s32MajorGPIO, GPIODev.s32MinorGPIO);
        s32Ret = register_chrdev_region(dev, MOD_GPIO_DEVICE_COUNT, MOD_GPIO_NAME);
    }
    else
    {
        s32Ret = alloc_chrdev_region(&dev, GPIODev.s32MinorGPIO, MOD_GPIO_DEVICE_COUNT, MOD_GPIO_NAME);
        GPIODev.s32MajorGPIO = MAJOR(dev);
    }

    if ( 0 > s32Ret)
    {
        GPIO_PRINT("Unable to get major %d\n", GPIODev.s32MajorGPIO);
        return s32Ret;
    }

    cdev_init(&GPIODev.cDevice, &GPIODev.GPIOFop);
    if (0!= (s32Ret= cdev_add(&GPIODev.cDevice, dev, MOD_GPIO_DEVICE_COUNT)))
    {
        GPIO_PRINT("Unable add a character device\n");
        unregister_chrdev_region(dev, MOD_GPIO_DEVICE_COUNT);
        return s32Ret;
    }

    GPIO_PRINT("Backlight turns on...\n");
    //GPIO chiptop initialization
    MDrv_GPIO_Init();

#if defined(CONFIG_MSTAR_TITANIA_BD_T2_CUS03_VBRD_BOARD_DVB_1)  || \
    defined(CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_ATSC_1) || \
    defined(CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_DVB_1) || \
    defined(CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MIBRD_BOARD_ATSC_1)
    //for MSTAR S5,PAD_GPIO_PM3 pull low to let panel power turn on
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM3); // panel control
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM3);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM3);

#ifdef	NOT_USED_4_CUS03    /* POWER ON/OFF will be set only by 8051Lite */
	MDrv_GPIO_Pad_Set(PAD_GPIO_PM4); // panel power on/off
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM4);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM4);
#endif

	MDrv_GPIO_Pad_Set(PAD_GPIO_PM2); // invert control
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM2);
//    MDrv_GPIO_Pull_High(PAD_GPIO_PM2);	//athens: Change INV_CTRL Init Value (Low --> High) 080819
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM2);	// venus board

	MDrv_GPIO_Pad_Set(PAD_GPIO_PM0); // Relay on power
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM0);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM0);

#if 1 /* MSTAR. by jjab */
	MDrv_GPIO_Pad_Set(90); // Reset Ursa
    MDrv_GPIO_Pad_Oen(90);
	MDrv_GPIO_Pull_Low(90);
	msleep(1000);
	MDrv_GPIO_Pull_High(90);
	msleep(1000);
    MDrv_GPIO_Pull_Low(90);
#endif

    //for MSTAR S5,PAD_PCM2_CE_N(GPIO42), PAD_UART_TX2, PAD_ET_RX_DV pull low
    MDrv_GPIO_Pad_Set(PAD_PCM2_CE_N); // CI enable
    MDrv_GPIO_Pad_Oen(PAD_PCM2_CE_N);
    MDrv_GPIO_Pull_High(PAD_PCM2_CE_N);

//    MDrv_GPIO_Pad_Set(PAD_UART_TX2); // power board, no use in S6 DVB
//    MDrv_GPIO_Pad_Oen(PAD_UART_TX2);
//    MDrv_GPIO_Pull_Low(PAD_UART_TX2);

    MDrv_GPIO_Pad_Set(PAD_I2S_IN_WS); // USB
    MDrv_GPIO_Pad_Oen(PAD_I2S_IN_WS);
#if (CONFIG_MSTAR_TITANIA_BD_T2_CUS03_VBRD_BOARD_DVB_1)  || \
    (CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_ATSC_1) || \
    (CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_DVB_1) || \
    (CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MIBRD_BOARD_ATSC_1)
	MDrv_GPIO_Pull_High(PAD_I2S_IN_WS);
#endif
#if defined(CONFIG_MSTAR_TITANIA_BD_T2_CUS03_MBRD_BOARD_ATSC_1)
    MDrv_GPIO_Pull_Low(PAD_I2S_IN_WS);
#endif

#ifdef NOT_USED_4_CUS03    /* TUNER RESET will be done by MSTAR's APPL */
	MDrv_GPIO_Pad_Set(PAD_ET_MDIO); // tuner reset
	MDrv_GPIO_Pad_Oen(PAD_ET_MDIO);
    MDrv_GPIO_Pull_High(PAD_ET_MDIO);
	msleep(60);
    MDrv_GPIO_Pull_Low(PAD_ET_MDIO);
	msleep(60);
    MDrv_GPIO_Pull_High(PAD_ET_MDIO);
#endif

	return 0;
#endif

//MSTAR GPIO SETTING
#if ! (defined(CUS03_S5_BOARD_GPIO) || defined(CUS03_S6_DVB_BOARD_GPIO) || defined(CUS03_S6_ATSC_BOARD_GPIO))


  #if (defined(MSTAR_BD_MST064C_GPIO))
	//MDrv_GPIO_Pad_Set(PAD_ET_TXD3);
	MDrv_GPIO_Pad_Oen(PAD_ET_TXD3);
	MDrv_GPIO_Pull_Low(PAD_ET_TXD3);

	//MDrv_GPIO_Pad_Set(PAD_ET_TXD2);
	MDrv_GPIO_Pad_Oen(PAD_ET_TXD2);
	MDrv_GPIO_Pull_Low(PAD_ET_TXD2);

  #else  //for MST55C/55E/64D,PAD_GPIO14 low to let backlight turn on
	//MDrv_GPIO_Pad_Set(PAD_GPIO14);
	MDrv_GPIO_Pad_Oen(PAD_GPIO14);
	MDrv_GPIO_Pull_Low(PAD_GPIO14);

	//MDrv_GPIO_Pad_Set(PAD_GPIO15);
	MDrv_GPIO_Pad_Oen(PAD_GPIO15);
	MDrv_GPIO_Pull_Low(PAD_GPIO15);
  #endif

#endif

#ifdef CUS03_S5_BOARD_GPIO
    //for MSTAR S5,PAD_GPIO_PM3 pull low to let panel power turn on
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM3);
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM3);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM3);
    //for MSTAR S5,PAD_PCM2_CE_N(GPIO42), PAD_UART_TX2, PAD_ET_RX_DV pull low
    MDrv_GPIO_Pad_Set(PAD_PCM2_CE_N);
    MDrv_GPIO_Pad_Set(PAD_UART_TX2);
    MDrv_GPIO_Pad_Set(PAD_ET_RX_DV);
    MDrv_GPIO_Pad_Oen(PAD_PCM2_CE_N);
    MDrv_GPIO_Pad_Oen(PAD_UART_TX2);
    MDrv_GPIO_Pad_Oen(PAD_ET_RX_DV);
    MDrv_GPIO_Pull_High(PAD_PCM2_CE_N); //athens: Change INV_CTRL Init Value (Low --> High) 080626
    MDrv_GPIO_Pull_Low(PAD_UART_TX2);
    MDrv_GPIO_Pull_Low(PAD_ET_RX_DV);
    //for MSTAR S5,PAD_GPIO_SAR3: Low->High->Low
    MDrv_GPIO_Pad_Set(PAD_GPIO_SAR3);
	MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR3);
    MDrv_GPIO_Pull_High(PAD_GPIO_SAR3);
	msleep(60);
    MDrv_GPIO_Pull_Low(PAD_GPIO_SAR3);

    //for MSTAR S5 Mlink
    MDrv_GPIO_Pull_High(PAD_GPIO14);
    MDrv_GPIO_Pad_Set(PAD_GPIO14);
	MDrv_GPIO_Pad_Oen(PAD_GPIO14);
    MDrv_GPIO_Pull_High(PAD_GPIO14);
#endif

#ifdef CUS03_S6_ATSC_BOARD_GPIO

   //for MSTAR S5,PAD_GPIO_PM3 pull low to let panel power turn on
#ifdef	NOT_USED_4_CUS03    /* PANEL_CTL will be set by MSTAR's APPL */
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM3); // panel control
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM3);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM3);
#endif

#ifdef	NOT_USED_4_CUS03    /* POWER ON/OFF will be set only by 8051Lite */
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM4); // panel power on/off
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM4);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM4);
#endif

//#ifdef	NOT_USED_4_CUS03    /* INV CTL will be set by MSTAR's APPL */
/* 	2008,06,19: USB LAN 
*/
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM2); // invert control
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM2);
    MDrv_GPIO_Pull_High(PAD_GPIO_PM2);
//#endif

    MDrv_GPIO_Pad_Set(PAD_GPIO_PM0); // Relay on power
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM0);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM0);

    MDrv_GPIO_Pad_Set(90); // Reset Ursa
    MDrv_GPIO_Pad_Oen(90);
    MDrv_GPIO_Pull_Low(90);
    msleep(1000);
    MDrv_GPIO_Pull_High(90);
    msleep(1000);
    MDrv_GPIO_Pull_Low(90);

#ifdef NOT_USED_4_ATSC
    MDrv_GPIO_Pad_Set(99); // Reset Triton
    MDrv_GPIO_Pad_Oen(99);
    MDrv_GPIO_Pull_High(99);
    msleep(60);
    //MDrv_GPIO_Pull_Low(99);

    //for MSTAR S5,PAD_PCM2_CE_N(GPIO42), PAD_UART_TX2, PAD_ET_RX_DV pull low
    MDrv_GPIO_Pad_Set(PAD_PCM2_CE_N); // CI enable
   MDrv_GPIO_Pad_Oen(PAD_PCM2_CE_N);
    MDrv_GPIO_Pull_High(PAD_PCM2_CE_N);


    MDrv_GPIO_Pad_Set(PAD_UART_TX2); // power board, no use in S6 DVB
    MDrv_GPIO_Pad_Oen(PAD_UART_TX2);
    MDrv_GPIO_Pull_Low(PAD_UART_TX2);
#endif

    MDrv_GPIO_Pad_Set(PAD_I2S_IN_WS); // USB
    MDrv_GPIO_Pad_Oen(PAD_I2S_IN_WS);
    MDrv_GPIO_Pull_Low(PAD_I2S_IN_WS);
    //MDrv_GPIO_Pull_Low(PAD_ET_RX_DV);

#ifdef NOT_USED_4_CUS03    /* TUNER RESET will be done by MSTAR's APPL */
    MDrv_GPIO_Pad_Set(PAD_ET_MDIO); // tuner reset
    MDrv_GPIO_Pad_Oen(PAD_ET_MDIO);
    MDrv_GPIO_Pull_High(PAD_ET_MDIO);
    msleep(60);
    MDrv_GPIO_Pull_Low(PAD_ET_MDIO);
    msleep(60);
    MDrv_GPIO_Pull_High(PAD_ET_MDIO);
#endif

    //for MSTAR S5,PAD_GPIO_SAR3: Low->High->Low
//    MDrv_GPIO_Pad_Set(PAD_GPIO_SAR3);
//    MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR3);
//    MDrv_GPIO_Pull_High(PAD_GPIO_SAR3);
//    msleep(60);
//    MDrv_GPIO_Pull_Low(PAD_GPIO_SAR3);

    //for MSTAR S5 Mlink
    /*
    MDrv_GPIO_Pull_High(PAD_GPIO14);
    MDrv_GPIO_Pad_Set(PAD_GPIO14);
    MDrv_GPIO_Pad_Oen(PAD_GPIO14);
    MDrv_GPIO_Pull_High(PAD_GPIO14);
    */

#endif

#ifdef CUS03_S6_DVB_BOARD_GPIO
    //for MSTAR S5,PAD_GPIO_PM3 pull low to let panel power turn on
#ifdef	NOT_USED_4_CUS03    /* PANEL_CTL will be set by MSTAR's APPL */
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM3); // panel control
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM3);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM3);
#endif

#ifdef	NOT_USED_4_CUS03    /* POWER ON/OFF will be set only by 8051Lite */
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM4); // panel power on/off
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM4);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM4);
#endif

//#ifdef	NOT_USED_4_CUS03    /* INV CTL will be set by MSTAR's APPL */
/* 	2008,06,19:	̺κ Ծϸ USB LAN  .  
*/
    MDrv_GPIO_Pad_Set(PAD_GPIO_PM2); // invert control
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM2);
    MDrv_GPIO_Pull_High(PAD_GPIO_PM2);
//#endif

    MDrv_GPIO_Pad_Set(PAD_GPIO_PM0); // Relay on power
    MDrv_GPIO_Pad_Oen(PAD_GPIO_PM0);
    MDrv_GPIO_Pull_Low(PAD_GPIO_PM0);

    MDrv_GPIO_Pad_Set(90); // Reset Ursa
    MDrv_GPIO_Pad_Oen(90);
    MDrv_GPIO_Pull_Low(90);
    msleep(1000);
    MDrv_GPIO_Pull_High(90);
    msleep(1000);
    MDrv_GPIO_Pull_Low(90);

    MDrv_GPIO_Pad_Set(99); // Reset Triton
    MDrv_GPIO_Pad_Oen(99);
    MDrv_GPIO_Pull_High(99);
    msleep(60);
    //MDrv_GPIO_Pull_Low(99);

    //for MSTAR S5,PAD_PCM2_CE_N(GPIO42), PAD_UART_TX2, PAD_ET_RX_DV pull low
    MDrv_GPIO_Pad_Set(PAD_PCM2_CE_N); // CI enable
    MDrv_GPIO_Pad_Oen(PAD_PCM2_CE_N);
    MDrv_GPIO_Pull_High(PAD_PCM2_CE_N);

    MDrv_GPIO_Pad_Set(PAD_UART_TX2); // power board, no use in S6 DVB
    MDrv_GPIO_Pad_Oen(PAD_UART_TX2);
    MDrv_GPIO_Pull_Low(PAD_UART_TX2);

    MDrv_GPIO_Pad_Set(PAD_I2S_IN_WS); // USB
    MDrv_GPIO_Pad_Oen(PAD_I2S_IN_WS);
    MDrv_GPIO_Pull_Low(PAD_I2S_IN_WS);
    //MDrv_GPIO_Pull_Low(PAD_ET_RX_DV);

#ifdef NOT_USED_4_CUS03    /* TUNER RESET will be done by MSTAR's APPL */
    MDrv_GPIO_Pad_Set(PAD_ET_MDIO); // tuner reset
    MDrv_GPIO_Pad_Oen(PAD_ET_MDIO);
    MDrv_GPIO_Pull_High(PAD_ET_MDIO);
    msleep(60);
    MDrv_GPIO_Pull_Low(PAD_ET_MDIO);
    msleep(60);
    MDrv_GPIO_Pull_High(PAD_ET_MDIO);
#endif

    //for MSTAR S5,PAD_GPIO_SAR3: Low->High->Low
//    MDrv_GPIO_Pad_Set(PAD_GPIO_SAR3);
//    MDrv_GPIO_Pad_Oen(PAD_GPIO_SAR3);
//    MDrv_GPIO_Pull_High(PAD_GPIO_SAR3);
//    msleep(60);
//    MDrv_GPIO_Pull_Low(PAD_GPIO_SAR3);

    //for MSTAR S5 Mlink
    /*
    MDrv_GPIO_Pull_High(PAD_GPIO14);
    MDrv_GPIO_Pad_Set(PAD_GPIO14);
    MDrv_GPIO_Pad_Oen(PAD_GPIO14);
    MDrv_GPIO_Pull_High(PAD_GPIO14);
    */
#endif

    return 0;
}

static void __exit mod_gpio_exit(void)
{
    GPIO_PRINT("%s is invoked\n", __FUNCTION__);

    cdev_del(&GPIODev.cDevice);
    unregister_chrdev_region(MKDEV(GPIODev.s32MajorGPIO, GPIODev.s32MinorGPIO), MOD_GPIO_DEVICE_COUNT);
}

module_init(mod_gpio_init);
module_exit(mod_gpio_exit);

EXPORT_SYMBOL(PfModeSem);

MODULE_AUTHOR("MSTAR");
MODULE_DESCRIPTION("GPIO driver");
MODULE_LICENSE("MSTAR");
