////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2007 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    mdrv_iomap.c
/// @brief  Memory IO remap Control Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
//#include "MsCommon.h"
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>  //added
#include <linux/timer.h> //added
#include <linux/device.h>
#include <asm/io.h>
#include <asm/types.h>

#include "mst_devid.h"
#include "mdrv_iomap.h"
#include "mdrv_types.h"
#include "mst_platform.h"
#include "mdrv_system.h"
//-------------------------------------------------------------------------------------------------
// Macros
//-------------------------------------------------------------------------------------------------
//#define MIOMAP_DPRINTK(fmt, args...) printk(KERN_WARNING"%s:%d " fmt,__FUNCTION__,__LINE__,## args)
#define MIOMAP_DPRINTK(fmt, args...)

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------

#define MOD_MIOMAP_DEVICE_COUNT     1
#define MOD_MIOMAP_NAME             "miomap"

// Define MIOMAP Device
U32 miomap_base;
U32 miomap_size;

#define MIOMAP_BASE                 miomap_base
#define MIOMAP_SIZE                 miomap_size


//-------------------------------------------------------------------------------------------------
//  Local Structurs
//-------------------------------------------------------------------------------------------------;

typedef struct
{
    int                         s32MIOMapMajor;
    int                         s32MIOMapMinor;
    void*                       dmaBuf;
    struct cdev                 cDevice;
    struct file_operations      MIOMapFop;
} MIOMapModHandle;

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------------------
// Forward declaration
//--------------------------------------------------------------------------------------------------
static int                      _MDrv_MIOMAP_Open (struct inode *inode, struct file *filp);
static int                      _MDrv_MIOMAP_Release(struct inode *inode, struct file *filp);
static int                      _MDrv_MIOMAP_MMap(struct file *filp, struct vm_area_struct *vma);
static int                      _MDrv_MIOMAP_Ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);

//-------------------------------------------------------------------------------------------------
// Local Variables
//-------------------------------------------------------------------------------------------------
static struct class *miomap_class;

static MIOMapModHandle MIOMapDev=
{
    .s32MIOMapMajor=               MDRV_MAJOR_MIOMAP,
    .s32MIOMapMinor=               MDRV_MINOR_MIOMAP,
    .cDevice=
    {
        .kobj=                  {.name= MOD_MIOMAP_NAME, },
        .owner  =               THIS_MODULE,
    },
    .MIOMapFop=
    {
        .open=                  _MDrv_MIOMAP_Open,
        .release=               _MDrv_MIOMAP_Release,
        .mmap=                  _MDrv_MIOMAP_MMap,
        .ioctl=                 _MDrv_MIOMAP_Ioctl,
    },
};

//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

static int _MDrv_MIOMAP_Open (struct inode *inode, struct file *filp)
{
    MIOMapModHandle *dev;

    MIOMAP_DPRINTK("LennyD\n");

    dev = container_of(inode->i_cdev, MIOMapModHandle, cDevice);
    // dev->dmaBuf=ioremap_nocache(MIOMap_BASE, MIOMap_SIZE);
    filp->private_data = dev;

    return 0;
}

static int _MDrv_MIOMAP_Release(struct inode *inode, struct file *filp)
{
    // MIOMapModHandle *dev = filp->private_data ;;

    MIOMAP_DPRINTK("LennyD\n");
    // iounmap(dev->dmaBuf) ;
    return 0;
}

static int _MDrv_MIOMAP_MMap(struct file *filp, struct vm_area_struct *vma)
{
    vma->vm_pgoff = MIOMAP_BASE >> PAGE_SHIFT;
#if 0
    vma->vm_flags = VM_WRITE|VM_MAYWRITE|VM_READ|VM_MAYREAD|VM_RESERVED|VM_DONTCOPY;
    vma->vm_flags |= VM_LOCKED;
#endif
    /* set page to no cache */
    pgprot_val(vma->vm_page_prot) &= ~_CACHE_MASK;
    pgprot_val(vma->vm_page_prot) |= _CACHE_UNCACHED;

    if (io_remap_pfn_range(vma, vma->vm_start,
                            MIOMAP_BASE >> PAGE_SHIFT, MIOMAP_SIZE,
                            vma->vm_page_prot))
    {
        return -EAGAIN;
    }
    return 0;
}

static int _MDrv_MIOMAP_Ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
    int         err= 0;

    /*
     * extract the type and number bitfields, and don't decode
     * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
     */
    if (MIOMAP_IOC_MAGIC!= _IOC_TYPE(cmd))
    {
        return -ENOTTY;
    }

    /*
     * the direction is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. `Type' is user-oriented, while
     * access_ok is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
    {
        err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
    }
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
    {
        err =  !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
    }
    if (err)
    {
        return -EFAULT;
    }

    // @FIXME: Use a array of function pointer for program readable and code size later
    switch(cmd)
    {
    //------------------------------------------------------------------------------
    // Signal
    //------------------------------------------------------------------------------
    case MIOMAP_IOC_CHIPINFO:
        {
            DrvMIOMap_ChipInfo_t i ;
            // Chip Type
            // Uranus 0x0101
            // Oberon 0x0102
            // Euclid 0x0103
            // T1     0x0201
            // T2     0x0202
            // T3     0x0203
            // Chip version
            // FPGA   0x0000
            // U01    0x0001
            // U02    0x0002
            // U03    0x0003

            #if defined(CONFIG_MSTAR_OBERON)
                i.u16chiptype    = 0x0102 ;
                i.u16chipversion = 0x0001 ;
                //printk("It's oberon now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_TITANIA2)
                i.u16chiptype    = 0x0202 ;
                i.u16chipversion = 0x0001 ;
                //printk("It's oberon now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_EUCLID)
                i.u16chiptype    = 0x0103 ;
                i.u16chipversion = 0x0000 ;
                //printk("It's oberon now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_TITANIA3)
                i.u16chiptype    = 0x0203 ;
                i.u16chipversion = 0x0000 ;
                //printk("It's oberon now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_TITANIA4)
                i.u16chiptype    = 0x0204 ;
                i.u16chipversion = 0x0000 ;
                //printk("It's titania4 now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_TITANIA8)
                i.u16chiptype    = 0x0208 ;
                i.u16chipversion = 0x0000 ;
                //printk("It's titania8 now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_TITANIA9)
                i.u16chiptype    = 0x0209 ;
                i.u16chipversion = 0x0000 ;
                //printk("It's titania9 now\n"); // test string
            #endif

            #if defined(CONFIG_MSTAR_URANUS4)
                i.u16chiptype    = 0x0104 ;
                i.u16chipversion = 0x0000 ;
                //printk("It's uranus4 now\n"); // test string
            #endif
            copy_to_user( (void *)arg, &i, sizeof(i) ) ;
        }
        break;

    case MIOMAP_IOC_SET_MAP:
        {
            DrvMIOMap_Info_t i ;

            copy_from_user( &i, (DrvMIOMap_Info_t*)arg, sizeof(i) ) ;
            MIOMAP_BASE = i.u32Addr ;
            MIOMAP_SIZE = i.u32Size ;
            //printk("MIOMAP_BASE=%x MIOMAP_SIZE=%x\n", MIOMAP_BASE, MIOMAP_SIZE);
        }
        break;

    case MIOMAP_IOC_FLUSHDCACHE:
        {
            DrvMIOMap_Info_t i ;

            copy_from_user( &i, (DrvMIOMap_Info_t*)arg, sizeof(i) ) ;
            _dma_cache_wback_inv((unsigned long)__va(i.u32Addr), i.u32Size) ;
        }
        break;

    case MIOMAP_IOC_GET_BLOCK_OFFSET:
        {
            DrvMIOMap_Info_t i ;
            copy_from_user( &i, (DrvMIOMap_Info_t*)arg, sizeof(i) ) ;
            MDrv_SYS_GetMMAP(i.u32Addr, &(i.u32Addr), &(i.u32Size)) ;
            copy_to_user( (void *)arg, &i, sizeof(i) ) ;
        }
        break ;
    default:
        printk("Unknown ioctl command %d\n", cmd);
        return -ENOTTY;
    }
    return 0;
}

MSYSTEM_STATIC int __init mod_miomap_init(void)
{
    int s32Ret;
    dev_t dev;

    miomap_base = 0x0 ;
    miomap_size = 0x0 ;

    miomap_class = class_create(THIS_MODULE, "miomap");
    if (IS_ERR(miomap_class))
    {
        return PTR_ERR(miomap_class);
    }

    if (MIOMapDev.s32MIOMapMajor)
    {
        dev = MKDEV(MIOMapDev.s32MIOMapMajor, MIOMapDev.s32MIOMapMinor);
        s32Ret = register_chrdev_region(dev, MOD_MIOMAP_DEVICE_COUNT, MOD_MIOMAP_NAME);
    }
    else
    {
        s32Ret = alloc_chrdev_region(&dev, MIOMapDev.s32MIOMapMinor, MOD_MIOMAP_DEVICE_COUNT, MOD_MIOMAP_NAME);
        MIOMapDev.s32MIOMapMajor = MAJOR(dev);
    }

    if ( 0 > s32Ret)
    {
        MIOMAP_DPRINTK("Unable to get major %d\n", MIOMapDev.s32MIOMapMajor);
        class_destroy(miomap_class);
        return s32Ret;
    }

    cdev_init(&MIOMapDev.cDevice, &MIOMapDev.MIOMapFop);
    if (0!= (s32Ret= cdev_add(&MIOMapDev.cDevice, dev, MOD_MIOMAP_DEVICE_COUNT)))
    {
        MIOMAP_DPRINTK("Unable add a character device\n");
        unregister_chrdev_region(dev, MOD_MIOMAP_DEVICE_COUNT);
        class_destroy(miomap_class);
        return s32Ret;
    }

    device_create(miomap_class, NULL, dev, NULL, MOD_MIOMAP_NAME);

    return 0;
}

MSYSTEM_STATIC void __exit mod_miomap_exit(void)
{
    cdev_del(&MIOMapDev.cDevice);
    unregister_chrdev_region(MKDEV(MIOMapDev.s32MIOMapMajor, MIOMapDev.s32MIOMapMinor), MOD_MIOMAP_DEVICE_COUNT);
    class_destroy(miomap_class);
}

#if defined(CONFIG_MSTAR_MSYSTEM) || defined(CONFIG_MSTAR_MSYSTEM_MODULE)
#else//#if defined(CONFIG_MSTAR_MSYSTEM) || defined(CONFIG_MSTAR_MSYSTEM_MODULE)
module_init(mod_miomap_init);
module_exit(mod_miomap_exit);

MODULE_AUTHOR("MSTAR");
MODULE_DESCRIPTION("MIOMAP driver");
MODULE_LICENSE("GPL");
#endif//#if defined(CONFIG_MSTAR_MSYSTEM) || defined(CONFIG_MSTAR_MSYSTEM_MODULE)
