////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2007 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    drvIR.c
/// @brief  IR Control Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
//#include "MsCommon.h"
#include <linux/autoconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>  //added
#include <linux/timer.h> //added
#include <asm/io.h>
#include <linux/semaphore.h>
#ifdef CONFIG_MSTAR_IR_INPUT_DEVICE
#include <media/ir-common.h>
#endif
#include "board/Board.h"
#include "mst_devid.h"
#include "mhal_ir_reg.h"
//#include "mdrv_ir_io.h" //alex_tung###
#include "mdrv_ir.h"
#include "chip_int.h"
#include "mdrv_system.h"

#if (IR_TYPE_SEL == IR_TYPE_OLD)
#include "IR_MSTAR_OLD.h"
#elif (IR_TYPE_SEL == IR_TYPE_NEW)
#include "IR_MSTAR_NEW.h"
#elif (IR_TYPE_SEL == IR_TYPE_MSTAR_DTV)
#include "IR_MSTAR_DTV.h"
#elif (IR_TYPE_SEL == IR_TYPE_RC_V16)
#include "IR_MSTAR_RC_V16.h"
#elif (IR_TYPE_SEL == IR_TYPE_MSTAR_RAW)
#include "IR_MSTAR_RAW.h"
#elif (IR_TYPE_SEL == IR_TYPE_CUS03_DTV)
#include "IR_CUS03_DTV.h"
#elif (IR_TYPE_SEL == IR_TYPE_MSTAR_FANTASY)
#include "IR_MSTAR_FANTASY.h"
#elif (IR_TYPE_SEL == IR_TYPE_MSTAR_SZ1)
#include "IR_MSTAR_SZ1.h"
#elif (IR_TYPE_SEL == IR_TYPE_SKYWORTH_SLIDE)
#include "IR_SKYWORTH_SLIDE.h"
#elif (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)
#include "IR_HISENSE_6I78.h"
#elif (IR_TYPE_SEL==IR_TYPE_CUS08_RC5)
#include "IR_CUS08_RC5.h"
#else
#include "IR_MSTAR_DTV.h"
#endif

#ifdef CONFIG_IR_SUPPLY_RNG
#include <linux/input.h>
#include <random.h>
#include "mhal_rng_reg.h"
#endif

#if (IR_TYPE_SEL == IR_TYPE_MSTAR_FANTASY)
#define MSTAR_FANTASY_PROTOCOL_SUPPORTED 1
#else
#define MSTAR_FANTASY_PROTOCOL_SUPPORTED 0
#endif

#ifdef CONFIG_MSTAR_IR_FANTASY_MODE
#ifdef MSTAR_FANTASY_PROTOCOL_SUPPORTED
#undef MSTAR_FANTASY_PROTOCOL_SUPPORTED
#endif
#define MSTAR_FANTASY_PROTOCOL_SUPPORTED 1

#if 0
#ifdef IR_MODE_SEL
#undef IR_MODE_SEL
#endif
#define IR_MODE_SEL             IR_TYPE_RAWDATA_MODE
#endif
#endif      //#if (MSTAR_IR_FANTASY_MODE)

extern IRModHandle IRDev;
//-------------------------------------------------------------------------------------------------
// Macros
//-------------------------------------------------------------------------------------------------
#define REG(addr)                   (*(volatile u32 *)(addr))
#define IR_PRINT(fmt, args...)      //printk("[IR][%05d] " fmt, __LINE__, ## args)

//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------
#ifdef CONFIG_MSTAR_IR_INPUT_DEVICE
IR_KEYTAB_TYPE ir_codes_mstar_tv[IR_KEYTAB_SIZE] =
{
	//
	//  <<-!!! THIS IRCODE IS MAPPED TO MSTAR DTV IR !!!->>
	//
	[ 0x50 ] = KEY_0,
	[ 0x49 ] = KEY_1,
	[ 0x55 ] = KEY_2,
	[ 0x59 ] = KEY_3,
	[ 0x4D ] = KEY_4,
	[ 0x51 ] = KEY_5,
	[ 0x5D ] = KEY_6,
	[ 0x48 ] = KEY_7,
	[ 0x54 ] = KEY_8,
	[ 0x58 ] = KEY_9,
	[ 0x4C ] = KEY_SWITCHVIDEOMODE, // STAR
	[ 0x5C ] = KEY_KBDILLUMTOGGLE,  // POUND

	[ 0x52 ] = KEY_UP,
	[ 0x13 ] = KEY_DOWN,
	[ 0x06 ] = KEY_LEFT,
	[ 0x1A ] = KEY_RIGHT,
	[ 0x0F ] = KEY_ENTER,

	[ 0x40 ] = KEY_HOME,
	[ 0x07 ] = KEY_MENU,
	[ 0x1B ] = KEY_BACK,

	[ 0x47 ] = KEY_SEND,
	[ 0x5B ] = KEY_END,

	[ 0x16 ] = KEY_VOLUMEUP,
	[ 0x15 ] = KEY_VOLUMEDOWN,

	[ 0x46 ] = KEY_POWER,
};

EXPORT_SYMBOL_GPL(ir_codes_mstar_tv);

struct mstar_ir
{
	struct input_dev       *input;
	struct ir_input_state   ir;
};

static struct mstar_ir *ir;

#endif /* CONFIG_MSTAR_IR_INPUT_DEVICE */

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------


#define IR_RAW_DATA_NUM	        4
//#define IR_FILTER_REPEAT_NUM    1

#if (IR_MODE_SEL == IR_TYPE_SWDECODE_MODE)
#define IR_SWDECODE_MODE_BUF_LEN        100
#endif

//#define IR_DEBUG
#ifdef IR_DEBUG
#define DEBUG_IR(x) (x)
#else
#define DEBUG_IR(x)
#endif

//-------------------------------------------------------------------------------------------------
//  Local Structurs
//-------------------------------------------------------------------------------------------------
typedef enum
{
    E_IR_KEY_PROPERTY_INIT,
    E_IR_KEY_PROPERTY_1st,
    E_IR_KEY_PROPERTY_FOLLOWING
} IRKeyProperty;


//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
static U8 bIRPass = 0;
static pid_t MasterPid = 0;
//--------------------------------------------------------------------------------------------------
// Forward declaration
//--------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// Local Variables
//-------------------------------------------------------------------------------------------------
#if (IR_MODE_SEL == IR_TYPE_RAWDATA_MODE)
static U8   _u8IRRawModeBuf[IR_RAW_DATA_NUM];
static U32  _u8IRRawModeCount;
#endif

struct
{
    wait_queue_head_t inq;
    struct semaphore sem;
    U32 data; // 0=nothing, defined in fantasy protocol
}fantasy_protocol;

#if (IR_MODE_SEL == IR_TYPE_SWDECODE_MODE)

#if (IR_TYPE_SEL != IR_TYPE_CUS08_RC5)
static U32  _u32IRData[IR_SWDECODE_MODE_BUF_LEN];
#endif
static U32  _u32IRCount=0;

#if (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)
static U8   _u8IRHeadReceived=0;
static U8   _u8IRRepeateDetect=0;
static U8   _u8IRRepeated=0;
static U8   _u8IRType=0;

#elif (IR_TYPE_SEL == IR_TYPE_CUS08_RC5)
static U16  _u16IrRc5Data=0;          // for store shift ir data
static U16  _u16PreIrRc5Data=0;          // for store previous ir data
static U8 _u8IrRc5Bits=0;                // for store bit count
static U8 _u8IrRc5LastBit=0;//IR_RC5_LAST_BIT;            // for store bit var
static unsigned long  _ulPrevKeyTime;
#endif
#endif

static U32  _u32_1stDelayTimeMs;
static U32  _u32_2ndDelayTimeMs;
static IRKeyProperty _ePrevKeyProperty;


static U8   _u8PrevKeyCode;

#if (IR_TYPE_SEL != IR_TYPE_SKYWORTH_SLIDE)
#if (IR_MODE_SEL == IR_TYPE_FULLDECODE_MODE)
static unsigned long  _ulPrevKeyTime;
#endif
#else
static BOOL _btoshiba_code;
static BOOL _brepeat_flag;
#endif
static unsigned long  _ulLastKeyPresentTime;

static MS_IR_KeyInfo _KeyReceived;   //temporary solution

//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------
static void _MDrv_IR_Timing(void);
irqreturn_t _MDrv_IR_ISR(int irq, void *dev_id);
//irqreturn_t _MDrv_IR_ISR(int irq, void *dev_id, struct pt_regs *regs);

static U8   _MDrv_IR_ParseKey(U8 u8KeyData);
static BOOL _MDrv_IR_GetKey(U8 *pu8Key, U8 *pu8System, U8 *pu8Flag);
#if 0
static BOOL _MDrv_IR_GetInputData(U32* rawData);
#endif
static unsigned long _MDrv_IR_GetSystemTime(void);

//-------------------------------------------------------------------------------------------------
/// Translate from IR key to internal key.
/// @param  u8KeyData  \b IN: IR key value.
/// @return translated internal IR key.
//-------------------------------------------------------------------------------------------------
static U8 _MDrv_IR_ParseKey(U8 u8KeyData)
{
    U8 u8RetVal;

    if ( bIRPass ) return KEYCODE_DUMMY;

    switch(u8KeyData)
    {
    case IRKEY_TV_RADIO        : u8RetVal = KEYCODE_TV_RADIO;         break;
    case IRKEY_CHANNEL_LIST    : u8RetVal = KEYCODE_CHANNEL_LIST;     break;
    case IRKEY_CHANNEL_FAV_LIST: u8RetVal = KEYCODE_CHANNEL_FAV_LIST; break;
    case IRKEY_CHANNEL_RETURN  : u8RetVal = KEYCODE_CHANNEL_RETURN;   break;
    case IRKEY_CHANNEL_PLUS    : u8RetVal = KEYCODE_CHANNEL_PLUS;     break;
    case IRKEY_CHANNEL_MINUS   : u8RetVal = KEYCODE_CHANNEL_MINUS;    break;

    case IRKEY_AUDIO           : u8RetVal = KEYCODE_AUDIO;            break;
    case IRKEY_VOLUME_PLUS     : u8RetVal = KEYCODE_VOLUME_PLUS;      break;
    case IRKEY_VOLUME_MINUS    : u8RetVal = KEYCODE_VOLUME_MINUS;     break;

    case IRKEY_UP              : u8RetVal = KEYCODE_UP;               break;
    case IRKEY_POWER           : u8RetVal = KEYCODE_POWER;            break;
    case IRKEY_EXIT            : u8RetVal = KEYCODE_EXIT;             break;
    case IRKEY_MENU            : u8RetVal = KEYCODE_MENU;             break;
    case IRKEY_DOWN            : u8RetVal = KEYCODE_DOWN;             break;
    case IRKEY_LEFT            : u8RetVal = KEYCODE_LEFT;             break;
    case IRKEY_SELECT          : u8RetVal = KEYCODE_SELECT;           break;
    case IRKEY_RIGHT           : u8RetVal = KEYCODE_RIGHT;            break;

    case IRKEY_NUM_0           : u8RetVal = KEYCODE_NUMERIC_0;        break;
    case IRKEY_NUM_1           : u8RetVal = KEYCODE_NUMERIC_1;        break;
    case IRKEY_NUM_2           : u8RetVal = KEYCODE_NUMERIC_2;        break;
    case IRKEY_NUM_3           : u8RetVal = KEYCODE_NUMERIC_3;        break;
    case IRKEY_NUM_4           : u8RetVal = KEYCODE_NUMERIC_4;        break;
    case IRKEY_NUM_5           : u8RetVal = KEYCODE_NUMERIC_5;        break;
    case IRKEY_NUM_6           : u8RetVal = KEYCODE_NUMERIC_6;        break;
    case IRKEY_NUM_7           : u8RetVal = KEYCODE_NUMERIC_7;        break;
    case IRKEY_NUM_8           : u8RetVal = KEYCODE_NUMERIC_8;        break;
    case IRKEY_NUM_9           : u8RetVal = KEYCODE_NUMERIC_9;        break;

    case IRKEY_MUTE            : u8RetVal = KEYCODE_MUTE;             break;
    case IRKEY_PAGE_UP         : u8RetVal = KEYCODE_PAGE_UP;          break;
    case IRKEY_PAGE_DOWN       : u8RetVal = KEYCODE_PAGE_DOWN;        break;
    case IRKEY_CLOCK           : u8RetVal = KEYCODE_CLOCK;            break;

    case IRKEY_INFO            : u8RetVal = KEYCODE_INFO;             break;
    case IRKEY_RED             : u8RetVal = KEYCODE_RED;              break;
    case IRKEY_GREEN           : u8RetVal = KEYCODE_GREEN;            break;
    case IRKEY_YELLOW          : u8RetVal = KEYCODE_YELLOW;           break;
    case IRKEY_BLUE            : u8RetVal = KEYCODE_BLUE;             break;
    case IRKEY_MTS             : u8RetVal = KEYCODE_MTS;              break;
    case IRKEY_NINE_LATTICE    : u8RetVal = KEYCODE_NINE_LATTICE;     break;
#if defined(DVB_SYSTEM)
    case IRKEY_TTX             : u8RetVal = KEYCODE_TTX;              break;
#elif defined(ATSC_SYSTEM)
    case IRKEY_CC              : u8RetVal = KEYCODE_CC;               break;
#endif
    case IRKEY_INPUT_SOURCE    : u8RetVal = KEYCODE_INPUT_SOURCE;     break;
    case IRKEY_CRADRD          : u8RetVal = KEYCODE_CRADRD;           break;
//    case IRKEY_PICTURE         : u8RetVal = KEYCODE_PICTURE;          break;
    case IRKEY_ZOOM            : u8RetVal = KEYCODE_ZOOM;             break;
    case IRKEY_DASH            : u8RetVal = KEYCODE_DASH;             break;
    case IRKEY_SLEEP           : u8RetVal = KEYCODE_SLEEP;            break;
    case IRKEY_EPG             : u8RetVal = KEYCODE_EPG;              break;
    case IRKEY_PIP             : u8RetVal = KEYCODE_PIP;              break;

  	case IRKEY_MIX             : u8RetVal = KEYCODE_MIX;              break;
    case IRKEY_INDEX           : u8RetVal = KEYCODE_INDEX;            break;
    case IRKEY_HOLD            : u8RetVal = KEYCODE_HOLD;             break;
    case IRKEY_PREVIOUS        : u8RetVal = KEYCODE_PREVIOUS;         break;
    case IRKEY_NEXT            : u8RetVal = KEYCODE_NEXT;             break;
    case IRKEY_BACKWARD        : u8RetVal = KEYCODE_BACKWARD;         break;
    case IRKEY_FORWARD         : u8RetVal = KEYCODE_FORWARD;          break;
    case IRKEY_PLAY            : u8RetVal = KEYCODE_PLAY;             break;
    case IRKEY_RECORD          : u8RetVal = KEYCODE_RECORD;           break;
    case IRKEY_STOP            : u8RetVal = KEYCODE_STOP;             break;
    case IRKEY_PAUSE           : u8RetVal = KEYCODE_PAUSE;            break;

    case IRKEY_SIZE            : u8RetVal = KEYCODE_SIZE;             break;
    case IRKEY_REVEAL          : u8RetVal = KEYCODE_REVEAL;           break;
    case IRKEY_SUBCODE         : u8RetVal = KEYCODE_SUBCODE;          break;

    default                    : u8RetVal = KEYCODE_DUMMY;            break;
    }

    return u8RetVal;
}


//-------------------------------------------------------------------------------------------------
/// Set the timing of IrDa at BOOT stage.
/// @return None
//-------------------------------------------------------------------------------------------------
static void _MDrv_IR_Timing(void)
{
    // header code upper bound
    REG(REG_IR_HDC_UPB) = IR_HDC_UPB;

    // header code lower bound
    REG(REG_IR_HDC_LOB) = IR_HDC_LOB;

    // off code upper bound
    REG(REG_IR_OFC_UPB) = IR_OFC_UPB;

    // off code lower bound
    REG(REG_IR_OFC_LOB) = IR_OFC_LOB;

    // off code repeat upper bound
    REG(REG_IR_OFC_RP_UPB) = IR_OFC_RP_UPB;

    // off code repeat lower bound
    REG(REG_IR_OFC_RP_LOB) = IR_OFC_RP_LOB;

    // logical 0/1 high upper bound
    REG(REG_IR_LG01H_UPB) = IR_LG01H_UPB;

    // logical 0/1 high lower bound
    REG(REG_IR_LG01H_LOB) = IR_LG01H_LOB;

    // logical 0 upper bound
    REG(REG_IR_LG0_UPB) = IR_LG0_UPB;

    // logical 0 lower bound
    REG(REG_IR_LG0_LOB) = IR_LG0_LOB;

    // logical 1 upper bound
    REG(REG_IR_LG1_UPB) = IR_LG1_UPB;

    // logical 1 lower bound
    REG(REG_IR_LG1_LOB) = IR_LG1_LOB;

    // timeout cycles
    REG(REG_IR_TIMEOUT_CYC_L) = IR_RP_TIMEOUT & 0xFFFF;
    //set up ccode bytes and code bytes/bits num
    REG(REG_IR_TIMEOUT_CYC_H_CODE_BYTE) = IR_CCB_CB | 0x30 | ((IR_RP_TIMEOUT >> 16) & 0x0F);

    REG(REG_IR_CKDIV_NUM_KEY_DATA) = IR_CKDIV_NUM;   // clock divider
}

static unsigned long _MDrv_IR_GetSystemTime(void)
{
    return((unsigned long)((jiffies)*(1000/HZ)));
    //return 0;
}

static void _MDrv_IR_ClearFIFO(void)
{
    unsigned long i;

    // Empty the FIFO
    for(i=0; i<8; i++)
    {
        U8 u8Garbage;

        if(REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & IR_FIFO_EMPTY)
            break;

        u8Garbage = REG(REG_IR_CKDIV_NUM_KEY_DATA) >> 8;
        REG(REG_IR_FIFO_RD_PULSE) |= 0x0001; //read
    }

}

#if 0
static BOOL _MDrv_IR_GetInputData(U32* rawData)
{
    BOOL bRet = FALSE;
#if (IR_MODE_SEL == IR_TYPE_RAWDATA_MODE)
    U8 i, j;
    for (j=0; j<IR_RAW_DATA_NUM; j++)
    {
        if ( REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & IR_FIFO_EMPTY)  // check FIFO empty
            break;

        _u8IRRawModeBuf[_u8IRRawModeCount++] = REG(REG_IR_CKDIV_NUM_KEY_DATA) >> 8;
        REG(REG_IR_FIFO_RD_PULSE) |= 0x0001; //read
#if 0
        if(_u8IRRawModeCount == IR_RAW_DATA_NUM)
        {
            _u8IRRawModeCount = 0;
            *rawData=((_u8IRRawModeBuf[3]<<24)|(_u8IRRawModeBuf[2]<<16)|(_u8IRRawModeBuf[1]<<8)|_u8IRRawModeBuf[0]);
            bRet = TRUE;
        }
#else
        if(_u8IRRawModeCount == IR_RAW_DATA_NUM)
        {
            _u8IRRawModeCount = 0;
            if( (_u8IRRawModeBuf[0]==IR_HEADER_CODE0) &&
                (_u8IRRawModeBuf[1]==IR_HEADER_CODE1) )
            {
                if(_u8IRRawModeBuf[2] == (U8)(~_u8IRRawModeBuf[3]))
                {
                    *rawData = _MDrv_IR_ParseKey(_u8IRRawModeBuf[2]);    // translate for MIPS

                    //set fantasy class to IR
                    *rawData |= (0x01 << 28);

                    //TODO: Implement repeat code later.

                    bRet = TRUE;
                    break;
                }
            }
        }
#endif
    }

    // Empty the FIFO
    _MDrv_IR_ClearFIFO();
#endif
    return bRet;
}
#endif

//-------------------------------------------------------------------------------------------------
/// ISR when receive IR key.
/// @return None
//-------------------------------------------------------------------------------------------------
irqreturn_t _MDrv_IR_ISR(int irq, void *dev_id)
//irqreturn_t _MDrv_IR_ISR(int irq, void *dev_id, struct pt_regs *regs)
{
    U8 u8Key=0, u8RepeatFlag=0;
    U8 u8System = 0;
#if (IR_MODE_SEL == IR_TYPE_SWDECODE_MODE)
    #if (IR_TYPE_SEL == IR_TYPE_SKYWORTH_SLIDE)
    U16 tmp;
    tmp=(((REG(REG_IR_SHOT_CNT_H_FIFO_STATUS)) & 0xF) << 16) | ((REG(REG_IR_SHOT_CNT_L))&0xFFFF);
    _u32IRData[_u32IRCount++] = tmp;
    if(tmp>10000)
    {
        _u32IRCount=0;
        _u32IRData[_u32IRCount++] = tmp;
    }
    else if(tmp> (IR_HDC_LOB)&&tmp < (IR_HDC_UPB))
    {
        _u32IRCount=1;
        _u32IRData[_u32IRCount++] = tmp;
    }

    if(_u32IRCount==2)
    {
        if ((_u32IRData[1] > (IR_HDC_LOB)) && (_u32IRData[1] < (IR_HDC_UPB)))
            _btoshiba_code=TRUE;
        else
            _btoshiba_code=FALSE;
    }
    else if(_u32IRCount==3)
    {
        if (_u32IRData[2] > IR_LG1_LOB && _u32IRData[2] < IR_LG1_UPB)   //repeat key
            _brepeat_flag=TRUE;
    }
    /*if(!btoshiba_code&&_u32IRCount>18)
    {
        for(tmp=0;tmp<18;tmp++)
            printk("<<<%d data %d\n",tmp,_u32IRData[tmp]);
        _u32IRCount=0;
    }
    else if(btoshiba_code&&_u32IRCount>33)
    {
        for(tmp=0;tmp<34;tmp++)
            printk("<<<%d data %d\n",tmp,_u32IRData[tmp]);
        _u32IRCount=0;
    }*/

    #elif (IR_TYPE_SEL == IR_TYPE_CUS08_RC5)

    U16 u16IrCounter,u16N_Shot;

    u16IrCounter=((REG(REG_IR_SHOT_CNT_H_FIFO_STATUS)&0xF) << 16) | ((REG(REG_IR_SHOT_CNT_L))&0xFFFF);
    u16N_Shot=(REG(REG_IR_SHOT_CNT_H_FIFO_STATUS))&0x10;

    if ((u16IrCounter > ONE_BIT_UB) || (u16IrCounter < HALF_BIT_LB))    // if ir counter so big, then reset and return
    {
        _u8IrRc5LastBit = TRUE;
        _u16IrRc5Data = 0;
        _u8IrRc5Bits = 0;
    }
    else
    {
        if (u16IrCounter >= HALF_BIT_LB && u16IrCounter <= HALF_BIT_UB)     //detect
        {
            _u8IrRc5Bits++;
            if (_u8IrRc5LastBit)
            {
                _u8IrRc5LastBit = FALSE;
                _u16IrRc5Data <<= 1;
                if (u16N_Shot==0) // Is nshot
                    _u16IrRc5Data |= 1;
            }
            else
            {
                _u8IrRc5LastBit = TRUE;
            }
        }
        else if (u16IrCounter >= ONE_BIT_LB && u16IrCounter <= ONE_BIT_UB)
        {
            if (_u8IrRc5LastBit)
            {
                _u16IrRc5Data <<= 1;
                if (u16N_Shot==0) // Is nshot
                    _u16IrRc5Data |= 1;
                _u8IrRc5Bits += 2;
            }
            else    //error stste
            {
                _u16IrRc5Data = 0;
                _u8IrRc5Bits = 0;
            }
        }

        if (_u8IrRc5Bits == IR_RC5_DATA_BITS)
        {
            if(u16N_Shot==0)
            {
                _u16IrRc5Data <<= 1;
                _u8IrRc5Bits++;
            }
        }
    }

    #elif (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)
    static unsigned long ulPreTime;
    U32 tmp;

    if ((_MDrv_IR_GetSystemTime() - ulPreTime > IR_TIMEOUT_CYC/1000))//||_u8IRRepeated) //timeout or not handler yet
    {
        _u32IRCount = 0;
		_u8IRRepeateDetect=0;
		_u8IRRepeated=0;
		_u8IRHeadReceived=0;
    }
    if (_u32IRCount <IR_SWDECODE_MODE_BUF_LEN)
    {
        tmp=((REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & 0xF) << 16) | REG(REG_IR_SHOT_CNT_L);
		if(tmp<IR_LG0_LOB)//error signal
		{
        	_u32IRCount = 0;
			_u8IRRepeateDetect=0;
			_u8IRRepeated=0;
			_u8IRHeadReceived=0;
		}

		if( tmp>(IR_HDC_LOB))//Head received
		{
			_u32IRCount=0;
			_u8IRHeadReceived=1;
		}
		if(_u8IRHeadReceived)//begin data received
		{
	        _u32IRData[_u32IRCount++] = tmp;

			if(_u8IRRepeateDetect)
			{
				if( tmp>(IR_OFC_RP_LOB) && tmp<(IR_OFC_RP_UPB) )
				{
					_u8IRRepeated=1;
					_u8IRHeadReceived=0;
				}
			}

		}

    }
    ulPreTime = _MDrv_IR_GetSystemTime();

    #else
    static unsigned long ulPreTime;
    if (_MDrv_IR_GetSystemTime() - ulPreTime > IR_TIMEOUT_CYC/1000) //timeout
    {
        _u32IRCount = 0;
    }

    if (_u32IRCount <IR_SWDECODE_MODE_BUF_LEN)
    {
        _u32IRData[_u32IRCount++] = ((REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & 0xF) << 16) | REG(REG_IR_SHOT_CNT_L);
    }
    ulPreTime = _MDrv_IR_GetSystemTime();
    #endif
#endif
#ifdef CONFIG_IR_SUPPLY_RNG
    {
        unsigned int u32Temp;
        unsigned short u16Temp;

        u16Temp = MIPS_REG(REG_RNG_OUT);
        memcpy((unsigned char *)&u32Temp+0, &u16Temp, 2);
        u16Temp = MIPS_REG(REG_RNG_OUT);
        memcpy((unsigned char *)&u32Temp+2, &u16Temp, 2);
        add_input_randomness(EV_MSC, MSC_SCAN, u32Temp);
    }
#endif
    //regs = NULL;
    if(MSTAR_FANTASY_PROTOCOL_SUPPORTED)
    {
//        U32 u32InputData=0;
//        if(_MDrv_IR_GetInputData(&u32InputData))

        if (_MDrv_IR_GetKey(&u8Key, &u8System, &u8RepeatFlag))
        {
            if(down_trylock(&fantasy_protocol.sem)==0)
            {
                if(fantasy_protocol.data!=0)
                {
                    //printk(KERN_WARNING"unprocessed fantasy protocol data!!\n");
                }
                else
                {
					#if (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)//for hisense need suport 2 ir head code ;
					U8 hisense_factory_ir=0;
					if(u8RepeatFlag&FCTORY_HEADER_RECEIVE)
					{
						u8RepeatFlag &= (~FCTORY_HEADER_RECEIVE);
	                    hisense_factory_ir=1;
					}
                    fantasy_protocol.data = (u8Key<<8 | u8RepeatFlag);

					if(hisense_factory_ir)//HISENSE_FACTORY IR
                    	fantasy_protocol.data |= (0x04 << 28);
					else
	                    fantasy_protocol.data |= (0x01 << 28);
					#else //
//                    fantasy_protocol.data=u32InputData;
                    fantasy_protocol.data = (u8Key<<8 | u8RepeatFlag);

                    //set fantasy class to IR
                    fantasy_protocol.data |= (0x01 << 28);
					#endif

                }
                up(&fantasy_protocol.sem);

                wake_up_interruptible(&fantasy_protocol.inq);
            }
        }
    }
    else if (_MDrv_IR_GetKey(&u8Key, &u8System, &u8RepeatFlag))
    {
        _ulLastKeyPresentTime = _MDrv_IR_GetSystemTime();
        IR_PRINT("_MDrv_IR_ISR() -> %x, %x\n", u8Key, u8RepeatFlag);

        //temporary solution, need to implement ring buffer for this
        _KeyReceived.u8Key = u8Key;
        _KeyReceived.u8System = u8System;
        _KeyReceived.u8Flag = u8RepeatFlag;
        _KeyReceived.u8Valid = 1;

       	if (IRDev.async_queue)
    		kill_fasync(&IRDev.async_queue, SIGIO, POLL_IN);
    }

    //Disabled already by SA_INTERRUPT at initial
    //MsOS_EnableInterrupt(E_FIQ_IR);

#ifdef CONFIG_MSTAR_IR_INPUT_DEVICE
    if (NULL != ir)
    { // <-@@@
        static U8 u8PreKey = IRKEY_DUMY;
        static unsigned int u32PreTime = 0;
        unsigned int now;

        now = jiffies_to_msecs(jiffies);

        /* the same keys must has interval larger then 200msec */
        if (u8PreKey != u8Key || now - u32PreTime > 200)
        {
            //printk(">"); // <-@@@
            ir_input_keydown(ir->input, &ir->ir, u8Key, (u8RepeatFlag << 8) | u8Key);

            /* up immediately to prevent repeat by input layer */
            ir_input_nokey(ir->input, &ir->ir);

            u8PreKey = u8Key;
            u32PreTime = now;

#if 0   // <-@@@ REMOVEME: dirty hack to enable/disable GWIN
            if (   IRKEY_PLAY == u8Key // disable GWIN
                || IRKEY_STOP == u8Key // enable GWIN
                )
            {
                #define BANK_MSK 0x0F
#if defined(CONFIG_MSTAR_TITANIA2)
                #define _RIU_MAP 0xBF800000 // T2
#else  // CONFIG_MSTAR_TITANIA2
                #define _RIU_MAP 0xBF200000 // T3
#endif // CONFIG_MSTAR_TITANIA2
                #define _RIU ((unsigned short volatile *) _RIU_MAP)

                U8 u8Bank;
                static U8 u8Enable = 0;

                u8Enable = (IRKEY_PLAY == u8Key) ? 0 : 1;

                u8Bank = _RIU[0x1FFE] & BANK_MSK;
                _RIU[0x1FFE] = (_RIU[0x1FFE] & (~0x020F)) | 0x0201;
                _RIU[0x1F00] = (_RIU[0x1F00] & (~0x0001)) | u8Enable;
                _RIU[0x1FFE] = ((_RIU[0x1FFE] & (~0x020F)) | 0x0200) | u8Bank;
            }
#endif
        }
        //printk("u8Key = 0x%02X, 0x%02X\n", u8Key, u8RepeatFlag); // <-@@@
    } // <-@@@
#endif

    return IRQ_HANDLED;
}


#if (IR_MODE_SEL == IR_TYPE_FULLDECODE_MODE)
//-------------------------------------------------------------------------------------------------
/// Get IR key. It is a non-blocking function.
/// @param pu8Key  \b IN: Return IR key value.
/// @param pu8Flag \b IN: Return IR repeat code.
///
/// @return TRUE:  Success
/// @return FALSE: No key or repeat key is faster than the specified period
//-------------------------------------------------------------------------------------------------
static BOOL _MDrv_IR_GetKey(U8 *pu8Key, U8 *pu8System, U8 *pu8Flag)
{
    unsigned long i;
    BOOL bRet;
    *pu8System = 0;

    if(REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & IR_FIFO_EMPTY)
    {
        bRet = FALSE;
    }
    else
    {
        *pu8Key = REG(REG_IR_CKDIV_NUM_KEY_DATA) >> 8;
        REG(REG_IR_FIFO_RD_PULSE) |= 0x0001; //read

    	//for(i=0;i<10000;i++);   // Delay
        udelay(1000*1);
        if ( REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & IR_RPT_FLAG )
            *pu8Flag = 1;
        else
            *pu8Flag = 0;
        bRet = TRUE;
    }

#if 1
    if(bRet)
    {
        if ( (_u8PrevKeyCode != *pu8Key) || (!*pu8Flag) )
        {
            _ePrevKeyProperty = E_IR_KEY_PROPERTY_INIT;
        }

        i = _MDrv_IR_GetSystemTime();
        if( _ePrevKeyProperty == E_IR_KEY_PROPERTY_INIT)
        {
            _u8PrevKeyCode     = *pu8Key;
            _ulPrevKeyTime    = i;
            _ePrevKeyProperty  = E_IR_KEY_PROPERTY_1st;
        }
        else if(_ePrevKeyProperty == E_IR_KEY_PROPERTY_1st)
        {
            if( (i - _ulPrevKeyTime) > _u32_1stDelayTimeMs)
            {
                _ulPrevKeyTime = i;
                _ePrevKeyProperty  = E_IR_KEY_PROPERTY_FOLLOWING;
            }
            else
            {
                bRet = FALSE;
            }
        }
        else //E_IR_KEY_PROPERTY_FOLLOWING
        {
            if( (i - _ulPrevKeyTime) > _u32_2ndDelayTimeMs)
            {
                _ulPrevKeyTime = i;
            }
            else
            {
                bRet = FALSE;
            }
        }
    }
#endif
#if 0
    if(bRet)
    {
        //if (!MDrv_SYS_GetDisplayControllerSeparated())
            *pu8Key = _MDrv_IR_ParseKey(*pu8Key);   // translate for MIPS
    }
#endif
    // Empty the FIFO
    _MDrv_IR_ClearFIFO();

    return bRet;
}

#elif (IR_MODE_SEL == IR_TYPE_RAWDATA_MODE)
//-------------------------------------------------------------------------------------------------
/// Get IR key.
/// @param pu8Key  \b IN: Return IR key value.
/// @param pu8Flag \b IN: Return IR repeat code.
///
/// @return TRUE:  Success
/// @return FALSE: Failure
//-------------------------------------------------------------------------------------------------
static BOOL _MDrv_IR_GetKey(U8 *pu8Key, U8 *pu8System, U8 *pu8Flag)
{
    BOOL bRet = FALSE;

    u32 i, j;
    *pu8System = 0;

    for (j=0; j<IR_RAW_DATA_NUM; j++)
    {
        if ( REG(REG_IR_SHOT_CNT_H_FIFO_STATUS) & IR_FIFO_EMPTY)  // check FIFO empty
            break;

        _u8IRRawModeBuf[_u8IRRawModeCount++] = REG(REG_IR_CKDIV_NUM_KEY_DATA) >> 8;
        REG(REG_IR_FIFO_RD_PULSE) |= 0x0001; //read

    	for(i=0;i<5;i++); //Delay

        if(_u8IRRawModeCount == IR_RAW_DATA_NUM)
        {
            _u8IRRawModeCount = 0;
            if( (_u8IRRawModeBuf[0]==IR_HEADER_CODE0) &&
                (_u8IRRawModeBuf[1]==IR_HEADER_CODE1) )
            {
                if(_u8IRRawModeBuf[2] == (U8)(~_u8IRRawModeBuf[3]))
                {
                    //if (!MDrv_SYS_GetDisplayControllerSeparated())
                        *pu8Key = _MDrv_IR_ParseKey(_u8IRRawModeBuf[2]);    // translate for MIPS

                    //TODO: Implement repeat code later.
                    *pu8Flag = 0;
                    bRet = TRUE;
                    break;
                }
            }
        }
    }

    // Empty the FIFO
    _MDrv_IR_ClearFIFO();

    return bRet;
}


#elif(IR_MODE_SEL == IR_TYPE_SWDECODE_MODE)
//-------------------------------------------------------------------------------------------------
/// Get IR key.
/// @param pu8Key  \b IN: Return IR key value.
/// @param pu8Flag \b IN: Return IR repeat code.
///
/// @return TRUE:  Success
/// @return FALSE: Failure
//-------------------------------------------------------------------------------------------------
static BOOL _MDrv_IR_GetKey(U8 *pu8Key, U8 *pu8System, U8 *pu8Flag)
{
    BOOL bRet = FALSE;

    #if (IR_TYPE_SEL != IR_TYPE_CUS08_RC5)
    U32 u8Byte, u8Bit;
    U8 u8IRSwModeBuf[IR_RAW_DATA_NUM];
    U32 *pu32IRData = NULL;
    *pu8System = 0;
    #else
    unsigned long i;
    #endif

    #if (IR_TYPE_SEL == IR_TYPE_SKYWORTH_SLIDE)
    //U8 u8tmp;

    if(_btoshiba_code)
    {
        if(_brepeat_flag)
        {
            *pu8Key = _u8PrevKeyCode;
            *pu8Flag = 1;
            _u32IRCount = 0;
            _btoshiba_code=FALSE;
            _brepeat_flag=FALSE;
            printk("repeat input key is 0x%x\n",*pu8Key);
            return TRUE;
        }

        if (_u32IRCount< (34))  //multiple 8
            return FALSE; //not complete yet
        else
        {
            for( u8Byte=0; u8Byte<IR_RAW_DATA_NUM; u8Byte++)
            {
               u8IRSwModeBuf[u8Byte] = 0;
            }
            pu32IRData = &_u32IRData[2];

            for( u8Byte=0; u8Byte<IR_RAW_DATA_NUM; u8Byte++)
            {
                for( u8Bit=0; u8Bit<8; u8Bit++)
                {
                    u32 u32BitLen = pu32IRData[u8Byte*8+u8Bit];
                    u8IRSwModeBuf[u8Byte] >>= 1;

                    if( u32BitLen > IR_LG0_LOB && u32BitLen < IR_LG0_UPB ) //0
                    {
                        u8IRSwModeBuf[u8Byte] |= 0x00;
                    }
                    else // if (u32BitLen > IR_LG1_LOB && u32BitLen < IR_LG1_UPB) //1
                    {
                        u8IRSwModeBuf[u8Byte] |= 0x80;
                    }
                    /*else
                    {
                        DEBUG_IR(printk(" invalid waveform,0x%x\n",u32BitLen));
                        for(u8tmp=0;u8tmp<_u32IRCount;u8tmp++)
                            printk("%d data=%d\n",u8tmp,_u32IRData[u8tmp]);
                        _u32IRCount = 0;
                        btoshiba_code=FALSE;
                        return FALSE;
                    }*/
                }
                if((u8IRSwModeBuf[0]!=IR_HEADER_CODE0)||(u8Byte==1&&u8IRSwModeBuf[1]!=IR_HEADER_CODE1))
                {
                    U8 u8tmp;
                    for(u8tmp=0;u8tmp<_u32IRCount;u8tmp++)
                            printk("%d data=%d\n",u8tmp,_u32IRData[u8tmp]);
                    _u32IRCount = 0;
                    _btoshiba_code=FALSE;
                    DEBUG_IR(printk("invalid header\n"));
                    return FALSE;
                }
                else if(u8Byte==3)
                {
                    if(u8IRSwModeBuf[2] == (u8)~u8IRSwModeBuf[3])
                    {
                        *pu8Key = u8IRSwModeBuf[2];
                        *pu8Flag = 0;
                        _u8PrevKeyCode=*pu8Key;
                        _u32IRCount = 0;
                        _btoshiba_code=FALSE;
                        DEBUG_IR(printk("input key is 0x%x\n",*pu8Key));
                        printk("input key is 0x%x\n",*pu8Key);
                        return TRUE;
                    }
                }
            }//for( u8Byte=0; u8Byte<IR_RAW_DATA_NUM; u8Byte++)
        }//else
    }//toshiba code
    else
    {
        if(_u32IRCount< (18))  //multiple 4
            return FALSE; //not complete yet
        else
        {
            for( u8Byte=0; u8Byte<2; u8Byte++)
            {
               u8IRSwModeBuf[u8Byte] = 0;
            }
            pu32IRData = &_u32IRData[2];
            for( u8Byte=0; u8Byte<2; u8Byte++)
            {
                for( u8Bit=0; u8Bit<8; u8Bit++)
                {
                    u32 u32BitLen = pu32IRData[u8Byte*8+u8Bit];
                    u8IRSwModeBuf[u8Byte] >>= 1;

                    if( u32BitLen > IR_LG0_LOB && u32BitLen < IR_LG0_UPB ) //0
                    {
                        u8IRSwModeBuf[u8Byte] |= 0x00;
                    }
                    else // if (u32BitLen > IR_LG1_LOB && u32BitLen < IR_LG1_UPB) //1
                    {
                        u8IRSwModeBuf[u8Byte] |= 0x80;
                    }
                    /*else
                    {
                        DEBUG_IR(printk(" invalid waveform,0x%x\n",u32BitLen));
                        bRet = FALSE;
                    }*/
                }
            }//for( u8Byte=0; u8Byte<2; u8Byte++)

            if(((u8IRSwModeBuf[0]&0xf0)>>4)==IR_HEADER_CODE3)
            {
                u8IRSwModeBuf[2]=u8IRSwModeBuf[0]&0x0f;
                u8IRSwModeBuf[3]=u8IRSwModeBuf[1]&0x0f;
                if((u8IRSwModeBuf[2]+u8IRSwModeBuf[3])==0x0f)
                {
                    switch(u8IRSwModeBuf[2])
                    {
                        case IRKEY_DESIL1:
                        case IRKEY_DESIL2:
                        case IRKEY_DESIL3:
                        case IRKEY_DESIL4:
                        case IRKEY_DESIL5:
                        case IRKEY_DESIL6:
                            *pu8Key=IRKEY_SLIDE_L1+(u8IRSwModeBuf[2]-IRKEY_DESIL1);
                             break;
                        case IRKEY_REVERSE1:
                        case IRKEY_REVERSE2:
                        case IRKEY_REVERSE3:
                        case IRKEY_REVERSE4:
                        case IRKEY_REVERSE5:
                        case IRKEY_REVERSE6:
                            *pu8Key=IRKEY_SLIDE_R1+(u8IRSwModeBuf[2]-IRKEY_REVERSE1);
                            break;
                    }
                    _u8PrevKeyCode=*pu8Key;
                    DEBUG_IR(printk("slide key 0x%x\n",*pu8Key));
                    printk("slide key 0x%x\n",*pu8Key);
                    *pu8Flag = 0;
                    bRet = TRUE;
                }
            }
            _u32IRCount = 0;
            _btoshiba_code=FALSE;
            return bRet;
        }
    }
    return bRet;

#elif (IR_TYPE_SEL == IR_TYPE_CUS08_RC5)

        if(_u8IrRc5Bits > IR_RC5_DATA_BITS)
        {
            if ((_u16IrRc5Data&0xF000)==0x3000)
            {
                if(_u16PreIrRc5Data==_u16IrRc5Data)
                    *pu8Flag = TRUE;

                *pu8Key = _u16IrRc5Data&0x3F;
                *pu8System = (_u16IrRc5Data&0x3C0)>>6;
                _u16PreIrRc5Data=_u16IrRc5Data;
                _u8IrRc5Bits=0;
                bRet = TRUE;
            }
        }

        if(bRet)
        {
            if ( (_u8PrevKeyCode != *pu8Key) || (!*pu8Flag) )
            {
                _ePrevKeyProperty = E_IR_KEY_PROPERTY_INIT;
            }

            i = _MDrv_IR_GetSystemTime();
            if( _ePrevKeyProperty == E_IR_KEY_PROPERTY_INIT)
            {
                _u8PrevKeyCode     = *pu8Key;
                _ulPrevKeyTime    = i;
                _ePrevKeyProperty  = E_IR_KEY_PROPERTY_1st;
            }
            else if(_ePrevKeyProperty == E_IR_KEY_PROPERTY_1st)
            {
                if( (i - _ulPrevKeyTime) > _u32_1stDelayTimeMs)
                {
                    _ulPrevKeyTime = i;
                    _ePrevKeyProperty  = E_IR_KEY_PROPERTY_FOLLOWING;
                }
                else
                {
                    bRet = FALSE;
                }
            }
            else //E_IR_KEY_PROPERTY_FOLLOWING
            {
                if( (i - _ulPrevKeyTime) > _u32_2ndDelayTimeMs)
                {
                    _ulPrevKeyTime = i;
                }
                else
                {
                    bRet = FALSE;
                }
            }
        }

        return bRet;

#else

    for( u8Byte=0; u8Byte<IR_RAW_DATA_NUM; u8Byte++)
    {
       u8IRSwModeBuf[u8Byte] = 0;
    }
	#if (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)
	if(_u8IRRepeated)//repeate key
	{
		_u8IRRepeated=0;
		_u8IRHeadReceived=0;//clear head receive flag
        *pu8Key = _u8PrevKeyCode;
        *pu8Flag = 1;
        bRet = TRUE;
		if(_u8IRType)
		*pu8Flag |=FCTORY_HEADER_RECEIVE;
        goto done;
	}
    if (_u32IRCount<(2+IR_RAW_DATA_NUM*8))
        return FALSE; //not complete yet
	#else
    if (_u32IRCount< 3+IR_RAW_DATA_NUM*8)
        return FALSE; //not complete yet
	#endif
    DEBUG_IR(printk("_u32IRCount=%d", _u32IRCount));
    for( u8Byte=0; u8Byte<_u32IRCount; u8Byte++)
    {
       DEBUG_IR(printk(" %d", _u32IRData[u8Byte]));
    }

    if( _u32IRData[0] > IR_HDC_LOB && _u32IRData[1] > IR_OFC_LOB+IR_LG01H_LOB && _u32IRData[1] < REG_IR_OFC_UPB+IR_LG01H_UPB )
    {
        pu32IRData = &_u32IRData[2];
        DEBUG_IR(printk(" H1 "));
    }
    else if( _u32IRData[1] > IR_HDC_LOB && _u32IRData[2] > IR_OFC_LOB+IR_LG01H_LOB && _u32IRData[2] < REG_IR_OFC_UPB+IR_LG01H_UPB )
    {
        pu32IRData = &_u32IRData[3];
        DEBUG_IR(printk(" H2 "));
    }
    else
    {
        DEBUG_IR(printk(" invalid leader code\n"));
        bRet = FALSE;
        goto done;
    }

    for( u8Byte=0; u8Byte<IR_RAW_DATA_NUM; u8Byte++)
    {
        for( u8Bit=0; u8Bit<8; u8Bit++)
        {
            u32 u32BitLen = pu32IRData[u8Byte*8+u8Bit];
            u8IRSwModeBuf[u8Byte] >>= 1;

            if( u32BitLen > IR_LG0_LOB && u32BitLen < IR_LG0_UPB ) //0
            {
                u8IRSwModeBuf[u8Byte] |= 0x00;
            }
            else if (u32BitLen > IR_LG1_LOB && u32BitLen < IR_LG1_UPB) //1
            {
                u8IRSwModeBuf[u8Byte] |= 0x80;
            }
            else
            {
                DEBUG_IR(printk(" invalid waveform,0x%x\n",u32BitLen));
                bRet = FALSE;
                goto done;
            }
        }
    }

    if(u8IRSwModeBuf[0] == IR_HEADER_CODE0)
    {
        if(u8IRSwModeBuf[1] == IR_HEADER_CODE1)
        {
            if(u8IRSwModeBuf[2] == (u8)~u8IRSwModeBuf[3])
            {
                *pu8Key = u8IRSwModeBuf[2];
                *pu8Flag = 0;
                bRet = TRUE;
				#if (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)
                _u8PrevKeyCode=*pu8Key;
				_u8IRRepeateDetect=1;
				_u8IRHeadReceived=0;
				_u8IRType=0;
				#endif

                goto done;
            }
        }
    }
	#if (IR_TYPE_SEL == IR_TYPE_HISENSE_6I78)
	if(u8IRSwModeBuf[0] == IR_FHEADER_CODE0)
	{
		if(u8IRSwModeBuf[1] == IR_FHEADER_CODE1)
        {
            if(u8IRSwModeBuf[2] == (u8)~u8IRSwModeBuf[3])
            {
                *pu8Key = u8IRSwModeBuf[2];
                *pu8Flag = 0;
				*pu8Flag |=FCTORY_HEADER_RECEIVE;
                bRet = TRUE;
                _u8PrevKeyCode=*pu8Key;
				_u8IRRepeateDetect=1;
				_u8IRHeadReceived=0;
				_u8IRType=1;
                goto done;
            }
        }
    }
	#endif
    DEBUG_IR(printk(" invalid data\n"));
    bRet = FALSE;

done:
    _u32IRCount = 0;
    return bRet;
    #endif
}
#endif


//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
/// Initialize IR timing and enable IR.
/// @return None
//-------------------------------------------------------------------------------------------------
void MDrv_IR_Init(void)
{
	#if((IR_MODE_SEL==IR_TYPE_RAWDATA_MODE) || (IR_MODE_SEL==IR_TYPE_FULLDECODE_MODE))
    //u32 i=0x00;
    #endif
	int result;

#if (IR_MODE_SEL == IR_TYPE_RAWDATA_MODE)
    _u8IRRawModeCount = 0;
#endif
    init_MUTEX(&fantasy_protocol.sem);
    init_waitqueue_head(&fantasy_protocol.inq);
    fantasy_protocol.data=0;


#if (IR_MODE_SEL == IR_TYPE_SWDECODE_MODE)
    _u32IRCount = 0;
#endif


    if(IR_MODE_SEL == IR_TYPE_RAWDATA_MODE)
    {
        REG(REG_IR_CTRL) = IR_TIMEOUT_CHK_EN |
                                   IR_INV            |
                                   IR_LG01H_CHK_EN   |
                                   IR_LDCCHK_EN      |
                                   IR_EN;
    }
    else
    {
        REG(REG_IR_CTRL) = IR_TIMEOUT_CHK_EN |
                                   IR_INV            |
                                   IR_RPCODE_EN      |
                                   IR_LG01H_CHK_EN   |
                                   IR_DCODE_PCHK_EN  |
                                   IR_CCODE_CHK_EN   |
                                   IR_LDCCHK_EN      |
                                   IR_EN;
    }

    _MDrv_IR_Timing();

    REG(REG_IR_CCODE) = ((u16)IR_HEADER_CODE1<<8) | IR_HEADER_CODE0;
    REG(REG_IR_SEPR_BIT_FIFO_CTRL) = 0xF00;
    REG(REG_IR_GLHRM_NUM) = 0x804;

#if (IR_MODE_SEL==IR_TYPE_FULLDECODE_MODE)
    REG(REG_IR_GLHRM_NUM) |= (0x3 <<12);
    REG(REG_IR_FIFO_RD_PULSE) |= 0x0020; //wakeup key sel
#elif (IR_MODE_SEL==IR_TYPE_RAWDATA_MODE)
    REG(REG_IR_GLHRM_NUM) |= (0x2 <<12);
    REG(REG_IR_FIFO_RD_PULSE) |= 0x0020; //wakeup key sel
#else
    REG(REG_IR_GLHRM_NUM) |= (0x1 <<12);
	#ifdef IR_INT_NP_EDGE_TRIG	//for N/P edge trigger
    	REG(REG_IR_SEPR_BIT_FIFO_CTRL) |= 0x3 <<12;
    #else
    	REG(REG_IR_SEPR_BIT_FIFO_CTRL) |= 0x2 <<12;
	#endif
#endif


#if((IR_MODE_SEL==IR_TYPE_RAWDATA_MODE) || (IR_MODE_SEL==IR_TYPE_FULLDECODE_MODE))
    // Empty the FIFO
    _MDrv_IR_ClearFIFO();
#endif

    _u32_1stDelayTimeMs = 0;
    _u32_2ndDelayTimeMs = 0;
    _ePrevKeyProperty = E_IR_KEY_PROPERTY_INIT;

    memset(&_KeyReceived, 0 , sizeof(_KeyReceived) );

    result = request_irq(E_FIQ_IR, _MDrv_IR_ISR, SA_INTERRUPT, "IR", NULL);
    if (result)
    {
        IR_PRINT("IR IRQ registartion ERROR\n");
	}
	else
    {
        IR_PRINT("IR IRQ registartion OK\n");
    }

    #if (defined(CONFIG_MSTAR_TITANIA)||defined(CONFIG_MSTAR_TITANIA2))

    #else
    // unmask IR IRQ on PM
    REG(REG_IRQ_MASK_IR) &= IRQ_UNMASK_IR;
    #endif
    //enable_irq(E_FIQ_IR);

#ifdef CONFIG_MSTAR_IR_INPUT_DEVICE
    { // <-@@@
        IR_KEYTAB_TYPE *ir_codes;
        int ir_type;
        struct input_dev *input_dev;
        int err;

        ir = kzalloc(sizeof(struct mstar_ir), GFP_KERNEL);
        input_dev = input_allocate_device();

        if (NULL == ir || NULL == input_dev)
        {
            err = -ENOMEM;
            goto MDrv_IR_Init_err_out_free;
        }

        /*
         * Android: add for fake touchscreen
         * some android app like Contacts or Music need touchscreen resource
         * add fake one here to avoid crash
         */
	set_bit(BTN_TOUCH, input_dev->keybit);
        input_dev->evbit[0] = BIT_MASK(EV_KEY) | BIT_MASK(EV_ABS);
        input_set_abs_params(input_dev, ABS_X, 60, 985, 0, 0);
        input_set_abs_params(input_dev, ABS_Y, 35, 1024, 0, 0);

        ir->input = input_dev;

        ir_type = IR_TYPE_RC5;
        ir_codes = ir_codes_mstar_tv;

        ir_input_init(input_dev, &ir->ir, ir_type, ir_codes);

        err = input_register_device(ir->input);

        if (err)
        {
            goto MDrv_IR_Init_err_out_free;
        }

        return;

MDrv_IR_Init_err_out_free:
        input_free_device(input_dev);
        kfree(ir);
    } // <-@@@
#endif
}
EXPORT_SYMBOL(MDrv_IR_Init);


//-------------------------------------------------------------------------------------------------
/// Set the IR delay time to recognize a valid key.
/// @param  u32_1stDelayTimeMs \b IN: Set the delay time to get the 1st key.
/// @param  u32_2ndDelayTimeMs \b IN: Set the delay time to get the following keys.
/// @return None
//-------------------------------------------------------------------------------------------------
void MDrv_IR_SetDelayTime(u32 u32_1stDelayTimeMs, u32 u32_2ndDelayTimeMs)
{
//TBD
    //u32 u32OldIntr;

    //u32OldIntr = MsOS_DisableAllInterrupts();

    _u32_1stDelayTimeMs = u32_1stDelayTimeMs;
    _u32_2ndDelayTimeMs = u32_2ndDelayTimeMs;

    //MsOS_RestoreAllInterrupts(u32OldIntr);
}
EXPORT_SYMBOL(MDrv_IR_SetDelayTime);


//-------------------------------------------------------------------------------------------------
/// Get IR key. It is a non-blocking function.
/// @param pu8Key  \b IN: Return IR key value.
/// @param pu8Flag \b IN: Return IR repeat code.
///
/// @return TRUE:  Success
/// @return FALSE: No key.
//-------------------------------------------------------------------------------------------------
BOOL MDrv_IR_GetKey(U8 *pu8Key, U8 *pu8System, U8 *pu8Flag)
{
    //eCos
    #if 0
    //u32 u32OldIntr;
    //MsOS_DiableAllInterrupts(u32OldIntr); //need ???

    //bRet = _MDrv_IR_GetKey(pu8Key, pu8Flag);

    //MsOS_RestoreAllInterrupts(u32OldIntr);
    #endif

    if (_KeyReceived.u8Valid)
    {
        *pu8Key = _KeyReceived.u8Key;
        *pu8System = _KeyReceived.u8System;
        *pu8Flag = _KeyReceived.u8Flag ;

        //memset(&_KeyReceived, 0 , sizeof(_KeyReceived) );

        return(TRUE);
    }
    else
    {
        return(FALSE);
    }
}

//-------------------------------------------------------------------------------------------------
/// Get IR Code. It is a non-blocking function.
/// @param pu8Key  \b IN: Return IR key value.
/// @param pu8Flag \b IN: Return IR repeat code.
///
/// @return TRUE:  Success
/// @return FALSE: No key.
//-------------------------------------------------------------------------------------------------
BOOL MDrv_IR_GetCode(U8 *pu8Key, U8 *pu8Flag)
{
    if (_KeyReceived.u8Valid)
    {
        *pu8Key = _KeyReceived.u8Key;
        *pu8Flag = _KeyReceived.u8Flag ;
        _KeyReceived.u8Valid = 0;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}
EXPORT_SYMBOL(MDrv_IR_GetCode);


//-------------------------------------------------------------------------------------------------
/// Return the time that IR key present. It is a non-blocking function.
/// @return Last key present time.
//-------------------------------------------------------------------------------------------------
unsigned long MDrv_IR_GetLastKeyTime(void)
{
    return _ulLastKeyPresentTime;
}


//-------------------------------------------------------------------------------------------------
/// Translate from IR key to internal key.
/// @param  u8Key  \b IN: IR key value.
/// @return translated internal IR key.
//-------------------------------------------------------------------------------------------------
U8 MDrv_IR_ParseKey(U8 u8Key)
{
    return _MDrv_IR_ParseKey(u8Key);
}

void MDrv_IR_EnableIR(U8 bEnable)
{
    irqreturn_t isrStatus;
    bIRPass = !bEnable;
    if (bEnable)
    {
        isrStatus = request_irq(E_FIQ_IR, _MDrv_IR_ISR, SA_INTERRUPT, "IR", NULL);
        //enable_irq(E_FIQ_IR);
    }
    else
    {
        disable_irq(E_FIQ_IR);
        //free_irq(E_FIQ_IR, NULL);
    }
}

BOOL MDrv_IR_IsFantasyProtocolSupported(void)
{
    return MSTAR_FANTASY_PROTOCOL_SUPPORTED;
}

ssize_t MDrv_IR_Read(struct file *filp, char __user *buf, size_t count, loff_t *f_pos)
{
    if(MSTAR_FANTASY_PROTOCOL_SUPPORTED&&count==sizeof(U32))
    {
        if (down_interruptible(&fantasy_protocol.sem))
        {
            return -ERESTARTSYS;
        }

        while(fantasy_protocol.data==0)
        {
            up(&fantasy_protocol.sem); /* release the lock */
            if (filp->f_flags & O_NONBLOCK)
            {
                return -EAGAIN;
            }

            if (wait_event_interruptible(fantasy_protocol.inq, (fantasy_protocol.data!=0)))
            {
                return -ERESTARTSYS;
            }

            if (down_interruptible(&fantasy_protocol.sem))
            {
                return -ERESTARTSYS;
            }
        }

        if(copy_to_user(buf,&fantasy_protocol.data,sizeof(U32)))
        {
            up(&fantasy_protocol.sem);
            return -EFAULT;
        }

        //success
        fantasy_protocol.data=0;
        up(&fantasy_protocol.sem);

        return count;

    }
    else
    {

        return 0;
    }
}
EXPORT_SYMBOL(MDrv_IR_Read);

unsigned int MDrv_IR_Poll(struct file *filp, poll_table *wait)
{
    if (MSTAR_FANTASY_PROTOCOL_SUPPORTED)
    {
        unsigned int mask=0;

        down(&fantasy_protocol.sem);
        poll_wait(filp, &fantasy_protocol.inq, wait);
        if (fantasy_protocol.data != 0)
        {
            mask |= POLLIN|POLLRDNORM;
        }
        up(&fantasy_protocol.sem);

        return mask;
    }
    else
    {
        return 0;
    }
}
EXPORT_SYMBOL(MDrv_IR_Poll);

void MDrv_IR_SetMasterPid(pid_t pid)
{
    MasterPid = pid;
}

pid_t MDrv_IR_GetMasterPid(void)
{
    return MasterPid;
}
