////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2006-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (MStar Confidential Information) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    mdrv_mpool.c
/// @brief  Memory Pool Control Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
//#include "MsCommon.h"
#include <linux/autoconf.h>
#include <linux/undefconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/kdev_t.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/wait.h>
#include <linux/cdev.h>
#include <linux/time.h>  //added
#include <linux/timer.h> //added
#include <linux/device.h>
#include <asm/io.h>
#include <asm/types.h>
#include <asm/mips-boards/prom.h>

#include "mst_devid.h"
#include "mdrv_mpool.h"
#include "mdrv_types.h"
#include "mst_platform.h"
#include "mdrv_system.h"

//-------------------------------------------------------------------------------------------------
// Macros
//-------------------------------------------------------------------------------------------------
//#define MPOOL_DPRINTK(fmt, args...) printk(KERN_WARNING"%s:%d " fmt,__FUNCTION__,__LINE__,## args)
#define MPOOL_DPRINTK(fmt, args...)

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define MOD_MPOOL_DEVICE_COUNT     1
#define MOD_MPOOL_NAME             "malloc"

// Define MPOOL Device
U32 mpool_base;
U32 mpool_size;
U32 linux_base;
U32 linux_size;


#if defined(CONFIG_MSTAR_TITANIA3) || defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9)
    #define HAL_MIU0_LEN 0x10000000
#else
    #define HAL_MIU0_LEN 0x08000000
#endif

//-------------------------------------------------------------------------------------------------
//  Local Structurs
//-------------------------------------------------------------------------------------------------;

typedef struct
{
    int                         s32MPoolMajor;
    int                         s32MPoolMinor;
    void*                       dmaBuf;
    struct cdev                 cDevice;
    struct file_operations      MPoolFop;
} MPoolModHandle;

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
extern void (*flush_cache_all)(void);

//--------------------------------------------------------------------------------------------------
// Forward declaration
//--------------------------------------------------------------------------------------------------
static int                      _MDrv_MPOOL_Open (struct inode *inode, struct file *filp);
static int                      _MDrv_MPOOL_Release(struct inode *inode, struct file *filp);
static int                      _MDrv_MPOOL_MMap(struct file *filp, struct vm_area_struct *vma);
static int                      _MDrv_MPOOL_Ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);

//-------------------------------------------------------------------------------------------------
// Local Variables
//-------------------------------------------------------------------------------------------------

static struct class *mpool_class;

static MPoolModHandle MPoolDev=
{
    .s32MPoolMajor=               MDRV_MAJOR_MPOOL,
    .s32MPoolMinor=               MDRV_MINOR_MPOOL,
    .cDevice=
    {
        .kobj=                  {.name= MOD_MPOOL_NAME, },
        .owner  =               THIS_MODULE,
    },
    .MPoolFop=
    {
        .open=                  _MDrv_MPOOL_Open,
        .release=               _MDrv_MPOOL_Release,
        .mmap=                  _MDrv_MPOOL_MMap,
        .ioctl=                 _MDrv_MPOOL_Ioctl,
    },
};

static unsigned int bMapCached = 1;

//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

static int _MDrv_MPOOL_Open (struct inode *inode, struct file *filp)
{
    MPoolModHandle *dev;

    dev = container_of(inode->i_cdev, MPoolModHandle, cDevice);
    // dev->dmaBuf=ioremap_nocache(MPOOL_BASE, MPOOL_SIZE);
    filp->private_data = dev;

    return 0;
}

static int _MDrv_MPOOL_Release(struct inode *inode, struct file *filp)
{
    // MPoolModHandle *dev = filp->private_data ;;

    // iounmap(dev->dmaBuf) ;
    return 0;
}


static int _MDrv_MPOOL_MMap(struct file *filp, struct vm_area_struct *vma)
{

    //printk("mpool_base=%x\n",mpool_base);
    //printk("linux_base=%x\n",linux_base);

    vma->vm_pgoff = mpool_base >> PAGE_SHIFT;

    /* set page to no cache */
    if (bMapCached)
    {
        pgprot_val(vma->vm_page_prot) &= ~_CACHE_MASK;
    }
    else
    {
        pgprot_val(vma->vm_page_prot) &= ~_CACHE_MASK;
        pgprot_val(vma->vm_page_prot) |= _CACHE_UNCACHED;
    }

    //io_remap length can't be zero
    if( linux_base > 0 )
    {
        if(io_remap_pfn_range(vma, vma->vm_start,
                    0 >> PAGE_SHIFT,linux_base,
                                         vma->vm_page_prot))
        {
            return -EAGAIN;
        }
    }

#if defined(CONFIG_MSTAR_TITANIA3) || defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9) || defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7) || defined(CONFIG_MSTAR_URANUS4)
    {
#if defined(CONFIG_MSTAR_TITANIA3)
        u16 miu0_size_reg_val=0;
#endif
        u32 miu0_len=0x8000000;

        unsigned long u32MIU0_MapStart=0;
        unsigned long u32MIU0_MapSize=mpool_size;
        unsigned long u32MIU1_MapStart=0;
        unsigned long u32MIU1_MapSize=0;

#if defined(CONFIG_MSTAR_TITANIA3)
        //check MIU0 size from MAU
        miu0_size_reg_val=REG_ADDR(REG_MAU0_MIU0_SIZE);
        if(miu0_size_reg_val&0x100)
        {
            miu0_len=0x10000000;
        }
#endif

#if defined(CONFIG_MSTAR_TITANIA8)
    miu0_len = 0x10000000;
#endif        

        //calculate map size & start
        if(mpool_base<miu0_len)
        {
            u32MIU0_MapStart=mpool_base;

            if((mpool_base+u32MIU0_MapSize)>miu0_len)
            {
                u32MIU0_MapSize=(miu0_len-mpool_base);
                u32MIU1_MapSize=mpool_size-u32MIU0_MapSize;
                u32MIU1_MapStart=MIPS_MIU1_BUS_BASE;
            }
        }
        else
        {
            u32MIU0_MapStart=0;
            u32MIU0_MapSize=0;
            u32MIU1_MapStart=(mpool_base-miu0_len)+MIPS_MIU1_BUS_BASE;
            u32MIU1_MapSize=mpool_size;

        }
        //printk("MPOOL MAP INFORMATION:\n");
        //printk("    MIU0 Length=0x%08X\n",miu0_len);
        //printk("    MIU0 MAP:0x%08lX,0x%08lX\n",u32MIU0_MapStart,u32MIU0_MapSize);
        //printk("    MIU1 MAP:0x%08lX,0x%08lX\n",u32MIU1_MapStart,u32MIU1_MapSize);


        if(u32MIU0_MapSize)
        {
            if (io_remap_pfn_range(vma, vma->vm_start+ mpool_base,
                                        u32MIU0_MapStart >> PAGE_SHIFT, u32MIU0_MapSize,
                                        vma->vm_page_prot))
            {
              return -EAGAIN;
            }

        }

        if(u32MIU1_MapSize)
        {
           if (io_remap_pfn_range(vma, vma->vm_start+mpool_base+ u32MIU0_MapSize,
                                       MIPS_MIU1_BUS_BASE >> PAGE_SHIFT, u32MIU1_MapSize,
                                       vma->vm_page_prot))
           {
             return -EAGAIN;
           }

        }


    }

#else
    if (io_remap_pfn_range(vma, vma->vm_start+mpool_base,
                           mpool_base >> PAGE_SHIFT, mpool_size,
                           vma->vm_page_prot))
    {
        return -EAGAIN;
    }
#endif
    return 0;
}


static int _MDrv_MPOOL_Ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
    int         err= 0;

    /*
     * extract the type and number bitfields, and don't decode
     * wrong cmds: return ENOTTY (inappropriate ioctl) before access_ok()
     */
    if (MPOOL_IOC_MAGIC!= _IOC_TYPE(cmd))
    {
        return -ENOTTY;
    }

    /*
     * the direction is a bitmask, and VERIFY_WRITE catches R/W
     * transfers. `Type' is user-oriented, while
     * access_ok is kernel-oriented, so the concept of "read" and
     * "write" is reversed
     */
    if (_IOC_DIR(cmd) & _IOC_READ)
    {
        err = !access_ok(VERIFY_WRITE, (void __user *)arg, _IOC_SIZE(cmd));
    }
    else if (_IOC_DIR(cmd) & _IOC_WRITE)
    {
        err =  !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
    }
    if (err)
    {
        return -EFAULT;
    }

    // @FIXME: Use a array of function pointer for program readable and code size later
    switch(cmd)
    {
    //------------------------------------------------------------------------------
    // Signal
    //------------------------------------------------------------------------------
    case MPOOL_IOC_INFO:
        {
            DrvMPool_Info_t i ;
            i.u32Addr = 0 ;
            i.u32Size = mpool_base+mpool_size ;
            copy_to_user( (void *)arg, &i, sizeof(i) );
        }
        break;
    case MPOOL_IOC_FLUSHDCACHE:
        {
#if defined(CONFIG_MSTAR_TITANIA3) || defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9) || defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7) || defined(CONFIG_MSTAR_URANUS4)
            DrvMPool_Info_t i ;
            unsigned long miu0_len;
#if defined(CONFIG_MSTAR_TITANIA3)
            unsigned short miu0_size_reg_val;
#endif
            copy_from_user(&i, (void __user *)arg, sizeof(i));
            miu0_len=HAL_MIU0_LEN;

#if defined(CONFIG_MSTAR_TITANIA3)
            //check MIU0 size from MAU
            miu0_size_reg_val=REG_ADDR(REG_MAU0_MIU0_SIZE);
            if(miu0_size_reg_val&0x100)
            {
                miu0_len=0x10000000;
            }
            else
            {
                miu0_len=0x08000000;
            }
#endif//defined(CONFIG_MSTAR_TITANIA3)
            if(i.u32Addr>(0+miu0_len))
            {
                printk("**************Dcache flush MIU1 = flush all cache\n");
                flush_cache_all();
            }
            else
            {
                _dma_cache_wback_inv((unsigned long)__va(i.u32Addr), i.u32Size);
            }
#else
           DrvMPool_Info_t i ;
            copy_from_user(&i, (void __user *)arg, sizeof(i));
            _dma_cache_wback_inv((unsigned long)__va(i.u32Addr), i.u32Size);
#endif
        }
        break;
    case MPOOL_IOC_GET_BLOCK_OFFSET:
        {
            DrvMPool_Info_t i ;
            copy_from_user( &i, (void __user *)arg, sizeof(i) );
            MDrv_SYS_GetMMAP(i.u32Addr, &(i.u32Addr), &(i.u32Size)) ;
            copy_to_user( (void __user *)arg, &i, sizeof(i) );
        }
        break ;
    case MPOOL_IOC_SET_MAP_CACHE:
        {
            copy_from_user(&bMapCached, (unsigned int*)arg, sizeof(bMapCached));
        }
        break;
    default:
        printk("Unknown ioctl command %d\n", cmd);
        return -ENOTTY;
    }
    return 0;
}

#if defined(CONFIG_MSTAR_TITANIA3) || defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9) || defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7) || defined(CONFIG_MSTAR_URANUS4)
static void _mod_mpool_map_miu1(void)
{
    unsigned long flags;

#if defined(CONFIG_MSTAR_TITANIA8)
    unsigned long entrylo0 = ((0x60000000>> 12) << 6) | (0<<3) | (1<< 2) | (1<< 1) | (1<< 0); // cacheable
    unsigned long entrylo1 = ((0x60000000>> 12) << 6) | (2<<3) | (1<< 2) | (1<< 1) | (1<< 0); // un-cacheable or 0x7
#else
    unsigned long entrylo0 = ((0x40000000>> 12) << 6) | (0<<3) | (1<< 2) | (1<< 1) | (1<< 0); // cacheable
    unsigned long entrylo1 = ((0x40000000>> 12) << 6) | (2<<3) | (1<< 2) | (1<< 1) | (1<< 0); // un-cacheable or 0x7
#endif
    local_irq_save(flags);
    add_wired_entry(entrylo0, entrylo1, 0xC0000000, PM_256M);
    local_irq_restore(flags);
}
#endif // #if defined(CONFIG_MSTAR_XXXX)

//extern unsigned int MDrv_SYS_GetDRAMLength(void);

MSYSTEM_STATIC int __init mod_mpool_init(void)
{
    int s32Ret;
    dev_t dev;

    //MDrv_SYS_GetMMAP(E_SYS_MMAP_LINUX_BASE, &mpool_size, &mpool_base);
    //mpool_size = MDrv_SYS_GetDRAMLength()-mpool_base ;


    get_boot_mem_info(LINUX_MEM, &linux_base, &linux_size);
    get_boot_mem_info(MPOOL_MEM, &mpool_base, &mpool_size);

       
    
    //printk( "\nMpool base=0x%08X\n", mpool_base );
    //printk( "\nMpool size=0x%08X\n", mpool_size );

    mpool_class = class_create(THIS_MODULE, "mpool");
    if (IS_ERR(mpool_class))
    {
        return PTR_ERR(mpool_class);
    }

    if (MPoolDev.s32MPoolMajor)
    {
        dev = MKDEV(MPoolDev.s32MPoolMajor, MPoolDev.s32MPoolMinor);
        s32Ret = register_chrdev_region(dev, MOD_MPOOL_DEVICE_COUNT, MOD_MPOOL_NAME);
    }
    else
    {
        s32Ret = alloc_chrdev_region(&dev, MPoolDev.s32MPoolMinor, MOD_MPOOL_DEVICE_COUNT, MOD_MPOOL_NAME);
        MPoolDev.s32MPoolMajor = MAJOR(dev);
    }

    if ( 0 > s32Ret)
    {
        MPOOL_DPRINTK("Unable to get major %d\n", MPoolDev.s32MPoolMajor);
        class_destroy(mpool_class);
        return s32Ret;
    }

    cdev_init(&MPoolDev.cDevice, &MPoolDev.MPoolFop);
    if (0!= (s32Ret= cdev_add(&MPoolDev.cDevice, dev, MOD_MPOOL_DEVICE_COUNT)))
    {
        MPOOL_DPRINTK("Unable add a character device\n");
        unregister_chrdev_region(dev, MOD_MPOOL_DEVICE_COUNT);
        class_destroy(mpool_class);
        return s32Ret;
    }

#if defined(CONFIG_MSTAR_TITANIA3) || defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9) || defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7) || defined(CONFIG_MSTAR_URANUS4)
    _mod_mpool_map_miu1();
    // only for testing
    // *((unsigned int*) 0xC0000000) = 0x12345678;
    // printk("[%s][%d] 0x%08x\n", __FILE__, __LINE__, *((unsigned int*) 0xC0000000));
    // printk("[%s][%d] 0x%08x\n", __FILE__, __LINE__, *((unsigned int*) 0x80000000));
    // printk("[%s][%d] 0x%08x\n", __FILE__, __LINE__, *((unsigned int*) 0xA0000000));
#endif // #ifdef CONFIG_MSTAR_XXXX

    device_create(mpool_class, NULL, dev, NULL, MOD_MPOOL_NAME);

    return 0;
}


MSYSTEM_STATIC void __exit mod_mpool_exit(void)
{
    cdev_del(&MPoolDev.cDevice);
    unregister_chrdev_region(MKDEV(MPoolDev.s32MPoolMajor, MPoolDev.s32MPoolMinor), MOD_MPOOL_DEVICE_COUNT);
    class_destroy(mpool_class);
}
#if defined(CONFIG_MSTAR_MPOOL) || defined(CONFIG_MSTAR_MPOOL_MODULE)
module_init(mod_mpool_init);
module_exit(mod_mpool_exit);

MODULE_AUTHOR("MSTAR");
MODULE_DESCRIPTION("MPOOL driver");
MODULE_LICENSE("GPL");
#endif//#if defined(CONFIG_MSTAR_MPOOL) || defined(CONFIG_MSTAR_MPOOL_MODULE)
