#ifndef NAND_DRV_LINUX_H
#define NAND_DRV_LINUX_H

#include <linux/delay.h>
#include <linux/mm.h>
#include <linux/highmem.h>
#include <asm/io.h>

#if defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7) || defined(CONFIG_MSTAR_JANUS) || \
	defined(CONFIG_MSTAR_URANUS4) || defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9)

#define REG_BANK_CLKGEN			(0x580)
#define REG_BANK_CHIPTOP		(0xF00)
#define REG_BANK_FCIE0			(0x8980)
#define REG_BANK_FCIE2			(0x8A00)

#endif

#if defined(CONFIG_MIPS)

#define KSEG02KSEG1(addr)				((void *)((U32)(addr)|0x20000000))

#define NAND_MIPS_WRITE_BUFFER_PATCH	1
#define NAND_MIPS_READ_BUFFER_PATCH		1

extern void Chip_Flush_Memory(void);
extern void Chip_Read_Memory(void);

#endif

//=====================================================
// HW parameters
//=====================================================
#define REG(Reg_Addr)			(*(volatile U16*)(Reg_Addr))
#define REG_OFFSET_SHIFT_BITS	2 

//----------------------------------
// should only one be selected
#define NC_SEL_FCIE2			0
#define NC_SEL_FCIE3			1 

#define IF_IP_VERIFY			0 // [CAUTION]: to verify IP and HAL code, defaut 0
//----------------------------------

#if defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7) || defined(CONFIG_MSTAR_URANUS4) || \
	defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9)
#define RIU_BASE				0xBF200000
#elif defined(CONFIG_MSTAR_JANUS)
#define RIU_BASE				0xA0200000
#endif

#define FCIE_REG_BASE_ADDR		(RIU_BASE+(REG_BANK_FCIE0<<REG_OFFSET_SHIFT_BITS))
#define FCIE_NC_CIFD_BASE		(RIU_BASE+(REG_BANK_FCIE2<<REG_OFFSET_SHIFT_BITS))

#if NC_SEL_FCIE3

#include "drvNand_reg_v3.h" 

#elif NC_SEL_FCIE2

#error "ERR: no drvNand_reg_v2.h so far." 

#endif

#define REG_WRITE_UINT16(reg_addr, val)		REG(reg_addr) = val
#define REG_READ_UINT16(reg_addr, val)		val = REG(reg_addr)
#define HAL_WRITE_UINT16(reg_addr, val)		(REG(reg_addr) = val)
#define HAL_READ_UINT16(reg_addr, val)		val = REG(reg_addr)
#define REG_SET_BITS_UINT16(reg_addr, val)	REG(reg_addr) |= (val)
#define REG_CLR_BITS_UINT16(reg_addr, val)	REG(reg_addr) &= ~(val)
#define REG_W1C_BITS_UINT16(reg_addr, val)	REG_WRITE_UINT16(reg_addr, REG(reg_addr)&(val))

#define RTC_REG_BASE_ADDR			(RIU_BASE+(REG_BANK_XSPHA<<REG_OFFSET_SHIFT_BITS))

//=====================================================
// Nand Driver configs
//=====================================================
#define NAND_DRIVER_ROMCODE			0 // for ROM Code
#define NAND_DRIVER_PROGRAMMER		0 // for Nand Programmer
#define NAND_DRIVER_OS				1 // for OS / File System
#define NAND_DRIVER_OS_NOS			(0 & NAND_DRIVER_OS)
#define NAND_DRIVER_OS_ECOS			(0 & NAND_DRIVER_OS)
#define NAND_DRIVER_OS_WCE			(0 & NAND_DRIVER_OS)
#define NAND_DRIVER_OS_LINUX		(1 & NAND_DRIVER_OS)

#define PARTITION_FATU_IDX			0
#define PARTITION_FATH_IDX			1
//-------------------------------
#define NAND_ENV_FPGA				1
#define NAND_ENV_ASIC				2
#define NAND_DRIVER_ENV				NAND_ENV_ASIC /* [CAUTION] */

//=====================================================
// debug option
//=====================================================
#define NAND_TEST_IN_DESIGN			0 

#define NAND_DEBUG_MSG				0

#if defined(NAND_DEBUG_MSG) && NAND_DEBUG_MSG
#define nand_printf	printk
#define NANDMSG(x)	(x)
#else
#define nand_printf	printk
#define NANDMSG(x)
#endif

#define UNFD_ECOS_PARAMETER_CHECK	1

//=====================================================
// HW Timer for Delay 
//=====================================================
//#define HW_TIMER_CLOCK				12*1000*1000

#define HW_TIMER_DELAY_1us			1
#define HW_TIMER_DELAY_1ms			1000
#define HW_TIMER_DELAY_1s			1000000

/*return Timer tick*/
U32 nand_hw_timer_delay(U32 u32usTick);

//=====================================================
// Pads Switch
//=====================================================
#define CHIPTOP_REG_BASE_ADDR		(RIU_BASE+(REG_BANK_CHIPTOP<<REG_OFFSET_SHIFT_BITS))

/*set bit 4, 7, 8; clr bit 5*/
#if defined(CONFIG_MSTAR_JANUS)
#define reg_nf_en					GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x20)
#elif defined(CONFIG_MSTAR_URANUS4)
#define reg_nf_en					GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x06)
#else
#define reg_nf_en					GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x6f)
#endif

/* clr bit6, 5, 4 */
#define reg_ts1_mode				GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x02)

/*clr bit 15*/
#define reg_allpad_in				GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x50)

/*clr bit 10*/
#define reg_boot_from_pf			GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x01)

/*clr bit 2, 3, 13*/
#define reg_test_pad				GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x70) 
/* check Package ID*/
#define reg_top_03					GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x03)
#define reg_cardio_puz				GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x24)
#define reg_cardio_pd				GET_REG_ADDR(CHIPTOP_REG_BASE_ADDR, 0x26)

#define IF_FCIE_SHARE_PINS			1	// 1: need to nand_pads_switch at HAL's functions.
										// 0: only nand_pads_switch in drvNand_Init.

/*
  if pin-shared with Card IF, need to call before every JOB_START.
*/
U32 nand_pads_switch(U32 u32EnableFCIE);

//=====================================================
// set FCIE clock
//=====================================================
#define MPLL_CLK_REG_BASE_ADDR		(RIU_BASE+(REG_BANK_CLKGEN<<REG_OFFSET_SHIFT_BITS))
#define reg_ckg_fcie				GET_REG_ADDR(MPLL_CLK_REG_BASE_ADDR, 0x64)

#if defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9)

#define NFIE_CLK_MASK				(BIT4|BIT3|BIT2)
#define NFIE_CLK_5_4M				(0)
#define NFIE_CLK_27M				(BIT2)
#define NFIE_CLK_32M				(BIT3)
#define NFIE_CLK_43M				(BIT3|BIT2)
#define NFIE_CLK_86M				(BIT4)

#else

#define NFIE_CLK_MASK				(BIT5|BIT4|BIT3|BIT2)
#define NFIE_CLK_300K				0
#define NFIE_CLK_750K				BIT2
#define NFIE_CLK_5_4M				BIT3
#define NFIE_CLK_13_5M				(BIT3|BIT2)
#define NFIE_CLK_18M				BIT4
#define NFIE_CLK_22_7M				(BIT4|BIT2)
#define NFIE_CLK_27M				(BIT4|BIT3)
#define NFIE_CLK_32M				(BIT4|BIT3|BIT2)
#define NFIE_CLK_43M				BIT5
#define NFIE_CLK_54M				(BIT5|BIT2)
#define NFIE_CLK_72M				(BIT5|BIT3)
#define NFIE_CLK_86M				(BIT5|BIT3|BIT2)

#endif

U32 nand_clock_setting(U32 u32ClkParam);

//=====================================================
// transfer DMA Address
//=====================================================
//#define MIU_BUS_WIDTH_BITS		3 // 8 bytes aligned
//#define NC_DMA_ADDR_R				0x6000000 // DRAM space: 0~128MB
//#define NC_DMA_ADDR_W				0x6800000

// return DMA address for FCIE3 registers
U32 nand_translate_DMA_address_Ex(U32 u32_DMAAddr, U32 u32_ByteCnt);
U32 nand_get_internal_DMA_address_Ex(U32 u32DMAAddr);

//=====================================================
// reset WatchDog
//=====================================================
void nand_reset_WatchDog(void);

//=====================================================
// misc
//=====================================================
//#define BIG_ENDIAN
#define LITTLE_ENDIAN

typedef struct _NAND_DRIVER_PLATFORM_DEPENDENT
{
	U8 *pu8_Spare;	
}NAND_DRIVER_PLAT, *P_NAND_DRIVER_PLAT;

U32 nand_platform_init(void);

#endif
