//=======================================================================
//  MStar Semiconductor - Unified Nand Flash Driver
//
//  drvNand_platform.c - Storage Team, 2009/08/20 
//
//  Design Notes: defines common platform-dependent functions.
//
//    1. 2009/08/25 - support C5 eCos platform
//
//=======================================================================
#include "../include/drvNand.h"

#define URANUS4_216P	0

#if defined(NAND_DRV_C5_ROM) && NAND_DRV_C5_ROM

#elif (defined(NAND_DRV_C5_ECOS) && NAND_DRV_C5_ECOS) || (defined(NAND_DRV_C5_NPRG) && NAND_DRV_C5_NPRG)
//=====================================================
// HW Timer for Delay 
//=====================================================
/*return Timer tick*/
U32 nand_hw_timer_delay(U32 u32usTick)
{
#if 1
	volatile U32 u32i, u32Tick;

	u32Tick = (u32usTick>>7)+ (u32usTick>>7) +
		      (u32usTick>>7)+ (u32usTick>>7) + 
		      (u32usTick>>7)+ (u32usTick>>7) +
		      (u32usTick>>7)+ (u32usTick>>6) +8;

	for(u32i=0; u32i<u32Tick; u32i++)
		;	// soft timer, temporarily coded.

	return u32Tick;
#else
    delay_us((u32usTick/TIMER_DELAY_1us)+1);
    return (u32usTick/TIMER_DELAY_1us)+1;
#endif
}

//=====================================================
// Pads Switch
//=====================================================

// if pin-shared with Card IF, need to call before every JOB_START.
U32 nand_pads_switch(U32 u32EnableFCIE)
{
	volatile U16 u16RegVal, u16Tmp;

	REG_SET_BITS_UINT16(NC_TEST_MODE, 0xD8<<8);
	
	if(u32EnableFCIE)
	{
		REG_SET_BITS_UINT16(reg_nf_en, BIT4|BIT7|BIT8);
		REG_CLR_BITS_UINT16(reg_nf_en, BIT5);

		REG_CLR_BITS_UINT16(reg_test_pad, BIT2|BIT3|BIT13);

		REG_SET_BITS_UINT16(reg_cardio_puz, BIT0|BIT1|BIT4);
		REG_CLR_BITS_UINT16(reg_cardio_puz, BIT2|BIT3);
		REG_SET_BITS_UINT16(reg_cardio_pd, BIT0|BIT1|BIT4);
		REG_CLR_BITS_UINT16(reg_cardio_pd, BIT2|BIT3);

		#if 0
        // check Package ID
        //[CAUTION] ask HW designer
		REG_READ_UINT16(reg_top_03, u16RegVal);
		if(0 == (u16RegVal & (BIT2|BIT1|BIT0)))
		{
		    REG_SET_BITS_UINT16(NC_TEST_MODE, BIT5);
			REG_READ_UINT16(NC_TEST_MODE, u16Tmp);
			//nand_printf("Package:%Xh, set Reg30h_bit5: %Xh\r\n", u16RegVal&7, u16Tmp);
		}
		else
		{	REG_CLR_BITS_UINT16(NC_TEST_MODE, BIT5);
			REG_READ_UINT16(NC_TEST_MODE, u16Tmp);
			//nand_printf("Package:%Xh, clr Reg30h_bit5: %Xh\r\n", u16RegVal&7, u16Tmp);
		}
		#endif
		
        #if NAND_DRIVER_ENV == NAND_ENV_FPGA
        REG_SET_BITS_UINT16(NC_TEST_MODE, BIT5);
		REG_READ_UINT16(NC_TEST_MODE, u16Tmp);
		nand_printf("FPGA set Reg30h bit5: %Xh\r\n", u16Tmp);
        #endif

        #if NAND_TEST_IN_DESIGN
        REG_READ_UINT16(reg_nf_en, u16RegVal);
        if(0==(u16RegVal & (BIT4|BIT7|BIT8)) || 1==(u16RegVal & BIT5))
        {
			NANDMSG(nand_printf("Err: switch pads fail! %Xh \r\n", u16RegVal));
			return 1; // error
        }
        #endif
	}
	//else
	{
	}

	REG_WRITE_UINT16(NC_PATH_CTL, BIT_NC_EN);


	return 0; // ok
}

//=====================================================
// set FCIE clock
//=====================================================
U32  nand_clock_setting(U32 u32ClkParam)
{

	REG_CLR_BITS_UINT16(reg_mclk_fcie_en, BIT5);
	
	REG_CLR_BITS_UINT16(reg_ckg_fcie, BIT6-1); // enable FCIE clk, set to lowest clk
	REG_SET_BITS_UINT16(reg_ckg_fcie, u32ClkParam);

	return 0;
}

//=====================================================
// transfer DMA Address
//=====================================================

// return DMA address for FCIE3 registers
U32 nand_translate_DMA_address_Ex(U32 u32_DMAAddr, U32 u32_ByteCnt)
{
	U32 invbuf = u32_DMAAddr;	
	
	if(invbuf & (HAL_DCACHE_LINE_SIZE-1)) 
	{
		/* for UNFD, the min u32_ByteCnt would be 512B. */
		u32_ByteCnt -= ((U32)invbuf & (HAL_DCACHE_LINE_SIZE-1));
        invbuf &= ~(HAL_DCACHE_LINE_SIZE-1);
        HAL_DCACHE_FLUSH( invbuf, HAL_DCACHE_LINE_SIZE );
        invbuf += HAL_DCACHE_LINE_SIZE;
    }

	HAL_DCACHE_FLUSH( invbuf, (u32_ByteCnt & ~(HAL_DCACHE_LINE_SIZE-1)) );
	//HAL_DCACHE_INVALIDATE( invbuf, (u32_ByteCnt & ~(HAL_DCACHE_LINE_SIZE-1)) );
    invbuf += u32_ByteCnt & ~(HAL_DCACHE_LINE_SIZE-1);

	if (u32_ByteCnt & (HAL_DCACHE_LINE_SIZE-1)) 
	{
		HAL_DCACHE_FLUSH( invbuf, HAL_DCACHE_LINE_SIZE );
    }
			
    return u32_DMAAddr + (256<<10);
}

U32 nand_get_internal_DMA_address_Ex(U32 u32DMAAddr)
{
	while(1);
}

//=====================================================
// used in ROM Code
// get an Address for Nand Device Context
//=====================================================
#if 0
static U32 nand_get_DevContext_address_Ex(void) // exposed API
{
	while(1);
	return 0xFFFFFFFF;
}
#endif

//=====================================================
// reset WatchDog
//=====================================================
void nand_reset_WatchDog(void)
{
	MDrv_ResetWatchdog();
}

//=====================================================
// misc
//=====================================================
// success: return UNFD_ST_SUCCESS
// error: return error codes
U32 nand_platform_init(void)
{
	NAND_DRIVER *pNandDrv = (NAND_DRIVER*)drvNand_get_DrvContext_address();

	pNandDrv->NandDrvPlat_t.pu8_Spare = (U8*)malloc(pNandDrv->u16_SpareByteCnt);
	if(NULL == pNandDrv->NandDrvPlat_t.pu8_Spare)
	{
		nand_printf("Error: nand_platform_init, ErrCode:%Xh \r\n", UNFD_ST_ERR_PLAT_INIT_0);
		return UNFD_ST_ERR_PLAT_INIT_0;
	}

	return UNFD_ST_SUCCESS;
}

#elif defined(NAND_DRV_LINUX) && NAND_DRV_LINUX

U32 nand_hw_timer_delay(U32 u32usTick)
{
	udelay(u32usTick);
	
	return u32usTick;
}

//=====================================================
// Pads Switch
//=====================================================

// if pin-shared with Card IF, need to call before every JOB_START.
U32 nand_pads_switch(U32 u32EnableFCIE)
{
	#if defined(NAND_DEBUG_MSG) && NAND_DEBUG_MSG
	U16 u16Tmp;
	#endif

	if(u32EnableFCIE)
	{
		#if defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_TITANIA7)
		REG_SET_BITS_UINT16(reg_nf_en, BIT4);
		#elif defined(CONFIG_MSTAR_JANUS)
		REG_SET_BITS_UINT16(reg_nf_en, BIT0);
		#elif defined(CONFIG_MSTAR_URANUS4)
		#if 0
		REG_CLR_BITS_UINT16(reg_nf_en, BIT3|BIT2|BIT1|BIT0);
		#if !(defined(URANUS4_216P) && URANUS4_216P)
		REG_SET_BITS_UINT16(reg_nf_en, BIT0);
		#else
		REG_SET_BITS_UINT16(reg_nf_en, BIT1);
		REG_CLR_BITS_UINT16(reg_ts1_mode, BIT6|BIT5|BIT4);
		#endif
		#endif
		REG_CLR_BITS_UINT16(reg_allpad_in, BIT15);
		REG_CLR_BITS_UINT16(NC_TEST_MODE, BIT5);
		REG_CLR_BITS_UINT16(reg_boot_from_pf, BIT10);
		#elif defined(CONFIG_MSTAR_TITANIA8)
		REG_SET_BITS_UINT16(reg_nf_en, BIT7);
		REG_CLR_BITS_UINT16(reg_allpad_in, BIT15);
		#elif defined(CONFIG_MSTAR_TITANIA9)
		REG_SET_BITS_UINT16(reg_nf_en, BIT4);
		REG_CLR_BITS_UINT16(reg_allpad_in, BIT15);
		#endif
	}

	#if defined(NAND_DEBUG_MSG) && NAND_DEBUG_MSG
	REG_READ_UINT16(reg_nf_en, u16Tmp);
	NANDMSG(nand_printf("reg_nf_en(%08X)=%04X\n", reg_nf_en, u16Tmp));
	REG_READ_UINT16(reg_allpad_in, u16Tmp);
	NANDMSG(nand_printf("reg_allpad_in(%08X)=%04X\n", reg_allpad_in, u16Tmp));
	REG_READ_UINT16(NC_TEST_MODE, u16Tmp);
	NANDMSG(nand_printf("NC_TEST_MODE(%08X)=%04X\n", NC_TEST_MODE, u16Tmp));
	REG_READ_UINT16(reg_boot_from_pf, u16Tmp);
	NANDMSG(nand_printf("reg_boot_from_pf(%08X)=%04X\n", reg_boot_from_pf, u16Tmp));
	#endif

	return 0; // ok
}

//=====================================================
// set FCIE clock
//=====================================================
U32 nand_clock_setting(U32 u32ClkParam)
{
	#if defined(NAND_DEBUG_MSG) && NAND_DEBUG_MSG
	U16 u16Tmp;
	#endif

	REG_CLR_BITS_UINT16(reg_ckg_fcie, BIT6-1); // enable FCIE clk, set to lowest clk
	REG_SET_BITS_UINT16(reg_ckg_fcie, u32ClkParam);
	
	#if defined(NAND_DEBUG_MSG) && NAND_DEBUG_MSG
	REG_READ_UINT16(reg_ckg_fcie, u16Tmp);
	NANDMSG(nand_printf("reg_ckg_fcie(%08X)=%08X\n", reg_ckg_fcie, u16Tmp));
	#endif
	
	return 0;
}

U32 nand_translate_DMA_address_Ex(U32 u32_DMAAddr, U32 u32_ByteCnt)
{
	#if defined(CONFIG_MSTAR_JANUS)
	return virt_to_phys((void*)u32_DMAAddr);
	#else
	return u32_DMAAddr;
	#endif
}

void nand_reset_WatchDog(void)
{
}

U32 nand_platform_init(void)
{
	return UNFD_ST_SUCCESS;
}

#else

#error "Error! no platform functions."

#endif
