#include <linux/module.h>
#include <linux/string.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>

#include <linux/semaphore.h>

#if defined(CONFIG_MSTAR_TITANIA4) || defined(CONFIG_MSTAR_JANUS) || defined(CONFIG_MSTAR_URANUS4) || \
	defined(CONFIG_MSTAR_TITANIA8) || defined(CONFIG_MSTAR_TITANIA9)
#define USING_FCIE3 1
#endif

#if defined(USING_FCIE3)
#include "include/drvNand.h"
#else
#include "drvNAND.h"
#endif

#if defined(USING_FCIE3)
U32	u32CurRow = 0;
U32 u32CurCol = 0;
extern U32 u32CIFDIdx;
extern U16 u16ByteIdxofPage;
extern U32 u32WriteLen;
#endif

//#define NAND_DEBUG(fmt, args...)	printk(fmt, ##args)
#define NAND_DEBUG(fmt, args...)

extern struct semaphore				PfModeSem;
#define PF_MODE_SEM(x)				(x)

extern unsigned int MDrv_SYS_GetMIU0Length(void);

/* These really don't belong here, as they are specific to the NAND Model */
static uint8_t scan_ff_pattern[] = { 0xff };

/* struct nand_bbt_descr - bad block table descriptor */
static struct nand_bbt_descr _titania_nand_bbt_descr = {
	.options		= NAND_BBT_2BIT | NAND_BBT_LASTBLOCK | NAND_BBT_VERSION | NAND_BBT_CREATE | NAND_BBT_WRITE,
	.offs			= 5,
	.len			= 1,
	.pattern		= scan_ff_pattern
};

static struct nand_ecclayout titania_nand_oob_16 = {
	.eccbytes		= 10,
	.eccpos			= {6,7,8,9,10,11,12,13,14,15},
	.oobfree		= 
	{
		{
			.offset = 0,
			.length = 5
		}
	}
};

static struct nand_ecclayout titania_nand_oob_64 = {
    .eccbytes		= 40,
    .eccpos			= 
    {
    	0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
    	0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
    	0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
    	0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F
    },
    .oobfree		=
    {
    	{
    		.offset = 1,
    		.length = 5
    	},
        {
        	.offset = 0x10,
        	.length = 6
        },
        {
        	.offset = 0x20,
        	.length = 6
        },
        {
        	.offset = 0x30,
        	.length = 6
        }
    }
};

static uint8_t bbt_pattern[] = {'B', 'b', 't', '0' };
static uint8_t mirror_pattern[] = {'1', 't', 'b', 'B' };

static struct nand_bbt_descr _titania_bbt_main_descr = {
	.options		= NAND_BBT_LASTBLOCK | NAND_BBT_CREATE | NAND_BBT_WRITE |
					  NAND_BBT_2BIT | NAND_BBT_VERSION | NAND_BBT_PERCHIP,
	.offs			= 1,
	.len			= 3,
	.veroffs		= 4,
	.maxblocks		= 4,
	.pattern		= bbt_pattern
};

static struct nand_bbt_descr _titania_bbt_mirror_descr = {
	.options		= NAND_BBT_LASTBLOCK | NAND_BBT_CREATE | NAND_BBT_WRITE |
					  NAND_BBT_2BIT | NAND_BBT_VERSION | NAND_BBT_PERCHIP,
	.offs			= 1,
	.len			= 3,
	.veroffs		= 4,
	.maxblocks		= 4,
	.pattern		= mirror_pattern
};

#ifdef CONFIG_MTD_CMDLINE_PARTS
extern int parse_cmdline_partitions(struct mtd_info *master, struct mtd_partition **pparts, char *);
#endif

#define MSTAR_NAND_SIZE						0x4000000

#define SZ_256KB							(256 * 1024)
#define SZ_512KB							(512 * 1024)
#define SZ_2MB								(2 * 1024 * 1024)
#define SZ_4MB								(4 * 1024 * 1024)
#define SZ_7MB								(7 * 1024 * 1024)
#define SZ_8MB								(8 * 1024 * 1024)
#define SZ_16MB								(16 * 1024 * 1024)
#define SZ_32MB								(32 * 1024 * 1024)

#define NAND_PART_PARTITION_TBL_OFFSET		0
#define NAND_PART_PARTITION_TBL_SIZE		SZ_256KB

#define NAND_PART_LINUX_BOOT_PARAM_OFFSET	(NAND_PART_PARTITION_TBL_OFFSET + NAND_PART_PARTITION_TBL_SIZE)
#define NAND_PART_LINUX_BOOT_PARAM_SIZE		SZ_256KB

#define NAND_PART_KERNEL_OFFSET				(NAND_PART_LINUX_BOOT_PARAM_OFFSET + NAND_PART_LINUX_BOOT_PARAM_SIZE)
#define NAND_PART_KERNEL_SIZE				SZ_4MB

#define NAND_PART_CHAKRA_BOOT_PARAM_OFFSET	(NAND_PART_KERNEL_OFFSET + NAND_PART_KERNEL_SIZE)
#define NAND_PART_CHAKRA_BOOT_PARAM_SIZE	SZ_512KB

#define NAND_PART_CHAKRA_BIN_OFFSET			(NAND_PART_CHAKRA_BOOT_PARAM_OFFSET + NAND_PART_CHAKRA_BOOT_PARAM_SIZE)
#define NAND_PART_CHAKRA_BIN_PARAM_SIZE		SZ_8MB

#define NAND_PART_ROOTFS_OFFSET				(NAND_PART_CHAKRA_BIN_OFFSET + NAND_PART_CHAKRA_BIN_PARAM_SIZE)
#define NAND_PART_ROOTFS_SIZE				SZ_4MB

#define NAND_PART_CONF_OFFSET				(NAND_PART_ROOTFS_OFFSET + NAND_PART_ROOTFS_SIZE)
#define NAND_PART_CONF_SIZE					SZ_256KB

#define NAND_PART_SUBSYSTEM_OFFSET			(NAND_PART_CONF_OFFSET + NAND_PART_CONF_SIZE)
#define NAND_PART_SUBSYSTEM_SIZE			SZ_2MB

#define NAND_PART_FONT_OFFSET				(NAND_PART_SUBSYSTEM_OFFSET + NAND_PART_SUBSYSTEM_SIZE)
#define NAND_PART_FONT_SIZE					SZ_4MB

#define NAND_PART_OPT_OFFSET				(NAND_PART_FONT_OFFSET + NAND_PART_FONT_SIZE)
#define NAND_PART_OPT_SIZE					SZ_8MB

#define NAND_PART_APPLICATION_OFFSET		(NAND_PART_OPT_OFFSET + NAND_PART_OPT_SIZE)
#define NAND_PART_APPLICATION_SIZE			(MSTAR_NAND_SIZE - NAND_PART_APPLICATION_OFFSET)

#if ( (NAND_PART_APPLICATION_OFFSET) >= MSTAR_NAND_SIZE)
    #error "Error: NAND partition is not correct!!!"
#endif

static const struct mtd_partition partition_info[] =
{
    {
    	.name   = "nand_partition_tbl",
    	.offset = NAND_PART_PARTITION_TBL_OFFSET,
        .size   = NAND_PART_PARTITION_TBL_SIZE
    },
    {
        .name   = "linux_boot_param",
        .offset = NAND_PART_LINUX_BOOT_PARAM_OFFSET,
        .size   = NAND_PART_LINUX_BOOT_PARAM_SIZE
    },
    {
        .name   = "kernel",
        .offset = NAND_PART_KERNEL_OFFSET,
        .size   = NAND_PART_KERNEL_SIZE
    },
    {
        .name   = "chakra_boot_param",
        .offset = NAND_PART_CHAKRA_BOOT_PARAM_OFFSET,
        .size   = NAND_PART_CHAKRA_BOOT_PARAM_SIZE
    },
    {
        .name   = "charkra_bin",
        .offset = NAND_PART_CHAKRA_BIN_OFFSET,
        .size   = NAND_PART_CHAKRA_BIN_PARAM_SIZE
    },
    {
        .name   = "rootfs",
        .offset = NAND_PART_ROOTFS_OFFSET,
        .size   = NAND_PART_ROOTFS_SIZE
    },
    {
		.name   = "conf",
		.offset = NAND_PART_CONF_OFFSET,
		.size   = NAND_PART_CONF_SIZE
    },
    {
		.name   = "subsystem",
		.offset = NAND_PART_SUBSYSTEM_OFFSET,
		.size   = NAND_PART_SUBSYSTEM_SIZE
    },
    {
		.name   = "font",
		.offset = NAND_PART_FONT_OFFSET,
		.size   = NAND_PART_FONT_SIZE
    },
    {
		.name   = "opt",
		.offset = NAND_PART_OPT_OFFSET,
		.size   = NAND_PART_OPT_SIZE
    },
    {
		.name   = "application",
		.offset = NAND_PART_APPLICATION_OFFSET,
		.size   = NAND_PART_APPLICATION_SIZE
    },
};

#define NUM_PARTITIONS ARRAY_SIZE(partition_info)

static struct mtd_info *nand_mtd = NULL;

static void _titania_nand_hwcontrol(struct mtd_info *mtdinfo, int cmd, unsigned int ctrl)
{
    NAND_DEBUG("NANDDrv_HWCONTROL()\n");

    if(ctrl & NAND_CTRL_CHANGE)
    {
		if(ctrl & NAND_NCE)
		{
			NAND_DEBUG("NAND_CTL_SETNCE\n");
			#if defined(USING_FCIE3)
			drvNAND_SetCEZ(0x00);
			#else
			MDrv_NAND_Set_Chip_Select(0x0);
			#endif
		}
		else
		{
			NAND_DEBUG("NAND_CTL_CLRNCE\n");
			#if defined(USING_FCIE3)
			drvNAND_SetCEZ(0x1);
			#else
			MDrv_NAND_Set_Chip_Select(0x1);
			#endif
		}
        
		if(ctrl & NAND_CLE)
        {
			NAND_DEBUG("NAND_CTL_SETCLE\n");
			#if defined(USING_FCIE3)
			drvNAND_SetCLE(0x01);
			#else
			MDrv_NAND_Set_Command_Latch(0x1);
			#endif
		}
		else
		{
			NAND_DEBUG("NAND_CTL_CLRCLE\n");
			#if defined(USING_FCIE3)
			drvNAND_SetCLE(0x0);
			#else
			MDrv_NAND_Set_Command_Latch(0x0);
			#endif
		}
		
		if(ctrl & NAND_ALE)
		{
			NAND_DEBUG("NAND_CTL_SETALE\n");
			#if defined(USING_FCIE3)
			drvNAND_SetALE(0x1);
			#else
			MDrv_NAND_Set_Address_Latch(0x1);
			#endif
		}
		else
		{
			NAND_DEBUG("NAND_CTL_CLRALE\n");
			#if defined(USING_FCIE3)
			drvNAND_SetALE(0x0);
			#else
			MDrv_NAND_Set_Address_Latch(0x0);
			#endif
		}
	}

	#if !defined(USING_FCIE3)	
	if(cmd != NAND_CMD_NONE)
    {
        MDrv_NAND_SendCmd(cmd);
    }
    #endif
}

static int _titania_nand_device_ready(struct mtd_info *mtdinfo)
{
	NAND_DEBUG("NANDDrv_DEVICE_READY()\n");
	
	#if defined(USING_FCIE3)
	drvNAND_WaitReady(1000);
	#else
    MDrv_NAND_WaitReady();
	#endif
	
	return 1;
}

static void _titania_nand_write_buf(struct mtd_info *mtd, const u_char *buf, int len)
{
#ifdef CONFIG_MS_NAND_MIU1_PATCH
	void *tmpBuf;
	int count = 0;
	unsigned int _MIU0_LEN=MDrv_SYS_GetMIU0Length();

    NAND_DEBUG("NANDDrv_WRITE_BUF() 0x%X\r\n",len);

    if( ((unsigned long)VA2PA(buf)) >= _MIU0_LEN  )
    {
    	NAND_DEBUG("NAND write DMA for MIU1\n");

    	if( (tmpBuf = kmalloc(len, GFP_ATOMIC | GFP_DMA)) == 0 )
    	{
    		printk(KERN_ERR "No memory for NAND write\n");
    		return;
    	}

    	while( (((unsigned long)VA2PA(tmpBuf)) >= _MIU0_LEN) || (count > 0x3FFF) )
    	{
    		printk(KERN_ERR "No memory for MIU0 NAND write\n");
    		kfree(tmpBuf);
    		tmpBuf = 0;
    		count++;
    	}

    	if( tmpBuf != 0 )
    	{
    		NAND_DEBUG("W buf = %p, tmpBuf = %p, len = %X\n", VA2PA(buf), VA2PA(tmpBuf), len);

    		memcpy(tmpBuf, buf, len);

    		MDrv_NAND_WriteBuf((U8 *)tmpBuf, len);

    		kfree(tmpBuf);
    	}
    }
    else
#endif
    {
    	#if defined(USING_FCIE3)
    	drvNAND_WriteBuf((U8 *const)buf, len);
    	#else
    	MDrv_NAND_WriteBuf((U8 *const)buf, len);
    	#endif
    }

    return;
}

static void _titania_nand_read_buf(struct mtd_info *mtd, u_char* const buf, int len)
{
#ifdef CONFIG_MS_NAND_MIU1_PATCH
	void *tmpBuf;
	int count = 0;
	unsigned int _MIU0_LEN=MDrv_SYS_GetMIU0Length();

    NAND_DEBUG("NANDDrv_READ_BUF()0x%X\n",len);

    if( ((unsigned long)VA2PA(buf)) > _MIU0_LEN )
    {
    	NAND_DEBUG("NAND read DMA for MIU1\n");
    	
    	if( (tmpBuf = kmalloc(len, GFP_ATOMIC | GFP_DMA)) == 0 )
    	{
    		printk(KERN_ERR "No memory for NAND write\n");
    		return;
    	}

    	while( (((unsigned long)VA2PA(tmpBuf)) >= _MIU0_LEN) || (count > 0x3FFF) )
    	{
    		printk(KERN_ERR "No memory for MIU0 NAND write\n");
    		kfree(tmpBuf);
    		tmpBuf = 0;
    		count++;
    	}

    	if( tmpBuf != 0 )
    	{
    		NAND_DEBUG("R buf = %p, tmpBuf = %p, len = %X\n", VA2PA(buf), VA2PA(tmpBuf), len);

    		MDrv_NAND_ReadBuf((U8*)tmpBuf, len);

    		memcpy(buf, (U8*)tmpBuf, len);

    		kfree(tmpBuf);
    	}
    }
    else
#endif
    {
    	#if defined(USING_FCIE3)
    	drvNAND_ReadBuf(buf, len);
    	#else
    	MDrv_NAND_ReadBuf(buf, len);
    	#endif
    }


    return;
}

static u16 _titania_nand_read_word(struct mtd_info *mtd)
{
    NAND_DEBUG("NANDDrv_READ_WORD()\n");
    return 0;
}

static u_char _titania_nand_read_byte(struct mtd_info *mtd)
{
    U8 u8Ret;
    
    NAND_DEBUG("NANDDrv_READ_BYTE()\n");
    
    PF_MODE_SEM(down(&PfModeSem));
    
#if defined(USING_FCIE3)
	u8Ret = drvNAND_ReadByte();
#else
    u8Ret = MDrv_NAND_ReadByte();
#endif

    PF_MODE_SEM(up(&PfModeSem));
    
    return (u8Ret);
}

/**
 * nand_wait - [DEFAULT]  wait until the command is done
 * @mtd:	MTD device structure
 * @this:	NAND chip structure
 * @state:	state to select the max. timeout value
 *
 * Wait for command done. This applies to erase and program only
 * Erase can take up to 400ms and program up to 20ms according to
 * general NAND and SmartMedia specs
 *
*/

static int _titania_nand_wait(struct mtd_info *mtd, struct nand_chip *this)
{
	#if defined(USING_FCIE3)
	U8 u8Tmp;
	#endif
    NAND_DEBUG("NANDDrv_WAIT()\n");
    
    #if defined(USING_FCIE3)
	drvNAND_WaitReady(1000);
	drvNand_Read_Status();
	NC_GetCIFD(&u8Tmp, 0, 1);
	return u8Tmp;
	#else
    MDrv_NAND_WaitReady();
	MDrv_NAND_Send_Read_Status_CMD(NAND_CMD_STATUS);
    return (MDrv_NAND_ReadByte());
	#endif
}

static void _titania_nand_cmdfunc(struct mtd_info *mtd, unsigned command,
                        int column, int page_addr)
{
	//U16 u16Tmp;
	
    PF_MODE_SEM(down(&PfModeSem));
    
    #if defined(USING_FCIE3)
    REG_CLR_BITS_UINT16(NC_SIGNAL, BIT_NC_WP_AUTO | BIT_NC_CE_AUTO | BIT_NC_CE_H | BIT_NC_CE_SEL_MASK);
    REG_SET_BITS_UINT16(NC_SIGNAL, BIT_NC_CHK_RB_HIGH | BIT_NC_WP_H);
    #else
    _fsinfo.u16NC_CFG0 &= ~(R_NC_CE_AUTO+R_NC_WP_AUTO);
    _fsinfo.u16NC_CFG0 |= (R_NC_CE_EN+R_NC_WP_EN);
    HAL_WRITE_UINT16(NC_CFG0,_fsinfo.u16NC_CFG0);
    #endif
    
    
	switch (command) {
        case NAND_CMD_READ0:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_READ0, page_addr: 0x%x, column: 0x%x.\n", page_addr, (column>>1));
            #if defined(USING_FCIE3)
            u32CurRow = page_addr;
            u32CurCol = column;
            #else
            MDrv_NAND_SendCmd(NAND_CMD_READ0);
            MDrv_NAND_SendAdr(page_addr,column);
            if(mtd->oobsize > 16)
            {
                MDrv_NAND_SendCmd(NAND_CMD_READSTART);
            }
            #endif
            break;

        case NAND_CMD_READ1:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_READ1.\n");
            #if defined(USING_FCIE3)
            u32CurRow = page_addr;
            u32CurCol = column;
            #else
            MDrv_NAND_SendCmd(NAND_CMD_READ1);
            MDrv_NAND_SendAdr(page_addr,column);
            #endif
            break;

        case NAND_CMD_READOOB:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_READOOB.\n");
            #if defined(USING_FCIE3)
			drvNAND_CmdReadOOB(page_addr, column);
			#else
            MDrv_NAND_ReadOOB(page_addr,column);
			#endif
            break;

        case NAND_CMD_READID:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_READID.\n");
            #if defined(USING_FCIE3)
            u32CIFDIdx = 0;
			NC_ReadID();
			#else
            drvNAND_READ_ID();
			#endif
            break;

        case NAND_CMD_PAGEPROG:
            /* sent as a multicommand in NAND_CMD_SEQIN */
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_PAGEPROG.\n");
            #if defined(USING_FCIE3)
            u32CurRow = page_addr;
            u32CurCol = column;
			#else
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_PAGEPROG.\n");
            MDrv_NAND_Cmd_PageProgram(NAND_CMD_PAGEPROG);
			#endif
            break;

        case NAND_CMD_ERASE1:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_ERASE1,  page_addr: 0x%x, column: 0x%x.\n", page_addr, column);
            #if defined(USING_FCIE3)
			NC_EraseBlk(page_addr);
			#else
            MDrv_NAND_Erase1(page_addr);
			#endif
            break;

        case NAND_CMD_ERASE2:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_ERASE2.\n");
            #if defined(USING_FCIE3)
			#else
            MDrv_NAND_Erase2();
			#endif
            break;

        case NAND_CMD_SEQIN:
            /* send PAGE_PROG command(0x1080) */
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_SEQIN/PAGE_PROG,  page_addr: 0x%x, column: 0x%x.\n", page_addr, column);
			#if defined(USING_FCIE3)
			u32CurRow = page_addr;
            u32CurCol = column;
			u16ByteIdxofPage = column;
			u32WriteLen = 0;
			#else
            MDrv_NAND_WritePage(page_addr,column);
			#endif
            break;

        case NAND_CMD_STATUS:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_STATUS.\n");
            #if defined(USING_FCIE3)
            u32CIFDIdx = 0;
            drvNand_Read_Status();
			#else
            MDrv_NAND_Send_Read_Status_CMD(NAND_CMD_STATUS);
			#endif
            break;

        case NAND_CMD_RESET:
        	NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_RESET.\n");
            #if defined(USING_FCIE3)
			NC_ResetNandFlash();
			#else
            drvNAND_FLASHRESET();
			#endif
            break;

        case NAND_CMD_STATUS_MULTI:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_STATUS_MULTI.\n");
            #if defined(USING_FCIE3)
            u32CIFDIdx = 0;
			drvNand_Read_Status();
			#else
            MDrv_NAND_Send_Read_Status_CMD(NAND_CMD_STATUS_MULTI);
			#endif
            break;

        case NAND_CMD_READSTART:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_READSTART.\n");
            #if defined(USING_FCIE3)
            
			#else
            MDrv_NAND_SendCmd(NAND_CMD_READSTART);
			#endif
            break;

        case NAND_CMD_CACHEDPROG:
            NAND_DEBUG("_titania_nand_cmdfunc: NAND_CMD_CACHEDPROG.\n");
            #if defined(USING_FCIE3)
            
			#else
            MDrv_NAND_Cmd_PageProgram(NAND_CMD_CACHEDPROG);
			#endif
            break;

        default:
            printk("_titania_nand_cmdfunc: error, unsupported command.\n");
            break;
	}
	
    PF_MODE_SEM(up(&PfModeSem));
    
	return;
}

static void _titania_nand_enable_hwecc(struct mtd_info *mtd, int mode)
{
    NAND_DEBUG("enable_hwecc\r\n");
    // default enable
}

static int _titania_nand_calculate_ecc(struct mtd_info *mtd, const u_char *dat, u_char *ecc_code)
{    
    NAND_DEBUG("calculate_ecc\r\n");
    
	return 0;
}

static int _titania_nand_correct_data(struct mtd_info *mtd, u_char *dat, u_char *read_ecc, u_char *calc_ecc)
{
    NAND_DEBUG("correct_data\r\n");
	#if defined(USING_FCIE3)
	return drvNand_CheckECC();
	#else
	return(MDrv_NAND_CheckECC());
	#endif
}

/*
 * Board-specific NAND initialization.
 * - hwcontrol: hardwarespecific function for accesing control-lines
 * - dev_ready: hardwarespecific function for  accesing device ready/busy line
 * - eccmode: mode of ecc, see defines
 */
static int __init mstar_nand_init(void)
{
	struct nand_chip *nand;
	int err = 0;	
	#if defined(USING_FCIE3)
	NAND_DRIVER *pNandDrv = (NAND_DRIVER*)drvNand_get_DrvContext_address();
	#endif

	/* Allocate memory for MTD device structure and private data */
	nand_mtd = kmalloc (sizeof(struct mtd_info) + sizeof (struct nand_chip), GFP_KERNEL);
	if (!nand_mtd) {
		printk ("Unable to allocate NAND MTD device structure.\n");
		return -ENOMEM;
	}

	/* Get pointer to private data */
	nand = (struct nand_chip *) (&nand_mtd[1]);

	/* Initialize structures */
	memset((char *) nand_mtd, 0, sizeof(struct mtd_info));
	memset((char *) nand, 0, sizeof(struct nand_chip));

	nand_mtd->priv = nand;

    NAND_DEBUG("NANDDrv_BOARD_NAND_INIT().\n");

    PF_MODE_SEM(down(&PfModeSem));
    
    #if defined(USING_FCIE3)
    REG_CLR_BITS_UINT16(NC_SIGNAL, BIT_NC_WP_AUTO | BIT_NC_CE_AUTO | BIT_NC_CE_H | BIT_NC_CE_SEL_MASK);
    REG_SET_BITS_UINT16(NC_SIGNAL, BIT_NC_CHK_RB_HIGH | BIT_NC_WP_H);
	drvNAND_Init();
	#else
    drvNAND_FLASH_INIT();
	#endif
    
    PF_MODE_SEM(up(&PfModeSem));

    /* please refer to include/linux/nand.h for more info. */

    nand->dev_ready						= _titania_nand_device_ready;
    nand->cmd_ctrl						= _titania_nand_hwcontrol;
    nand->ecc.mode						= NAND_ECC_HW;

	#if defined(USING_FCIE3)
	if( pNandDrv->u32_PageByteCnt == 2048 )
	#else
    if(_fsinfo.eFlashConfig & FLASH_2KPAGE)
    #endif
    {
        nand->ecc.size					= 2048;
        nand->ecc.bytes					= 40;
        nand->ecc.layout				= &titania_nand_oob_64;
        _titania_nand_bbt_descr.offs	= 0;
    }
    else
    {
		nand->ecc.size					= 512;
		nand->ecc.bytes					= 10;
		nand->ecc.layout				= &titania_nand_oob_16;
    }
    
    nand->ecc.hwctl						= _titania_nand_enable_hwecc;
    nand->ecc.correct					= _titania_nand_correct_data;
    nand->ecc.calculate					= _titania_nand_calculate_ecc;
    
    #if defined(USING_FCIE3)
	if( pNandDrv->u32_PageByteCnt == 2048 )
	#else
    if(_fsinfo.eFlashConfig & FLASH_2KPAGE)
    #endif
		nand->options					= NAND_USE_FLASH_BBT | NAND_NO_AUTOINCR;
	else
		nand->options					= NAND_USE_FLASH_BBT;
	
    nand->waitfunc						= _titania_nand_wait;
    nand->read_byte						= _titania_nand_read_byte;
    nand->read_word						= _titania_nand_read_word;
    nand->read_buf						= _titania_nand_read_buf;
    nand->write_buf						= _titania_nand_write_buf;
    nand->chip_delay					= 20;
    nand->cmdfunc						= _titania_nand_cmdfunc;
    nand->badblock_pattern				= &_titania_nand_bbt_descr; //using default badblock pattern.
    nand->bbt_td						= &_titania_bbt_main_descr;
    nand->bbt_md						= &_titania_bbt_mirror_descr;
    
    #if defined(USING_FCIE3)
	REG_CLR_BITS_UINT16(NC_SIGNAL, BIT_NC_WP_AUTO | BIT_NC_CE_AUTO | BIT_NC_CE_H | BIT_NC_CE_SEL_MASK);
    REG_SET_BITS_UINT16(NC_SIGNAL, BIT_NC_CHK_RB_HIGH | BIT_NC_WP_H);
	#else
    MDrv_NAND_Set_Write_Protect(1);
	#endif
	
	if ((err = nand_scan(nand_mtd, 1)) != 0) {
		printk("can't register NAND\n");
		return -ENOMEM;
	}

#ifdef CONFIG_MTD_CMDLINE_PARTS
    {
        int mtd_parts_nb = 0;
        struct mtd_partition *mtd_parts = 0;
        mtd_parts_nb = parse_cmdline_partitions(nand_mtd, &mtd_parts, "hynix-nand");
        if (mtd_parts_nb > 0) {
            add_mtd_partitions(nand_mtd, mtd_parts, mtd_parts_nb);
            printk("parse_cmdline_partitions: add_mtd_partitions\n");
        }
        else {
            #if 0
            add_mtd_device(nand_mtd);
            printk("parse_cmdline_partitions: add_mtd_device\n");
            #else
            add_mtd_partitions(nand_mtd, partition_info, NUM_PARTITIONS); // use default partition table if parsing cmdline partition fail
            #endif
        }
    }
#else
	add_mtd_partitions(nand_mtd, partition_info, NUM_PARTITIONS);
#endif
    return 0;
}

static void __exit mstar_nand_cleanup(void)
{
}

module_init(mstar_nand_init);
module_exit(mstar_nand_cleanup);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("MStar");
MODULE_DESCRIPTION("MStar MTD NAND driver");
