/*
 * Copyright (C) 2008, OGAWA Hirofumi
 * Released under GPL v2.
 */

#include "fatent.h"
#include "exfat.h"

inline void exfat_ent_init(struct exfat_ent *fatent)
{
	fatent->bh = NULL;
}

inline void exfat_ent_release(struct exfat_ent *fatent)
{
	brelse(fatent->bh);
}

/* FIXME: should be outline those funcs? must rethink those */

inline int exfat_ent_read(struct inode *inode,
				 struct exfat_ent *fatent,
				 u32 clusnr, u32 *result_clusnr)
{
	struct super_block *sb = inode->i_sb;
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	sector_t blocknr;
	__le32 *ent;

	blocknr = sbi->fat_blocknr + (clusnr >> sbi->fpb_bits);
	if (!fatent->bh || fatent->bh->b_blocknr != blocknr) {
		brelse(fatent->bh);
		/*
		if(blocknr != sbi->fat_bh->b_blocknr){
			sbi->fat_bh = sb_bread(sb, blocknr);	
			if(!sbi->fat_bh)
			return -EIO;
		}
		get_bh(sbi->fat_bh);
		fatent->bh = sbi->fat_bh;
		*/
		fatent->bh = sb_bread(sb, blocknr);
		if (!fatent->bh)
			return -EIO;
	}

	ent = (__le32 *)fatent->bh->b_data;
	*result_clusnr = le32_to_cpu(ent[clusnr & (sbi->fpb - 1)]);

	return 0;
}

inline int exfat_ent_prepare(struct inode *inode,
				    struct exfat_ent *fatent,
				    u32 clusnr, u32 *result_clusnr)
{
	struct super_block *sb = inode->i_sb;
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	__le32 *ent;
	sector_t blocknr;

	blocknr = sbi->fat_blocknr + (clusnr >> sbi->fpb_bits);
	/*
		if(blocknr != sbi->fat_bh->b_blocknr){
			sbi->fat_bh = sb_bread(sb, blocknr);	
			if(!sbi->fat_bh)
			return -EIO;
		}
		get_bh(sbi->fat_bh);
		fatent->bh = sbi->fat_bh;
	*/
	
	fatent->bh = sb_bread(sb,blocknr);
	if (!fatent->bh)
		return -EIO;

	fatent->offset = clusnr & (sbi->fpb - 1);
	
	ent = (__le32 *)fatent->bh->b_data;
	*result_clusnr = le32_to_cpu(ent[fatent->offset]);

	
	MY_DEBUG(0,"blocknr=%d,offset=%d,result_clusnr=%x \n",sbi->fat_blocknr + (clusnr >> sbi->fpb_bits),fatent->offset,*result_clusnr);

	return 0;
}

inline void exfat_ent_write(struct inode *inode,
				   struct exfat_ent *fatent,
				   u32 new_clusnr)
{
	__le32 *ent = (__le32 *)fatent->bh->b_data;
	ent[fatent->offset] = cpu_to_le32(new_clusnr);
}
