/*
 * Copyright (C) 2008, OGAWA Hirofumi
 * Released under GPL v2.
 */

#include <linux/buffer_head.h>
#include <linux/mpage.h>
#include <linux/hash.h>
#include <linux/writeback.h>
#include "exfat.h"
#include "cluster.h"
#include "fatent.h"


 extern int g_flag;

int exfat_get_block(struct inode *inode, sector_t iblock,
		    struct buffer_head *bh_result, int create)
{
	struct super_block *sb = inode->i_sb;
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);
	unsigned long blocksize, max_blocks, mapped_blocks;
	sector_t blocknr, last_iblock;
	u32 iclusnr, last_iclusnr,clus_len;
	int err, offset;

	static int idisplay = 0;
	static int aaaa = 140930;
	static int flag = 1;
	
//	if(create == 0)
//		idisplay = 0;

	if(g_flag == 1)
		idisplay = 1;

	blocksize = 1 << inode->i_blkbits;
	last_iblock = (exi->phys_size+ (blocksize - 1))
		>> inode->i_blkbits;

	if (!create && iblock >= last_iblock)
		return -EIO;
//	printk("iblock=%d,last_iblock=%d,create=%d \n",iblock,last_iblock,create);	
	MY_DEBUG(idisplay,"inodenum=%d,iblock=%d,last_iblock=%d,create=%d,exi->phys_size=%lld \n",inode->i_ino,iblock,last_iblock,create,exi->phys_size);	

	/*
	 * Find the available blocks already allocating
	 */
	last_iclusnr = (last_iblock + (sbi->bpc - 1)) >> sbi->bpc_bits;
	iclusnr = iblock >> sbi->bpc_bits;
	offset = iblock & (sbi->bpc - 1);

	max_blocks = bh_result->b_size >> inode->i_blkbits;
	mapped_blocks = 0;

	MY_DEBUG(idisplay,"iclusnr=%d,last_iclusnr=%d,offset=%d,max_blocks=%d \n",iclusnr,last_iclusnr,offset,max_blocks);
	
	if (iclusnr < last_iclusnr) {
		/* Some clusters are available */
		struct exfat_clus_map cmap;
		u32 clus_len;

		clus_len = (max_blocks + (sbi->bpc - 1)) >> sbi->bpc_bits;
		MY_DEBUG(idisplay,"[]max_blocks=%d, clus_len=%d,iclusnr=%d\n",max_blocks, clus_len,iclusnr);

		if(1){
			err = exfat_get_cluster(inode, iclusnr, clus_len, &cmap);
			if (err)
				return err;

			if (!exfat_ent_valid(sbi, cmap.clusnr)) {
				PRINTK(KERN_ERR "exFAT: unexpected FAT entry"
				       " (start cluster 0x%08x, entry 0x%08x)\n",
				       exi->clusnr, cmap.clusnr);
				return -EIO;
			}
		}
		else{
			cmap.clusnr = aaaa;
			cmap.len = 1;
		}

		blocknr = exfat_clus_to_blknr(sbi, cmap.clusnr) + offset;
		mapped_blocks = (cmap.len << sbi->bpc_bits) - offset;
		MY_DEBUG(idisplay,"[]@!@! blocknr=%d,mapped_blocks=%d,offset=%d,cmap.clusnr=%d \n",blocknr,mapped_blocks,offset,cmap.clusnr);
		if (iblock < last_iblock) {
			/* Some blocks are available */
			//unsigned long avail_blocks = last_iblock - iblock;

			//mapped_blocks = min(mapped_blocks, avail_blocks);
			//mapped_blocks = min(mapped_blocks, max_blocks);

			map_bh(bh_result, sb, blocknr);
			bh_result->b_size = max_blocks << inode->i_blkbits;
			return 0;
		}
		/* We can allocate blocks in this last cluster */
	}
	/* FIXME: write is not supported */
//	DEBUG0_ON(create);


	/*
	 * Allocate new blocks
	 */
	MY_DEBUG(idisplay,"iiblock=%d,exi->phys_size=%lld ,mapped_blocks=%d\n",iblock,exi->phys_size,mapped_blocks);		 
//	DEBUG_ON(iblock != exi->phys_size >> inode->i_blkbits,
//		 "iblock %llu, phys_size %lld\n", (llu)iblock, exi->phys_size);
	DEBUG_ON(mapped_blocks >= sbi->bpc,
		 "mapped_blocks %lu\n", mapped_blocks);

//	return -EIO;

	clus_len = iclusnr - last_iclusnr;
	if(clus_len == 0)
		clus_len = 1;
	/* FIXME: multiple cluster allocation would be desirable. */
	/* We have to allocate new cluster */
	err = exfat_add_cluster(inode, &iclusnr);

//	if(iclusnr & 0x10000)
//		idisplay = 1;

	if (err)
		return err;

//	aaaa = iclusnr; 
//	flag = 0;

	mapped_blocks += sbi->bpc ;//- offset;		//???

	MY_DEBUG(idisplay,"mapped_blocks=%d\n",mapped_blocks);

	mapped_blocks = min(mapped_blocks, max_blocks);

	MY_DEBUG(idisplay,"111 mapped_blocks=%d\n",mapped_blocks);

	set_buffer_new(bh_result);

	blocknr = exfat_clus_to_blknr(sbi, iclusnr);// + offset;

	MY_DEBUG(idisplay,"iclusnr = %d,blocknr=%d \n",iclusnr,blocknr);	
	
	map_bh(bh_result, sb, blocknr);
	bh_result->b_size = mapped_blocks << inode->i_blkbits;

	return 0;
}

static int exfat_readpage(struct file *file, struct page *page)
{
	MY_DEBUG(0,"######### \n");
	return mpage_readpage(page, exfat_get_block);
}

static int exfat_readpages(struct file *file, struct address_space *mapping,
			   struct list_head *pages, unsigned nr_pages)
{
	MY_DEBUG(0,"######### \n");
	return mpage_readpages(mapping, pages, nr_pages, exfat_get_block);
}

static int exfat_writepage(struct page *page, struct writeback_control *wbc)
{
	MY_DEBUG(0,"exfat_writepage######### \n");
	return block_write_full_page(page, exfat_get_block, wbc);
}

static int exfat_writepages(struct address_space *mapping,
			  struct writeback_control *wbc)
{
	MY_DEBUG(0,"exfat_writepages######### \n");
	return mpage_writepages(mapping, wbc, exfat_get_block);
}



static sector_t exfat_bmap(struct address_space *mapping, sector_t iblock)
{
	sector_t blocknr;

	/* exfat_get_cluster() assumes the requested blocknr isn't truncated. */
	mutex_lock(&mapping->host->i_mutex);
	blocknr = generic_block_bmap(mapping, iblock, exfat_get_block);
	mutex_unlock(&mapping->host->i_mutex);

	return blocknr;
}

static int exfat_write_begin(struct file *file, struct address_space *mapping,
				loff_t pos, unsigned len, unsigned flags,
				struct page **pagep, void **fsdata)
{
	int ret ;

	static int idisplay = 0;
	
	*pagep = NULL;

	MY_DEBUG(idisplay,"1111inode num=%d,len= %d\n",mapping->host->i_ino,len);

	ret = cont_write_begin(file, mapping, pos, len, flags, pagep, fsdata,
				exfat_get_block,
				&EXFAT_I(mapping->host)->phys_size);		

	MY_DEBUG(idisplay,"isize = %lld ,physize = %d \n",mapping->host->i_size,EXFAT_I(mapping->host)->phys_size);
//	if(EXFAT_I(mapping->host)->phys_size >= 2147319808)
//		idisplay = 1;
	return ret;
}

static int exfat_write_end(struct file *file, struct address_space *mapping,
				loff_t pos, unsigned len, unsigned copied,
				struct page *pagep, void *fsdata)
{
	struct inode *inode = mapping->host;
	int err;
	static int idisplay = 0;

	MY_DEBUG(idisplay,"inode num=%d,physize = %lld,len= %d,copied=%d\n",mapping->host->i_ino,EXFAT_I(mapping->host)->phys_size,len,copied);
	
	err = generic_write_end(file, mapping, pos, len, copied, pagep, fsdata);
	if (!(err < 0) && !(EXFAT_I(inode)->attrib & EXFAT_ATTR_ARCH)) {
		inode->i_mtime = inode->i_ctime = CURRENT_TIME_SEC;
		EXFAT_I(inode)->attrib |= EXFAT_ATTR_ARCH;
		mark_inode_dirty(inode);
	}
	MY_DEBUG(idisplay,"isize = %lld \n",mapping->host->i_size);

//	if(EXFAT_I(mapping->host)->phys_size >= 2147319808)
//		idisplay = 1;

	return err;
}


static const struct address_space_operations exfat_aops = {
	.readpage		= exfat_readpage,
	.readpages		= exfat_readpages,
	.writepage		= exfat_writepage,
	.writepages		= exfat_writepages,
	.sync_page		= block_sync_page,
	.write_begin		= exfat_write_begin,
	.write_end		= exfat_write_end,
	.bmap			= exfat_bmap,
//	.invalidatepage		= ext4_da_invalidatepage,
//	.releasepage		= ext4_releasepage,
//	.direct_IO		= ext4_direct_IO,
//	.migratepage		= buffer_migrate_page,
//	.is_partially_uptodate	= block_is_partially_uptodate,
};

int exfat_getattr(struct vfsmount *mnt, struct dentry *dentry,
		  struct kstat *stat)
{
	struct inode *inode = dentry->d_inode;
	generic_fillattr(inode, stat);
	stat->blksize = EXFAT_SB(inode->i_sb)->clus_size;
	return 0;
}


static int exfat_free(struct inode *inode, int iclus)
{
	struct super_block *sb = inode->i_sb;
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);
	int clusnr,err,next_clus,last_iclusnr,blocksize;
	struct exfat_clus_map cmap;
	struct exfat_ent exfatent;
	sector_t last_iblock;

	int idisplay = 0;

	if(exi->clusnr == 0)
		return 0;

	blocksize = 1 << inode->i_blkbits;
	last_iblock = (exi->phys_size+ (blocksize - 1))
		>> inode->i_blkbits;
	last_iclusnr = (last_iblock + (sbi->bpc - 1)) >> sbi->bpc_bits;

	MY_DEBUG(idisplay,"iclus=%d,last_iclusnr=%d,exi->clusnr=%d \n",iclus,last_iclusnr,exi->clusnr);

	clusnr = exi->clusnr;	

	if(iclus){
		exfat_get_cluster(inode, iclus-1, 0 , &cmap);		//get the cluster want to free from
		clusnr = 	cmap.clusnr;

		MY_DEBUG(idisplay,"clusnr=%d \n",clusnr);

		exfat_ent_init(&exfatent);	
		err = exfat_ent_read(inode, &exfatent, clusnr, &next_clus);
		MY_DEBUG(idisplay,"next_clus=%d,err=%d \n",clusnr,err);
		if(err)
			return -EIO;

		if(next_clus == EXFAT_ENT_EOF){
			exfat_ent_release(&exfatent);
			return 0;
		}
		else if(next_clus == EXFAT_ENT_FREE || next_clus == EXFAT_ENT_BAD){
			exfat_fs_panic(sb, "Invalid FAT chain");
			return -EIO;
		}
		else{
			err = exfat_write_entry(inode, clusnr, EXFAT_ENT_EOF);
			if(err)
				return err;
		}
			
		clusnr = next_clus;
	}

	err = exfat_free_clusters(inode, clusnr,last_iclusnr - iclus);
	MY_DEBUG(idisplay,"clusnr=%d,err=%d\n",clusnr,err);
	
	return err;
}


/*
 * helper function for fat_flush_inodes.  This writes both the inode
 * and the file data blocks, waiting for in flight data blocks before
 * the start of the call.  It does not wait for any io started
 * during the call
 */
static int writeback_inode(struct inode *inode)
{
	int ret;
	struct address_space *mapping = inode->i_mapping;
	struct writeback_control wbc = {
	      .sync_mode = WB_SYNC_NONE,
	      .nr_to_write = 0,
	};
	/* if we used WB_SYNC_ALL, sync_inode waits for the io for the
	* inode to finish.  So WB_SYNC_NONE is sent down to sync_inode
	* and filemap_fdatawrite is used for the data blocks
	*/
	ret = sync_inode(inode, &wbc);
	if (!ret)
	       ret = filemap_fdatawrite(mapping);
	return ret;
}

/*
 * write data and metadata corresponding to i1 and i2.  The io is
 * started but we do not wait for any of it to finish.
 *
 * filemap_flush is used for the block device, so if there is a dirty
 * page for a block already in flight, we will not wait and start the
 * io over again
 */
int exfat_flush_inodes(struct super_block *sb, struct inode *i1, struct inode *i2)
{
	int ret = 0;
	if (i1)
		ret = writeback_inode(i1);
	if (!ret && i2)
		ret = writeback_inode(i2);
	if (!ret) {
		struct address_space *mapping = sb->s_bdev->bd_inode->i_mapping;
		ret = filemap_flush(mapping);
	}
	return ret;
}


void  exfat_truncate(struct inode *inode)
{
	struct super_block *sb = inode->i_sb;
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);

	const unsigned int cluster_size = sbi->clus_size;
	int nr_clusters;
	int idisplay = 0;

	MY_DEBUG(idisplay,"\n ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ \n");

	/*
	 * This protects against truncating a file bigger than it was then
	 * trying to write into the hole.
	 */
	nr_clusters = (inode->i_size + (cluster_size - 1)) >> sbi->clus_bits;
	MY_DEBUG(idisplay,"nr_clusters=%d,inode->i_size=%lld\n",nr_clusters,inode->i_size);
	exfat_free(inode,nr_clusters);
	exfat_flush_inodes(sb, inode, NULL);
}

static const struct file_operations exfat_file_ops = {
	.llseek		= generic_file_llseek,
	.read		= do_sync_read,
	.write		= do_sync_write,
	.aio_read	= generic_file_aio_read,
	.aio_write	= generic_file_aio_write,
//	.unlocked_ioctl	= fat_generic_ioctl,
#ifdef CONFIG_COMPAT
//	.compat_ioctl	= fat_compat_dir_ioctl,
#endif
	.mmap		= generic_file_mmap,
	.open		= generic_file_open,
//	.fsync		= file_fsync,
	.splice_read	= generic_file_splice_read,
	.splice_write	= generic_file_splice_write,
};

static const struct inode_operations exfat_file_inode_ops = {
	.truncate	= exfat_truncate,
//	.permission	= ext4_permission,
//	.setattr	= ext4_setattr,
	.getattr	= exfat_getattr
#ifdef CONFIG_EXT4DEV_FS_XATTR
//	.setxattr	= generic_setxattr,
//	.getxattr	= generic_getxattr,
//	.listxattr	= ext4_listxattr,
//	.removexattr	= generic_removexattr,
#endif
//	.fallocate	= ext4_fallocate,
//	.fiemap		= ext4_fiemap,
};

/* See comment in fs/fat/inode.c */
/* FIXME: rethink those hash stuff */
void exfat_hash_init(struct super_block *sb)
{
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	int i;

	spin_lock_init(&sbi->inode_hash_lock);
	for (i = 0; i < EXFAT_HASH_SIZE; i++)
		INIT_HLIST_HEAD(&sbi->inode_hashtable[i]);
}

static inline u32 exfat_hash(struct super_block *sb, sector_t blocknr,
			     unsigned long offset)
{
	u32 key = (((loff_t)blocknr << sb->s_blocksize_bits) | offset)
		>> EXFAT_CHUNK_BITS;
	return hash_32(key, EXFAT_HASH_BITS);
}

void exfat_attach(struct inode *inode, struct exfat_parse_data *pd)
{
	struct super_block *sb = inode->i_sb;
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);
	u32 hashval = exfat_hash(sb, pd->bhs[0]->b_blocknr, pd->bh_offset);
	struct hlist_head *head = sbi->inode_hashtable + hashval;
	int i;

	spin_lock(&sbi->inode_hash_lock);
	for (i = 0; i < pd->nr_bhs; i++)
		exi->de_blocknr[i] = pd->bhs[i]->b_blocknr;
	exi->de_offset = pd->bh_offset;
	exi->de_size = pd->size;
	exi->nr_bhs = pd->nr_bhs;
	hlist_add_head(&exi->i_hash, head);
	spin_unlock(&sbi->inode_hash_lock);
}

static void __exfat_detach(struct exfat_inode_info *exi)
{
	exi->de_blocknr[0] = 0;
	exi->de_offset = -1;
	exi->de_size = -1;
}

void exfat_detach(struct inode *inode)
{
	struct exfat_sb_info *sbi = EXFAT_SB(inode->i_sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);

	spin_lock(&sbi->inode_hash_lock);
//	__exfat_detach(exi);
	hlist_del_init(&exi->i_hash);
	spin_unlock(&sbi->inode_hash_lock);
}

struct inode *exfat_ilookup(struct super_block *sb, sector_t blocknr,
			    unsigned long offset)
{
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	u32 hashval = exfat_hash(sb, blocknr, offset);
	struct hlist_head *head = sbi->inode_hashtable + hashval;
	struct hlist_node *_p;
	struct exfat_inode_info *exi;
	struct inode *inode = NULL;

	spin_lock(&sbi->inode_hash_lock);
	hlist_for_each_entry(exi, _p, head, i_hash) {
		DEBUG_ON(exi->vfs_inode.i_sb != sb,
			 "vfs.sb %p, sb %p\n", exi->vfs_inode.i_sb, sb);
		if (exi->de_blocknr[0] != blocknr)
			continue;
		if (exi->de_offset != offset)
			continue;
		inode = igrab(&exi->vfs_inode);
		if (inode)
			break;
	}
	spin_unlock(&sbi->inode_hash_lock);
	return inode;
}

static void exfat_fill_inode(struct inode *inode, unsigned long ino,
			     struct exfat_chunk_dirent *dirent,
			     struct exfat_chunk_data *data)
{
	struct exfat_sb_info *sbi = EXFAT_SB(inode->i_sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);

	inode->i_ino = ino;
	inode->i_nlink = 1;	/*  Windows use 1 even if directory.*/
	inode->i_uid = sbi->opts.uid;
	inode->i_gid = sbi->opts.gid;
	inode->i_version = 1;
	/* FIXME: inode->i_size is "signed", have to check overflow? */
	inode->i_size = le64_to_cpu(data->size);
	/* FIXME: sane time conversion is needed */
	exfat_time_exfat2unix(&inode->i_mtime, dirent->mtime, dirent->mdate,
			      dirent->mtime_cs);
	exfat_time_exfat2unix(&inode->i_atime, dirent->atime, dirent->adate, 0);
	inode->i_ctime.tv_sec = 0;
	inode->i_ctime.tv_nsec = 0;
	inode->i_blocks = (inode->i_size + (sbi->clus_size - 1)) >> 9;
/*	inode->i_generation = get_seconds(); FIXME: don't support nfs? */
	if (dirent->attrib & cpu_to_le16(EXFAT_ATTR_DIR)) {
		inode->i_mode = S_IFDIR | sbi->opts.dmode;
		inode->i_op = &exfat_dir_inode_ops;
		inode->i_fop = &exfat_dir_ops;
	} else {
		inode->i_mode = S_IFREG | sbi->opts.fmode;
		inode->i_op = &exfat_file_inode_ops;
		inode->i_fop = &exfat_file_ops;
		inode->i_mapping->a_ops = &exfat_aops;
	}
#if 0
	/* FIXME: separate S_NOCMTIME to S_NOCTIME and S_NOMTIME,
	 * then exFAT/FAT can use S_NOCTIME. */
	inode->i_flags |= S_NOCTIME;

	/* FIXME: how handle these attrib? */
	if (dirent->attrib & cpu_to_le16(EXFAT_ATTR_SYSTEM)) {
		if (sbi->opts.sys_immutable)
			inode->i_flags |= S_IMMUTABLE;
	}
	if (dirent->attrib & cpu_to_le16(EXFAT_ATTR_RO)) {
		if (sbi->opts.sys_immutable)
			inode->i_flags |= S_IMMUTABLE;
	}
#endif
	exi->phys_size = inode->i_size;
	if (exi->phys_size & (sbi->clus_size - 1)) {
		exi->phys_size = (exi->phys_size + sbi->clus_size - 1)
			& ~((loff_t)sbi->clus_size - 1);
	}

	exi->clusnr = le32_to_cpu(data->clusnr);
	exi->attrib = dirent->attrib;
	exi->data_flag = data->flag;

	__exfat_detach(exi);
}

struct inode *exfat_iget(struct super_block *sb,
			 struct exfat_parse_data *pd,
			 struct exfat_chunk_dirent *dirent,
			 struct exfat_chunk_data *data)
{
	struct inode *inode;
	unsigned long ino;

	/* FIXME: this can use ilookup5 (with inode_lock). Which one is fast? */
	inode = exfat_ilookup(sb, pd->bhs[0]->b_blocknr, pd->bh_offset);
	if (inode)
		return inode;

	inode = new_inode(sb);
	if (!inode)
		return ERR_PTR(-ENOMEM);

	ino = iunique(sb, EXFAT_RESERVED_INO);
	exfat_fill_inode(inode, ino, dirent, data);

	exfat_attach(inode, pd);
	insert_inode_hash(inode);

	return inode;
}

struct inode *exfat_new_internal_inode(struct super_block *sb,
				       unsigned long ino, u16 attrib,
				       u32 clusnr, u64 size)
{
	struct inode *inode;
	struct exfat_chunk_dirent dirent = {
		.attrib	= cpu_to_le16(attrib),
	};
	struct exfat_chunk_data data = {
		.flag	= EXFAT_DATA_NORMAL,
		.clusnr	= cpu_to_le32(clusnr),
		.size	= cpu_to_le64(size),
	};

	inode = new_inode(sb);
	if (inode) {
		exfat_fill_inode(inode, ino, &dirent, &data);
		/* The internal inode doesn't have timestamp */
		inode->i_flags |= S_NOATIME | S_NOCMTIME;
	}
	return inode;
}

struct inode *exfat_rootdir_iget(struct super_block *sb)
{
	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	struct inode *inode;
	struct exfat_clus_map cmap;
	int err;

	err = -ENOMEM;
	inode = exfat_new_internal_inode(sb, EXFAT_ROOT_INO, EXFAT_ATTR_DIR,
					 sbi->rootdir_clusnr, 0);
	if (!inode)
		goto error;

	/* Get last iclusnr to set ->i_size */
	err = exfat_get_cluster(inode, EXFAT_ENT_EOF, 0, &cmap);
	if (err)
		goto error_iput;

	if (!exfat_ent_eof(sbi, cmap.clusnr)) {
		PRINTK(KERN_ERR "exFAT: found invalid FAT entry 0x%08x"
		       " for root directory\n", cmap.clusnr);
		err = -EIO;
		goto error_iput;
	}

	inode->i_size = cmap.iclusnr << sbi->clus_bits;
	EXFAT_I(inode)->phys_size = inode->i_size;
	inode->i_blocks = (inode->i_size + (sbi->clus_size - 1)) >> 9;

	insert_inode_hash(inode);

	return inode;

error_iput:
	iput(inode);
error:
	return ERR_PTR(err);
}

int exfat_write_inode(struct inode *inode, int wait)
{
	struct super_block *sb = inode->i_sb;
//	struct exfat_sb_info *sbi = EXFAT_SB(sb);
	struct exfat_inode_info *exi = EXFAT_I(inode);
	struct buffer_head *bh_dir = NULL,*bh_data= NULL,*bh_name = NULL;
	struct exfat_chunk_data *data = NULL;
	struct exfat_chunk_dirent* dir = NULL;
	struct exfat_chunk_name* name = NULL;
	int offset = 0,err = 0,index = 0,i,cross_sector_flag = 0,nr_subchunks,name_left;
	unsigned long blocksize;
	u16 checksum;
	int idisplay = 0;

	MY_DEBUG(idisplay,"$$$$$$$ inode num = %d\n",inode->i_ino);
	
	if (inode->i_ino == EXFAT_ROOT_INO)
		return 0;

	blocksize = 1 << inode->i_blkbits;
	
	offset = exi->de_offset;
	offset += EXFAT_CHUNK_SIZE;
	MY_DEBUG(idisplay,"offset = %d,blocknr=%d\n",exi->de_offset,exi->de_blocknr[0]);	

	if(offset < blocksize)
		goto write_data;
	
write_dir:		//write dir
	DEBUG_ON(offset != 512,"bug");
	offset -= blocksize;
	bh_dir = sb_bread(sb, exi->de_blocknr[index]);
	MY_DEBUG(idisplay,"exi->de_blocknr[%d] = %d\n",index,exi->de_blocknr[index]);
	if (!bh_dir) {
		printk(KERN_ERR "FAT: unable to read inode block "
		       "for updating (i_pos )\n");
		return -EIO;
	}
	dir = (struct exfat_chunk_dirent*)(bh_dir->b_data+ exi->de_offset);

	index ++;
	cross_sector_flag = 1;

write_data:
	bh_data = sb_bread(sb, exi->de_blocknr[index]);
	MY_DEBUG(idisplay,"offset=%d,exi->de_blocknr[%d] = %d\n",offset,index,exi->de_blocknr[index]);
	if (!bh_data) {
		printk(KERN_ERR "FAT: unable to read inode block "
		       "for updating (i_pos )\n");
		return -EIO;
	}
	data = (struct exfat_chunk_data *)(bh_data->b_data+ offset);
	if(dir == NULL)
		dir = (struct exfat_chunk_dirent *)(data - 1);	//offset to dirent;

	if (!S_ISDIR(inode->i_mode)){
		data->size = data->size2 = cpu_to_le64(inode->i_size);
		if(data->size == 0)
			exi->data_flag = EXFAT_DATA_NORMAL;
	}
		
	MY_DEBUG(idisplay,"&&&&&&&&&&&&&&&& data->clusnr = %d,clunr = %d \n",data->clusnr,exi->clusnr);
	
	data->clusnr = exi->clusnr;	
	data->flag = exi->data_flag;


	//re-calculate the checksum
	//chunk_dirent
	checksum = exfat_checksum16(0, dir, 2);
	checksum = exfat_checksum16(checksum, (u8 *)dir + 4, EXFAT_CHUNK_SIZE - 4);

	//chunk_data
	checksum = exfat_checksum16(checksum, data, EXFAT_CHUNK_SIZE);

	offset += EXFAT_CHUNK_SIZE;	//offset to chunk_name
	name = (struct exfat_chunk_name*)(data+ 1);

	MY_DEBUG(idisplay,"&&&&&&&&&&&&&&&& offset = %d \n",offset);
	
write_name:
	nr_subchunks = dir->sub_chunks - 1;
	name_left = nr_subchunks * EXFAT_CHUNK_SIZE;

	MY_DEBUG(idisplay,"&&&&&&&&&&&&&&&& name_left = %d \n",name_left);

	while(name_left){
	
		if(offset >= blocksize){		//
			index ++;
			offset -= blocksize;
			if(bh_name)
				brelse(bh_name);
			bh_name = sb_bread(sb, exi->de_blocknr[index]);
			MY_DEBUG(idisplay,"exi->de_blocknr[%d] = %d\n",index,exi->de_blocknr[index]);
			if (!bh_name) {
				printk(KERN_ERR "FAT: unable to read inode block "
				       "for updating (i_pos )\n");
				return -EIO;
			}
			name = (struct exfat_chunk_name*)(bh_name->b_data + offset);	
		}

		checksum = exfat_checksum16(checksum, name, EXFAT_CHUNK_SIZE);	

		name ++;
		name_left -= EXFAT_CHUNK_SIZE;
		offset += EXFAT_CHUNK_SIZE;
	}
	
	MY_DEBUG(idisplay," orgchksum = %x,checksum = %x,cross_sector_flag=%d \n",dir->checksum,checksum,cross_sector_flag);

	
	if(bh_name)
		brelse(bh_name);

	if(checksum == dir->checksum){	//if it is not changed, do not need to save
		if(cross_sector_flag)
			brelse(bh_dir);
	
		brelse(bh_data);
	}
	else{
		dir->checksum = checksum;
		if(cross_sector_flag){
			mark_buffer_dirty(bh_dir);
			sync_dirty_buffer(bh_dir);
			brelse(bh_dir);
		}
		
		mark_buffer_dirty(bh_data);	
		sync_dirty_buffer(bh_data);
		brelse(bh_data);
	}
	MY_DEBUG(idisplay,"&&&&&&&&&&&&&&&& ret = %d \n",err);

	return err;
}


int exfat_sync_inode(struct inode *inode)
{
	return exfat_write_inode(inode, 1);
}



