/*
 * Copyright (C) 2008, OGAWA Hirofumi
 * Released under GPL v2.
 */

#include "exfat.h"
#include <linux/buffer_head.h>
#include "exfat_fs.h"


extern struct timezone sys_tz;

#define C_32(s, p)	do {						\
	C(s, (p)[ 0]); C(s, (p)[ 1]); C(s, (p)[ 2]); C(s, (p)[ 3]);	\
	C(s, (p)[ 4]); C(s, (p)[ 5]); C(s, (p)[ 6]); C(s, (p)[ 7]);	\
	C(s, (p)[ 8]); C(s, (p)[ 9]); C(s, (p)[10]); C(s, (p)[11]);	\
	C(s, (p)[12]); C(s, (p)[13]); C(s, (p)[14]); C(s, (p)[15]);	\
	C(s, (p)[16]); C(s, (p)[17]); C(s, (p)[18]); C(s, (p)[19]);	\
	C(s, (p)[20]); C(s, (p)[21]); C(s, (p)[22]); C(s, (p)[23]);	\
	C(s, (p)[24]); C(s, (p)[25]); C(s, (p)[26]); C(s, (p)[27]);	\
	C(s, (p)[28]); C(s, (p)[29]); C(s, (p)[30]); C(s, (p)[31]);	\
} while (0)

u32 exfat_checksum32(u32 sum, const void *__buf, int size)
{
#define C(s, v)		do { s = __exfat_checksum32(s, v); } while (0)
	const u8 *buf = __buf;
	int i;

	for (i = 0; (i + 511) < size; i += 512) {
		C_32(sum, &buf[i + 32 *  0]); C_32(sum, &buf[i + 32 *  1]);
		C_32(sum, &buf[i + 32 *  2]); C_32(sum, &buf[i + 32 *  3]);
		C_32(sum, &buf[i + 32 *  4]); C_32(sum, &buf[i + 32 *  5]);
		C_32(sum, &buf[i + 32 *  6]); C_32(sum, &buf[i + 32 *  7]);
		C_32(sum, &buf[i + 32 *  8]); C_32(sum, &buf[i + 32 *  9]);
		C_32(sum, &buf[i + 32 * 10]); C_32(sum, &buf[i + 32 * 11]);
		C_32(sum, &buf[i + 32 * 12]); C_32(sum, &buf[i + 32 * 13]);
		C_32(sum, &buf[i + 32 * 14]); C_32(sum, &buf[i + 32 * 15]);
	}
	for (; (i + 31) < size; i += 32)
		C_32(sum, &buf[i]);
	for (; i < size; i++)
		C(sum, buf[i]);

	return sum;
#undef C
}

u16 exfat_checksum16(u16 sum, const void *__buf, int size)
{
#define C(s, v)		do { s = __exfat_checksum16(s, v); } while (0)
	const u8 *buf = __buf;
	int i;

	for (i = 0; (i + 31) < size; i += 32)
		C_32(sum, &buf[i]);
	for (; i < size; i++)
		C(sum, buf[i]);
	
	return sum;
#undef C
}

int exfat_conv_u2c(struct nls_table *nls,
		   const __le16 **uni, unsigned int *uni_left,
		   unsigned char **out, unsigned int *out_left)
{
	wchar_t wc;
	int len,err=0;

	int idisplay = 0,i;

	len = exfat_utf16_to_utf8(&err, *uni, *uni_left, out, *out_left);

	for(i = 0 ; i < *uni_left ; i ++)
		MY_DEBUG(idisplay," %x \n",*uni + i);	
	MY_DEBUG(idisplay,"char = %s\n",*out);
	MY_DEBUG(idisplay,"len = %d,err = %d\n",len ,err);
	if(err)
		return err;

	(*uni)++;
	(*uni_left) -= len;
	
	*out += len;
	*out_left -= len;

	return 0;
}


int exfat_conv_u2c_1(struct nls_table *nls,
		   const __le16 **uni, unsigned int *uni_left,
		   unsigned char **out, unsigned int *out_left)
{
	wchar_t wc;
	int len;

	int idisplay = 1;

	while (*uni_left) {
		wc = le16_to_cpu(**uni);
		if (!wc)
			return -EINVAL;

		len = nls->uni2char(wc, *out, *out_left);
		MY_DEBUG(idisplay,"wc = %x, *out = %x,*out_left = %d,len=%d \n",wc, *out, *out_left,len);
		if (len < 0)
			return len;

		(*uni)++;
		(*uni_left)--;
		*out += len;
		*out_left -= len;
	}

	return 0;
}

int exfat_conv_c2u(struct nls_table *nls,
		   const unsigned char **in, unsigned int *in_left,
		   wchar_t **uni, unsigned int *uni_left)
{
	int len,err = 0;
	int i ;

	int idisplay = 0;

	len = exfat_utf8_to_utf16(&err, *in, *in_left, uni);
	MY_DEBUG(idisplay,"char = %s \n",*in);
	for( i = 0 ; i < *in_left ; i ++ )
		MY_DEBUG(idisplay," %x ",*uni + i);
	MY_DEBUG(idisplay,"len=%d,err=%d\n",len,err);
	if(err)
		return err;

	*in += len;
	*in_left -= len;

	(*uni)+= len;
	(*uni_left)-= len;

	return 0;
}


int exfat_conv_c2u_1(struct nls_table *nls,
		   const unsigned char **in, unsigned int *in_left,
		   wchar_t **uni, unsigned int *uni_left)
{
	int len;
	int i ;

	while (*in_left) {
		if (!*uni_left)
			return -ENAMETOOLONG;

		len = nls->char2uni(*in, *in_left, *uni);
		if (len < 0)
			return len;

		*in += len;
		*in_left -= len;
		(*uni)++;
		(*uni_left)--;
	}

	return 0;
}


/*
 * The epoch of exFAT timestamp is 1980.
 *     :  bits :     value
 * date:  0 -  4: day	(1 -  31)
 * date:  5 -  8: month	(1 -  12)
 * date:  9 - 15: year	(0 - 127) from 1980
 * time:  0 -  4: sec	(0 -  29) 2sec counts
 * time:  5 - 10: min	(0 -  59)
 * time: 11 - 15: hour	(0 -  23)
 */
#define SECS_PER_MIN	60
#define SECS_PER_HOUR	(60 * 60)
#define SECS_PER_DAY	(SECS_PER_HOUR * 24)
#define UNIX_SECS_1980	315532800L
#if BITS_PER_LONG == 64
#define UNIX_SECS_2108	4354819200L
#endif
/* days between 1.1.70 and 1.1.80 (2 leap days) */
#define DAYS_DELTA	(365 * 10 + 2)
/* 120 (2100 - 1980) isn't leap year */
#define YEAR_2100	120
#define IS_LEAP_YEAR(y)	(!((y) & 3) && (y) != YEAR_2100)

/* Linear day numbers of the respective 1sts in non-leap years. */
static time_t days_in_year[] = {
	/* Jan  Feb  Mar  Apr  May  Jun  Jul  Aug  Sep  Oct  Nov  Dec */
	0,   0,  31,  59,  90, 120, 151, 181, 212, 243, 273, 304, 334, 0, 0, 0,
};

/* Convert a exFAT time/date pair to a UNIX date (seconds since 1 1 70). */
void exfat_time_exfat2unix(struct timespec *ts, __le16 __time, __le16 __date,
			   u8 time_cs)
{
	u16 time = le16_to_cpu(__time), date = le16_to_cpu(__date);
	time_t second, day, leap_day, month, year;

	year  = date >> 9;
	month = max(1, (date >> 5) & 0xf);
	day   = max(1, date & 0x1f) - 1;

	leap_day = (year + 3) / 4;
	if (year > YEAR_2100)		/* 2100 isn't leap year */
		leap_day--;
	if (IS_LEAP_YEAR(year) && month > 2)
		leap_day++;

	second =  (time & 0x1f) << 1;
	second += ((time >> 5) & 0x3f) * SECS_PER_MIN;
	second += (time >> 11) * SECS_PER_HOUR;
	second += (year * 365 + leap_day
		   + days_in_year[month] + day
		   + DAYS_DELTA) * SECS_PER_DAY;

	second += sys_tz.tz_minuteswest * SECS_PER_MIN;

	if (time_cs) {
		ts->tv_sec = second + (time_cs / 100);
		ts->tv_nsec = (time_cs % 100) * 10000000;
	} else {
		ts->tv_sec = second;
		ts->tv_nsec = 0;
	}
}

/* Convert linear UNIX date to a exFAT time/date pair. */
void exfat_time_unix2exfat(struct timespec *ts, __le16 *time, __le16 *date,
			   u8 *time_cs)
{
	time_t second = ts->tv_sec;
	time_t day, leap_day, month, year;

	second -= sys_tz.tz_minuteswest * SECS_PER_MIN;

	/* Jan 1 GMT 00:00:00 1980. But what about another time zone? */
	if (second < UNIX_SECS_1980) {
		*time = 0;
		*date = cpu_to_le16((0 << 9) | (1 << 5) | 1);
		if (time_cs)
			*time_cs = 0;
		return;
	}
#if BITS_PER_LONG == 64
	if (second >= UNIX_SECS_2108) {
		*time = cpu_to_le16((23 << 11) | (59 << 5) | 29);
		*date = cpu_to_le16((127 << 9) | (12 << 5) | 31);
		if (time_cs)
			*time_cs = 199;
		return;
	}
#endif

	day = second / SECS_PER_DAY - DAYS_DELTA;
	year = day / 365;
	leap_day = (year + 3) / 4;
	if (year > YEAR_2100)		/* 2100 isn't leap year */
		leap_day--;
	if (year * 365 + leap_day > day)
		year--;
	leap_day = (year + 3) / 4;
	if (year > YEAR_2100)		/* 2100 isn't leap year */
		leap_day--;
	day -= year * 365 + leap_day;

	if (IS_LEAP_YEAR(year) && day == days_in_year[3]) {
		month = 2;
	} else {
		if (IS_LEAP_YEAR(year) && day > days_in_year[3])
			day--;
		for (month = 1; month < 12; month++) {
			if (days_in_year[month + 1] > day)
				break;
		}
	}
	day -= days_in_year[month];

	*time = cpu_to_le16(((second / SECS_PER_HOUR) % 24) << 11
			    | ((second / SECS_PER_MIN) % 60) << 5
			    | (second % SECS_PER_MIN) >> 1);
	*date = cpu_to_le16((year << 9) | (month << 5) | (day + 1));
	if (time_cs)
		*time_cs = (ts->tv_sec & 1) * 100 + ts->tv_nsec / 10000000;
}

void exfat_fs_panic(struct super_block *sb, const char *fmt, ...)
{
	va_list args;

	printk(KERN_ERR "exFAT: Filesystem panic (dev %s)\n", sb->s_id);

	printk(KERN_ERR "       ");
	va_start(args, fmt);
	vprintk(fmt, args);
	va_end(args);
	printk("\n");

	if (!(sb->s_flags & MS_RDONLY)) {
		sb->s_flags |= MS_RDONLY;
		printk(KERN_ERR "       File system has been set read-only\n");
	}
}

void exfat_dump_bh(struct buffer_head  *bh)
{
	int isize = 512 , i = 0;
	printk("[exfat_dump_bh] begin \n");
	for(i = 0 ; i < isize ; i ++)
		printk(" %x ",bh->b_data[i]);
	printk("[exfat_dump_bh] end \n");	
} 


static int utf16_to_utf8_size(int *p_errno, const __le16 *ins, const int ins_len, int outs_len)
{
	int i, ret = -1;
	int count = 0;
	int surrog;

	surrog = 0;
	for (i = 0; i < ins_len && ins[i]; i++) {
		unsigned short c = le16_to_cpu(ins[i]);
		if (surrog) {
			if ((c >= 0xdc00) && (c < 0xe000)) {
				surrog = 0;
				count += 4;
			} else 
				goto fail;
		} else
			if (c < 0x80)
				count++;
			else if (c < 0x800)
				count += 2;
			else if (c < 0xd800)
				count += 3;
			else if (c < 0xdc00)
				surrog = 1;
#if NOREVBOM
			else if ((c >= 0xe000) && (c < 0xfffe))
#else
			else if (c >= 0xe000)
#endif
				count += 3;
			else 
				goto fail;
		if (count > outs_len) {
			*p_errno = ENAMETOOLONG;
			goto out;
		}
	}
	if (surrog) 
		goto fail;

	ret = count;
out:
	return ret;
fail:
	*p_errno = EILSEQ;
	goto out;
}

/*
 * exfat_utf16_to_utf8 - convert a little endian UTF16LE string to an UTF-8 string
 * @ins:	input utf16 string buffer
 * @ins_len:	length of input string in utf16 characters
 * @outs:	on return contains the (allocated) output multibyte string
 * @outs_len:	length of output buffer in bytes
 *
 * Return -1 with errno set if string has invalid byte sequence or too long.
 */
int exfat_utf16_to_utf8(	int *p_errno, const __le16 *ins, 
							const int ins_len,
			      				char **outs, int outs_len)
{
	char *t;
	int i, ret = -1;
	__le16 halfpair;

	halfpair = 0;

	if (!*outs) 
		DEBUG_ON(1,"bug");

//	size = utf16_to_utf8_size(p_errno, ins, ins_len, outs_len);

	t = *outs;

	for (i = 0; i < ins_len && ins[i]; i++) {
	    unsigned short c = le16_to_cpu(ins[i]);
			/* size not double-checked */
		if (halfpair) {
			if ((c >= 0xdc00) && (c < 0xe000)) {
				*t++ = 0xf0 + (((halfpair + 64) >> 8) & 7);
				*t++ = 0x80 + (((halfpair + 64) >> 2) & 63);
				*t++ = 0x80 + ((c >> 6) & 15) + ((halfpair & 3) << 4);
				*t++ = 0x80 + (c & 63);
				halfpair = 0;
			} else 
				goto fail;
		} else if (c < 0x80) {
			*t++ = c;
	    	} else {
			if (c < 0x800) {
			   	*t++ = (0xc0 | ((c >> 6) & 0x3f));
			        *t++ = 0x80 | (c & 0x3f);
			} else if (c < 0xd800) {
			   	*t++ = 0xe0 | (c >> 12);
			   	*t++ = 0x80 | ((c >> 6) & 0x3f);
		        	*t++ = 0x80 | (c & 0x3f);
			} else if (c < 0xdc00)
				halfpair = c;
			else if (c >= 0xe000) {
				*t++ = 0xe0 | (c >> 12);
				*t++ = 0x80 | ((c >> 6) & 0x3f);
			        *t++ = 0x80 | (c & 0x3f);
			} else 
				goto fail;
	        }
	}
	*t = '\0';
	ret = t - *outs;
out:
	return ret;
fail:
	*p_errno = EILSEQ;
	goto out;
}


/* 
 * This converts one UTF-8 sequence to cpu-endian Unicode value
 * within range U+0 .. U+10ffff and excluding U+D800 .. U+DFFF
 *
 * Return the number of used utf8 bytes or -1 with errno set 
 * if sequence is invalid.
 */
static int utf8_to_unicode(int *p_errno, u32 *wc, const char *s, int ins_len)
{
    	unsigned int byte = *((const unsigned char *)s);

        if(ins_len < 0)
              return -1;
					/* single byte */
	if (byte == 0) {
		*wc = (u32) 0;
		return 0;
	} else if (byte < 0x80) {
		*wc = (u32) byte;
		return 1;
					/* double byte */
	} else if (byte < 0xc2) {
		goto fail;
	} else if (byte < 0xE0) {
		if (ins_len < 2)
			goto fail;
		if ((s[1] & 0xC0) == 0x80) {
			*wc = ((u32)(byte & 0x1F) << 6)
			    | ((u32)(s[1] & 0x3F));
			return 2;
		} else
			goto fail;
					/* three-byte */
	} else if (byte < 0xF0) {
		if (ins_len < 3)
			goto fail;
		if (((s[1] & 0xC0) == 0x80) && ((s[2] & 0xC0) == 0x80)) {
			*wc = ((u32)(byte & 0x0F) << 12)
			    | ((u32)(s[1] & 0x3F) << 6)
			    | ((u32)(s[2] & 0x3F));
			/* Check valid ranges */
#if NOREVBOM
			if (((*wc >= 0x800) && (*wc <= 0xD7FF))
			  || ((*wc >= 0xe000) && (*wc <= 0xFFFD)))
				return 3;
#else
			if (((*wc >= 0x800) && (*wc <= 0xD7FF))
			  || ((*wc >= 0xe000) && (*wc <= 0xFFFF)))
				return 3;
#endif
		}
		goto fail;
					/* four-byte */
	} else if (byte < 0xF5) {
		if (ins_len < 4)
			goto fail;
		if (((s[1] & 0xC0) == 0x80) && ((s[2] & 0xC0) == 0x80)
		  && ((s[3] & 0xC0) == 0x80)) {
			*wc = ((u32)(byte & 0x07) << 18)
			    | ((u32)(s[1] & 0x3F) << 12)
			    | ((u32)(s[2] & 0x3F) << 6)
			    | ((u32)(s[3] & 0x3F));
		/* Check valid ranges */
		if ((*wc <= 0x10ffff) && (*wc >= 0x10000))
			return 4;
		}
		goto fail;
	}
fail:
	*p_errno = EILSEQ;
	return -1;
}



static int utf8_to_utf16_size(int *p_errno, const char *s, int ins_len)
{
	int ret = -1;
	unsigned int byte;
	size_t count = 0;
    
       if(ins_len <= 0)
           goto out;

	while (ins_len-- && (byte = *((const unsigned char *)s++))) {
		if (++count >= PATH_MAX) 
			goto fail;
		if (byte >= 0xF5) {
			*p_errno = EILSEQ;
			goto out;
		}
		if (!ins_len ||!*s) 
			break;
		if (byte >= 0xC0) 
			s++, ins_len--;
		if (!ins_len ||!*s) 
			break;
		if (byte >= 0xE0) 
			s++, ins_len--;
		if (!ins_len ||!*s) 
			break;
		if (byte >= 0xF0) {
			s++, ins_len--;
			if (++count >= PATH_MAX)
				goto fail;
		}
	}
	ret = count;
out:
	return ret;
fail:
	*p_errno = ENAMETOOLONG;
	goto out;
}


/**
 * exfat_utf8_to_utf16 - convert a UTF-8 string to a UTF-16LE string
 * @ins:	input multibyte string buffer
 * @outs:	on return contains the (allocated) output utf16 string
 * @outs_len:	length of output buffer in utf16 characters
 * 
 * Return -1 with errno set.
 */
int exfat_utf8_to_utf16(int *p_errno, const char *ins, int ins_len, __le16 **outs)
{
	const char *t = ins;
	u32 wc;
	__le16 *outpos;
	int shorts, ret = -1;

       if(ins_len <= 0)
           goto fail;

	shorts = utf8_to_utf16_size(p_errno, ins, ins_len);
	if (shorts < 0)
		goto fail;

	if (!*outs) 
		DEBUG_ON(1,"byg");

	outpos = *outs;

	while(ins_len>0) {
		int m  = utf8_to_unicode(p_errno, &wc, t, ins_len);
		if (m < 0)
			goto fail;
		if (wc < 0x10000)
			*outpos++ = cpu_to_le16(wc);
		else {
			wc -= 0x10000;
			*outpos++ = cpu_to_le16((wc >> 10) + 0xd800);
			*outpos++ = cpu_to_le16((wc & 0x3ff) + 0xdc00);
		}
		if (m == 0)
			break;
		t += m;
              ins_len -= m;
	}
	
	ret = outpos - *outs;
fail:
	return ret;
}

