/*
 * debug.h - NTFS kernel debug support. Part of the Linux-NTFS project.
 *
 * Copyright (c) 2001-2004 Anton Altaparmakov
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _LINUX_NTFS_DEBUG_H
#define _LINUX_NTFS_DEBUG_H

#include <linux/fs.h>


#define ntfs_log_debug(format, a...)   do{}while(0)
#define ntfs_log_trace(format, a...)   do{}while(0)
#define ntfs_log_perror(format, a...)   do{}while(0)
#define ntfs_log_error(format, a...)   do{}while(0)
#define ntfs_log_enter(format, a...) do{}while(0)
#define ntfs_log_leave(format, a...) do{}while(0)
#define ntfs_log_info(format, a...) do{}while(0)
#define ntfs_log_warning(format, a...) do{}while(0)


s64 ntfs_pread(struct super_block *sb, const s64 pos, s64 count, void *b);
s64 ntfs_pwrite(struct super_block *sb, const s64 pos, s64 count,
		const void *b);


extern void __ntfs3g_warning(const char *function, const struct super_block *sb,
		const char *fmt, ...);
#define ntfs3g_warning(sb, f, a...)	__ntfs3g_warning(__func__, sb, f, ##a)

extern void __ntfs_error(const char *function, const struct super_block *sb,
		const char *fmt, ...);
#define ntfs3g_error(sb, f, a...)		__ntfs3g_error(__func__, sb, f, ##a)

#define NTFS3G_BUG_TRACE( state)  \
do{  \
    if(unlikely(state))   \
   {    \
       printk("%s:%d:\n", __FILE__, __LINE__);    \
       printk("    assert(%s)\n", #state);    \
       BUG();   \
   }    \
}while(0)

#endif /* _LINUX_NTFS_DEBUG_H */
